use Test::Simple tests => 11;
use strict;

use DE_EPAGES::Core::API::Script qw (RunScript);
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject ExistsObjectByPath LoadObjectByPath );
use DE_EPAGES::XML::API::DateTimeFormatXML;

my $Parser = DE_EPAGES::XML::API::DateTimeFormatXML->new();
my $ROOTPATH = '/Test';

sub Main {
    ConnectTestSite();

    ok( Init(), "initialization" );

    my $Calendar = LoadObjectByPath($ROOTPATH);

    # 10:00 locked 10:30
    $Calendar->setLock({
        'StartDate' => $Parser->parse_datetime('2007-10-15T10:00:00'),
        'EndDate'   => $Parser->parse_datetime('2007-10-15T10:30:00'),
        'IsLocked'  => 1
    });
    ok(  $Calendar->isLocked($Parser->parse_datetime('2007-10-15T10:00:00')), '10:00 locked');
    ok( !$Calendar->isLocked($Parser->parse_datetime('2007-10-15T10:30:00')), '10:30 not locked');
    ok(  $Calendar->isLocked($Parser->parse_datetime('2007-10-15T10:15:00')), '10:15 locked');

    ok(  !$Calendar->isLocked(
        $Parser->parse_datetime('2007-10-15T09:30:00'),
        $Parser->parse_datetime('2007-10-15T10:00:00'),
    ), '09:30-10:00 not locked');

    my $DayList = $Calendar->dayList($Parser->parse_datetime('2007-10-15T10:30:00'));
    ok( scalar (grep { $_->{'IsLocked'} } @$DayList) == 1, 'one entry is locked');
    ok( $DayList->[20]->{'IsLocked'}, 'time 10:00 until 10:30 is locked');

    $Calendar->set({'AppointmentInterval'=>'15'}); # was 30 (default) before
    $DayList = $Calendar->dayList($Parser->parse_datetime('2007-10-15T10:30:00'));
    ok( scalar (grep { $_->{'IsLocked'} } @$DayList) == 2, 'two entries are locked on 15min step');
    ok( $DayList->[40]->{'IsLocked'}, 'time 10:00 until 10:15 is locked');
    ok( $DayList->[41]->{'IsLocked'}, 'time 10:15 until 10:30 is locked');

    ok( Cleanup(), "cleanup" );

    DisconnectSite();
}

sub Cleanup {
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    Cleanup();
    LoadRootObject()->insertChild('Calendar', {'Alias'=>'Test'});
    return 1;
}

RunScript(Sub=>\&Main, Debug=>1);
