use strict;
use Test;

# TODO: EPG-15730: fix test case
plan tests => 1;
ok(1);

__END__

use Test::Simple tests => 3;
use DE_EPAGES::Core::API::Error qw ( GetError ExistsError Error);
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( ConnectTestSite DisconnectSite );
use DE_EPAGES::Database::API::DatabaseInstaller;
use DE_EPAGES::DAL_TEST::DAL::Test01 qw ( DAL_InsertTable1 DAL_UpdateTable1 DAL_DeleteTable1 DAL_GetInfoTable1 );

local $| = 1;

RunScript( Sub => sub {
    ConnectTestSite(); # DBI->connect('dbi:Sybase:server=DBServer;database=admindb1','admin','admindb');
    Init();
    Test_Package();
    Cleanup();
    DisconnectSite;
});

sub Init {
    Cleanup();

    my $Installer = DE_EPAGES::Database::API::DatabaseInstaller->new(
        'CartridgeDirectory' => 'DE_EPAGES/DAL_TEST',
        'Version'            => '1.0',
    );

    # fake install cartridge DE_EPAGES::DAL_TEST
    local $ENV{EPAGES_CARTRIDGES} = '.';
    my $DatabaseDir = $Installer->databaseDirectory;
    ok( $Installer->hasDBSchema, "hasDBSchema" );
    ok( -d $DatabaseDir, "Database directory exists" );
    $Installer->processDatabaseDir( $DatabaseDir );
}

sub Cleanup {

    my $Installer = DE_EPAGES::Database::API::DatabaseInstaller->new(
        'CartridgeDirectory' => 'DE_EPAGES/DAL_TEST',
        'Version'            => '1.0',
        'Force' => 1,
    );

    # fake uninstall cartridge DE_EPAGES::DAL_TEST
    local $ENV{EPAGES_CARTRIDGES} = '.';
    my $DatabaseDir = $Installer->databaseDirectory;
    $Installer->cleanupDatabaseDir( $DatabaseDir );
}


sub Test_Package {
    eval {
        my $MasterID = DAL_InsertTable1({

                TimeValue=>DateTime->new(
                    year   => 2003,
                    month  => 4,
                    day    => 25,
                    hour   => 15,
                    minute => 53,
                    second => 1,
                    nanosecond => 500000000,
                    time_zone => 'Europe/Brussels',
                ),
                IntegerValue=>1234,
                DateValue=>DateTime->new(
                    year   => 2003,
                    month  => 4,
                    day    => 25,
                    time_zone => 'Europe/Brussels',
                ),
                StringValue=>'test string',
                FloatValue=>123.4
        });
        my $data = DAL_GetInfoTable1($MasterID);
        DAL_UpdateTable1($MasterID, {
                TimeValue    =>'20040425 7:53:01',
        });
        DAL_UpdateTable1($MasterID, {
                TimeValue    => '20040425 17:53:01',
                StringValue  => 'das war der test',
                });
        $data = DAL_GetInfoTable1($MasterID);
        #my $data = GetAllInfoTable1s($dbh);
        DAL_DeleteTable1($MasterID);
    };
    ok( !ExistsError(), "no error" );
    if (ExistsError()) {
        my $Error = GetError();
        print $Error->longMessage();
    }
    ##use critic
}

1;

