use strict;
use Test;

# TODO: EPG-15730: fix test case
plan tests => 1;
ok(1);

__END__

use DE_EPAGES::Core::API::Error qw ( GetError ExistsError Error);
use DE_EPAGES::Database::API::Connection qw ( ConnectToSite );
use DE_EPAGES::DAL_TEST::DAL::Sybase::Test01;
use DE_EPAGES::DAL_TEST::DAL::Sybase::Test02;

BEGIN {
    local $SIG{__DIE__}  = \&DE_EPAGES::Core::API::Error::DieErrorHandler;
    local $SIG{__WARN__} = \&DE_EPAGES::Core::API::Error::DieErrorHandler;
    push @INC, '.';
    plan tests => 2;
};

sub Main {
    Initialize();
    ok ( Test_Package1() );
    ok ( Test_Package2() );
}

sub ErrorDefinition { return {
    NO_EQUAL_LINE => "Output is different at line #AtLine"
}; }

sub Test_Package1 {
    eval {
        ##no critic qw(ProhibitMissingImports)
        my $DAL='DE_EPAGES::DAL_TEST::DAL::Sybase::Test01';
        my $dbh = ConnectToSite(); # DBI->connect('dbi:Sybase:server=DBServer;database=admindb1','admin','admindb');
        my $MasterID = $DAL->InsertTable1($dbh, {
                TimeValue=>'20030425 15:53:01',
                IntegerValue=>1234,
                DateValue=>'20030425',
                StringValue=>'test string',
                FloatValue=>123.4
        });
        my $data = $DAL->GetInfoTable1($dbh, $MasterID);
        $DAL->UpdateTable1($dbh, $MasterID, {
                TestTimeValue    =>'20040425 7:53:01',
        });
        $DAL->UpdateTable1($dbh, $MasterID, {
                TimeValue    => '20040425 17:53:01',
                StringValue  => 'das war der test',
                });
        $data = $DAL->GetInfoTable1($dbh, $MasterID);
        $data = $DAL->GetAllInfoTable1s($dbh);
        $data = $DAL->GetAllPKeyTable1s($dbh);
        $DAL->DeleteTable1($dbh, $MasterID);
        $dbh->disconnect;
        ##use critic
    };
    if (ExistsError()) {
        my $Error = GetError();
        print $Error->longMessage();
        return 0;
    }
    return 1;
}

my @TABLE2_DATA = (
    {
        'MasterID'  => 1,
        'MasterID2' => 2,
        'MasterID3' => 3,
        'Name'      => 5,
        'Name2'     => 5,
        'Name3'     => 6,
    },{
        'MasterID'  => 1,
        'MasterID2' => 3,
        'MasterID3' => 4,
        'Name'      => 5,
        'Name2'     => 6,
        'Name3'     => 7,
    }
);
sub Test_Package2 {
    eval {
        my $dbh = ConnectToSite(); # DBI->connect('dbi:Sybase:server=DBServer;database=admindb1','admin','admindb');
        my $DAL='DE_EPAGES::DAL_TEST::DAL::Sybase::Test02';
        foreach my $data (@TABLE2_DATA) {
            $DAL->InsertTable2($dbh, $data);
        }
        my $data = $DAL->GetAllInfoTable2s($dbh);
        Error('WRONG_COUNT') unless scalar @$data == 2;

        $data = $DAL->GetInfoTable2sByName($dbh, 5);
        Error('WRONG_COUNT') unless scalar @$data == 2;

        $data = $DAL->GetInfoTable2sByName($dbh, 5, 6);
        Error('WRONG_COUNT') unless scalar @$data == 1;

        $data = $DAL->GetInfoTable2sByMasterID($dbh, 1);
        Error('WRONG_COUNT') unless scalar @$data == 2;

        $data = $DAL->GetInfoTable2sByMasterID($dbh, 1, 2);
        Error('WRONG_COUNT') unless scalar @$data == 1;

        foreach my $data (@TABLE2_DATA) {
            $DAL->DeleteTable2($dbh, @$data{qw (MasterID MasterID2 MasterID3)});
        }
        $dbh->disconnect;
    };
    if (ExistsError()) {
        my $Error = GetError();
        print $Error->longMessage();
        return 0;
    }
    return 1;
}

sub Initialize {
    ##no critic qw(ProhibitPackageVars)
    $bin::ISSRV::AppServerIP = 12.12.12.12 if not defined $bin::ISSRV::AppServerIP;
    $bin::ISSRV::AppServerPort = 1234 if not defined $bin::ISSRV::AppServerPort;
    ##use critic
    local $ENV{SCRIPT_NAME} = 'UseDAL.t';
}

sub Cleanup {
}


local $| = 1;
Main(@ARGV);

1;

