use strict;
use Test::Simple tests => 19;
use Perl::Critic;
use DE_EPAGES::Core::API::Script qw( RunScript );
use lib ("$ENV{EPAGES_CARTRIDGES}/DE_EPAGES/Core/Scripts");
use Perl::Critic::Policy::DE_EPAGES::Security::ProhibitVariablesInSQLStatements;

# Tests Perl::Critic::Policy::DE_EPAGES::Security::ProhibitVariablesInSQLStatements
#

sub Main {

    my $critic = Perl::Critic->new(
        -theme    => 'epages',
        -severity => 5,
        -profile  => 'NONE',
    );
    ok( defined $critic, "Perl::Critic initialized" );

    $critic->add_policy( -policy => 'DE_EPAGES::Security::ProhibitVariablesInSQLStatements' );
    my $policyLoaded = scalar grep { $_ =~ 'DE_EPAGES::Security::ProhibitVariablesInSQLStatements' }
    $critic->policies();
    ok( $policyLoaded, "Policy loaded" );

    # =======================================================================================
    #     Injection Tests
    # =======================================================================================
    ##no critic qw(ProhibitImplicitNewlines ProhibitMissingImports ProhibitUnusedVariables)
    my $sourceCode = q(
        $dbi->do("SELECT from table" .
        " where name = $OverManLines" .
        " AND was = anderes");
    );
    ok( $critic->critique( \$sourceCode ), "Variable in SQL select over more than one line." );

    $sourceCode = q(
        my $sqlString = "inSert INTO $was";
    );
    ok( !$critic->critique( \$sourceCode ), "Variable as table name - OK!." );

    $sourceCode = q(
        my $sqlString = "inSert INTO $was values ($value)";
    );
    ok( $critic->critique( \$sourceCode ), "Variable in q SQL Select." );

    $sourceCode = q(
        my $sqlString = "inSert INTO $was SELECT * FROM othertable";
    );
    ok( !$critic->critique( \$sourceCode ), "Subselect" );

    $sourceCode = q(
        my $sqlString = "inSert INTO $was SELECT * FROM othertable WHERE val = $val;
    ); # "
    ok( $critic->critique( \$sourceCode ), "Subselect with variable" );

    $sourceCode = q(
        my $kompliziert = qq(SELECT
        FROM what
        WHERE $InQQString = 10
        AND x0y
        BLA);
    );
    ok( $critic->critique( \$sourceCode ), "Variable in qq SQL select." );

    $sourceCode = q(
        die "Missing attribute alias for index $wrong" unless defined $wrong ;
    );
    ok( !$critic->critique( \$sourceCode ), "Variable in die, not SQL." );

    $sourceCode = q(
        print "This is not an mysql update, $wrong dude!";
    );
    ok( !$critic->critique( \$sourceCode ), "Variable in print, not SQL." );

    $sourceCode = q(
        $dbi->do('INSERT INTO mailtypetemplate (mailtypetemplateid, pagetypeid, hastofield, hassubject, hasadditionaltext, isalwaysactive, hasheader) VALUES (?, ?, ?, ?, ?, ?, ?)',
             [ @{$wrong}{qw( MailTypeTemplateID PageTypeID HasToField HasSubject HasAdditionalText IsAlwaysActive HasHeader )} ]
    );
    );
    ok( !$critic->critique( \$sourceCode ), "SQL with ?" );

    my ( $table, $column );
    $sourceCode = "alter table $table DROP $column";

    ok( !$critic->critique( \$sourceCode ), "SQL with ? before WHERE" );

    $sourceCode = <<'END_CODE';
TestInteger( 'ShopID', $ShopID );
$dbi->createTempTable('export_ids (objectid int)');
$dbi->execute("insert into $exportids (objectid) select objectid from object where siteid = ?", [$ShopID]);
END_CODE

    ok( !$critic->critique( \$sourceCode ),
        "SQL with matching previous lines (ShopTransferService)" );

    $sourceCode = q (
        $dbi->do("delete from $tableids where objectid in (select d.objectid from $deleteids d)");
    );
    ok( !$critic->critique( \$sourceCode ), "Subselect SQL with allowed variables" );

    $sourceCode = q (
        $dbi->do("delete from $tableids where objectid in (select d.objectid from $deleteids d where a = $var)");
    );
    ok( $critic->critique( \$sourceCode ), "Subselect SQL with variable" );

    $sourceCode = q (
        $dbi->do("delete from $tableids where a = $b and objectid in (select d.objectid from $deleteids d where a = b)");
    );
    ok( $critic->critique( \$sourceCode ), "Subselect SQL with variable in first select" );

    $sourceCode = q (
        $dbi->do("delete from $tableids where objectid in (select d.objectid from $deleteids d where a = b) and a = $b");
    );
    ok( $critic->critique( \$sourceCode ),
        "Subselect SQL with variable in first select after subquery" );

    $sourceCode = q (
        $dbi->do("delete from $tableids where objectid = " . $b);
    );
    ok( $critic->critique( \$sourceCode ), "Concatenated variable" );

    $sourceCode = q (
        $dbi->do('delete from $tableids2 where objectid = ' . $b);
    );
    ok( $critic->critique( \$sourceCode ), "Concatenated variable (2)" );

    #    $sourceCode = q (
    #        $aaResult = $dbi->execute("select count(1) from $Table where $Column = ?",[$SiteID]);
    #    );
    #    ok( !$critic->critique(\$sourceCode), "Select with variable as column name (valid)" );
}

RunScript( Sub => \&Main );
