use strict;
use Test::More;
use Perl::Critic;
use DE_EPAGES::Core::API::Script qw( RunScript );
use lib ("$ENV{EPAGES_CARTRIDGES}/DE_EPAGES/Core/Scripts");
use Perl::Critic::Policy::DE_EPAGES::Variables::ProhibitUnusedVariablesEnhanced;

# Tests Perl::Critic::Policy::DE_EPAGES::Variables::ProhibitUnusedVariablesEnhanced
#

sub Main {

    my $critic = Perl::Critic->new(
        -profile  => 'NONE',
        -severity => 2
    );
    ok( defined $critic, "Perl::Critic initialized" );
    $critic->add_policy( -policy => 'Perl::Critic::Policy::DE_EPAGES::Variables::ProhibitUnusedVariablesEnhanced' );
    my @policies = $critic->policies();
    my $policyLoaded = scalar grep { $_ =~ 'DE_EPAGES::Variables::ProhibitUnusedVariablesEnhanced' }
    @policies;
    ok( $policyLoaded, "Policy loaded" );
    
    my @violations;
    my $sourceCode;
    
    ##no critic qw( ProhibitImplicitNewlines )

    $sourceCode = q(
    use strict;
        sub x {
            # shifted arguments should be ok.
            my ( $c , $d ) = @_;
        }
    );

    @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 0, "shifted arguments in sub. ") or diag(@violations);

    $sourceCode = q(
    use strict;
        sub x {
            my $x = '10';
        }
    );

    my @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 1, "simple unused variable.") or diag(@violations);

    $sourceCode = q(
    use strict;
        sub x {
            my $x = 10;
            $x = $x + 1;
        }
    );

    my @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 0, "simple used variable") or diag(@violations);

    $sourceCode = q(
    use strict;
        sub x {
            my $x = 10;
            while (false) {
                $x += 1;
            }
        }
    );

    my @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 0, "simple used variable in loop") or diag(@violations);

    $sourceCode = q(
    use strict;
        sub x {
            my $x;
        }
    );

    my @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 1, "uninitialized variable") or diag(@violations);

    $sourceCode = q(
    use strict;
        sub x {
            my $x;
        }
        sub y {
            my $x = 10;
        }
    );

    my @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 2, "uninitialized variable in different subs") or diag(@violations);

    $sourceCode = q(
    use strict;
        sub x {
            my $x = "abc";
            print qq/ $x /;
        }
    );

    my @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 0, "general quote operator") or diag(@violations);

    $sourceCode = q(
    use strict;
        sub x {
            my $x = "abc";
            print "test" if (/\Q$x\E/);
        }
    );

    my @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 0, "variable in regex") or diag(@violations);

    $sourceCode = q(
    use strict;
        sub x {
            my $x = "abc";
            print q/ $x /;
        }
    );

    my @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 1, "non-interpolating quote operator") or diag(@violations);

    $sourceCode = q(
    use strict;
        sub x {
            my $x = "abc";
            print `$x`;
        }
    );

    my @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 0, "quotelike operator (backticks)") or diag(@violations);

    $sourceCode = q(
    use strict;
        sub x {
            my @abc = qw ( test qwer );
            print $abc[0];
        }
    );

    @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 0, "array element") or diag(@violations);

    my $sourceCode = q(
    use strict;
        sub x {
            my %abc = (test => 'test1' );
            print "${abc{test}}";
        }
    );

    @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 0, "hash element") or diag(@violations);

    my $sourceCode = q(
    use strict;
        sub x {
            my $abc = "abc";
            print "${abc}";
        }
    );

    my @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 0, "bracketed variable in string") or diag(@violations);

    $sourceCode = q(
    use strict;
        sub x {
            my @abc = qw( 123 );
            print "$abc[0]";
        }
    );

    my @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 0, "array element in string") or diag(@violations);

    $sourceCode = q(
    use strict;
        sub x {
            my @abc = qw( 123 );
            print "${abc[0]}";
        }
    );

    my @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 0, "bracketed array element in string") or diag(@violations);

    $sourceCode = q(
    use strict;
        sub x {
            my ( $c , $d ) = @_;
            my ($e , $f);
            
            my $a = "";
            my $b = 2;
            print <<EOL;
            Test $e Blbalba $a
EOL
            my $x = '1';
            my $y;
            my @z = ();
            if ($x eq '1') {
                my $bla = 'test' . $y . $z[0] . " $a $b ";
            }
        }
    );

    @violations = $critic->critique( \$sourceCode );
    is(scalar @violations, 2, "heredocument and some mixed code. ") or diag(@violations);
}

RunScript( Sub => \&Main );
done_testing();