use Test::More tests => 1;

use strict;
use DE_EPAGES::Database::API::Connection qw ( Connect );
use DE_EPAGES::Core::API::Script qw( RunScript );

my $DataSource = 'dbi:Sybase:charset=utf8;server=DBServer;database=sitedb';
my ($Login, $Password, $Site) = ('usr_sitedb', 'epages', 'Site');
my $dbi;
##no critic qw(ProhibitImplicitNewlines)
my %Statements = (
    'Pre'   => [
        'CREATE TABLE o (id INTEGER PRIMARY KEY, value VARCHAR(30) NOT NULL)',
        'CREATE TABLE p (id INTEGER PRIMARY KEY)',
        'CREATE TABLE pp (id INTEGER PRIMARY KEY)',
        'delete from pp',
        'delete from p',
        'delete from o',
        'INSERT INTO o VALUES (0, \'Base\')',
        'INSERT INTO o VALUES (1, \'Hallo\')',
        'INSERT INTO p VALUES (1)',
    ],
    'User1' => [
        'DECLARE @ObjectID             int
            SELECT @ObjectID = 1 + max(id) FROM o HOLDLOCK
            INSERT INTO o VALUES (@ObjectID, \'User1\')
            SELECT @ObjectID',
        'INSERT INTO p VALUES (?)',
    ],
    'User2' => [
        'DELETE FROM p WHERE id=1',
        'DELETE FROM o WHERE id=1',
        'DECLARE @ObjectID             int
            SELECT @ObjectID = 1 + max(id) FROM o HOLDLOCK
            INSERT INTO o VALUES (@ObjectID, \'User2\')
            SELECT @ObjectID',
        'INSERT INTO pp VALUES (?)'

    ],
    'Post'   => [
        'DROP TABLE pp',
        'DROP TABLE p',
        'DROP TABLE o',
    ],
);

sub Pre {
    Post(); # cleanup
    my $User = 'Pre';
    my $aStatements = $Statements{$User};
    $dbi->execute($_) foreach @$aStatements;
}

sub Post {
    my $User = 'Post';
    my $aStatements = $Statements{$User};
    eval { $dbi->execute($_) } foreach @$aStatements;
}

sub User1 {
    my $User = 'User1';
    my $aStatements = $Statements{$User};
    $dbi->transaction(sub {
        my $aaResult;my $Counter = 0;
        print "$User counter $Counter\n";
        $aaResult = $dbi->execute($aStatements->[$Counter++]);
        ok ($aaResult->[0][0] == 2, 'object 1 inserted');
        print "$User counter $Counter\n";
        sleep 7;

        $aaResult = $dbi->execute($aStatements->[$Counter++], [$aaResult->[0][0]]);
        print "$User counter $Counter\n";
        ok (1, 'product inserted');
        sleep 5;
    });
}

sub User2 {
    my $User = 'User2';
    my $aStatements = $Statements{$User};
    $dbi->transaction(sub {
        sleep 5;
        my $aaResult;my $Counter = 0;
        print "$User counter $Counter\n";
        $aaResult = $dbi->execute($aStatements->[$Counter++]);
        sleep 5;
        print "$User counter $Counter\n";
        $aaResult = $dbi->execute($aStatements->[$Counter++]);
        print "$User counter $Counter\n";
        $aaResult = $dbi->execute($aStatements->[$Counter++]);
        print "$User $aaResult->[0][0]\n";
        $aaResult = $dbi->execute($aStatements->[$Counter++], [$aaResult->[0][0]]);
        print "$User counter $Counter\n";
        sleep 5;
    });
}

RunScript(
    Debug=>1,
    Sub => sub {
        unless( scalar @ARGV ) {
            ok(1, "OK");
            exit 0;
        }
        $dbi = Connect($DataSource, $Login, $Password, $Site);
        foreach (@ARGV) {
            Pre()   if $_ == 0;
            User1() if $_ == 1;
            User2() if $_ == 2;
            Post()  if $_ == 3;
        }
    }
);


1;

