/*
	Copyright (c) 2006-2007, epages Software GmbH
	All Rights Reserved.

	epages.html $Revision: 1.6.4.1 $

*/
dojo.provide("epages.event.domdispatcher");
dojo.require("epages.html.element");
dojo.require("epages.html");

dojo.declare("epages.event.DomDispatcher",
	function() {
		this._events = {};
	},
	{

		_monitoredEvents: $A(),
		_events: null,

		add: function(/* String */eventType, /*function|DOMNode|Array.DOMNode*/nodeList, /*function*/callback, /*Boolean*/outside, /*Document|Document[]*/docs) {
			// description:
			//		add an event to be monitored and dispatched
			//	eventType:
			//		name of an event, like "click"
			//	nodeList:
			//		a single DOM node or an array of DOM nodes to be monitored
			//	outside:
			//		dispatch event if it occurs outside or inside the monitored nodes
			if (outside == null) outside = false;
			if (this._events[eventType] === undefined) this._events[eventType] = [];
			this._events[eventType].push({
				'outside': outside,
				'nodes': nodeList,
				'callback': callback
			});
			if (!this._monitoredEvents.exists(eventType)) this._registerEventHandler(eventType, docs);
		},

		_registerEventHandler: function(eventType, docs) {
			if (docs == null) docs = [document];
			if (!dojo.isArrayLike(docs)) docs = [docs];
			for (var i = 0; i < docs.length; i++) {
				$E(docs[i]).addEventListener(eventType, dojo.hitch(this, 'handleEvent'), true);
			}
			// avoid double entries and recursion
			if (!this._monitoredEvents.exists(eventType)) this._monitoredEvents.push(eventType);
		},

		_unregisterEventHandler: function(eventType, docs) {
			if (docs == null) docs = [document];
			if (!dojo.isArrayLike(docs)) docs = [docs];
			for (var i = 0; i < docs.length; i++) {
				$E(docs[i]).removeEventListener(eventType, dojo.hitch(this, 'handleEvent'), true);
			}
			// update monitor list
			var EventIndex = this._monitoredEvents.find(eventType);
			if (EventIndex != null) {
				this._monitoredEvents.remove(EventIndex);
			}
		},

		handleEvent: function(DomEvent) {
			// summary: if this happens, the window is probably not longer member of the dom
			if (window.epages == null) return;
			var events = this._events[DomEvent.type];
			if (events.length === undefined) return;
			for (var i = 0; i < events.length; i++) {
				var event = events[i];
				var nodes = event.nodes;
				if (typeof nodes == 'function') nodes = nodes();
				// dispatch event
				if (event.nodes != null && epages.html.isInnerEvent(DomEvent, nodes) != event.outside) {
					event.callback(DomEvent);
				}
			}
		}

	}
);

dojo.publish('epages/addGlobalSymbol', ['$D', window]);
if ($D === undefined) $D = new epages.event.DomDispatcher();

epages.onIframeStart.push(function(parentWin, iframeWin) {
	$D._monitoredEvents.each(function(el) {
		$D._registerEventHandler(el, iframeWin.document)
	});
});
