/* 
 * @charset			utf-8		(ÄÖÜ äöü ß)
 *
 * Copyright (c) 2006-2010, epages GmbH
 * All Rights Reservedojo.
 * epages.fx.kenburns $Revision: 1.1.2.3 $
 */
 
	dojo.provide('epages.fx.kenburns');
	dojo.require('epages.fx');
	
	dojo.declare('epages.fx.kenburns',[epages.fx._base],{
	// summary: special effect with zoom and move animation.
		
		/*
		 * init instance
		 */	
		constructor: function(/*node*/node,/*object?*/params ) {
			// summary: constructor
			// params: 	!Attention! if kenburns is used on an imageNode before the image source is loaded: params.width and params.height are required
			//					when no params are given than random values are used
			//
			// example:	
			//					|var example = new epages.fx.kenburns( node , {
			//					|	'duration':3000,		// duration in milliseconds
			//					|	'easing':'linear',	// easingType of dojo.fx.easing
			//					|	'width':800,				
			//					|	'height':600,					
			//					|	'zoom':[1.6,1.3],		// [ start, end ]	zoom factor: in use => [ start*width , end*width ] and [ start*height , end*height ] 	min:1 , max:2
			//					|	'focus':[0.2,0.8],	// [ x, y ]			focus point: in use => [ x*width , y*height ]  											min:0 , max:1
			//					|	'fadeIn':1000				// fadeInTime in milliseconds
			//					|	'fadeOut':1000			// fadeOutTime in milliseconds
			//					|} );
			//					|example.play();
			var params = dojo.mixin(
					{
						duration: 2000,
						easing: 'linear',
						focus: 'auto',
						zoom: 'auto',
						width: dojo.style( this.node , 'width' ),
						height: dojo.style( this.node , 'height' ),
						containerWidth: dojo.style( this.containerNode , 'width' ),
						containerHeight: dojo.style( this.containerNode , 'height' )
					}, 
					params
				);
			
			if( params.width && params.height ) {
				
				var proporties = {};
					
				// set original size
				var origWidth = params.width;
				var origHeight = params.height;
				// init size
				if( (params.width/params.height) <= (params.containerWidth/params.containerHeight) ) {
					params.width = params.containerWidth;
					params.height = params.height * params.width / origWidth;
				}
				else{
					params.width = params.width * params.height / origHeight;
					params.height = params.containerHeight;
				}
				// init zoom 
				if(typeof(params.zoom)=='array' || typeof(params.zoom)=='object') {
					var zoomStart = params.zoom[0];
					var zoomEnd = params.zoom[1];
					zoomStart = zoomStart<1 ? Math.random()+1 : (zoomStart>2 ? Math.random()+1 : zoomStart );
					zoomEnd = zoomEnd<1 ? Math.random()+1 : (zoomEnd>2 ? Math.random()+1 : zoomEnd );
				}
				else {
					var zoomStart = Math.random()+1; 
					var zoomEnd = Math.random()+1;
				}
				// init focus 
				if(typeof(params.focus)=='array' || typeof(params.focus)=='object') {
					var focusX = params.focus[0];
					var focusY = params.focus[1];
					focusX = focusX<0 ? Math.random() : (focusX>1 ? Math.random() : focusX);
					focusY = focusY<0 ? Math.random() : (focusY>1 ? Math.random() : focusY);
				}
				else {
					var focusX = Math.random(); 
					var focusY = Math.random();
				}
				// init proporties
				proporties = {
					top:{
						unit:'px'
					},
					left:{
						unit:'px'
					},
					width:{
						start: zoomStart*params.width,
						end: zoomEnd*params.width,
						unit:'px'
					},
					height:{
						start: zoomStart*params.height,
						end: zoomEnd*params.height,
						unit:'px'
					}
				};
				proporties.top.start 	= focusY>=0.5 ? 0 : params.containerHeight-proporties.height.start;
				proporties.top.end 		= Math.max(params.height*focusY*-1,params.containerHeight-proporties.height.end);
				proporties.left.start 	= focusX>=0.5 ? 0 : params.containerWidth-proporties.width.start;
				proporties.left.end 	= Math.max(params.width*focusX*-1,params.containerWidth-proporties.width.end);
				
				dojo.style(this.containerNode,{
					'position': dojo.style(this.containerNode,'position')=='static' ? 'relative' : 'absolute'
				});
				dojo.style(this.node,{
					'position':'absolute'
				});
				
				if(params.fadeIn) {
					dojo.style(this.node,{
						'opacity':0
					});
					this.addFadeIn(params.fadeIn);
				};
				this.addAnimation({
					duration : params.duration,
					easing: dojo.fx.easing[params.easing],
					properties: proporties,
					onEnd: params.onEnd || function(){}
				});
				if(params.fadeOut) {
					this.addFadeOut(params.fadeOut,params.duration-params.fadeOut);
				};
			}
		}
	});