/*
	Copyright (c) 2006-2007, epages Software GmbH
	All Rights Reserved.

	epages.html $Revision: 1.15.4.1 $

*/
dojo.provide("epages.html");
dojo.require('epages.epages');

dojo.mixin(epages.html, {
	getChildNodesByTagName: function(/*XMLNode*/parentNode, /*String*/childTagName) {
		// summary:
		//  Returns an array of direct children matching the childTagName tag name.
		// description:
		//  Only direct children will be returned, so this differs from getElementsByTagName().
		// example:
		//  |/*
		//  |<ul>
		//  |  <li>node 1</li>
		//  |  <li>node 2</li>
		//  |</ul>
		//  |*/
		//  |listItems = epages.com.getChildNodesByTagName(listNode, "li");
		//  |// listItems has now two children
		returnValue = [];
		for (var i = 0; i < parentNode.childNodes.length; i++) {
			var e = parentNode.childNodes[i];
			if (e.nodeName == childTagName) {
				returnValue.push(e);
			}
		}
		return returnValue; // Array
	},

	getElementsByClassName: function (/*String*/className, /*DomNode*/domNode) {
		if(domNode == null) domNode = dojo.body();
		return dojo.query("." + className, domNode);
	},

	replaceClass: function(/*DomNode|String*/node, /*String*/newClassStr, /*String*/ oldClassStr) {
	  var regEx = new RegExp("(^|\\s+)" + oldClassStr + "($|\\s+)", "g");
	  node.className = node.className.replace(regEx, "$1" + newClassStr + "$2");
	},

	removeChildren: function(/*DomNode|String*/node) {
		while(node.firstChild){ node.removeChild(node.firstChild); }
	},

	disableHrefsAndActions: function (/* DomNode */containerNode) {
		if(containerNode == null) containerNode = dojo.body();

		// disable links
		var aTags = containerNode.getElementsByTagName('a');
		for (var i = 0; i < aTags.length; i++) {
			var tag = aTags[i];
			tag.style.cursor='default';
			if(typeof tag.getAttribute("onclick")=="string" && (tag.getAttribute("onclick") || "").match("openWindow")) {
				tag.onclick = function(){return false};
			}
			dojo.connect(tag, "onclick", function(evt) {
				dojo.stopEvent(evt);
				return;
			});
			// change href
			if(tag.getAttribute("href"))		
				tag.removeAttribute("href");
			// change target
			if(tag.getAttribute("target"))
				tag.removeAttribute("target");
		}
		// disable all forms
		var formTags = containerNode.getElementsByTagName('form');
		for (var i = 0; i < formTags.length; i++) {
			formTags[i].onsubmit = function() { return false; };
		}
		// disable all form elements
		var elementList = ['input', 'button', 'select', 'option', 'a'];
		var deleteAttributes = ['onchange','onclick','onselect','onkeyup','onkeypress','onmouseup'];
		for (var j = 0; j < elementList.length; j++) {
			var inputTags = containerNode.getElementsByTagName(elementList[j]);
			for (var i = 0; i < inputTags.length; i++) {
				var e = inputTags[i];
				//if (e.type == 'submit') e.type = 'button';
				for (var k = 0; k < deleteAttributes.length; k++) {										
					var attrib = e.getAttribute(deleteAttributes[k]);
					
					if(attrib) {
						if((typeof attrib == "string") || (typeof attrib == "function")) {
							dojo.connect(e, deleteAttributes[k], dojo, "stopEvent");					
							e.setAttribute(deleteAttributes[k], 'return false;');
						}
					}
				}
			}
		}
	},

	getHref: function (/* DomNode<a>*/linkNode) {
		// returns: real (not transformed absolutely) href content of an A-tag-domNode
		if (linkNode.outerHTML) {
			// IE
			var html = linkNode.outerHTML;
			var href = html.match(/\s*href\s*\=\s*["'](.*?)["']/i);
			if(href && href[1]) return href[1];
			return null;
		}
		// others
		return linkNode.getAttribute("href");
	},

	isInnerEvent: function(/* Event */evt, /* DOMNode | array.DOMNode */allowedNodes) {
		// summary: usefull for blur events, that could be triggered by clicking on another dom node
		// returns: whether the given event happened outside of the allowed nodes
		if (!dojo.isArray(allowedNodes)) allowedNodes = [ allowedNodes ];
		var target = evt.explicitOriginalTarget;
		for (var i = 0; i < allowedNodes.length; i++) {
			if (dojo.isDescendant(target, allowedNodes[i])) return true;
		}
		return false;
	},

	encodeEntity: function(string, fixBreaks) {
		if (fixBreaks == null) fixBreaks = false;
		var rx = new RegExp("\r?\n", "g");
		var lines = epages.string.split(string, rx);
		var textnode = document.createTextNode('');
		var node = document.createElement('span');
		node.appendChild(textnode);
		for (var i = 0; i < lines.length; i++) {
			textnode.nodeValue = lines[i];
			lines[i] = node.innerHTML;
		}
		return lines.join(fixBreaks ? "<br/>\n" : "\n");
	},

	decodeEntity: function(string, fixBreaks) {
		if (fixBreaks == null) fixBreaks = false;
		var node = document.createElement('span');
		node.innerHTML = string;
		var value = node.firstChild.nodeValue;
		if (fixBreaks)
			value = value.replace(/\<br\s*\/?\>\r?\n/gi, "\n");
		return value;
	},

	insertAtCursor: function(myValue, fieldId, doc) {
		if (doc === undefined) doc = window.document;
		var myField = $(fieldId, doc);
		//IE support
		if (doc.selection) {
			myField.focus();
			sel = doc.selection.createRange();
			sel.text = myValue;
		}
		//MOZILLA/NETSCAPE support
		else if (myField.selectionStart || myField.selectionStart == '0') {
			var startPos = myField.selectionStart;
			var endPos = myField.selectionEnd;
			myField.value = myField.value.substring(0, startPos)
				+ myValue
				+ myField.value.substring(endPos, myField.value.length);
			myField.selectionStart = startPos + myValue.length;
			myField.selectionEnd = startPos + myValue.length;
		} else {
			myField.value += myValue;
		}
		dojo.require("epages.event");
		epages.event.fire(myField,'change');
	}
});

