/*
	Copyright (c) 2006-2007, epages Software GmbH
	All Rights Reserved.

	epages.html.element $Revision: 1.37.4.2 $

*/
dojo.provide('epages.html.element');
dojo.require("epages.event");
dojo.require("epages.html");

epages.html.Element = function (object) {
	//summary: epages.html.Element extends standard object with missing functions
	if (object === undefined) {
		throw new Error("epages.html.Element: initialize without object");
	} else if (typeof object == "string") {
		object = $(object);
		if (object === undefined) throw new Error("epages.html.Element: initialize without object, unknown element id");
	}
	this.object = object;
};

dojo.extend(epages.html.Element, {
	object: undefined,

	get: function () {
		//summary: getter function for value of element with special treatment for radiobox & checkbox
		if((dojo.hasClass(this.object,"Radio")|| dojo.hasClass(this.object,"Checkbox"))  && !this.object.checked){
			return '';
		}
		return this.object.value || this.object.originalValue || '';
	},

	set: function(value) {
		//summary: setter function for value of element with speacial treatment for radiobox & checkbox
		var type = this.object.type;
		switch (type) {
			case 'checkbox':
			case 'radio':
				var items = this.object.form.elements;
				var name = this.object.name;
				dojo.forEach(items, function(object) {
					if (object.type == type && object.name == name && object.value == value)
						object.checked = true;
				});
				break;
			default:
				this.object.value = value;
		}
	},

	connectChangeEvent: function (callback) {
		//summary: connect onchange event to element but in case of element is radio use click event
		dojo.connect(this.object, (this.object.type == 'radio' ? "onclick" : "onchange"), callback);
	},

	callOnChange: function () {
		//summary: fire onchange event for the element
		epages.event.fire(this.object,'change');
	},

	firstElement: function (nodeType, nodeName) {
		var element = undefined;
		var data = this.object.childNodes;
		for(var counter = 0; counter < data.length && element === undefined; counter++) {
			// find element, if nodeType == 1 then nodeName has to be equal
			if (nodeType != 1               && data[counter].nodeType == nodeType ||
					data[counter].nodeType == 1 && data[counter].nodeName == nodeName) {
				element = data[counter];
			}
		}
		return (element);
	},

	firstComment: function () {
		var commentElement = this.firstElement(8, 'COMMENT');
		return (commentElement === undefined ? undefined : commentElement.data);
	},

	getIframeDocument: function (iframe) {
		if (iframe === undefined && this.object.tagName.toLowerCase() == 'iframe') iframe = this.object;
		if (iframe === undefined) return iframe;
		return (iframe.contentDocument === undefined) ? iframe.contentWindow.document : iframe.contentDocument;
	},

	addClassName: function (/*string*/className) {
		dojo.addClass(this.object, className);
	},

	removeClassName: function (/*string*/className) {
		dojo.removeClass(this.object, className);
	},

	changeClassName: function (/*string*/fromClassName, /*string*/toClassName) {
	  epages.html.replaceClass(this.object, toClassName, fromClassName);
	},

	getAbsolutePosition: function(baseNode) {
	// summary: get absolute position to base node
		var node = this.object;
		var x = 0, y = 0;
		while (node && node != baseNode) {
			if (node.offsetLeft !== undefined && node.offsetTop !== undefined) {
				x += node.offsetLeft;
				y += node.offsetTop;
			}
			node = node.offsetParent;
		}
		return {'x': x, 'y': y};
	},

	getPositionToParent: function(parentNode) {
		// summary: calcualte position to the specified parent node
		// parentNode: should be an offsetParent (position: absolute or relative)
		var node = this.object;
		var x = 0, y = 0;
		var position = dojo.getComputedStyle(parentNode).position;
		if(position != "relative" && position != "absolute") {
			console.warn("parentNode should be an offsetParent (position: absolute or relative) in "+ this.delcaredClass);
		}
		var lastOffsetParent = undefined;
		while (node != undefined && parentNode !== node) {
			if(lastOffsetParent !== node.offsetParent) {			// skip offset values which are already added (jump from offsetparent to offsetparent)
				if (node.offsetLeft !== undefined && node.offsetTop !== undefined) {
					x += node.offsetLeft;
					y += node.offsetTop;
				}
				lastOffsetParent = node.offsetParent;
			}
			node = node.parentNode;
		}
		return {'x': x, 'y': y};
	},

	getRelativePosition: function() {
		// summary: calcualte position to the next parentnode with position relative|absolute
		var node = this.object;
		var x = 0, y = 0;
		var position = dojo.getComputedStyle(node).position;
		while (node != undefined && position != "relative" && position != "absolute") {
			if (node.offsetLeft !== undefined && node.offsetTop !== undefined) {
				x += node.offsetLeft;
				y += node.offsetTop;
			}
			node = node.offsetParent;
			position = dojo.getComputedStyle(node).position;
		}
		return {'x': x, 'y': y};
	},

	setCss: function(/*Object*/attributes) {
		for (var attributeName in attributes) {
			this.object.style[attributeName] = attributes[attributeName];
		}
	},

	makeWidgets: function() {
		dojo.parser.parse(this.object);
	},

	toQueryString: function() {
		return encodeURIComponent(this.object.name) + '=' + encodeURIComponent(this.get());
	},

	addEventListener: function(/*String*/eventName, /*function*/callBack, /*boolean?false*/bubbleUp) {
		// summary: cross-browser implementation of node.addEventListener from Mozilla
		if (bubbleUp == null) bubbleUp = false;
		eventName = eventName.replace(/^on/, '');
		if (this.object.addEventListener) {
			return this.object.addEventListener(eventName, callBack, bubbleUp);
		} else if(this.object.attachEvent) {
			return this.object.attachEvent('on' + eventName, callBack);
		}
		throw new Error('element.addEventListener: could not add event listener for event ' + eventName);
	},

	removeEventListener: function(/*String*/eventName, /*function*/callBack, /*boolean?false*/bubbleUp) {
		//summary: cross-browser implementation of node.removeEventListener from Mozilla
		if (bubbleUp == null) bubbleUp = false;
		eventName = eventName.replace(/^on/, '');
		if (this.object.removeEventListener) {
			return this.object.removeEventListener(eventName, callBack, bubbleUp);
		} else if(this.object.detachEvent) {
			return this.object.detachEvent('on' + eventName, callBack);
		}
		console.debug('element.removeEventListener: could not remove event listener for event ' + eventName + (typeof this.object.detachEvent));
		return false;
	},

	replaceChild: function(newNode, oldNode) {
		newNode = this._changeNodeDocument(newNode, oldNode.ownerDocument);
		this.object.replaceChild(newNode, oldNode);
	},

	appendChild: function(childNode) {
		childNode = this._changeNodeDocument(childNode, this.object.ownerDocument);
		this.object.appendChild(childNode);
	},

	_changeNodeDocument: function(node, newDoc) {
		// summary: in IE you can not insert a node in a document, that has been created in another document
		// so we simply clone the node in correct document by inserting its outer HTML
		if (node.ownerDocument != newDoc && node.outerHTML != null) {
			var nodeHtml = node.outerHTML;
			node = newDoc.createElement('span');
			node.outerHTML = nodeHtml;
		}
		return node;
	},

	
	getLabel: function() {
		// summary: this.object needs an id and there has to exist a label node for this id
		// returns: assigned label for this node
		var id = this.object.id;
		if (id == null) return;
		var labels = this.object.ownerDocument.getElementsByTagName('label');
		for (var i = 0; i < labels.length; i++) {
			if (labels[i].htmlFor == id) return labels[i].innerHTML;
		}
	},

	isEmpty: function(){
		//summary: check if object has methodes or properties assigned
		for(var el in this.object){
			return false;
		}
		return true;
	}
});

$E = function (/*Object*/object) {
	return new epages.html.Element(object); // epages.html.Element
};
