/*
	Copyright (c) 2006-2007, epages Software GmbH
	All Rights Reserved.

	epages.io.Json $Revision: 1.19.2.2 $

*/
dojo.provide('epages.io.json');

epages.io.Json = function () {};
dojo.extend(epages.io.Json, {

	defaultErrorCallback: function(error,args) {
  	var response = args.xhr.status == 404 ? {} : dojo.fromJson(args.xhr.responseText);
    console.warn('epages.io.Json: error in request');

		if (response.Errors !== undefined) {
			dojo.publish("epages/interfaceerror", [{
				code:	    response.Errors[0].Reason,
				message:  response.Errors[0].Message,
				guid:     response.Errors[0].GUID,
				vars:     {
	          				url:		  args.url,
	          				status:	  args.xhr.status,
	          				response: response
	          		  }
			}]);
		}
	},

	loadSync: function (/*String*/url, /*object?*/formData, /*Function?*/errorCallback) {
		// summary:
		//		returns a javascript object, which was loaded via http from given url.
		// 		additional parameters will be added in the post.
		// example:
		//		JsonLoader = new epages.io.Json();
		//		Answer = JsonLoader.loadSync('http://www.mydomain.de/test.Json', { filename: "test.Json" });
		//		// returns JsonDocument object
		var returnData;
		var requestMethod = formData !== undefined ? dojo.xhrPost : dojo.xhrGet;
		var _this = this;
		var opt = {
			url: 						url,
			sync: 					true,
			preventCache:		epages.constants.preventCache,
			error:					function(error,args) {
												returnData = {
													'error': {
														error: error,
														args: args,
														data: dojo.fromJson(args.xhr.responseText)
													}
												};
												if (errorCallback != null)
													errorCallback({'error':[error,args]});
												else
													_this.defaultErrorCallback(error,args);
											},
			load: 					function(data, evt){
												var result = dojo.fromJson(data);
												if(result.Errors){
													returnData = {
														'error': {
															data: result
														}
													};
												}
												else{
													returnData = { 'data': dojo.fromJson(data) };
												}
											}
		}
		// test cases doesnt allow posts
		if(url.match('\.(js|json)$')) {
			requestMethod = dojo.xhrGet;
		} else {
			dojo.mixin(opt, {
				headers:	{ Accept: 'text/x-json' },
				content:	formData
			});
		}
		requestMethod(opt);
		return returnData; /*Object*/
	},

	loadAsync: function (/*String*/url, /*Function*/callback, /*object?*/formData, /*Function?*/errorCallback) {
		// summary:
		//		loads a document via http from given url.
		//		the function doesnt wait for the result.
		//		if the result is available the callback function is called with the data.
		// example:
		//		JsonLoader = new epages.io.Json();
		//		JsonDoc = JsonLoader.loadAsync('http://www.mydomain.de/test.Json', function (data) { /* do what you want */ });
		//		// returns JsonDocument object
		var requestMethod = formData !== undefined ? dojo.xhrPost : dojo.xhrGet;
		var _this = this;
		var opt = {
			url: 			url,
			error:		function(error,args) {
									var c = errorCallback || callback;
									var response = dojo.fromJson(args.xhr.responseText);
									if (c) {
										c({
											'error': [ error, args ],
											'data': (typeof response == "object") ? dojo.fromJson(args.xhr.responseText) : {}
										});
									} else {
										_this.defaultErrorCallback(error,args);
									}
								},
			load: 		function(data, ioArgs) {
									if (callback != null){
										var result = dojo.fromJson(data);
										if(result.Errors){
											callback({
												'error': {
													data: result
												}
											});
										} else {
											callback({ 
												'data': result, 
												'ioArgs': ioArgs
											});
										}
									}
								}
		};
		// test cases doesnt allow posts
		if(url.match('\.(js|json)$')) {
			requestMethod = dojo.xhrGet;
		} else {
			dojo.mixin(opt, {
				headers:	{ Accept: 'text/x-json' },
				content:	formData
			});
		}
		requestMethod(opt);
		return;
	}
});
