/*
	Copyright (c) 2006-2007, epages Software GmbH
	All Rights Reserved.

	epages.io.translation $Revision: 1.24.4.2 $

*/
dojo.provide('epages.io.translation');
dojo.require('epages.io.xml');
dojo.require('epages.lang.array');
dojo.require('epages.lang.hash');

epages.io.Translation = function (/*String?*/ url, /*String?*/ locale) {
		this.translation = new epages.lang.Hash();
		this.loadedFiles = new epages.lang.Array();
		if( url !== undefined)
			if(locale===undefined)
				this.load(url);
			else
				this.loadLocale(url, (locale!="auto") ? locale : undefined);
}

dojo.extend(epages.io.Translation, epages.io.Xml.prototype);
dojo.extend(epages.io.Translation, {
	translation: null,
	loadedFiles: null,
	useLocale:	false,

	load: function (/*String*/ url, /*Boolean?*/ skipError) {
		if (this.loadedFiles.exists(url)) return true; // Boolean
		if (skipError === undefined) skipError = false;
		var urlData = this.loadSync(url);
		this.loadedFiles.push(url);
		if (urlData === undefined) {
			if (! skipError) throw new Error('translation undefined');
			return false; //Boolean
		}
		var tags = urlData.getElementsByTagName('Translation');
		for(var counter = 0; counter < tags.length; counter++) {
			var thisElement = tags[counter];
			// is translation
			if (thisElement.nodeType == 1) {
				// ignore <notrans>
				var data = '';
				for (var i = 0; i < thisElement.childNodes.length; i++) {
					var child = thisElement.childNodes[i];
					data += child.nodeName.match(/notrans/i) ? child.firstChild.data : child.data
				}
				this.translation.set(thisElement.getAttribute('Keyword'), data);
			}
		}
		return true; //Boolean
	},

	loadLocale: function (/*String*/ url, /*String?*/ locale) {
    // summary: load translation data (of language and locale), the locale will be trans
		// url: file name without locale and extension (.xml)
		// locale: locale string (RFC 3066; eg en-us)
		if (locale === undefined) locale = dojo.locale;

		var locParts = locale.split('-');
		var loadLanguage = this.load(url + '.' + locParts[0] + '.xml', true);
		// load locale dependend translation, later
		if (this.useLocale && locParts.length > 1)
		  this.load(url + '.' + locParts[0] + '_' + locParts[1].toUpperCase() + '.xml', true);
		return loadLanguage; //Boolean
	},

	exists: function (/*String*/key) {
		return this.translation.existsKey(key) || (this != epages.io.dictionary && epages.io.dictionary.translation.existsKey(key));
	},

	get: function (/*String*/ key, /*Object?*/vars) {
		// summary:
		//		returns the value of an specified language tag.
		//		returns {key} if the language tag doesnt exist.
		// example:
		//   | var translation = new epages.io.Translation();
		//   | translation.load('translation.de.xml');
		//   |   image       : 'test.png',
		//   |   description : 'long description, long description'
		//   | });
		//   | string = translation.get('Image');
		//   | // returns 'Bild'
		var locName = this.translation.get(key);
		if(locName === undefined && this != epages.io.dictionary)
			return epages.io.dictionary.get(key, vars);
		return (locName === undefined) ? '{' + key + '}' : this.replaceVars(locName, vars); // String
	},

	replaceLanguageTags: function (/*String*/ txt) {
		// summary:
		//		replace all language tags in a string
		//		returns the "translated" string
		// example:
		// 	  var translation = new epages.io.Translation(translation.de.xml');
		// 		var mystring = "<div>{Image}</div>";
		// 		mystring=this.Translation.replaceLanguageTags(mystring);
		var templateRegExp = new RegExp('{(.*?)}', 'g');
		return txt.replace( templateRegExp, dojo.hitch(this, function(match, tag) {
			return this.get(tag);
		})); // String
	},

	replaceVars: function(/*String*/ txt, /*Object?*/ vars) {
	  var varsRegExp = new RegExp('#([a-zA-Z_][a-zA-Z0-9_]*)', 'g');
		return txt.replace( varsRegExp, dojo.hitch(vars, function(match, varName) {
		  return this[varName] !== undefined ? this[varName] : match;
		})); // String
	}

});

epages.io.dictionary = new epages.io.Translation(dojo.moduleUrl('epages', "translation"),"auto");
