/*
	Copyright (c) 2006-2007, epages Software GmbH
	All Rights Reserved.

	epages.lang.array $Revision: 1.7.52.1 $

*/
dojo.provide('epages.lang.array_test');
dojo.require('epages.lang.array');

dojo.extend(epages.lang.Array, {
	test: function(/*epages.Test*/eTest) {
		// summary:
		//		test this package
		// example:
		// 	  var array = new epages.lang.Array();
		//		boolean = array.test(eTest);
		//		// returns true if all test are successfully
		eTest.setPlanned(30);

		this.push('Test', 'Array', 'Hash', 'epages');
		eTest.ok(1, 'pushed');
		eTest.ok(this.length == 4, 'initial length');

		eTest.ok(this.match(/a/).length == 3, 'length with /a/');
		eTest.ok(this.match('e').length == 2, "length with 'e'");

		var it = function (element) {
			return element == 'epages';
		};
		var itSelf = function (element) {
			return this.length == 1;
		};
		eTest.ok(this.grep(it).length == 1, "length with iterator '==epages'");
		eTest.ok(this.grep(itSelf, this.grep(it)).length == 4, "length with iterator 'local this'");
		eTest.ok(this.grep(function (element) { return element == 'epages' } ).length == 1, "length with function '==epages'");


		var integers = new epages.lang.Array([1, 2, 3, 4]);
		var sum = 0;
		integers.each( function(item){ sum += item; });
		eTest.ok(sum == 10, 'sum with each method');

		var newArray = integers.map( function(item){ return item + 1; } );
		eTest.ok(newArray[0] == 2, 'map + 1');

		this.push('pushTest');
		var el = this.pop();
		eTest.ok(el == 'pushTest', 'push + pop');

		this.unshift('unshiftTest');
		 el = this.shift();
		eTest.ok(el == 'unshiftTest', 'unshift + shift');

		var newArray = new epages.lang.Array(["Homer", "Marge", "Bart", "Lisa", "Maggie"]);
		eTest.ok(newArray.exists("Homer"), "key exists");
		eTest.ok(!newArray.exists("Smithers"), "key doesn't exist");
		eTest.ok(newArray.find("Bart") == 2, "correct index found");
		eTest.ok(newArray.find("Smithers") === null, "incorrect index");
		eTest.ok(newArray.length == 5, "correct length before remove");
		newArray.remove(2);
		eTest.ok(1, "remove() returned");
		eTest.ok(!newArray.exists("Bart"), "index not found anymore");
		eTest.ok(newArray.length == 4, "correct length ater remove");
		newArray.remove(100);
		eTest.ok(1, "remove() returned after illegal index");

		var newArray = new epages.lang.Array(["Homer", "Marge", "Bart"]);
		var compArray = ["Homer", "Marge", "Bart"];
		eTest.ok(newArray.equals(compArray), "equals() works for JS arrays");
		compArray.push("Lisa");
		eTest.ok(!newArray.equals(compArray), "not equals() works for JS arrays (different length)");
		newArray.push("Maggie");
		eTest.ok(!newArray.equals(compArray), "not equals() works for JS arrays (different elements)");

		var newArray = new epages.lang.Array(["Homer", "Marge", "Bart", "Homer", "Marge"]);
		var compArray = new epages.lang.Array(['Homer','Marge','Bart']);
		newArray.unique();
		eTest.ok(newArray.equals(compArray), "unique works "+newArray);

		var newArray = new epages.lang.Array(["Homer", "Marge", "Bart"]);
		var compArray = new epages.lang.Array(["Homer", "Marge", "Bart"]);
		eTest.ok(newArray.equals(compArray), "equals() works for epages.lang.Arrays");
		compArray.push("Lisa");
		eTest.ok(!newArray.equals(compArray), "not equals() works for epages.lang.Arrays (different length)");
		newArray.push("Maggie");
		eTest.ok(!newArray.equals(compArray), "not equals() works for epages.lang.Arrays (different elements)");
		newArray.push(undefined);
		newArray.push(null);
		var expected = '["Homer","Marge","Bart","Maggie",undefined,null]';
		var s = newArray.toString();
		eTest.ok(s == expected, "toString():" + s);

		var merge1 = new epages.lang.Array(["Homer", "Marge", "Bart"]);
		var merge2 = new epages.lang.Array(["Lisa", "Maggie"]);
		merge1.merge(merge2);
		eTest.ok(merge1.length == 5, "merge():" + merge1);

		merge2.insertAt(1, "Marge");
		eTest.ok(merge2.toString() == '[\"Lisa\",\"Marge\",\"Maggie\"]', "insertAt():" + merge2);

		return eTest.planned == eTest.correct ? true : false; // Boolean
	}

});
