/*
	Copyright (c) 2006-2007, epages Software GmbH
	All Rights Reserved.

	epages.lang.hash $Revision: 1.5.52.1 $

*/
dojo.provide('epages.lang.hash_test');
dojo.require('epages.lang.hash');

dojo.extend(epages.lang.Hash, {
	test: function(/*epages.Test*/eTest) {
		// summary:
		//		test this package
		// example:
		// 	  var hash = new epages.lang.Hash();
		//		boolean = hash.test(eTest);
		//		// returns true if all test are successfully
		eTest.setPlanned(24);

		var hash = this;

		var setValue = 'Value1';
		hash.set('Test', setValue);
		eTest.ok(1, 'set returned');

		var existsKey = hash.existsKey('Test');
		eTest.ok(existsKey, 'key found');

		var existsKey = hash.existsKey('Test1234567890');
		eTest.ok(!existsKey, 'key not found');

		var existsValue = hash.existsValue(setValue);
		eTest.ok(existsValue, 'value found');

		var setValue2 = 123;
		hash.set('Test123', '123');
		var existsValue = hash.existsValue(setValue2);
		eTest.ok(!existsValue, 'value not found / other type');

		var existsValue = hash.existsValue(setValue + '1234567890');
		eTest.ok(!existsValue, 'value not found / other string');

		var getValue = hash.get('Test');
		eTest.ok(1, 'get returned');

		eTest.ok((getValue == setValue), 'correct value');
		hash.set('Value1', setValue + '2');

		eTest.test(function () {
			return hash.keys().sort().join(',') == 'Test,Test123,Value1';
		}, 'keys');

		eTest.ok((hash.values().sort().join(',') == '123,Value1,Value12'), 'values');

		hash.set('funcTion', function (bool) { return bool ? false: true; });
		eTest.ok(! hash.get('funcTion')(true), 'true function value');
		eTest.ok(hash.get('funcTion')(false), 'false function value');

		hash.remove('Test');
		eTest.ok(1, 'remove returned for existing key');

		hash.remove('Test0987654321');
		eTest.ok(1, 'remove returned for undefined key');

		existsKey = hash.existsKey('Test');
		eTest.ok(!existsKey, 'node was not found after remove()');

		var hash2 = new epages.lang.Hash({ key: 'schluessel', value: 'wert'});
		eTest.ok(hash2.get('key') == 'schluessel', 'new Hash from Object');
		eTest.ok(hash2.existsValue('schluessel'), 'existing string value');

		var hash3 = new epages.lang.Hash(hash2);
		eTest.ok(hash3.get('value') == 'wert', 'new Hash from Hash');

		var hash4 = new epages.lang.Hash({ 'schluessel': 'value', 'enc&Schluessel':'enc&Value\'', 'isundef': undefined, 'isnull': null});

		var e = 'schluessel=value&enc%26Schluessel=enc%26Value\'';
		var s = hash4.toQueryString();
		eTest.ok(s == e, 'toQueryString()' + s);

		var e = '{"schluessel":"value","enc&Schluessel":"enc&Value\\\'","isundef":undefined,"isnull":null}';
		var s = hash4.toString();
		eTest.ok(s == e, 'toString()' + s);

		hash4.set('schluessel', ['value1', 'value2']);
		var e = 'schluessel=value1&schluessel=value2&enc%26Schluessel=enc%26Value\'';
		var s = hash4.toQueryString();
		eTest.ok(s == e, 'toQueryString()' + s);

		var testObject = new Object();
		var testObject2 = new Object();
		hash.set('TestObject', testObject);
		hash.set('TestObjectUndef', undefined);
		eTest.ok(hash.existsValue(testObject), 'object exists');
		eTest.ok(!hash.existsValue(testObject2), 'object doesnt exist');
		eTest.ok(hash.existsKey('TestObjectUndef'), 'undefined object key exist');

		return eTest.planned == eTest.correct ? true : false;
	}
});
