/*
	Copyright (c) 2006-2007, epages Software GmbH
	All Rights Reserved.

	epages.lang.uri $Revision: 1.5.4.1 $

*/
dojo.provide('epages.lang.uri_test');
dojo.require('epages.lang.uri');

epages.lang.uri.test = function(/*epages.Test*/eTest) {
	// summary:
	//		test this package
	// example:
	//		boolean = new epages.lang.uri.test(eTest);
	//		// returns true if all test are successfully
	eTest.setPlanned(30);

	var compare = {
		href: 'http://mweber.jena.epages.de/epages/DemoShop.admin/de_DE/?ViewAction=HTB-ViewWizardSelectCategory&ObjectID=6943#anchor',
		protocol: 'http',
		host: 'mweber.jena.epages.de',
		path: '/epages/DemoShop.admin/de_DE/',
		query: '?ViewAction=HTB-ViewWizardSelectCategory&ObjectID=6943',
		anchor: '#anchor',
		parameters: {
			ViewAction: 'HTB-ViewWizardSelectCategory',
			ObjectID: '6943'
		}
	};
	var result = epages.lang.uri.parts('http://mweber.jena.epages.de/epages/DemoShop.admin/de_DE/?ViewAction=HTB-ViewWizardSelectCategory&ObjectID=6943#anchor');
	eTest.ok(result.href == compare.href, 'parts - href')
	eTest.ok(result.protocol == compare.protocol, 'parts - protocol')
	eTest.ok(result.host == compare.host, 'parts - host')
	eTest.ok(result.path == compare.path, 'parts - path')
	eTest.ok(result.query == compare.query, 'parts - query')
	eTest.ok(result.anchor == compare.anchor, 'parts - anchor')
	eTest.ok(result.parameters.ViewAction == compare.parameters.ViewAction, 'parts - parameters')
	eTest.ok(result.parameters.ObjectID == compare.parameters.ObjectID, 'parts - parameters')

	var result = epages.lang.uri.fromParts(compare);
	eTest.ok(result == "http://mweber.jena.epages.de/epages/DemoShop.admin/de_DE/?ViewAction=HTB-ViewWizardSelectCategory&ObjectID=6943#anchor", 'fromParts')

	var compare = {
		href: '#anchor',
		protocol: '',
		host: '',
		path: '',
		query: '',
		anchor: '#anchor',
		parameters: {}
	};
	var result = epages.lang.uri.parts('#anchor');
	eTest.ok(result.href == compare.href, 'parts 2 - href')
	eTest.ok(result.protocol == compare.protocol, 'parts 2 - protocol')
	eTest.ok(result.host == compare.host, 'parts 2 - host')
	eTest.ok(result.path == compare.path, 'parts 2 - path')
	eTest.ok(result.query == compare.query, 'parts 2 - query')
	eTest.ok(result.anchor == compare.anchor, 'parts 2 - anchor')

	var result = epages.lang.uri.fromParts(compare);
	eTest.ok(result == "#anchor", 'fromParts 2')

	var compare = {
		href: 'test/abc.html#anchor',
		protocol: '',
		host: '',
		path: 'test/abc.html',
		query: '',
		anchor: '#anchor',
		parameters: {}
	};
	var result = epages.lang.uri.parts('test/abc.html#anchor');
	eTest.ok(result.href == compare.href, 'parts 3 - href')
	eTest.ok(result.protocol == compare.protocol, 'parts 3 - protocol')
	eTest.ok(result.host == compare.host, 'parts 3 - host')
	eTest.ok(result.path == compare.path, 'parts 3 - path')
	eTest.ok(result.query == compare.query, 'parts 3 - query')
	eTest.ok(result.anchor == compare.anchor, 'parts 3 - anchor')

	var result = epages.lang.uri.fromParts(compare);
	eTest.ok(result == "test/abc.html#anchor", 'fromParts 3')

	// getRelative
	eTest.ok( "../SubFolder2" == epages.lang.uri.getRelative('/Folder/SubFolder1', '/Folder/SubFolder2'), "epages.lang.uri.getRelative - sibling" );
	eTest.ok( "" == epages.lang.uri.getRelative('/Folder/SubFolder1', '/Folder/SubFolder1'), "epages.lang.uri.getRelative - identic" );
	eTest.ok( "../../../SubFolder2/SubSubFolder" == epages.lang.uri.getRelative('/Folder/SubFolder1/SubSubFolder/SubSubSubFolder', '/Folder/SubFolder2/SubSubFolder'), "epages.lang.uri.getRelative - deep" );
	eTest.ok( "SubSubFolder/SubSubSubFolder" == epages.lang.uri.getRelative('/Folder/SubFolder', '/Folder/SubFolder/SubSubFolder/SubSubSubFolder'), "epages.lang.uri.getRelative - base is deep parent" );
	eTest.ok( "../../" == epages.lang.uri.getRelative('/Folder/SubFolder/SubSubFolder/SubSubSubFolder', '/Folder/SubFolder'), "epages.lang.uri.getRelative - base is deep child" );
	eTest.ok( "http://domain1.net/Folder/SubFolder" == epages.lang.uri.getRelative('http://domain2.net/Folder', 'http://domain1.net/Folder/SubFolder'), "epages.lang.uri.getRelative - different domains" );
	eTest.ok( "../../MediaGallery/Friedenstaube.jpg" == epages.lang.uri.getRelative('/WebRoot/Store/Shops/DemoShop/Styles/Boxes/', '/WebRoot/Store/Shops/DemoShop/MediaGallery/Friedenstaube.jpg'), "epages.lang.uri.getRelative - trailing slash" );


	return eTest.planned == eTest.correct ? true : false; // Boolean
}

