/* 
 * @charset			utf-8		(ÄÖÜ äöü ß)
 *
 * Copyright (c) 2006-2010, epages GmbH
 * All Rights Reservedojo.
 * epages.mimetype $Revision: 1.1.2.2.4.1 $
 */
 
(function(dojo) {
	dojo.provide('epages.mimetype');

	dojo.declare('epages._mimetype',null,{
	// summary: 		mimetype and filetype handling
	// description: 	privat class
	// TODO:			improve this class and make it to a public class
	//					by preserving the epages.mimetype API 
	
		/*
		 * private
		 */
		 _MIMETYPE_MAP: {
			// image
			'image/png':																	['png' ],
			'image/jpeg':																	['jpe', 'jpeg', 'jpg' ],
			'image/gif':																	['gif' ],
			'image/bmp':																	['bmp' ],
			'image/vnd.microsoft.icon':														['ico' ],
			'image/tiff':																	['tif', 'tif' ],
			'image/svg+xml':																['svg', 'svgz' ],
			'image/x-win-bitmap':															['cur' ],
			// audio/video
			'audio/basic':																	['au'],
			'audio/mpeg':																	['mp3' ],
			'audio/mp4a-latm':																['m4p', 'm4b', 'm4a' ],
			'audio/x-wav':																	['wav' ],
			'audio/ogg':																	['ogg', 'oga', 'spx' ],
			'audio/flac':																	['flac' ],
			'audio/x-ms-wma':																['wma' ],
			'audio/x-pn-realaudio':															['ra','ram' ],
			'application/vnd.rn-realmedia':													['rm' ],
			'video/ogg':																	['ogv' ],
			'video/mp4':																	['mp4' ],
			'video/mpeg':																	['mpe', 'mpg', 'mpeg', 'mpga' ],
			'video/quicktime':																['qt', 'mov' ],
			'video/x-msvideo':																['avi' ],
			'video/x-flv':																	['flv' ],
			'video/x-ms-wmv':																['wmv'],
			// web
			'text/plain':																	['txt' ],
			'text/html':																	['htm', 'html', 'shtml', 'php', 'php4' ],
			'text/css':																		['css' ],
			'text/x-actionscript':															['as'],
			'application/javascript':														['js' ],
			'application/json':																['json' ],
			'application/xml':																['xml', 'plist' ],
			'application/x-shockwave-flash':												['swf' ],
			// archives
			'application/zip':																['zip' ],
			'application/gzip':																['gz' ],
			'application/x-tar':															['tar' ],
			'application/x-rar-compressed':													['rar' ],
			'application/x-msdownload':														['exe', 'msi' ],
			'application/vnd.ms-cab-compressed':											['cab' ],
			// adobe
			'application/pdf':																['pdf' ],
			'image/vnd.adobe.photoshop':													['psd' ],
			'application/postscript':														['ai', 'eps', 'ps' ],
			// ms office
			'application/msword':															['doc' ],
			'application/x-dot':															['dot' ],
			'application/rtf':																['rtf' ],
			'application/vnd.ms-excel':														['xls','xla' ],
			'application/vnd.ms-powerpoint':												['ppt' ],
			'application/vnd.openxmlformats-officedocument.wordprocessingml.document': 		['docx'],
			'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet':			['xlsx'],
			//
			'application/x-director':														['cdr']
		},
		
		_MIMETYPE_GROUPS: {
			'all':				['*/*' ],
			'icons':			['image/vnd.microsoft.icon', 'image/x-win-bitmap' ],
			'images':			['image/png', 'image/jpeg', 'image/gif' ],
			'documents':		['application/pdf', 'application/msword', 'application/vnd.ms-excel', 'application/rtf', 'application/vnd.openxmlformats-officedocument.wordprocessingml.document', 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet' ],
			'sounds':			['audio/mpeg', 'audio/x-ms-wma', 'audio/ogg', 'audio/flac', 'audio/x-ms-wma' ],
			'video':			['video/x-msvideo', 'video/mp4', 'video/mpeg', 'video/x-ms-wmv', 'video/x-flv' ]
		},
		
		_MIMETYPE_PREVIEWABLE: 	['image/png', 'image/jpeg', 'image/gif', 'image/vnd.microsoft.icon' ],
		
		_ICON_MAP: {
			'ico_l_image':		['image/png','image/jpeg','image/gif','image/bmp','image/vnd.microsoft.icon','image/tiff','image/svg+xml','image/x-win-bitmap'],
			'ico_xl_box':		[this._unknow,'text/x-actionscript','audio/basic','audio/x-wav','audio/mpeg','audio/x-ms-wma','audio/ogg','audio/flac','application/msword','application/x-dot','application/vnd.ms-excel','application/vnd.ms-powerpoint','text/plain','text/css','application/javascript','application/json','application/xml'],
			'ico_xl_file_avi':	['video/x-msvideo','video/mpeg','video/mp4'],
			'ico_xl_file_wmv':	['video/x-ms-wmv'],
			'ico_xl_file_swf':	['video/x-flv','application/x-shockwave-flash'],
			'ico_xl_file_mov':	['video/quicktime','video/ogg'],
			'ico_xl_file_rm':	['application/vnd.rn-realmedia'],
			'ico_xl_file_ram':	['audio/x-pn-realaudio'],
			'ico_xl_file_dcr':	['application/x-director'],
			'ico_xl_file_pdf':	['application/pdf'],
			'ico_xl_file_htm':	['text/html'],
			'ico_m_folder':		['application/zip','application/x-rar-compressed','application/gzip','application/x-tar']
		},
		
		_mimes: null,
		_unknow: 'application/octet-stream',
		
		/*
		 * public
		 */	 
		
		/*
		 * init instance
		 */	
		constructor: function( mime ) {	
			this._mimes = [ mime || this._unknow ];				
		},
		
		/*
		 * methods
		 */	
		mimeBySuffix: function( suffix , firstOnly ) {
			suffix = suffix.toLowerCase();
			var ret = [];
			var map = this._MIMETYPE_MAP;
			
			for( var i in map ) {
				if( dojo.indexOf(map[i], suffix)>=0 ) {
					ret.push(i);
					
					if( firstOnly ) {
						break;
					}
				}
			};
			
			this._mimes = ret.length<0 ? [this._unknow] : ret;
			
			return firstOnly ? ret[0] : ret;
		},
		
		isGroup: function( group ) {
			var mimesGroup = this._MIMETYPE_GROUPS[group];
			
			if( group=='all' ) {
				return true;
			}
			else if(!mimesGroup) {
				return false;
			}
			else {
				for( var i=0,l=this._mimes.length ; i<l ; i++ ) {
					if( dojo.indexOf(mimesGroup, this._mimes[i])>=0 ) {
						return true;
					}
				}
			}
			
			return false;
		},
		
		isPreviewable: function() {
			var prev = this._MIMETYPE_PREVIEWABLE;
			
			for( var i=0,l=this._mimes.length ; i<l ; i++ ) {
				if( dojo.indexOf(prev, this._mimes[i])>=0 ) {
					return true;
				}
			}
			
			return false;
		},
		
		getIcon: function() {
			
			for( var i=0,l=this._mimes.length ; i<l ; i++ ) {
				var map = this._ICON_MAP;
				for( var ii in map ) {
					if( dojo.indexOf(map[ii], this._mimes[i])>=0 ) {
						return ii;
					}
				}
			}
			
			return 'ico_xl_box';
		}
		
	});
	
	epages.mimetype = {
		mimeBySuffix: function( suffix , firstOnly ) {
		// summary: get mimetypeby suffix
			return new epages._mimetype().mimeBySuffix( suffix , firstOnly );
		},
		isPreviewable: function( mime ) {
		// summary: to check is the givn mimetype previewable
			return new epages._mimetype( mime ).isPreviewable();
		},
		isGroup: function( mime , group ) {
		// summary: to check is the givn mimetype in a mimetype group
			return new epages._mimetype( mime ).isGroup( group );
		},
		getGroup: function( group ) {
		// summary: return a group of mimetypes
			var instance = new epages._mimetype();
			return instance._MIMETYPE_GROUPS[group] || instance._MIMETYPE_GROUPS['all'];
		},
		isGroupBySuffix: function( suffix , group ) {
		// summary: to check is the givn fileSuffix in a mimetype group
			var ret = false,
				instance = new epages._mimetype();
				
				instance.mimeBySuffix( suffix );
	
			return instance.isGroup( group );
		},
		getIcon: function( mime ) {
		// summary: returns a SpriteIconClass by the givn mimetype
			return new epages._mimetype( mime ).getIcon();
		}
	};
	
})(dojo);