/*
	Copyright (c) 2006-2007, epages Software GmbH
	All Rights Reserved.

	epages.preload $Revision: 1.3.4.1 $

*/
dojo.provide('epages.preload');
dojo.require('epages.epages');

epages.Preload = {
	// summary:
	//  preloads the given sources
	// description:
	// |preloads given sources listed in the "files" array
	// |never call the _preloadFiles method directly - simply push the necessary file into the "files" array
	// |there are two required properties: "file" and "priority"
	// |  the "file" property is the path / src to your file
	// |  the "priority" property states the importance of that particular file (e. g. images usually have got a lower priority than js packages)
	// example:
	// | dojo.addOnLoad(function() {
	// |   epages.Preload.add({file:"ep_megapackage.js", priority: 9});
	// | });
	start: function() {
		// summary:
		//  starts / restarts the preloading
		this.stopped = false;
		this._preloadFiles();
	},
	stop: function() {
		// summary:
		//  stops the preload operation on the next queued element
		this.stopped = true;
	},
	add: function(fileObject) {
		// summary:
		//  adds a file to the "files" array
		if(fileObject.file && fileObject.priority) {
			var isAlreadyAdded = false;
			for(var i = 0; i < this.files.length; i++) {
				if(this.files[i].file == fileObject.file) {
					isAlreadyAdded = true
					break;
				}
			}
			if(!isAlreadyAdded) {
				this.files.push(fileObject);
			}
		}
	},
	stopped: false,
	_preloadFiles: function() {
		if(!this.stopped && this.files.length > 0) {
			this.nextPreloadFile = this.files[0];
			for(var i = 0; i < this.files.length; i++) {
				if(this.files[i].priority >= this.nextPreloadFile.priority) {
					this.nextPreloadFile = this.files[i];
				}
			}
			if(this.nextPreloadFile.priority != -1) {
				this.nextPreloadFile.priority = -1;
				dojo.xhrGet({
					url:  this.nextPreloadFile.file,
					sync: false,
					load: function(){
						epages.Preload._preloadFiles();
						if(djConfig.isDebug){
							console.debug('epages.Preload loaded : ',epages.Preload.nextPreloadFile.file);
						}
					},
					error: function(){
						epages.Preload._preloadFiles();
						if(djConfig.isDebug){
							console.warn('epages.Preload could not load : ',epages.Preload.nextPreloadFile.file);
						}
					}
				});
			}else {
				// since we are through we can be a bit lazy - but we surely have to monitor the "files" array for realtime changes
				setTimeout(function() {epages.Preload._preloadFiles()}, 500);
			}
		}
	},
	files: [],
	nextPreloadFile: undefined
};

dojo.addOnLoad(function() {
	if(epages.vars.BusyLayerActive){
		dojo.subscribe("uimessage/stopBusy",function(owner, timeout, duration){
			setTimeout(function(){
				epages.Preload.start();
			},timeout);
		});
	}
	else{
		setTimeout(function() {
			epages.Preload.start();
		}, 100);
	}
})