/*
	Copyright (c) 2006-2007, epages Software GmbH
	All Rights Reserved.

  epages.profile $Revision: 1.3.4.1 $

*/

dojo.provide("epages.profile");
dojo.require("epages.lang.hash");

epages.profile = {
	statistik: {},
	isrunning: undefined,
	takeStatistik: function(/*String*/statId,/*function*/statFunc){
		// summary:
		//		wraps a time measurement around a specified function and writes the needed time to epages.profile.statistik
		// description: 
		//    |values inside statistik are all measurings under der StatistikID
		//		|inside this you have:
		//		|	sum - total time needed over all iterations of this function
		//		|	count - number of itterations of this function
		//		|	min - minimal needed time
		//		|	max - maximal needet time
		//		|	innersum - how many time needed the function if it was called again from itself (called again inside)
		// example:
		// 	|	epages.profile.takeStatistik('myFunctionName',function(){console.debug('Hello World');});
		//	|	or if you want to measure a function call inside a widget
		//	|	epages.profile.takeStatistik('focusNode',dojo.hitch(this,function(){this.myfunc(this.params)}));
		var startTime = new Date();
		var isRunning = epages.profile.isrunning;
		epages.profile.isrunning = 1;
		statFunc();
		epages.profile.isrunning = 0;

		var runningTime = new Date() - startTime;
		// sum, count, min, max, innersum
		if(! $H(epages.profile.statistik).existsKey(statId)){
			epages.profile.statistik[statId] = [0,0,0,0,0];
		}
		var currentStat = epages.profile.statistik[statId];
		currentStat[isRunning ? 4 : 0]+= runningTime;
		currentStat[1]++;
		if(runningTime > currentStat[3]){
			currentStat[3] = runningTime;
		}
		if(runningTime < currentStat[2]){
			currentStat[2] = runningTime;
		}

	},
	showStatistik: function(){
		// summary:
		//		show epages.profile.statistik inside the console
		// example:
		//	| epages.profile.showStatistik();
		var stats = this.statistik;
		var sum = 0;
		$H(stats).each(function(p){
			sum += p.value[0];
			console.debug(p.key, $A(p.value).toString());
		});
		console.debug('Sum all: '+sum);

	}
};

