/*
	Copyright (c) 2006-2007, epages Software GmbH
	All Rights Reserved.

	epages.style.processor $Revision: 1.30.4.4.2.2 $

*/
dojo.provide('epages.style.processor');
dojo.require('epages.lang.hash');
dojo.require('epages.lang.array');
dojo.require('epages.lang.uri');
dojo.require('epages.html.element');
dojo.require("epages.browser");

epages.style.Processor = function (/*DomNode */container,/*String*/imageBaseUrl, /*Boolean*/ cachePrevention, /*Document?*/doc) {
	// summary:
	//    initialize the style processor.
	//    all changes of input and select elements of the given container starts the style processing.
	// container: DomNode [deprecated] - no longer in use (!!!fixme) -> just pass undefined
	// imageBaseUrl: String - object (current object) related base url e.g. /WebRoot/Store/Shops/DemoShop/Styles/Milestones
	// cachePrevention: Boolean - append cache prevention string (timestamp) to file refernces e.g. WebRoot/[..]/MediaGallery/CampingSite.gif?CachePrevention=1282899274013
	// doc: Document? - target document for style rules
	// example:
	//  |  var styleProcessor = new epages.style.Processor($('myform'),'/WebRoot/Store/Shops/DemoShop/');
	//  | styleProcessor.addRule(
	//  |    'NavBarFont',
	//  |    ".(%Class%) div, .(%Class%) span, .(%Class%) table, .(%Class%) td, .(%Class%) p, .(%Class%) ul li, .(%Class%) ol li, .(%Class%) blockquote, .(%Class%) input"
	//  |  );
	//  |  styleProcessor.addInput(
	//  |    'NavBarFont',                   // rule name
	//  |    'NavBarTop',                    // CSS ClassName
	//  |    'font-family',                  // CSS AttributeName
	//  |    $('TopFontFamily'),             // input
	//  |    function(el) {                  // callback
	//  |      if(el.value="") { alert("new value is empty"); }
	//  |    }
	//  |  );
	//  |  styleProcessor.fill();
	this.rules  = new epages.lang.Hash();
	this.inputs = new epages.lang.Hash();
	this.imageBaseUrl = imageBaseUrl;
	this.cachePrevention = cachePrevention;
	this.doc = doc || dojo.doc;
}

epages.style.Processor.prototype = {
	rules       : undefined,    // epages.lang.Hash - registerd css rules -> key: <css-selector:String> value <css-attributes:String>
	inputs      : undefined,    // epages.lang.Hash - registerd input elements (event trigger) e.g. {}
	imageBaseUrl: undefined,    // String - base url of file location
	cachePrevention: undefined, // Boolean - append cache prevention string on file url or not
	doc         : undefined,    // window.document - document to work on
	defaults    : {
		'text-decoration' : function (Value) { return 'text-decoration: '  + (Value == "1" ? 'underline' : 'none'  ); },
		'font-style' :      function (Value) { return 'font-style: '       + (Value == "1" ? 'italic'    : 'normal'); },
		'font-weight':      function (Value) { return 'font-weight: '      + (Value == "1" ? 'bold'      : 'normal'); },
		'navbar-horizontalalign':   function (Value) {
			if(Value == "left") {
				return 'text-align:'+Value+'; margin: 0 auto 0 0';
			} else if(Value == "right") {
				return 'text-align:'+Value+'; margin: 0 0 0 auto';
			} else {
				return 'text-align:'+Value+'; margin: 0 auto 0 auto';
			}
		},
		'box-horizontalalign':   function (Value) {
			if(Value == "left") {
				return 'margin: 0 auto 0 0';
			} else if(Value == "right") {
				return 'margin: 0 0 0 auto';
			} else {
				return 'margin: 0 auto 0 auto';
			}
		},
		'background-image':       function(Value, Owner) {
			if(Value!="none" && Value!="") {
				var myBaseURL = epages.lang.uri.isAbsolute(Value) ? '' : Owner.imageBaseUrl;
				return 'background-image: url(' + myBaseURL+ Value+Owner.attachCachePrevention()+')'
			} else {
			 return 'background-image: none';
			}
		},
		'background-image-right': function(Value, Owner) {
			if(Value!="none" && Value!="") {
				var myBaseURL = epages.lang.uri.isAbsolute(Value) ? '' : Owner.imageBaseUrl;
				return 'background-image: url(' + myBaseURL+ Value+Owner.attachCachePrevention()+') !important; background-position: top right'
			} else {
			 return 'background-image: none';
			}
		},
		'background-image-right-norepeat': function(Value, Owner) {
			if(Value!="none" && Value!="") {
				var myBaseURL = epages.lang.uri.isAbsolute(Value) ? '' : Owner.imageBaseUrl;
				return 'background-image: url(' + myBaseURL+ Value+Owner.attachCachePrevention()+') !important; background-position: top right !important; background-repeat: no-repeat'
			} else {
			 return 'background-image: none';
			}
		},
		'background-image-left': function(Value, Owner) {
			if(Value!="none" && Value!="") {
				var myBaseURL = epages.lang.uri.isAbsolute(Value) ? '' : Owner.imageBaseUrl;
				return 'background-image: url(' + myBaseURL+ Value+Owner.attachCachePrevention()+') !important; background-position: top left'
			} else {
			 return 'background-image: none';
			}
		},
		'background-image-left-norepeat': function(Value, Owner) {
			if(Value!="none" && Value!="") {
				var myBaseURL = epages.lang.uri.isAbsolute(Value) ? '' : Owner.imageBaseUrl;
				return 'background-image: url(' + myBaseURL+ Value+Owner.attachCachePrevention()+') !important; background-position: top left !important; background-repeat: no-repeat'
			} else {
			 return 'background-image: none';
			}
		},
		'background-image-bottom-right-norepeat': function(Value, Owner) {
			if(Value!="none" && Value!="") {
				var myBaseURL = epages.lang.uri.isAbsolute(Value) ? '' : Owner.imageBaseUrl;
				return 'background-image: url(' + myBaseURL+ Value+Owner.attachCachePrevention()+') !important; background-position: bottom right !important; background-repeat: no-repeat'
			} else {
			 return 'background-image: none';
			}
		},
		'background-image-bottom-left-norepeat':  function(Value, Owner) {
			if(Value!="none" && Value!="") {
				var myBaseURL = epages.lang.uri.isAbsolute(Value) ? '' : Owner.imageBaseUrl;
				return 'background-image: url(' + myBaseURL+ Value+Owner.attachCachePrevention()+') !important; background-position: bottom left !important; background-repeat: no-repeat'
			} else {
			 return 'background-image: none';
			}
		},
		'background-image-clear':   function(Value) {
			 return 'background-image: none';
		},
		'background-repeat': function(Value) {
			return 'background-repeat: '+ (Value=='1' ? 'repeat' : 'no-repeat');
		},
		'width-and-max-width': function(Value) {
			 return 'width:'+Value+' !important; max-width:'+Value;
		}
	},    // Object - hash with default css attribute function -> key <css-attribute> value <function>

	defaultCallbacks : {
		'update-button-active' : function (object, navbar) {
			this.updateButtonStyle(navbar.replace(/^NavBar/,''), 'ButtonActive');
		},
		'update-button-both' : function (object, navbar) {
			navbar = navbar.replace(/^NavBar/,'');
			this.updateButtonStyle(navbar, 'Button');
			this.updateButtonStyle(navbar, 'ButtonActive');
		},
		'set-imageset-left-backgound' : function (object, navbar) {
			if (object.value=="0") {
				this.set($('ImageSetLeft'+navbar.replace(/^NavBar/,'')+'BackgroundImage'));
			}
		},
		'set-imageset-right-backgound' : function (object, navbar) {
			this.set($('ImageSetRight'+navbar.replace(/^NavBar/,'')+'BackgroundImage'));
		},
		'set-imageset-right-boxbody' : function (object, navbar) {
			this.set($('ImageSetRight'+navbar.replace(/^NavBar/,'')+'BoxBodyImage'));
		},
		'set-imageset-right-box' : function (object, navbar) {
			this.set($('ImageSetRight'+navbar.replace(/^NavBar/,'')+'BoxImage'));
		},
		'set-imageset-boxbody-both' : function (object, navbar) {
			if(object.value=='0') {
				navbar = navbar.replace(/^NavBar/,'');
				this.set($('ImageSetRight'+navbar+'BoxBodyImage'));
				this.set($('ImageSetLeft'+navbar+'BoxBodyImage'));
			}
		},
		'set-imageset-box-both' : function (object, navbar) {
			if(object.value=='0') {
				navbar = navbar.replace(/^NavBar/,'');
				this.set($('ImageSetRight'+navbar+'BoxImage'));
				this.set($('ImageSetLeft'+navbar+'BoxImage'));
			}
		},
		'set-imageset-background-both' : function (object, navbar) {
			if(object.value=='0') {
				navbar = navbar.replace(/^NavBar/,'');
				this.set($('ImageSetRight'+navbar+'BackgroundImage'));
				this.set($('ImageSetLeft'+navbar+'BackgroundImage'));
			}
		},
		'set-boximage' : function (object, navbar) {
			navbar = navbar.replace(/^NavBar/,'');
			// clear background image
			this.set($(navbar+'BoxImageHidden')); // clear imageset floatin image
			this.set($(navbar+'BoxImage')); // set uploaded image again
			if(object.value=="") {
				this.set($('ImageSetRight'+navbar+'BoxImage'));
				this.set($('ImageSetLeft'+navbar+'BoxImage'));
			} else {
				this.set($(navbar+'BoxImageTiling')); // set tiling (repeat)
			}
		},
		'set-boxbodyimage' : function (object, navbar) {
			navbar = navbar.replace(/^NavBar/,'');
			// clear background image
			this.set($(navbar+'BoxBodyImageHidden')); // clear imageset floatin image
			this.set($(navbar+'BoxBodyImage')); // set uploaded image again
			if(object.value=="") {
				this.set($('ImageSetRight'+navbar+'BoxBodyImage'));
				this.set($('ImageSetLeft'+navbar+'BoxBodyImage'));
			} else {
				this.set($(navbar+'BoxBodyImageTiling')); // set tiling (repeat)
			}
		},
		'set-backgroundimage' : function (object, navbar) {
			navbar = navbar.replace(/^NavBar/,'');
			// clear background image
			this.set($(navbar+'BackgroundImageHidden')); // clear imageset floatin image
			this.set($(navbar+'BackgroundImage')); // set uploaded image again
			if(object.value=="") {
				if ($('ImageSetRight'+navbar+'BackgroundImage') != null) this.set($('ImageSetRight'+navbar+'BackgroundImage'));
				this.set($('ImageSetLeft'+navbar+'BackgroundImage'));
			} else {
				this.set($(navbar+'BackgroundImageTiling')); // set tiling (repeat)
			}
		}
	},	// Object - hash with default callback functions to set css to a designable element -> key <set-elmentname> value <function>

	attachCachePrevention: function() {
	// summary: generate cache prevention string e.g. "?CachePrevention=1234567890"
		var date = new Date();
		return (this.cachePrevention) ? "?CachePrevention="+date.getTime() : "";	// String
	},

	addRule: function(/*String*/ruleName, /*String*/CSSKey, /*Object*/CSSAttributes) {
		// summary:
		//    Adds a style rule to the processor.
		//    The rule name is a short keyword for a CSS key.
		//    The CSSKey can include a placeholder '(%Class)' (also multiple times) for a CSS class,
		//    which will be defined via the 'addInput' method.
		// example:
		//  |  var styleProcessor = new epages.style.Processor();
		//  |  styleProcessor.addRule(
		//  |    'NavBarFont',
		//  |    ".(%Class%) div, .(%Class%) span, .(%Class%) table, .(%Class%) td, .(%Class%) p, .(%Class%) ul li, .(%Class%) ol li, .(%Class%) blockquote, .(%Class%) input"
		//  |  );
		//  |  styleProcessor.addRule('ContentPadding', ".ContentArea");
		//  |  styleProcessor.addRule('NavBarContainer', ".(%Class%) .PropertyContainer");
		//  |  styleProcessor.addRule('ThisClass', ".(%Class%)");
		if (CSSAttributes === undefined) {
			CSSAttributes = {};
		}
		var rule = new epages.lang.Hash();
		rule.set('Key', CSSKey);
		rule.set('Attributes', CSSAttributes);
		this.rules.set(ruleName, rule);
	},

	addInput: function(/*String*/ruleName, /*String*/className, /*String*/attributeName, /*Object|String*/inputObject, /*Function?*/callback) {
		// summary:
		//    Adds a input element to the processor.
		//    Attention: The name of input fields has to be unique.
		// example:
		//  |  var styleProcessor = new epages.style.Processor();
		//  |  styleProcessor.addInput(
		//  |    'NavBarFont',                   // rule name
		//  |    'NavBarTop',                    // CSS ClassName
		//  |    'font-family',                  // CSS AttributeName
		//  |    $('TopFontFamily'),             // input
		//  |  );
		//  |  styleProcessor.addInput('NavBarContainer', 'NavBarLeft', 'width', $('BoxWidth'), function (el) {
		//  |    $('LeftNavBarWidth').value = (parseInt($('BoxWidth').getValue().replace(/px/, '')) + parseInt($('LeftPaddingRight').getValue().replace(/px/, '')) + parseInt($('LeftPaddingLeft').getValue().replace(/px/, ''))) + 'px';
		//  |    styleProcessor.set($('LeftNavBarWidth'));
		//  |  });
		if (typeof inputObject == 'string') {
			var inputObjectId = inputObject;
			inputObject = $(inputObjectId);
			if (inputObject == null) {
				//console.debug('epages.style.Processor.addInput: could not find input with id ' + inputObjectId + ', ruleName:' + ruleName);
				return;
			}
		} else if (inputObject == null) {
			//console.debug('epages.style.Processor.addInput cant add undefined object, rulename: ' + ruleName + '. attributeName:' + attributeName);
			return;
		}
		if (callback !== undefined && typeof callback != 'function') {
			callback = this.defaultCallbacks[callback];
		}
		// console.debug('epages.style.Processor.addInput add object for rule: ' + ruleName + '.');
		// console.debug(inputObject);
		// console.debug('epages.style.Processor.addInput add object: ' + inputObject.name + '.');
		dojo.connect(inputObject, 'onchange', this, '_set');
		this.inputs.set(inputObject.name, {
			rule:       ruleName,
			className:  className,
			attribute:  attributeName,
			target:     inputObject,
			callback:   callback
		});
	},

	_updateStyleSheet: function(/* String */ rule, /*String*/ text) {
		// summary:
		//    add a CSS rule with given text.
		// rule: String - CSS Selector
		// rule: String - CSS Text (Attributes) e.g. color: red;
		// example:
		//  |  var styleProcessor = new epages.style.Processor();
		//  |  styleProcessor._updateStyleSheet('.NavBarTop div, .NavBarTop span', 'padding-left: 5px');

		// we have to be sure that we don't conflict with pseudo classes like :hover
		if(!rule.match("html")) {rule = "html body " + rule.replace(/,/g, " , html body")}
		if(!rule.match(" a")) {text += ' !important'};
		if (epages.Browser.engine == 'MSIE') {
			var rules = rule.split(/,\s+/);
			var sheet = this.doc.styleSheets[this.doc.styleSheets.length-1];
			for(var i=0; i< rules.length; i++) {
				sheet.addRule(rules[i], text);
			}
		} else {
			var styleRules = this.doc.getElementById('StyleRules');
			if(!styleRules){
				styleRules = this.doc.createElement("div");
				styleRules.id = "StyleRules";
				this.doc.body.appendChild(styleRules);
			}
			var styleContainer = this.doc.createElement("div");
			styleContainer.innerHTML = "<div class='HideElement'>.<style type='text/css'>" + rule + '{' + text + '}' + "</style></div>";
			styleRules.appendChild(styleContainer);
		}
	},

	_set: function (/*Event*/evt) {
		// summary:
		//    call back function for event observer.
		var inputNode = evt.target;
		if (inputNode.name.match(/image/i) && inputNode.value.match(/^MediaGallery\//)) {
			inputNode.value = epages.lang.uri.getRelative(this.imageBaseUrl, epages.vars.SiteRoot + '/' + inputNode.value);
		}
		var isError=inputNode.getAttribute("ep_error");
		if (isError != 1) this.set(inputNode, true);
	},

	set: function (/*Object*/inputObject, /*Boolean?*/doCallback) {
		// summary:
		//    set style sheet information for this input element.
		//    Returns true if input rule was processed.
		if (doCallback === undefined) {
			doCallback = false;
		}
		var value = $E(inputObject).get();
		if(value === undefined) {
			console.warn('Value is undefined in '+this.declaredClass+'. inputObject:', inputObject);
		}
		var ruleInput = this.inputs.get(inputObject.name);
		if (ruleInput === undefined) {
			//console.debug('no rule for inputfield "' + inputObject.id + '" (' + inputObject.name + ')');
			return 0; // Boolean
		}
		var ruleName      = ruleInput.rule;      // eg: NavBarFont
		var className     = ruleInput.className; // eg: NavBarTop
		var attributeName = ruleInput.attribute; // eg: font-family
		var KeyTemplate = this.rules.get(ruleName).get('Key');
		var CSSKey = KeyTemplate.replace( /\(%Class%\)/g , className);
		// get value definition, could be a function/string/undefined
		var attributeDefinition = this.rules.get(ruleName).get('Attributes')[attributeName];
		if (attributeDefinition === undefined) {
			attributeDefinition = this.defaults[attributeName];
		}

		// get real css value
		var CSSValue = undefined;
		switch (typeof attributeDefinition) {
			case 'function':
				CSSValue = attributeDefinition(value, this);
				break;
			case 'string':
				CSSValue = attributeDefinition.replace( /\(%Value%\)/gi , value);
				break;
			default:
				CSSValue = attributeName + ': ' + value;
		}
		this._updateStyleSheet(CSSKey, CSSValue);
		if (doCallback && ruleInput.callback !== undefined) {
			ruleInput.callback.call(this, inputObject, className);
		}
		return 1; // Boolean
	},

	fill: function () {
		// summary:
		//    calls processor rules for each input (set stylesheet info to input field values).
		var currentThis = this;
		var iterator = function(el) {
			currentThis.set(el.target, true);
		};
		var values = this.inputs.values();
		$A(values).each(iterator);
	},

	updateButtonStyle: function (navBar, buttonType) {
		// summary: helper function for button style rule update
		// desctiption: Sets or clears the current and the opposite button image. Furthermore it sets the
		//              left and right ImageSet images on demand and activates image tiling.
		// example:
		// | styleProcessor.addInput('NavigationElement','Header','background-image-left-norepeat',$('HeaderButton'),function(el) {
		// |   updateButtonStyle('Header','Button');
		// | });
		this.set($(navBar+buttonType));
		this.set($(navBar+"Right"+buttonType));
		if(($(navBar+buttonType+'Hidden').value=="0" || $(navBar+buttonType+'Hidden').value=="") &&
			 $(navBar+buttonType).value=="" &&
			 $(navBar+"Right"+buttonType).value=="") {
			// (hide ImageSet=0 or empty) && (no left image) && (no right image) --> show ImageSet images
			this.set($('ImageSetLeft'+navBar+buttonType));
			this.set($('ImageSetRight'+navBar+buttonType));
		} else {
			// set image tiling style (background-repeat)
			// this is only possible with uploaded images - not with image set images
			this.set($(navBar+buttonType+'Tiling'));
		}
	}
};
