/*
	Copyright (c) 2006-2007, epages Software GmbH
	All Rights Reserved.

  epages.validate.css_test $Revision: 1.7.4.1 $

*/
dojo.provide('epages.validate.css_test');
dojo.require('epages.validate.css');

epages.validate.css.test = /*Boolean*/ function(/*epages.Test*/eTest) {
	// summary:
	//		test this package
	// example:
	// 	  var template = new epages.Template();
	//		boolean = template.test(eTest);
	//		// returns true if all test are successfully
	eTest.setPlanned(38);
	eTest.ok(1, 'loaded');

	// test isColor
	var values = $H({
		'#111111' 	: true,
		'#1111' 		: false,
		'#111a11' 	: true,
		'#111111a' 	: false,
		'blue' 			: true,
		'kill' 			: false,
		''					: true,
		'yellowg' 	: false
	});
	values.each(function(pair) {
		var result = epages.validate.css.isColor(pair.key);
		if (pair.value) {
			eTest.ok(result === true,'is color: ' + pair.key);
		} else {
			eTest.ok(result !== true,'no color: ' + pair.key);
		}
	});

	// test isWidth (== isSize)
	values = $H({
		'10in' 	: true,
		'10px' 	: true,
		'50 pt' : false,
		'blue' 	: false,
		'100%' 	: true,
		'0' 	 	: true
	});
	values.each(function(pair) {
		var result = epages.validate.css.isWidth(pair.key);
		if (pair.value) {
			eTest.ok(result === true,'is width: ' + pair.key);
		} else {
			eTest.ok(result !== true,'no width: ' + pair.key + " (" + result + ")" );
		}
	});

	// test isSize - LOWER_THAN_MIN
	var settings = { min: 10};
	var v = "5px";
	var result = epages.validate.css.isSize(v, settings);
	eTest.ok(result == "LOWER_THAN_MIN", "isSize / test lower than minimum: value="+v+" : min="+settings.min);

	// test isSize - GREATER_THAN_MAX
	settings = { max: 1.123};
	v = "5.4321mm";
	result = epages.validate.css.isSize(v, settings);
	eTest.ok(result == "GREATER_THAN_MAX", "isSize / test greater than maximum: value="+v+" : max="+settings.max);

	// test isSize - NOT_IN_RANGE
	settings = { min: 10, max: 12.123};
	v = "15.4321mm";
	result = epages.validate.css.isSize(v, settings);
	eTest.ok(result == "NOT_IN_RANGE", "isSize / test not in range: value="+v+" : min="+settings.min+" : max="+settings.max);

	// convert some colors
	values = $A([
		['#57A', 					"#5577aa"],
		['#ABC123', 			"#abc123"],
		['#57a', 					"#5577aa"],
		['#abc123', 			"#abc123"],
		['', 							"none"],
		[' ', 						"none"],
		['NONE', 						"none"],
		['none', 						"none"],
		['transparent', 	"transparent"],
		['TRANSPARENT', 	"transparent"],
		['TRANspARent', 	"transparent"],
		['papayawhip', 		"#ffefd5"],
		['myowncolor',		false],
		['FFFFFF',				false],
		['C3A',						false],
		['AG7ZNA',				false],
		['rgb(1,2,3)', 		'#010203'],
		['rgb(51,72,223)','#3348df'],
		['rgb(1,2,1111)',	false],
		['rgb(,5,0)',			false]
	]);
	values.each(function(e) {
		var result = epages.validate.css.color2Hex(e[0]);
		if (e[1] === false) {
			eTest.ok(true, 'color2Hex no color: "' + e[0] + '"');
		} else {
			eTest.ok(result === e[1],'color2Hex correct color: "' + e[0] + '" => ' + e[1] + ' == ' + result);
		}
	});

}