/*
	Copyright (c) 2006-2007, epages Software GmbH
	All Rights Reserved.
*/
dojo.provide("epages.widget.BingMapsWidget");
dojo.require("epages.widget.LocalizedWidget");

dojo.declare("epages.widget.BingMapsWidget", [epages.widget.LocalizedWidget],	{
		templatePath		: dojo.moduleUrl('epages.widget', "templates/BingMaps.html"),
		translationName	: dojo.moduleUrl('epages.widget', "templates/BingMaps"),
		popup						:	null,
		hideOnBlur			: false,
		disabled				: '',
		popupclass			:	'',
		// class variable which may be changed when including the widget on a page
		// specifys the IDs of the input/select fields the widget extracts the bing map find parameters from
		inputMapping		:	'{"streetID":"Street","zipID":"Zipcode","cityID":"City","countryID":"CountryID","useAsValues":false}',
		searchNodeIds		:	undefined,
		
		postMixInProperties : function(){
			// summary: check if widget is disabled / check for parameters supplied for the widget
			
			this.inherited("postMixInProperties", arguments);
			this.disabled = this.disabled.toLowerCase();
			this.disabled = (this.disabled =="true" || this.disabled =="disabled" || this.disabled == "1") ? true : false;
			this.searchNodeIds = dojo.fromJson(this.inputMapping);
		},
		
		postCreate  : function() {
			// summary: connect widget to clock event
			
			this.inherited("postCreate", arguments);
			//attach click event
			dojo.connect(this.buttonNode, 'onclick', this, 'toggleMap');
		},		
									
		_encode : function (/*String*/string) {
			// summary: 
			//     prepare a string to use it with HTTP/Get
			// description: 
			//     Transfer chars which can't be used in a GET string to ones which can
			//		 e.q. ,,,...
			// example: 
			//     | var result = _encode("");
			// returns: 
			//     encoded string
			string = string.replace(/\r\n/g,"\n");
			var utftext = "";

			for (var n = 0; n < string.length; n++) {

				var c = string.charCodeAt(n);

				if (c < 128) {
					utftext += String.fromCharCode(c);
				}
				else if((c > 127) && (c < 2048)) {
					utftext += String.fromCharCode((c >> 6) | 192);
					utftext += String.fromCharCode((c & 63) | 128);
				}
				else {
					utftext += String.fromCharCode((c >> 12) | 224);
					utftext += String.fromCharCode(((c >> 6) & 63) | 128);
					utftext += String.fromCharCode((c & 63) | 128);
				}
			}
			return utftext;
		},
		disable: function() {
			// summary: disable button
			this.disabled = true;
		},

		enable: function() {
			// summary: enable button
			this.disabled = false;
		},
				
		_createMapPopup: function() {
			// summary: Create popup
			this.popup = new epages.widget.Popup({
				parent:     this.buttonNode,
				around:     this.buttonNode,
				hideOnBlur: this.hideOnBlur,
				o :					{orient:{'TL':'BL'}},
				'class':    this['popupclass']
			}, this.popupNode);
			this.attr('value',this.value);
		},
		
		toggleMap: function() {
			// summary: 
			//    prepare and show the popup
			// description: 
			//    extract the search parameters for the bing maps find and add an 
			//    iframe containing the map to the popup
			if(!this.disabled) {
				if (this.popup == null){ 
					this._createMapPopup();
				}
				var searchString = "";
				if(this.searchNodeIds.useAsValues){
					searchString = "s="+this.searchNodeIds.streetID+"&"+
												 "z="+this.searchNodeIds.zipID+"&"+
												 "c="+this.searchNodeIds.cityID+"&"+
												 "i="+this.searchNodeIds.countryID;					
				}else{
					searchString = "s="+dojo.byId(this.searchNodeIds.streetID).value+"&"+
												 "z="+dojo.byId(this.searchNodeIds.zipID).value+"&"+
												 "c="+dojo.byId(this.searchNodeIds.cityID).value;
				
					var obj = dojo.byId(this.searchNodeIds.countryID);
					var country =obj.options[obj.selectedIndex].text.split(" ")[0];									 
					searchString += "&i="+country;
				}
				
				searchString = escape(this._encode(searchString));

				this.contentNode.innerHTML = '<iframe  src="'+dojo.moduleUrl('epages.widget')+'/BingMapiFrame.html?'+searchString+'" height="420" width="420" style="border:1px solid #999;">Alternativtext</iframe>';
				this.popup.toggle();
			}
		},
		closePopup: function() {
			// summary: remove the iframe and the bing map
			this.popup.hide();
			this.contentNode.innerHTML = '';
		}
	}
);
