/*
	Copyright (c) 2006-2007, epages Software GmbH
	All Rights Reserved.
*/
dojo.provide("epages.widget.Bubble");
dojo.require("epages.widget");
dojo.require('epages.html.element');

dojo.declare("epages.widget.Bubble", [dijit._Widget, dijit._Templated],	{
		optionsstring   : "",              // string - setup options as string
		options         : undefined,       // object - setup options (see setupContent for more)
		typeClass       : "",              // string - (Additional css class e.g. Wide)

		/* widget properties */
		templatePath    : dojo.moduleUrl('epages.widget', "templates/Bubble.html"),

		/* widget life cycle */
		postMixInProperties: function() {
			this.inherited("postMixInProperties", arguments);
			// prepare settings
			if(this.options===undefined) this.options = { content: '' };

			var optFromString = (this.optionsstring != "" ) ? dojo.fromJson(this.optionsstring) : undefined;
			if(optFromString) {
				dojo.mixin(this.options, optFromString);
			}
		},

		postCreate: function() {
			this.inherited("postCreate", arguments);
			this.setupContent(this.options);
			// events
			dojo.connect(this.closeNode, 'onclick', this, 'hide');
			dojo.connect(window, "onkeypress", this, "onKeypress");
			dojo.connect(window, "onresize", this, "hide");
			dojo.connect(document.body, "onkeypress", this, "onKeypress");
		},

		/* public methods */

		setupContent: function(/*Object*/opt) {
			// summary: setup bubble content
			// opt:{
			//       content     - widget, string
			//       x           - integer
			//       y           - integer
			//       typeClass   - string (Additional css class e.g. Wide)
			//       orientation - string -> specifies the location of the bubble arrow (left right bottom top-right)
			//     }
			
			// mix in options
			if(opt) {
				dojo.mixin(this.options, opt);
			}

			// remove old nodes
			while(this.containerNode.firstChild){
				this.containerNode.removeChild(this.containerNode.firstChild);
			}

			// set content
			if(this.options.content) {
				if(typeof this.options.content == "object" && this.options.content.domNode != undefined) {
					// is widget
					this.containerNode.appendChild(this.options.content.domNode);
				} else if(typeof this.options.content == "string") {
					// is string
					this.containerNode.innerHTML = this.options.content;
				} else if (typeof this.options.content == "object" && this.options.content.nodeType){
					this.containerNode.appendChild(this.options.content);
				} else {
					console.debug('Bubble.setupContent needs a domnode a widget or a string as content.');
				}
			}

			var xOffset = 0;
			var yOffset = 0;

			if(this.options.orientation=="" || this.options.orientation=="top") {
				dojo.removeClass(this.domNode, "Left");
				dojo.removeClass(this.domNode, "Right");
				dojo.removeClass(this.domNode, "TopRight");
				dojo.removeClass(this.domNode, "Bottom");
			}

			if(this.options.orientation == "top-right") {
				var typeClass = this.options.typeClass;

				if(typeClass && typeClass.match("Wide")) {
					xOffset = -330;
				} else {
					xOffset = -230;
				}

				dojo.addClass(this.domNode, "TopRight");
				dojo.removeClass(this.domNode, "Left");
				dojo.removeClass(this.domNode, "Bottom");
				dojo.removeClass(this.domNode, "Right");
			}

			if(this.options.orientation == "right") {
				var typeClass = this.options.typeClass;

				if(typeClass && typeClass.match("Wide")) {
					xOffset = -380;
				} else {
					xOffset = -280;
				}

				yOffset = -24;
				dojo.addClass(this.domNode, "Right");
				dojo.removeClass(this.domNode, "Left");
				dojo.removeClass(this.domNode, "Bottom");
				dojo.removeClass(this.domNode, "TopRight");
			}

			if(this.options.orientation == "left") {
				yOffset = -24;
				dojo.addClass(this.domNode, "Left");
				dojo.removeClass(this.domNode, "Right");
				dojo.removeClass(this.domNode, "Bottom");
				dojo.removeClass(this.domNode, "TopRight");
			}

			if(this.options.orientation == "bottom" ) {
				dojo.addClass(this.domNode, "Bottom");
				dojo.removeClass(this.domNode, "Left");
				dojo.removeClass(this.domNode, "Right");
				dojo.removeClass(this.domNode, "TopRight");
			}

			// set position
			if(this.options.x !== undefined && this.options.y !== undefined) {
				$E(this.domNode).setCss({
					'left': 		(this.options.x + xOffset) + 'px',
					'top': 			(this.options.y + yOffset) + 'px',
					'zIndex': 	99998
				});
			}
			// replace css class
			if(this.options.typeClass !== undefined) {
				dojo.removeClass(this.domNode, this.typeClass);
				dojo.addClass(this.domNode, this.options.typeClass)
				this.typeClass = this.options.typeClass;
			}
		},

		show: function() {
		// summary: function to show the bubble
			if(this.options.orientation == "bottom") {
				this.domNode.style.visibility = 'hidden';
				this.domNode.style.display = 'block';
					var curHeight = this.layerOverIframeNode.offsetHeight || this.layerOverIframe.clientHeight;
					$E(this.domNode).setCss({
						'top': (this.options.y - curHeight) + 'px',
						visibility: 'visible'
					});
			} else {
				this.domNode.style.display = 'block';
			}
		},

		hide: function() {
		// summary: function to hide the bubble
			this.domNode.style.display = 'none';
		},

		/* events */
		onKeypress: function(evt) {
		// summary: keypress event controller
			if(evt.keyCode == dojo.keys.ESCAPE) {
				this.hide();
			}
		}
	}
);