/*
	Copyright (c) 2006-2007, epages Software GmbH
	All Rights Reserved.

	epages.widget.Button $Revision: 1.35.4.1 $

*/
dojo.provide("epages.widget.Button");
dojo.require("epages.widget");
dojo.require('epages.string');
dojo.require("epages.event.interactionrestrictor");

epages.widget.ButtonChangeForms = []; // global array of all buttons

dojo.declare(
	"epages.widget.Button",
	[dijit._Widget], {


		name:    '', // String - name of button
		button:  '', // String - execute|batch|confirm|submit
		'class': '', // String - additional class parameter for outer div element
		onclick: '', // function - execute this string on click
		value:   '', // String - label of button
		disabled:'', // String - disable button like submit
		active:  '', // String - enable active class on startup
		iconSrc: '', // String - path to any icon image
		submit:  '', // String - do submit on click
		triggerClass: '',
		disableOnClick: '',
		buttonContainerNode: undefined,// DomNode - always the original input element wich is wrapped by buttonContainerNode

		_returnFalse: "", // return false handle container

		_removeEventList: undefined,
		_onClickEvents: undefined,

		postCreate: function() {
			this.inherited('postCreate', arguments);

			var el = this.domNode;

			var type;
			var triggerClass = '';
			if (dojo.hasClass(el, 'RunButton')) {type = 'batch'}
			else if (dojo.hasClass(el, 'SaveButton'))   type = 'submit';
			else if (dojo.hasClass(el, 'DeleteButton')) type = 'confirm';
			else if (dojo.hasClass(el, 'CancelButton')) type = 'confirm';
			else if (dojo.hasClass(el, 'ExtraButton'))  { type = 'execute'; triggerClass="ExtraButtonTrigger"; }
			else if (dojo.hasClass(el, 'SearchButton')) type = 'execute';
			else { type = 'confirm'; }
			this.button = type;

			var submit = '';
			if (el.type == 'image' ||el.type == 'submit') {submit = 'true'}
			var disabled = '';
			if (el.disabled) {disabled = 'true'}
			if (el.getAttribute("triggerClass") != null && el.getAttribute("triggerClass") != '') {
				triggerClass = el.getAttribute("triggerClass");
			}

			this.triggerClass = triggerClass;
			this.disabled = disabled;
			this.submit = submit;

			//create outer element
			var buttonContainerNode = document.createElement("span");

			//export all classes to new button tag
			buttonContainerNode.setAttribute((document.all ? "className" : "class"), this["class"]);
			if (!dojo.hasClass(buttonContainerNode, 'Button')) {
				dojo.addClass(buttonContainerNode, 'Button');
			}
			dojo.addClass(buttonContainerNode, 'EnableHover');

			//insert button at the right place
			this.domNode.parentNode.insertBefore(buttonContainerNode, this.domNode);

			//transform old input into new button
			this.domNode.setAttribute((document.all ? "className" : "class"), "ButtonContainer");
			buttonContainerNode.appendChild(this.domNode);

			// is a submit button ?
			this.submit = this.submit != '' || this.button == 'submit';

			// disableOnClick - default settings for undefined
			if(this.disableOnClick == "" && (this.domNode.type == "submit" || this.domNode.href))
				this.disableOnClick = true;

			// disableOnClick - to bool
			this.disableOnClick = epages.string.toBoolean(this.disableOnClick);

			// is disabled ?
			this.disabled = this.disabled != '' || this.button == 'submit';
			if (this.disabled) {
				this.domNode.disabled = true;
				this._returnFalse = dojo.connect(this.domNode, "click", this, function(){return false});
				dojo.addClass(buttonContainerNode, 'Disabled');
			}

			// is active on startup
			if (this.active!='') {
				dojo.addClass(buttonContainerNode, 'Active');
			}

			// insert icon if an image url exists
			if (this.iconSrc) {
				var buttonIconContainer = document.createElement("span");

				if(this.onclick) {buttonIconContainer.onclick = this.onclick}
				if(this.onmousedown) {buttonIconContainer.onmousedown = this.onmousedown}
				if(this.onmouseup) {buttonIconContainer.onmouseup = this.onmouseup}
				if(this.onmouseover) {buttonIconContainer.onmouseover = this.onmouseover}

				buttonIconContainer.form = this.domNode.form;
				buttonIconContainer.className = "ButtonIconContainer";
				//buttonIconContainer.style.background = "url(" + this.iconSrc + ") no-repeat center";

				buttonIconContainer.innerHTML = "<i class='Sprite " + this.iconSrc + "'></i>";

				buttonContainerNode.insertBefore(buttonIconContainer, this.domNode);
				dojo.addClass(this.domNode, "ButtonIconPadding");
			}

			// add events to trigger activation of buttons
			this.form = this.domNode.form;
			this._removeEventList = []; // remove events later if button is enabled
			this._onClickEvents = []; // event list for click, enable or disable

			if (this.button == 'execute' || this.button == 'submit' || this.button == 'batch' ) {
				var inputs = this.form.elements;
				// filter triggerClass or remove NoButtonTrigger
				if (this.triggerClass != '') {
					inputs = dojo.filter(inputs, function (el) { return dojo.hasClass(el, this.triggerClass); }, this);
				} else {
					inputs = dojo.filter(inputs, function (el) { return ! dojo.hasClass(el, 'NoButtonTrigger'); });
				}
				// filter BatchTrigger
				if (this.button == 'batch') {
					inputs = dojo.filter(inputs, function (el) { return dojo.hasClass(el, 'RunButtonTrigger'); });
				} else {
					inputs = dojo.filter(inputs, function (el) { return !dojo.hasClass(el, 'RunButtonTrigger'); });
				}
				// add change events to enable buttons
				this.registerInputs(inputs);
			}

			// buttonContainerNode as widget property
			this.buttonContainerNode = buttonContainerNode;

			// register on click
			if (!this.disabled) {
				this._registerEvents();
			}

		},

		registerInput: function(el) {
			var it=el.type;
			if (it == "hidden" || it == "text" || it == "password" || it == "file" || it == "textarea") {
				this._removeEventList.push(dojo.connect(el, 'keydown', this, '_OnKeyPressInput')); // fastest activation
				this._removeEventList.push(dojo.connect(el, 'change', this, '_OnKeyPressInput')); // need for other javascripts
				this._removeEventList.push(dojo.connect(el, 'paste',  this, '_OnKeyPressInput')); // need for paste with mouse (right context menu)
				this._removeEventList.push(dojo.connect(el, 'blur', this, '_OnKeyPressInput')); // need for select old entry from browser history
			} else if (it == "checkbox" || it == "radio") {
				this._removeEventList.push(dojo.connect(el, 'click', this, '_OnKeyPressInput'));
				this._removeEventList.push(dojo.connect(el, 'keypress', this, '_OnKeyPressInput'));
				this._removeEventList.push(dojo.connect(el, 'change', this, '_OnKeyPressInput'));
			} else if (el.tagName == "SELECT") {
				this._removeEventList.push(dojo.connect(el, 'change', this, '_OnKeyPressInput'));
			}
		},

		registerInputs: function(inputs){
			dojo.forEach(inputs, function (el) {
				this.registerInput(el);
			}, this);
		},

		_OnKeyPressInput: function (evt) {
			if(!this.domNode.parentNode.className.match("Active")) {
				var t = evt.target;
				if(t.className.match("DialogError") || (evt.type =='blur' && (t.type == 'text' || t.type == 'textarea') && t.defaultValue === t.value)) return;

				if(epages.vars.activeButton) {
					dojo.removeClass(epages.vars.activeButton, "Active");
				}
				epages.vars.activeButton = this.domNode.parentNode;

				dojo.addClass(this.domNode.parentNode, 'Active');
				dojo.removeClass(this.domNode.parentNode, 'Disabled');

				if (this.disabled) this.enable();

				if (this.button == 'submit') {
					if (!document.title.match(/\*$/)) document.title+="*";
					epages.widget.ButtonChangeForms.push(this.form);
				}
			}
		},

		_OnClick: function (evt) {
			if(!this.disabled && this.disableOnClick) {
				dojo.publish("interactionRestrictor/suspend", ["interactionRestrictorFor_" + this.id, this.domNode.parentNode, true]);
				/* prevent double click */
				setTimeout(dojo.hitch(this, function(){this.disable()}), 20);

				setTimeout(dojo.hitch(this, function() {
					this.enable();
					dojo.removeClass(this.domNode.parentNode, 'Active');
					dojo.publish("interactionRestrictor/permit", ["interactionRestrictorFor_" + this.id]);
				}), 2000);
			}
			if (this.button == 'confirm' || this.button == 'submit') {
				epages.widget.ButtonChangeForms = [];
			} else if (this.button != 'batch'){
				var formNode = this.form;
				epages.widget.ButtonChangeForms = dojo.filter(epages.widget.ButtonChangeForms, function (el) {
					return el != formNode;
				});
			}
		},


		_OnMouseDown: function() {
		  //summary: emulates the ":active" pseudoclass in the MSIE
			dojo.addClass(this.domNode.parentNode, 'ButtonClicked');
			dojo.addClass(this.domNode, 'ButtonContainerClicked');
		},

		_OnMouseUp: function() {
			dojo.removeClass(this.domNode.parentNode, 'ButtonClicked');
			dojo.removeClass(this.domNode, 'ButtonContainerClicked');
		},

		_registerEvents: function () {
			this._onClickEvents.push(dojo.connect(this.domNode,'click', this, '_OnClick'));
			this._onClickEvents.push(dojo.connect(this.domNode,'mousedown', this, '_OnMouseDown'));
			this._onClickEvents.push(dojo.connect(this.domNode,'mouseup', this, '_OnMouseUp'));
			this._onClickEvents.push(dojo.connect(this.domNode,'mouseout', this, '_OnMouseUp'));
		},

		enable: function () {
				this.disabled = false;
				this.domNode.disabled = false;
				dojo.disconnect(this._returnFalse);

				// switch classes
				dojo.addClass(this.domNode.parentNode, 'Active');
				dojo.removeClass(this.domNode.parentNode, 'Disabled');
				if(this._onClickEvents.length == 0) {
					this._registerEvents();
				}
		},

		disable: function () {
			this.disabled = true;
			this.domNode.disabled = true;

			dojo.disconnect(this._returnFalse);
			this._returnFalse = dojo.connect(this.domNode, "click", this, function(){return false});

			// switch classes
			dojo.removeClass(this.domNode.parentNode, 'Active');
			dojo.addClass(this.domNode.parentNode, 'Disabled');

			// unregister events
			dojo.forEach(this._onClickEvents, dojo.disconnect);
			this._onClickEvents = [];
		},

		disableIfEmpty: function (value) {
			if (value == "") this.disable();
			else this.enable();
		},

		hide: function() {
			dojo.addClass(this.buttonContainerNode, "HideElement");
		},

		show: function() {
			dojo.removeClass(this.buttonContainerNode, "HideElement");
		}
	}
);

epages.onUnloadFunctions.push(function() {
	return epages.widget.ButtonChangeForms.length > 0;
});

epages.widget.TransformButtons = function (domNode) {
	dojo.forEach(dojo.query('.Button', domNode), function (el) {
		if (el.tagName != 'INPUT' || el.type == 'hidden') return;

		var type;
		var triggerClass = '';
		if (dojo.hasClass(el, 'RunButton')) {type = 'batch'}
		else if (dojo.hasClass(el, 'SaveButton'))   type = 'submit';
		else if (dojo.hasClass(el, 'DeleteButton')) type = 'confirm';
		else if (dojo.hasClass(el, 'CancelButton')) type = 'confirm';
		else if (dojo.hasClass(el, 'ExtraButton'))  { type = 'execute'; triggerClass="ExtraButtonTrigger"; }
		else if (dojo.hasClass(el, 'SearchButton')) type = 'execute';
		else { type = 'confirm'; }


		var submit = '';
		if (el.type == 'image' ||el.type == 'submit') {submit = 'true'}
		var disabled = '';
		if (el.disabled) {disabled = 'true'}
		if (el.getAttribute("triggerClass") != null && el.getAttribute("triggerClass") != '') {
			triggerClass = el.getAttribute("triggerClass");
		}
		new epages.widget.Button({
			button:  type,
			name:    el.name,
			value:   el.value,
			'class': el.className,
			onclick: el.onclick,
			iconSrc: el.getAttribute("iconsrc"),
			submit:  submit,
			disabled: disabled,
			triggerClass: triggerClass
			 }, el);
	});
}

// convert old buttons
dojo.addOnLoad(function () {epages.widget.TransformButtons(dojo.body())});