/*
	Copyright (c) 2006-2009, epages Software GmbH
	All Rights Reserved.

	epages.widget.DndTree $Revision: 1.2.4.1 $
*/

dojo.provide("epages.widget.DndTree");
dojo.require("dijit.Tree");

// note: need to declare node before tree because DndTree refers to DndTreeNode class
dojo.declare(
	"epages.widget.DndTreeNode",
	[dijit._TreeNode],
	{
		/* widget life cycle */
		postCreate: function(){
				this.inherited("postCreate", arguments);
				dojo.connect(this.domNode, 'mousemove', this,   '_dndOnMousemove');
				dojo.connect(this.domNode, "onmousedown", this, '_dndOnMousedown');

				if(epages.browser.engine!= "MSIE") {	// needed in ff for dnd
					this.domNode.style.position="relative";
				}
		},

		/* methods: getter, setter */

		_checkIsChild: function(/* treeNode */ checkNode) {
		// summary: check if draggedItem is a child node
			var node = this;
			var isChild = false;
			while(node !== this.tree && !isChild) {
				if(node === checkNode) {
					isChild = true;
				}
				if(node.getParent) {
					node = node.getParent();
				} else {
					console.warn("Invalid drag item in _checkIsChild / "+this.declaredClass);
				}
			}
			return isChild;
		},

		_checkChildrenAllowed: function(/* widget or widget[] */ draggesItems) {
		// summary: stub function to check wether children are allowed or not
			return true;
		},

		_checkDropAllowed: function(draggesItems, dropTargetNode) {
		// summary: check if drop is allowed in this category
			if(this.tree.getCurrentDragHandler() == this.tree.id) {
				if(dropTargetNode === this.tree) {
					this.tree.setDropForbidden(false);
					return false;
				}
				var isChild = dropTargetNode._checkIsChild(draggesItems);
				this.tree.setDropForbidden(isChild);
				return !isChild;
			}
			return true;
		},

		/* events */
		_dndOnMousedown : function(evt) {
			// summary: pass node to tree function _treeNodeOnMouseDown
			this.tree._treeNodeOnMouseDown(this, evt);
			dojo.stopEvent(evt);
		},

		_dndOnMousemove: function(evt) {
			if(this.tree._isDraggingInside) {
				var draggesItems = this.tree.getCurrentDragItems();
				var canHaveChildren = this._checkChildrenAllowed(draggesItems);
				var targetNode = evt.target;
				var yOff = (evt.layerY) ? evt.layerY : evt.y;
				var cHeight = this.domNode.clientHeight;
				var isOnBottom = (yOff > 11 && cHeight <22) ? true : false;
				var pos = this._getPositionToParent(this.tree.domNode);
				var dropTargetNode=(canHaveChildren && isOnBottom) ? this: this.getParent()
				var isDropAllowed = this._checkDropAllowed(draggesItems, dropTargetNode);

				var xOff = (canHaveChildren && isOnBottom) ? 30 : 10;
				if(isOnBottom) {
					pos.y += 19;
				} else {
					pos.y -= 2;
				}

				dojo.style(this.tree._dndGhostNode, "top",pos.y+"px");
				dojo.style(this.tree._dndGhostNode, "left",(pos.x + xOff)+"px");
				dojo.style(this.tree._dndGhostNode, "display","");

				if(!this.tree.getDropForbidden()) {
					dojo.addClass(document.body, "CanDrop");
				} else {
					dojo.removeClass(document.body, "CanDrop");
				}
				this.tree.setDropTargetNode(dropTargetNode, this, (isOnBottom) ? 1 : -1 );
				dojo.stopEvent(evt);
			}
		},

		/* help functions */
		_getPositionToParent: function(parentNode) {
			// summary: calcualte position to the specified parentnode -> copied from epages.html.element.js which is buggy in current ep version
			var node = this.domNode;
			var x = 0, y = 0;

			while (node != undefined && parentNode !== node) {
				if (node.offsetLeft !== undefined && node.offsetTop !== undefined) {
					x += node.offsetLeft;
					y += node.offsetTop;
				}
				node = node.offsetParent;
			}
			return {'x': x, 'y': y};
		}
});

dojo.declare(
	"epages.widget.DndTree",
	[dijit.Tree], {
		TreeNodeClass        : epages.widget.DndTreeNode,
		dropListener         : "",   // String - comma seperated list

		eventNames           :         undefined,
		eventNamesDefault    : {
			drop          : 'drop',    // String - dnd drop
			move          : 'move',    // String - dnd move
			startDragging : 'startDragging',  // String - dnd start dragging
			stopDnd       : 'stopDnd'  // String - dnd stop dnd - dnd end / dnd cancelled
		},

		_dropTarget           : undefined,
		_lastSelectedDragNode : undefined,

		/* avatar */
		_dndAvatarNode        : undefined,
		_dndAvatarContentNode : undefined,
		_isAvatarVisible      : false,

		/* ghost */
		_dndGhostNode         : undefined,

		/* dnd state */
		_startDragging        : false,		// Boolean - indicates that dragging is triggered (mousedown on selected element)
		_isDragging           : false,		// Boolean - indicates that a user really drags items
		_isDraggingInside     : false,		// Boolean - indicates that a user really drags items inside the tree
		_mousmoveInsideTree   : false,		// Boolean - helper property

		/* widget life cycle */

		postCreate: function() {
			this.inherited("postCreate", arguments);

			this.domNode.style.position="relative";
			this._dropTarget = {
				dropNode : undefined,					// DomNode - drop target node (container/parent)
				dropForbidden : undefined,		// Boolean - indcates that a drop target node is a forbidden target
				relatedNode : undefined,			// DomNode - drop before or after this element
				currentDragItems: undefined,	// currntly dragged items (items in dnd avatar)
				insertDir : -1,								// Integer - insert direction: -1 = before / 1 = after
				handler : undefined						// String - id of current drag handler
			};

			this._buildDndGhost();
			this._buildDndAvatar();
			this._adjustEventNames();
			this._appendDndEvents();
		},

		/* override methods */
		_setChildren: function(/* Object[] */ childrenArray){
			// summary:
			//	| Sets the children of this node.
			//	| Sets this.isExpandable based on whether or not there are children
			// 	| Takes array of objects like: {label: ...} (_TreeNode options basically)
			//	| See parameters of _TreeNode for details.
			
			// epages change: create treeNodes of type referred in this.TreeNodeClass
			this.destroyDescendants();

			this.state = "LOADED";
			var nodeMap= {};
			if(childrenArray && childrenArray.length > 0){
				this.isExpandable = true;
				if(!this.containerNode){ // maybe this node was unfolderized and still has container
					this.containerNode = this.tree.containerNodeTemplate.cloneNode(true);
					this.domNode.appendChild(this.containerNode);
				}

				// Create _TreeNode widget for each specified tree node
				dojo.forEach(childrenArray, function(childParams){
					var child = new this.TreeNodeClass(dojo.mixin({
						tree: this.tree,
						label: this.tree.getLabel(childParams.item)
					}, childParams));
					this.addChild(child);
					var identity = this.tree.store.getIdentity(childParams.item);
					nodeMap[identity] = child;
					if(this.tree.persist){
						if(this.tree._openedItemIds[identity]){
							this.tree._expandNode(child);
						}
					}
				}, this);

				// note that updateLayout() needs to be called on each child after
				// _all_ the children exist
				dojo.forEach(this.getChildren(), function(child, idx){
					child._updateLayout();
				});
			}else{
				this.isExpandable=false;
			}

			if(this._setExpando){
				// change expando to/form dot or + icon, as appropriate
				this._setExpando(false);
			}

			if(this.isTree && this._hideRoot){
				// put first child in tab index if one exists.
				var fc = this.getChildren()[0];
				var tabnode = fc ? fc.labelNode : this.domNode;
				tabnode.setAttribute("tabIndex", "0");
			}

			// create animations for showing/hiding the children (if children exist)
			if(this.containerNode && !this._wipeIn){
				this._wipeIn = dojo.fx.wipeIn({node: this.containerNode, duration: 150});
				this._wipeOut = dojo.fx.wipeOut({node: this.containerNode, duration: 150});
			}

			return nodeMap;
		},

		/* prepare dnd */
		_adjustEventNames: function() {
			this.eventNames = {};
			for(var name in this.eventNamesDefault) {
				if (this.eventNames[name] === undefined) {
					this.eventNames[name] = this.id + "/" + this.eventNamesDefault[name];
				}
			}
		},

		_buildDndGhost: function() {
			this._dndGhostNode = document.createElement("div");
			dojo.addClass(this._dndGhostNode, "DndTreeDropGhost");
			dojo.style(this._dndGhostNode, "top","0");
			dojo.style(this._dndGhostNode, "left","0");
			dojo.style(this._dndGhostNode, "display","none");
			this.domNode.appendChild(this._dndGhostNode);
		},

		_buildDndAvatar: function() {
			this._dndAvatarNode = document.createElement("div");
			dojo.addClass(this._dndAvatarNode, "HideElement DndAvatar")
			this._dndAvatarContentNode = document.createElement("div");
			this._dndAvatarContentNode.className="Inner";
			this._dndAvatarNode.appendChild(this._dndAvatarContentNode);
			document.body.appendChild(this._dndAvatarNode);
		},

		_appendDndEvents: function() {
			dojo.connect(this.domNode, 'onmousemove', this, '_onmousemoveDomNode');
			dojo.connect(document, 'onmousemove', this,     '_onmousemoveOutSide');
			dojo.connect(window, "onkeypress", this,        '_windowOnKeypress');
			dojo.connect(document.body, "onkeypress", this, '_windowOnKeypress');
			dojo.connect(document, 'onmousemove', this,     '_documentOnMousemove');
			dojo.connect(document, 'onmouseup', this,       '_documentOnMouseup');

			//drop listener
			var dropListener = this.dropListener.split(",");
			for(var el in dropListener) {
				dojo.subscribe(dropListener[el]+"/drop", this, "_onDrop");
				dojo.subscribe(dropListener[el]+"/startDragging", this, "_onStartDragging");
				dojo.subscribe(dropListener[el]+"/move", this, "_onDndMove");

				if(dropListener[el] != this.id) {
					dojo.subscribe(dropListener[el]+"/stopDnd", this, "_stopDnd");
				}
			}
		},

		/* methods: getter, setter, ... */
		setCurrentDragItems : function(/* Array or Object */ items, /* string */ handler)  {
		// summary: saves currently dragged items  (set by listeners)
			this._dropTarget.currentDragItems = items;
			this._dropTarget.handler = handler;
		},

		getCurrentDragItems : function()  {
		// summary: returns currently dragged items (set by listeners)
			return this._dropTarget.currentDragItems;
		},

		getCurrentDragHandler : function()  {
		// summary: returns id of current draghandler (set by listeners)
			return this._dropTarget.handler;
		},

		// ??? simplify duplicate function?
		setSelectedDragNode: function(/* treeNode */ node) {
		// summary: sets value of _lastSelectedDragNode (set by events - internal selection)
			this._lastSelectedDragNode = node;
		},

		getSelectedDragNode: function() {
		// summary: returns value of _lastSelectedDragNode (get by events - internal selection)
			return this._lastSelectedDragNode;
		},

		setDropTargetNode: function(/* treeNode */ dropNode, /* treeNode */ relatedNode, /* int */ insertDir) {
		// summary: sets the drop container target
		// dropNode: drop Container (parent)
		// relatedNode: drop before or after this tree node
		// insertDir: insert direction: -1 = before / 1 = after
			this._dropTarget.dropNode=dropNode;
			this._dropTarget.relatedNode = relatedNode;
			this._dropTarget.insertDir= insertDir;
		},

		getDropTargetNode : function() {
		//summary: returns the tree node which is currently selected by dnd as container
			return this._dropTarget.dropNode;
		},

		setDropForbidden: function(/* boolean */ forbid) {
		// summary: sets value dropForbidden
			this._dropTarget.dropForbidden = forbid;
		},

		getDropForbidden: function() {
		// summary: returns value dropForbidden
			return this._dropTarget.dropForbidden;
		},

		getDropRelatedDir : function() {
		//summary: returns the direction (above or below) where to drop elements relative to the related tree node
			return this._dropTarget.insertDir;
		},

		/* events */
		_onmousemoveDomNode : function(evt) {
		// summary: set state isDraggingInsideTree
			this._mousmoveInsideTree = true;
			if(this._isDragging)
				this._isDraggingInside = true;
		},

		_onmousemoveOutSide : function(evt) {
		  // summary: set state: draggging outside tree
			
			// mousemove outside tree
			if(!this._mousmoveInsideTree) {
				this.setDropTargetNode(undefined, undefined, -1);
				dojo.style(this._dndGhostNode, "display","none");
				dojo.removeClass(document.body, "CanDrop");
				this._isDraggingInside = false;
			}
			this._mousmoveInsideTree=false;
		},

		_windowOnKeypress :function(evt) {
			// summary: captures keypress event on window / ESC stops dnd
			if(evt.keyCode == dojo.keys.ESCAPE) {
				this._stopDnd();
			}
		},

		_treeNodeOnMouseDown: function(treeNode, evt) {
			// summary: prepare for dragging (start dragging)
			this._startDragging=true;
			this.setSelectedDragNode(treeNode);
			// prepare dnd avatar
			var itemNames = treeNode.label;
			this._dndAvatarContentNode.innerHTML = itemNames;

			//avoid text selection
			document.body.onselectstart = function() {return false};
			this.domNode.style.MozUserSelect = "none";
		},

		_documentOnMousemove: function(evt) {
			if(this._startDragging && !this._isDragging) {
				var pos = this._getMousePosition(evt);
				dojo.publish(this.eventNames.startDragging, [{
					x: pos.x,
					y: pos.y,
					items: this.getSelectedDragNode(),
					handler: this.id
				}]);	// real start dragging starts with moving not onmousedown
			}
			if(this._startDragging) {
				this._isDragging=true;
				var pos = this._getMousePosition(evt);
				// avatar
				this._dndAvatarNode.style.left = (pos.x + 10)+"px";
				this._dndAvatarNode.style.top= (pos.y +10)+"px";
				if(!this._isAvatarVisible)
					dojo.removeClass(this._dndAvatarNode, "HideElement");

				var treeNode = this.getSelectedDragNode();
				dojo.addClass(treeNode.domNode, "StartDragging");

				// publish event
				dojo.publish(this.eventNames.move, [{
					x: pos.x,
					y: pos.y,
					items: this.getSelectedDragNode(),
					handler: this.id
				}]);
			}
		},

		_documentOnMouseup: function(evt) {
			// summary: drop currently dragged tree node (publish topic)
			if(this._isDragging) {
				var pos = this._getMousePosition(evt);
				dojo.publish(this.eventNames.drop, [{
					x: pos.x,
					y: pos.y,
					items: this.getSelectedDragNode(),
					handler: this.id
				}]);
			}
			this._stopDnd();
		},

		/* listerns / subscriptions */

		onDrop: function(dndObj) {
			// summary: User overridable function / called by _onDrop
			// tags: extendable
		},

		_onDrop: function(dndObj) {
			//summary: called by listeners after drop
			if(this._isDraggingInside && !this.getDropForbidden()) {
				this.onDrop(dndObj);
			}
			this._stopDnd();
		},

		_stopDnd: function() {
			// summary: stops current dragging (does not publish drop)
			this._startDragging=false;

			this._isDragging=false;
			this._isDraggingInside = false;

			dojo.publish(this.eventNames.stopDnd, [{
					x: undefined,
					y: undefined,
					items: this.getSelectedDragNode(),
					handler: this.id
			}]);
			dojo.addClass(this.domNode, "StartDragging");

			// hide avatar
			dojo.addClass(this._dndAvatarNode, "HideElement");

			// hide ghost
			dojo.style(this._dndGhostNode, "display","none");
			dojo.removeClass(document.body, "CanDrop");

			// reset item display
			var items = this.getCurrentDragItems();

			if(items) {
				if(items.length !== undefined) {
					for(var el in items) {
						if(items[el] && items[el].domNode) {	// widgets can be destroyed on drop
							dojo.removeClass(items[el].domNode, "StartDragging");
						}
					}
				} else {
					if(items && items.domNode)  {	// widgets can be destroyed on drop
						dojo.removeClass(items.domNode, "StartDragging");
					}
				}
			}

			// reset dnd target / drag items / drop state
			this.setDropTargetNode(undefined, undefined, -1);
			this.setCurrentDragItems(undefined, undefined);
			this.setDropForbidden(false);

			//allow text selection
			document.body.onselectstart = function(){};
			// this.domNode.style.MozUserSelect = ""; --> this somehow harms drop in tree
		},

		_onStartDragging: function() {
			// summary: set state: someone is dragging something on the page (document)
			this._isDragging = true;
		},

		_onDndMove: function(opt) {
			// summary: sets the currently dragged transferred via listener
			this.setCurrentDragItems(opt.items, opt.handler);
		},

		/* help functions */
		_getMousePosition: function(evt) {
			// summary: returns mouse position on screen
			if(epages.Browser.engine=="MSIE") {
				var x= evt.clientX;
				var y= evt.clientY;
			} else {
				var x= evt.pageX;
				var y= evt.pageY;
			}
			return { x:x, y:y };
		}
});