/*
	Copyright (c) 2006-2007, epages Software GmbH
	All Rights Reserved.

	epages.widget.FormElements $Revision: 1.1.2.1 $

*/
(function(dj){

	dj.provide("epages.widget.FormElements");
	
	dj.require("dojo.date.locale");
	dj.require("dojo.regexp");
	dj.require('epages.epages');
	dj.require('epages.localize');
	dj.require('epages.browser');
	dj.require('epages.event');
	dj.require("epages.widget.LocalizedWidget");

/* START * ep */
	var _separator = {
			decimal: 		epages.vars.Locale.decimalSep,
			thousand: 		epages.vars.Locale.thousandSep
		},
		_format = {
			date:			epages.localize.getDojoDateTimePattern( epages.vars.Locale.dateFormat ),
			datetime:		epages.localize.getDojoDateTimePattern( epages.vars.Locale.datetimeFormat ),
			month:			'MM',
			time:			epages.localize.getDojoDateTimePattern( epages.vars.Locale.timeFormat ),
			week:			'w'
		},
		_pattern = {
			date:			dojo.date.locale.regexp({ selector:'date',	datePattern:_format.date,		timePattern:'' }),
			datetime:		dojo.date.locale.regexp({ selector:'date',	datePattern:_format.datetime,	timePattern:'' }),
			month: 			dojo.date.locale.regexp({ selector:'date',	datePattern:_format.month,		timePattern:'' }),
			time:			dojo.date.locale.regexp({ selector:'date',	datePattern:_format.time,		timePattern:'' }),
			week: 			dojo.date.locale.regexp({ selector:'date',	datePattern:_format.week,		timePattern:'' }),
			email:			dj.regexp.escapeString('^[A-Za-z0-9._-]{1,}@[A-Za-z0-9._-]{2,}[A-Za-z]{2,4}', '^([]{})$'),
			number:			dj.regexp.escapeString('^[0-9'+_separator.thousand+']{1,}['+_separator.decimal+']{0,1}[0-9]{0,}$', '^([]{})$')
		};
		 
	window.ep = {
		locale:{
			getFormat: function(type) {
				return _format[type];
			},
			getPattern: function(type) {
				return _pattern[type];
			},
			getSeparator: function(type) {
				return _separator[type];
			}
		},
		math:{
			unixAddTime: function( unix , add ) {
				return unix+add + ( (new Date((unix+add)*1000).getTimezoneOffset() - new Date(unix*1000).getTimezoneOffset()) * 60 );
			}
		},
		parse:{
			date2unix: function( input , type ) {
				var ret = dj.date.locale.parse( 
							input, 
							{ 
								selector: 		'date', 
								datePattern: 	ep.locale.getFormat(type), 
								timePattern: 	''
							}
						);
				return ret ? ( ret.getTime() / 1000 ) : null;
			},
			unix2date: function( input , type ) {
				return dj.date.locale.format( 
							new Date(input*1000), 
							{ 
								selector: 		'date', 
								datePattern: 	ep.locale.getFormat(type), 
								timePattern: 	'' 
							}
						);
			},
			str2number: function( input ) {
				var ret = parseFloat( (input+'')
							.replace( new RegExp( dj.regexp.escapeString('[^0-9' + ep.locale.getSeparator('decimal') + ']', '^([]{})$'), 'g') , '' )
							.replace( new RegExp( dj.regexp.escapeString('[' + ep.locale.getSeparator('decimal') + ']', 	'^([]{})$'), 'g') , '.' ) );
							
				return isNaN(ret) ? null : ret;
			},
			number2str: function( input ) {
				return (input+'').replace( /\./g , ep.locale.getSeparator('decimal') );
			}
		}
	};
/* END *** ep */




	
	
	dj.declare("epages.widget.FormElements", [dijit._Widget], {
		
		node 			: null,				// node of form element
		containerNode	: null,				// 
		form			: null,				// 
		type			: 'text',			// 
		attributes		: {},				// attributes of form element
		
		_stepInterval	: null,
		_stepCount		: null,
		_stepSpeed		: 250,
		
		_patterns 		: {
							'[:space:]' 			: '\\ \n\r\t',
							'[:alpha:]' 			: 'A-Za-z',
							'[:digit:]' 			: '0-9',
							
							'[[:date:]]' 			: ep.locale.getPattern('date'),
							'[[:datetime:]]' 		: ep.locale.getPattern('datetime'),
							'[[:datetime-local:]]' 	: ep.locale.getPattern('datetime'),
							'[[:email:]]' 			: ep.locale.getPattern('email'),
							'[[:month:]]' 			: ep.locale.getPattern('month'),
							'[[:number:]]' 			: ep.locale.getPattern('number'),
							'[[:tel:]]' 			: '[0-9]{0,}',
							'[[:time:]]' 			: ep.locale.getPattern('time'),
							'[[:url:]]' 			: '[0-9]{0,}',
							'[[:week:]]' 			: ep.locale.getPattern('week'),
							'[[:money:]]' 			: '[0-9]{0,}[,]{0,}[0-9]{0,2}'
						  },
		ATTRIBUTES 		: ['autofocus','required','pattern','min','max','name','accept','checked','disabled','list','maxlength','multiple','placeholder','step','value'],
		/**
		 * widget life cycle
		 */
		postCreate: function() {
			this.inherited("postCreate", arguments);
			
			var node = this.srcNodeRef,
				type = dj.indexOf(['TEXTAREA','SELECT'],node.nodeName)>=0 ? node.nodeName.toLowerCase() : (node.type=='datetime-local' ? 'datetime' : node.type)
				attributes = {},
				pattern = dj.attr(node,'pattern');
						
			// get attributes from node
			for( var i=0,l=this.ATTRIBUTES.length ; i<l ; i++ ) {
				var name = this.ATTRIBUTES[i];
				var value = dj.attr(node,name);
				
				// parse min, max & step to number
				if( dj.indexOf(['max','min','step'],name) >= 0 ) {
					var val = ep.parse.str2number(value);
					
					if( val!==null ) {
						attributes[name] = val;
					};
				}
				// is defined
				else if( value !== undefined ) { 
					attributes[name] = value;
				};
			};
			
			// connect events to input
			if( dj.indexOf(['color','date','datetime','email','month','number','search','tel','text','time','url','week'],type) >= 0 ) {
				if( type != 'text' ) {
					node = this._changeType(node,'text');
				};
				dj.connect( node , 'keyup' , this , 'validate' );
				dj.connect( node , 'blur' , this , 'validate' );
			};
			
			// set min & max for month & week
			if( type == 'month' ) {
				attributes.min = Math.min( 12 , Math.max( 1 , attributes.min || 1 ) );
				attributes.max = Math.min( 12 , Math.max( 1 , attributes.max || 12 ) );
			}
			else if( type == 'week' ) {
				attributes.min = Math.min( 53 , Math.max( 1 , attributes.min || 1 ) );
				attributes.max = Math.min( 53 , Math.max( 1 , attributes.max || 53 ) );
			}
			
			// set special patterns
			if( dj.indexOf(['date','datetime','email','month','tel','time','url','week'],type) >= 0 ) {
				attributes.pattern = '[[:'+type+':]]';
			}
			else if( type == 'number' ) {
				attributes.pattern = attributes.pattern ? this.replacePattern( attributes.pattern ) : '[[:'+type+':]]';
			}
			
			// add parsed pattern to formElement
			if( attributes.pattern ) {
				attributes.pattern = this.replacePattern( attributes.pattern );
				dj.attr( node , 'pattern' , attributes.pattern )
			};
			
			// build container
			var self = this;
			var containerNode = dj.create('span',{
					'class': 'EpForm ' + type.substr(0,1).toUpperCase() + type.substr(1)
				},node,'after');
				containerNode.appendChild(node);
			
			// build stepper
			if( dj.indexOf(['date','datetime','month','number','time','week'],type) >= 0 ) {
				var stepUpNode = dj.create('a',{
						'href':'javascript:;',
						'class':'StepUp'
					});
				
				var stepDownNode = dj.create('a',{
						'href':'javascript:;',
						'class':'StepDown'
					});

				containerNode.appendChild(stepUpNode);
				containerNode.appendChild(stepDownNode); 
				
				if( attributes.disabled ) {
					dj.addClass( containerNode , 'Disabled');
				}
				else {
					dj.connect( stepUpNode , 'mousedown' , function(){ self._stepStart(+1) } );
					dj.connect( stepUpNode , 'mouseup'   , function(){ self._stepStop() } );
					dj.connect( stepUpNode , 'mouseout'  , function(){ self._stepStop() } );
					
					dj.connect( stepDownNode , 'mousedown', function(){ self._stepStart(-1) } );
					dj.connect( stepDownNode , 'mouseup'  , function(){ self._stepStop() } );
					dj.connect( stepDownNode , 'mouseout' , function(){ self._stepStop() } );
				}
			};
			
			// for checkbox and radio
			if( dj.indexOf(['checkbox','radio'],type) >= 0 ) {
				if( attributes.checked ) {
					dj.addClass( containerNode , 'Checked');
				}
				
				if( attributes.disabled ) {
					dj.addClass( containerNode , 'Disabled');
				}
				else {
					dj.connect( containerNode , 'click' , this , '_toggleChecked' );
				}
			};
			
			// push local vars to instance
			this.node = this.srcNodeRef =  node;
			this.form = node.form;
			this.containerNode = containerNode;
			this.type = type;
			this.attributes = attributes; 
		},
		
		_toggleChecked 	: function() {
			if( dj.attr( this.node , 'checked' ) ) {
				this._uncheck();
			}
			else {
				this._check();
			};
			this.validate();
		},
		
		_check: function() {		
			if( this.type == 'radio' ) {
				var nodes = this.form.elements[this.node.name];
				for( var i=0,l=nodes.length ; i<l ; i++ ) {
					dj.attr( nodes[i] , 'checked' ,false );
					dj.removeClass( nodes[i] , 'Checked' );
					dj.removeClass( nodes[i].parentNode , 'Checked' );
					if( dj.attr( nodes[i] , 'required' ) ) {
						this.attributes.required = true;
					}
				};
			}
			dj.attr( this.node , 'checked' , true );
			dj.addClass( this.node , 'Checked' );
			dj.addClass( this.containerNode , 'Checked' );
			epages.event.fire(this.node,'change');
		},
		
		_uncheck: function() {		
			if( this.type != 'radio' ) {
				dj.attr( this.node , 'checked' ,false );
				dj.removeClass( this.node , 'Checked' );
				dj.removeClass( this.containerNode , 'Checked' );
				epages.event.fire(this.node,'change');
			}
		},
		
		_changeType 	: function( node , type ) {
			var _node = node;
				node = dj.clone(_node)
				dj.attr( node , 'type' , type );
				dj.place( node , _node , 'after' );
				node.parentNode.removeChild(_node);
			return node;
		},
							
		_stepAction: function( inc ) {
			var self = this;
			
			this._stepCount++;
			
			if( dj.indexOf([10,30,50],this._stepCount)>=0 ) {	
			
				this._stepSpeed = this._stepSpeed / 2;

				window.clearInterval( this._stepInterval );
				this._stepInterval = window.setInterval( function(){ self._stepAction(inc) } , this._stepSpeed );
			};
			
			var value = dj.attr( this.node , 'value' );
			var ret;
			var step;
			var attrStep = this.attributes.step;
			var attrType = this.type;
			var attrMin = this.attributes.min;
			var attrMax = this.attributes.max;			
			
			if( dj.indexOf(['date','datetime','time'],attrType) >= 0 ) {
				
				switch( attrType ) {
					case 'date':
						step = inc * (attrStep || 86400);
					break;
					case 'datetime':
						step = inc * (attrStep || 3600);
					break;
					case 'time':
						step = inc * (attrStep || 60);
					break;
				};

				ret = ep.parse.date2unix( value , this.type );				
				ret = ret!==null ? ep.math.unixAddTime(ret,step) : (new Date().getTime() / 1000);
				ret = (attrMin && attrMin > ret) ? attrMin : (attrMax && attrMax < ret) ? attrMax : ret;
				ret = ep.parse.unix2date( ret , attrType );
			}
			else if( dj.indexOf(['month','week'],attrType) >= 0 ) {
				step = inc * (attrStep || 1);
				
				ret = parseInt(value);
				ret = isNaN(ret) ? 1 : ret+step;
				ret = (attrMin && attrMin > ret) ? attrMin : (attrMax && attrMax < ret) ? attrMax : ret;
			}
			else {
				step = inc * (attrStep || 1);
				
				ret = ep.parse.str2number(value);
				ret = ret!==null ? ret+step : 1;
				ret = (attrMin && attrMin > ret) ? attrMin : (attrMax && attrMax < ret) ? attrMax : ret;
				ret = ep.parse.number2str(ret);
			};
			
			dj.attr( this.node , 'value' , ret );
		},
							
		_stepStart: function( inc ) {
			var self = this;
			
			this._stepCount = 0;
			this._stepSpeed = 250;
			
			this._stepAction(inc);
			this._stepInterval = window.setInterval( function(){ self._stepAction(inc) } , this._stepSpeed );
		},
							
		_stepStop: function() {
			window.clearInterval( this._stepInterval );
			epages.event.fire(this.node,'blur');
		},
		
		validate: function() {
			var value = dj.attr( this.node , 'value' );
			var valide = false;	
			
			if( this.type == 'checkbox' && ( (dj.attr(this.node,'checked') && this.attributes.required) || !this.attributes.required ) ) {
				valide = true;
			};
			
			if( this.type == 'radio' && ( (dj.attr(this.node,'checked') && this.attributes.required) || !this.attributes.required ) ) {
				valide = true;
			};
			
			if( (value=='' && !this.attributes.required) || ( this.attributes.pattern && value.match(new RegExp(this.attributes.pattern))) ) {
				
				valide = true;
				
				var val = ep.parse.str2number(value);
				if( this.attributes.min && this.attributes.min > val ) {
					valide = false;
				}
				if( this.attributes.max && this.attributes.max < val ) {
					valide = false;
				}
			};
			
			if( valide ) {	
				dj.removeClass( this.node , 'DialogError' )
				dj.removeClass( this.containerNode , 'DialogError' )
			}
			else {
				dj.addClass( this.node , 'DialogError' )
				dj.addClass( this.containerNode , 'DialogError' )
			};
			
			epages.event.fire(this.form,'change')
		},
		
		replacePattern: function( pattern ) {
			
			var patterns = this._patterns;
			
			for( var i in patterns ) {
				pattern = pattern.replace( i , patterns[i] );
			}
			
			return pattern;
		},
	
		destroy : function() {
			this.inherited("destroy", arguments);
		}
	});

})(dojo);