/*
	Copyright (c) 2006-2007, epages Software GmbH
	All Rights Reserved.

	epages.widget.LocalizedWidget $Revision: 1.18.2.1 $
*/
dojo.provide("epages.widget.LocalizedWidget");
dojo.require("epages.widget.TypedWidget");
dojo.require("epages.io.translation");
dojo.require("epages.html.element");
dojo.require("dojo.string");

dojo.declare("epages.widget.LocalizedWidget", [epages.widget.TypedWidget], {
	// summary:Base class for all widgets which use an additional translation file
	
	// all LocalizedWidgets share the same translation object
	translation: 			new epages.io.Translation(),
	translationName: 	null,

	constructor: function() {
		this.translation.loadLocale(this.getTranslationName());
	},

	translate: function(/*String*/key, /*Object?*/vars) {
		return this.translation.get(key, vars);
	},

	getTranslationName: function () {
		// summary: return the translation file name (without locale and .xml)
		// description: convert this.templatePath to this.translationName of translation name is null
		// example:
		//   | dojo.declare(
		//   |   "epages.cartridges.de_epages.design.Toolcollection",
		//   |   [epages.widget.LocalizedWidget], {
		//   |     widgetType  : "Toolcollection",
		//   |     templatePath: dojo.uri.dojoUri("../epages/cartridges/de_epages/design/widget/Toolcollection/Toolcollection.html"),
		//   |     translationName: dojo.uri.dojoUri("../epages/cartridges/de_epages/design/widget/templates/translation"),
		//   |     widgetsInTemplate : true
		//   | });
		// returns: translationName
		if (this.translationName == null)
			this.translationName = this.templatePath.toString().replace('.html','');
		return this.translationName; //String
	},

	createNodesFromText:  function(/*String*/text){
		return dojo._toDom(this.translation.replaceLanguageTags(text));
	},


	buildRendering: function(){
		// summary:
		//    Construct the UI for this widget from a template, setting this.domNode.
    // description: 
		//    Lookup cached version of template, and download to cache if it
		//    isn't there already.  Returns either a DomNode or a string, depending on
		//    whether or not the template contains ${foo} replacement parameters.
		this._attachPoints = [];
		var cached = this.getCachedTemplate(this.templatePath, this.templateString, this._skipNodeCache, this.translation);


		var node;
		if(dojo.isString(cached)){
//##### START of epages changes #####
			var className = this.declaredClass, _this = this;
			// Cache contains a string because we need to do property replacement
			// do the property replacement
			var tstr = dojo.string.substitute(cached, this, function(value, key){
				if(key.charAt(0) == '!'){ value = _this[key.substr(1)]; }
				if(typeof value == "undefined"){ throw new Error(className+" template:"+key); } // a debugging aide
				if(!value){ return ""; }

				// Substitution keys beginning with ! will skip the transform step,
				// in case a user wishes to insert unescaped markup, e.g. ${!foo}
				return key.charAt(0) == "!" ? value :
					// Safer substitution, see heading "Attribute values" in
					// http://www.w3.org/TR/REC-html40/appendix/notes.html#h-B.3.2
					value.toString().replace(/"/g,"&quot;"); //TODO: add &amp? use encodeXML method?
			}, this);
			node = this.createNodesFromText(tstr);

//##### END of epages changes #####
		}else{
			// if it's a node, all we have to do is clone it
			node = cached.cloneNode(true);
		}

		this.domNode = node;

		// recurse through the node, looking for, and attaching to, our
		// attachment points which should be defined on the template node.
		this._attachTemplateNodes(node);

				if(this.widgetsInTemplate){
				// Make sure dojoType is used for parsing widgets in template.
				// The dojo.parser.query could be changed from multiversion support.
				var parser = dojo.parser, qry, attr;
				if(parser._query != "[dojoType]"){
					qry = parser._query;
					attr = parser._attrName;
					parser._query = "[dojoType]";
					parser._attrName = "dojoType";
				}

				// Store widgets that we need to start at a later point in time
				var cw = (this._startupWidgets = dojo.parser.parse(node, {
					noStart: !this._earlyTemplatedStartup
				}));

				// Restore the query.
				if(qry){
					parser._query = qry;
					parser._attrName = attr;
				}

				this._supportingWidgets = dijit.findWidgets(node);

				this._attachTemplateNodes(cw, function(n,p){
					return n[p];
				});
			}

		this._fillContent(this.srcNodeRef);
	},

	getCachedTemplate: function(templatePath, templateString, alwaysUseString, translation){
		// summary:
		//    static method to get a template based on the templatePath or
		//    templateString key
		// templatePath: String
		//    the URL to get the template from. dojo.uri.Uri is often passed as well.
		// templateString: String?
		//    a string to use in lieu of fetching the template from a URL
		// returns:
		//  Either string (if there are ${} variables that need to be replaced) or just
		//  a DOM tree (if the node can be cloned directly)

		// is it already cached?
		var tmplts = dijit._Templated._templateCache;
		var key = templateString || templatePath;
		var cached = tmplts[key];
		if(cached){
			return cached;
		}

		// If necessary, load template string from template path
		if(!templateString){
			templateString =dojo.cache(templatePath, {sanitize: true});
		}

		templateString = dojo.string.trim(templateString);
		if(templateString.match(/\$\{([^\}]+)\}/g) || alwaysUseString){
			// there are variables in the template so all we can do is cache the string
			return (tmplts[key] = templateString); //String
		}else{
			// there are no variables in the template so we can cache the DOM tree
			return (tmplts[key] = this.createNodesFromText(templateString)); //Node
		}
	}

});
