/*
	Copyright (c) 2006-2008, epages Software GmbH
	All Rights Reserved.

	epages.widget.Slider $Revision: 1.8.4.1 $

*/
dojo.provide('epages.widget.Slider');
dojo.require('epages.widget');
dojo.require("epages.lang.hitch");

dojo.declare("epages.widget.Slider", [dijit._Widget,dijit._Templated], {
	timerLeft         : null,
	timerRight        : null,
	slideSpeed        : 0,
	currentWidth      : 0,   // Integer - total width of the container that holds all images
	menuWidth         : 0,   // Integer - width of the visible part of the slider
	currentMoveState  : '',  // String - current State of Animation, left, right or empty
	maxSpeed          : 15,  // Integer - max Speed of Animation on the End of the Slider increase to speed up Slider
	totalOffsetLeft   : 0,   // Integer - total Offset of the Slider to the left side of the browser

	templatePath    : dojo.moduleUrl("epages.widget", "templates/Slider.html"),

	postCreate: function() {
		this.inherited("postCreate", arguments);
		// summary: standard init routine
		// description: Link needed events to appropriate functions
		// 		and set all global variables needed in other functions.
		//		Also hide the horizontal fallback scrollbar and revise
		//		the height of the main element.
		//
		//		An Image Slide frame must look like this:
		//		...and YES, every char has a purpose!
		//		E. g. to ensure a fallback alternative for
		//		people without javascript.
		//		for a detailed example look at the SliderTest.html
		// example:
		//	|	<div dojoType="epages.widget.Slider" maxSpeed="50" class="ImageSlide">
		//	|		<nobr style="width:50px"> //total Width of Container width of picture * number of pictures
		//	|		      <a href="#go">
		//	|		        <img src="pic1.gif"/>
		//	|		      </a>
		//	|		      <a href="#go">
		//	|		        <img src="pic2.gif"/>
		//	|		      </a>
		//	|		      <a href="#go">
		//	|		        <img src="pic3.gif"/>
		//	|		      </a>
		//	|					...
		//	|		    </nobr>
		//	|	</div>
		dojo.connect(window, 'onresize', this, '_arrangeResizedDocument');
		this.domNode.style.overflow = 'hidden';
		this.domNode.style.height = this.crossSlide.offsetHeight+'px';
		this.menuWidth = this.domNode.offsetWidth;
		var crossSlideWidth = (this.crossSlide.style.width != "") ? parseInt(this.crossSlide.style.width) : 0;
		this.currentWidth = this.crossSlide.offsetWidth? this.crossSlide.offsetWidth : crossSlideWidth;
		this.crossSlide.style.left = 0;
		this.totalOffsetLeft = this.domNode.offsetLeft;
		this._appendEvents();
	},

	_appendEvents: function() {
		dojo.connect(this.domNode,'onmousemove', this, '_doMotion');
		dojo.connect(this.domNode,'onmouseout', this, '_stopMotion');
	},

	_updateAfterEvent: function() {
		// summary: update the image slide line
		// description: We must force the css engine of some browsers
		//    to refresh the style on the whole image slide.
		//    Otherwise the styles will not be updated and e. g.
		//    the :hover class misfires for all prior invisible images.
		// example:
		//  |  _updateAfterEvent();
		this.domNode.style.clear = 'both';
		this.domNode.style.clear = 'none';
	},

	_moveLeft: function() {
		// summary: move the image line inside the container to the left
		// description: First we check if it is necessary to move the
		//		image line and if "yes" we move it as long the cursor indicates
		//    that it should move. To not confuse other functions we state
		//		that we are currently really moving it to the left. MOVE!
		// example:
		//  | _moveLeft();
		this.currentMoveState = 'left';
		if(parseInt(this.crossSlide.style.left) > (this.menuWidth - this.currentWidth)){
			this.crossSlide.style.left = parseInt(this.crossSlide.style.left) - this.slideSpeed + 'px';
		}
		this.timerLeft = setTimeout(epages.lang.hitch(this, this._moveLeft), 10);
	},

	_moveRight: function(){
		// summary: move the image line inside the container to the rights
		// description: First we check if it is necessary to move the
		//		image line and if "yes" we move it as long the cursor indicates
		//    that it should move. To not confuse other functions we state
		//		that we are currently really moving it to the right. MOVE!
		// example:
		//  | _moveRight();
		this.currentMoveState = 'right';
		if(parseInt(this.crossSlide.style.left) < 0){
			this.crossSlide.style.left = parseInt(this.crossSlide.style.left) + this.slideSpeed + 'px';
		}
		this.timerRight = setTimeout(epages.lang.hitch(this, this._moveRight), 10);
	},

	_doMotion: function(/*Object?*/ mouseEvent){
		// summary: the heart of our sliding effect
		// description: It does all the stuff like calculating the speed of
		//		the image slide, observing the cursor for new motions and
		//		initiating the move[...]() functions.
		// example:
		//  | mynode.onmousemove = function(mouseEvent){
		//  |   _doMotion(mouseEvent);
		//  |   _updateAfterEvent();
		//  | }
		var mouseXPoint = mouseEvent.clientX;
		var currentYPosition = mouseEvent.clientX - this.totalOffsetLeft;
		var edgeBound = (this.menuWidth / 2);
		if (currentYPosition > edgeBound){
			this.slideSpeed = (currentYPosition - edgeBound) / this.menuWidth / 2 * this.maxSpeed;
			clearTimeout(this.timerRight);
			if (this.currentMoveState != 'left'){
				this._moveLeft();
			}
		} else if (currentYPosition < edgeBound){
			this.slideSpeed = (edgeBound - currentYPosition) / this.menuWidth / 2 * this.maxSpeed;;
			clearTimeout(this.timerLeft);
			if (this.currentMoveState != 'right'){
			this._moveRight();
			}
		}else{
			this.slideSpeed = 0;
		}
	},

	_stopMotion: function(/*Object?*/ mouseEvent){
		// summary: stops all slide motions
		// description: If called with a mouse position where the slide
		//    should be stopped - we have to immediately clear all
		//		active timeouts that would call the move[...] functions.
		//		Furthermore we state that there is no motion active anymore.
		// example:
		//  | mynode.onmouseout = function(mouseEvent){
		//  |   _stopMotion(mouseEvent);
		//  |   _updateAfterEvent();
		//  | }
		if ((window.event && !this.domNode.contains(event.toElement)) || (mouseEvent && mouseEvent.currentTarget && mouseEvent.currentTarget!= mouseEvent.relatedTarget && !this._checkNode(mouseEvent.currentTarget, mouseEvent.relatedTarget))){
			clearTimeout(this.timerLeft);
			clearTimeout(this.timerRight);
			this.currentMoveState = "";
		}
	},

	_checkNode: function(/*Object?*/ currentTarget, /*Object?*/ relatedTarget) {
		// summary: check if we are in the right element
		// description: Naturally there is one browser that behaves unintentional.
		//		This time nearly all browsers fail. Without this routine
		//		Firefox would refer the mousemove event to every single image
		//		(the motion would only start if the cursor points an image).
		//		Netscape would inherit the mousemove event to the whole
		//		document body and the Internet Explorer would produce sensless
		//		error messages because it does not know the attribute.
		// example:
		//  |  var canIPleaseKillTheTimout = _checkNode(currentTarget, relatedTarget);
		if (relatedTarget !== null){
			while (relatedTarget.parentNode){
				if ((relatedTarget = relatedTarget.parentNode) == currentTarget){
					return true; // Boolean
				}
			}
		}
	},

	_arrangeResizedDocument: function() {
		// summary: adjust the whole thing on window resizing
		// description: We have to resize and realign the slide
		//		if the size of the browser window changes.
		// example:
		//  | window.onresize = _arrangeResizedDocument;
		this.menuWidth = this.domNode.offsetWidth;
		this.crossSlide.style.left = this.menuWidth - this.currentWidth + 'px';
	}
});