/*
	Copyright (c) 2006-2007, epages Software GmbH
	All Rights Reserved.

	epages.widget.Slides $Revision: 1.43.2.5 $

*/
dojo.provide('epages.widget.Slides');
dojo.require('epages.widget.LocalizedWidget');
dojo.require('epages.browser');
dojo.require('epages.event.interval');
dojo.require("dojo.fx.easing");
dojo.require("dojo.fx");


dojo.declare("epages.widget.Slides", [epages.widget.LocalizedWidget], {
	templatePath:       dojo.moduleUrl("epages.widget", "templates/Slides.html"),
	imagePath:          epages.themeUrl('images'),
	translationName:    dojo.moduleUrl("epages.widget", "templates/translation"),
	loaderImagePath:    epages.themeUrl('images') + "/img_busy.gif",
	errorImagePath:     epages.themeUrl('images') + "/slideshow_img_broken_image.png",

	controlBarImages: [
		[epages.themeUrl('images') + "/slideshow_img_bg.png", epages.themeUrl('images') + "/slideshow_img_bg_small.png", epages.themeUrl('images') + "/slideshow_img_bg_smaller.png"],
		[epages.themeUrl('images') + "/slideshow_ico_l_rewind.png", epages.themeUrl('images') + "/slideshow_ico_s_rewind.png"],
		[epages.themeUrl('images') + "/slideshow_ico_l_pause.png", epages.themeUrl('images') + "/slideshow_ico_s_pause.png"],
		[epages.themeUrl('images') + "/slideshow_ico_l_play.png", epages.themeUrl('images') + "/slideshow_ico_s_play.png"],
		[epages.themeUrl('images') + "/slideshow_ico_l_overview.png", epages.themeUrl('images') + "/slideshow_ico_s_overview.png"],
		[epages.themeUrl('images') + "/slideshow_ico_l_details.png", epages.themeUrl('images') + "/slideshow_ico_s_details.png"],
		[epages.themeUrl('images') + "/slideshow_ico_l_forward.png", epages.themeUrl('images') + "/slideshow_ico_s_forward.png"]
										],

	delay:                 3500,
	protraction:           1200,
	imageWidth:            "450px",
	imageHeight:           "300px",
	tiny:                  false,
	stopped:               false,
	overview:              true,				// flag: enable / disable detailed (large) view
	imageClickOverview:    true,
	showName:              true,
	showDescription:       true,
	showControls:          true,
	showMisc:              true,
	showIndex:             0,

	_images:               undefined,
	_oldShowIndex:         0,
	_loadedIndex:          0,
	_loadedImages:         0,
	_pauseIcon:            0,
	_playIcon:             0,
	_fadeEasing:           dojo.fx.easing.quadOut,
	_overviewArea:         null,
	_overviewBackground:   null,
	_overviewImageCounter: null,
	_controlBarFadeOut:    null,
	_playerBeacon:         null,
	_hiddenDetails:        false,
	_changingImage:        true,
	_globalOverview:       true,
	_reAlignReady:         true,

	_delayExec: function(task, delay) {
		return setTimeout(dojo.hitch(this, task), delay);
	},

	_getAlphaImage: function (imageSrc, sizing) {

		var imageContainer = document.createElement("span");
		var imageHolder = document.createElement("span");

		var filterClassString = "";
		if(epages.Browser.engine == "MSIE" && imageSrc.match(".png")) {
			sizing = sizing ? "scale" : "crop";
			filterClassString = "class='AlphaImage'"
			imageHolder.style.zoom = "1";
			imageHolder.style.filter = "progid:DXImageTransform.Microsoft.AlphaImageLoader(src='" + imageSrc + "', sizingMethod='" + sizing + "')";
		};

		var image = "<img " + filterClassString + " alt='' src='" + imageSrc + "' onerror=''/>";

		imageHolder.innerHTML = image;
		imageContainer.appendChild(imageHolder);
		imageContainer.className = "Image";

		return imageContainer;
	},

	_drawControlBar: function() {

		if(this.tiny) {this.controlNode.className = "ControlBarMini"}
		if(!this.showControls) {this.controlNode.style.display = "none"}

		var backgroundImageIndex = 0;
		if(!this.overview || !this.showDescription) {
			backgroundImageIndex = 1;
		}else if(!this.overview && !this.showDescription) {
			backgroundImageIndex = 2;
		}

		var controlBarBg = this.tiny ? document.createElement("div") : this._getAlphaImage(this.controlBarImages[0][backgroundImageIndex]);
		controlBarBg.className = "ControlBarBg";
		this.controlNode.appendChild(controlBarBg);

		if(!this.overview && this.showControls) {
			controlBarBg.parentNode.style.width = (controlBarBg.offsetWidth - (this.tiny ? 16 : 32)) + "px";
		}
		if(!this.showDescription && this.showControls) {
			controlBarBg.parentNode.style.width = (controlBarBg.offsetWidth - (this.tiny ? 16 : 32)) + "px";
		}

		var rewindIcon = this._getAlphaImage(this.controlBarImages[1][this.tiny ? 1 : 0]);
		rewindIcon.className = "SlideshowButton";
		this.controlNode.appendChild(rewindIcon);
		dojo.connect(rewindIcon, "onclick", this, "_rewindClick");

		var pauseIcon = this._getAlphaImage(this.controlBarImages[2][this.tiny ? 1 : 0]);
		pauseIcon.className = "SlideshowButton";
		if(this.stopped) {pauseIcon.style.position = "absolute";pauseIcon.style.visibility = "hidden";}
		this.controlNode.appendChild(pauseIcon);
		dojo.connect(pauseIcon, "onclick", this, "_pauseClick");
		this._pauseIcon = pauseIcon;

		playIcon = this._getAlphaImage(this.controlBarImages[3][this.tiny ? 1 : 0])
		playIcon.className = "SlideshowButton";
		if(!this.stopped) {playIcon.style.position = "absolute";playIcon.style.visibility = "hidden";}
		this.controlNode.appendChild(playIcon);
		dojo.connect(playIcon, "onclick", this, "_playClick");
		this._playIcon = playIcon;

		if(this.overview) {
			var overviewIcon = this._getAlphaImage(this.controlBarImages[4][this.tiny ? 1 : 0]);
			overviewIcon.className = "SlideshowButton";
			this.controlNode.appendChild(overviewIcon);
			dojo.connect(overviewIcon, "onclick", this, "_overviewClick");
		}

		if(this.showDescription) {
			var detailsIcon = this._getAlphaImage(this.controlBarImages[5][this.tiny ? 1 : 0]);
			detailsIcon.className = "SlideshowButton";
			this.controlNode.appendChild(detailsIcon);
			dojo.connect(detailsIcon, "onclick", this, "_detailsClick");
		}

		var forwardIcon = this._getAlphaImage(this.controlBarImages[6][this.tiny ? 1 : 0]);
		forwardIcon.className = "SlideshowButton";
		this.controlNode.appendChild(forwardIcon);
		dojo.connect(forwardIcon, "onclick", this, "_forwardClick");

		dojo.style(this.controlNode, "opacity", 0);

		return [rewindIcon, playIcon, pauseIcon, overviewIcon ? overviewIcon : null, detailsIcon ? detailsIcon : null, forwardIcon];
	},

	_controlBarButtonOver: function(eventObject) {
		dojo.addClass(eventObject.currentTarget, "Highlighted");
		document.onselectstart = function() {return false};
	},

	_controlBarButtonOut: function(eventObject) {
		dojo.removeClass(eventObject.currentTarget, "Highlighted");
		dojo.removeClass(eventObject.currentTarget, "Hit");
		document.onselectstart = function() {return true};
	},

	_controlBarButtonDown: function(eventObject) {
		dojo.addClass(eventObject.currentTarget, "Hit");
	},

	_controlBarButtonUp: function(eventObject) {
		dojo.removeClass(eventObject.currentTarget, "Hit");
	},

	_showControlBar: function() {
		clearTimeout(this._controlBarFadeOut);
		dojo.fadeIn({node: this.controlNode, duration:350, easing: this._fadeEasing}).play();
	},

	_hideControlBar: function() {
		this._controlBarFadeOut = this._delayExec(function() {
			if(this.controlNode.offsetWidth == 0) {
				dojo.style(this.controlNode, "opacity", 0);
			}else{
				dojo.fadeOut({node: this.controlNode, duration:350}).play();
			}
		}, 1000);
	},

	_drawErrorImage: function(eventObject) {
		var imgNode = eventObject.currentTarget;
		imgNode.src = this.errorImagePath;
		imgNode.parentNode.style.filter = "progid:DXImageTransform.Microsoft.AlphaImageLoader(src='" + this.errorImagePath + "', sizingMethod='scale')";
	},

	_playClickStartTimer: function() {
		this._playIcon.style.position = "absolute";
		this._playIcon.style.visibility = "hidden";
		this._pauseIcon.style.position = "relative";
		this._pauseIcon.style.visibility = "visible";
		this._playerBeacon = epages.event.interval.set(dojo.hitch(this, this._setNextImage), this.delay);
	},

	_playClick: function() {
		this._setNextImage();
		this._playClickStartTimer();
	},

	_pauseClick: function() {
		this._pauseIcon.style.position = "absolute";
		this._pauseIcon.style.visibility = "hidden";
		this._playIcon.style.position = "relative";
		this._playIcon.style.visibility = "visible";
		epages.event.interval.clear(this._playerBeacon);
	},

	_overviewClick: function(image) {

		/* we don't want any mouse events in here */
		if(typeof(image) == "object") {image = null}

		this._pauseClick();

		if(this._loadedIndex >= this.showIndex && !this._overviewArea) {
			var overviewBackground = document.createElement("div");
			overviewBackground.className = "OverviewBackground";
			dojo.style(overviewBackground, "opacity", 0);

			dojo.addClass(dojo.body(), "OverviewAreaVisible");

			overviewBackground.innerHTML = "<div class='Background'></div>";

			var overviewArea = document.createElement("div");
			overviewArea.className = "OverviewArea";

			var info = this._images[this.showIndex];

			if(!info) {
				var info = {};
				info.imageTable = this._createImageTable();
				info.imageTable.getElementsByTagName("b")[0].appendChild(this._getAlphaImage(epages.themeUrl('images') + "/plain.gif"));
				this._images.push(info);
			}else {
				this._oldShowIndex = this.showIndex;
			}

			var infoTable = this._images[this.showIndex].imageTable.cloneNode(true);

			with(infoTable.getElementsByTagName("b")[0]) {
				innerHTML = "";
				appendChild(this._getAlphaImage(epages.themeUrl('images') + "/plain.gif"));
			}

			overviewArea.appendChild(infoTable);

			var backgroundEventLayer = document.createElement("div");
			backgroundEventLayer.className = "BackgroundEventLayer";
			overviewArea.getElementsByTagName("td")[0].appendChild(backgroundEventLayer)

			var rewindIcon = this._getAlphaImage(epages.themeUrl("images") + "/slideshow_ico_l_rewind_overview.png");
			rewindIcon.className = "Image SlideshowButton";
			dojo.connect(rewindIcon, "onclick", this, "_setPrevImage");
			dojo.connect(rewindIcon, "onmouseover", this, "_controlBarButtonOver");
			dojo.connect(rewindIcon, "onmouseout", this, "_controlBarButtonOut");
			dojo.connect(rewindIcon, "onmousedown", this, "_controlBarButtonDown");
			dojo.connect(rewindIcon, "onmouseup", this, "_controlBarButtonUp");
			var rewindIconContainer = document.createElement("i");
			rewindIconContainer.appendChild(rewindIcon);
			rewindIconContainer.className = "OverviewRewindButton";

			var forwardIcon = this._getAlphaImage(epages.themeUrl("images") + "/slideshow_ico_l_forward_overview.png");
			forwardIcon.className = "Image SlideshowButton";
			dojo.connect(forwardIcon, "onclick", this, "_setNextImage");
			dojo.connect(forwardIcon, "onmouseover", this, "_controlBarButtonOver");
			dojo.connect(forwardIcon, "onmouseout", this, "_controlBarButtonOut");
			dojo.connect(forwardIcon, "onmousedown", this, "_controlBarButtonDown");
			dojo.connect(forwardIcon, "onmouseup", this, "_controlBarButtonUp");
			var forwardIconContainer = document.createElement("i");
			forwardIconContainer.appendChild(forwardIcon);
			forwardIconContainer.className = "OverviewForwardButton";

			var closeIcon = this._getAlphaImage(epages.themeUrl("images") + "/layer_img_closebutton.png");
			closeIcon.className = "Image SlideshowButton";
			closeIcon.style.clip = "rect(0, 40px, 20px, 0)";
			dojo.connect(closeIcon, "onclick", this, "_destroyOverview");
			dojo.connect(closeIcon, "onmouseover", this, "_controlBarButtonOver");
			dojo.connect(closeIcon, "onmouseout", this, "_controlBarButtonOut");
			var closeIconContainer = document.createElement("i");
			closeIconContainer.appendChild(closeIcon);
			closeIconContainer.className = "OverviewCloseButton";

			var overviewImageCounter = document.createElement("b");
			overviewImageCounter.className = "OverviewImageCounter";
			if(!this.showMisc) {overviewImageCounter.style.display = "none"}

			dojo.style(overviewArea, "background", "url(" + this.loaderImagePath + ") center no-repeat");
			dojo.style(overviewArea.getElementsByTagName("span")[0], "opacity", 0);

			var overviewImageSrc = info.imageLarge ? info.imageLarge : info.image;
			var overviewImage = overviewArea.getElementsByTagName("img")[0];
			overviewImage.src = image || overviewImageSrc;
			overviewImage.style.height = "auto";
			overviewImage.style.width = "auto";
			if(overviewImage.src.match(".ico") || overviewImage.src.match(".cur")) {
				overviewImage.parentNode.style.filter = "fade()";
			}else {
				overviewImage.style.filter = "alpha(opacity=0)";
				overviewImage.parentNode.style.filter = "progid:DXImageTransform.Microsoft.AlphaImageLoader(src='" + overviewImage.src + "', sizingMethod='scale')";
			}
			dojo.body().appendChild(overviewBackground);
			dojo.body().appendChild(overviewArea);

			var imageLargeDuplicated = true;
			var prevImage = this._images[0];

			for(var i = 0; i < this._images.length; i++) {
				if(this._images[i].imageLarge != prevImage.imageLarge) {
					imageLargeDuplicated = false;
					break;
				}else {
					prevImage = this._images[i];
				}
			}

			if(this._images.length > 1 && !image && !imageLargeDuplicated) {
				if(this.showControls) {
					overviewArea.getElementsByTagName("span")[1].appendChild(rewindIconContainer);
					overviewArea.getElementsByTagName("span")[1].appendChild(forwardIconContainer);
				}
				overviewArea.getElementsByTagName("span")[1].appendChild(overviewImageCounter);
			}
			overviewArea.getElementsByTagName("span")[1].appendChild(closeIconContainer);

			if(overviewImage.complete) {
				this._delayExec(this._drawOverviewImage, 1);
			}else{
				dojo.connect(overviewImage, "onload", this, "_drawOverviewImage");
			}

			dojo.connect(overviewImage, "onerror", this, "_drawErrorImage");
			dojo.connect(backgroundEventLayer, "onclick", this, "_destroyOverview");

			overviewBackground.style.zoom = 1;
			dojo.style(overviewBackground, "opacity", 1);

			this._overviewBackground = overviewBackground;
			this._overviewArea = overviewArea;
			this._overviewImageCounter = overviewImageCounter;

			this._changeOverviewDimensions();

		}
	},

	_changeOverviewDimensions: function() {
		if(this._overviewArea != null) {

			var overviewImage = this._overviewArea.getElementsByTagName("img")[0];
			var windowHeight = window.innerHeight ? window.innerHeight : document.documentElement.clientHeight;
			var windowWidth = window.innerWidth ? window.innerWidth : document.documentElement.clientWidth;

			var currentHeight = 0;

			if(windowHeight > dojo.body().offsetHeight + 10) {
				currentHeight = windowHeight;
			}else{
				currentHeight = dojo.body().offsetHeight + 10;
			}
			this._overviewBackground.style.height = currentHeight + "px";
			this._overviewArea.style.height = windowHeight + "px";

			setTimeout(dojo.hitch(this, function() {
				var absoluteWidth = this._images[this.showIndex]._overviewWidth;
				var absoluteHeight = this._images[this.showIndex]._overviewHeight;

				if(absoluteWidth && absoluteHeight && (absoluteWidth > windowWidth - 50 || absoluteHeight > windowHeight - 50)) {
					if((windowWidth / windowHeight) / (absoluteWidth / absoluteHeight) > 1) {
						overviewImage.style.height = (windowHeight - 50) + "px";
						overviewImage.style.width = "auto";
					}else {
						overviewImage.style.width = (windowWidth - 50) + "px";
						overviewImage.style.height = "auto";
					}
				}
				this._reAlignOverview();
			}), 1);

		}
	},

	_reAlignOverview: function() {
		if(this._overviewArea != null && this._reAlignReady) {
			this._reAlignReady = false;
			this._delayExec(function(){
				if(this._overviewArea != null) {
					var scrollAmount = dojo.body().pageYOffset ? dojo.body().pageYOffset : document.documentElement.scrollTop;
					dojo.fx.slideTo({node: this._overviewArea, top: scrollAmount, duration: 350, easing: this._fadeEasing}).play()
				}
				this._reAlignReady = true;
			}, 50);
		}
	},

	_detailsClick: function() {
		if(this._hiddenDetails) {
			this._hiddenDetails = false;
			this.descriptionNode.style.visibility = "visible";
			this.miscNode.style.visibility = "visible";
		}else{
			this._hiddenDetails = true;
			this.descriptionNode.style.visibility = "hidden";
			this.miscNode.style.visibility = "hidden";
		}
	},

	_forwardClick: function() {
		this._pauseClick();
		this._setNextImage();
	},

	_rewindClick: function() {
		this._pauseClick();
		this._setPrevImage();
	},

	_drawOverviewImage: function() {
		this._delayExec(function() {
			var overviewImage = this._overviewArea.getElementsByTagName("img")[0];

			if(!this._images[this.showIndex]._overviewWidth) {
				overviewImage.style.position = "absolute";
				this._images[this.showIndex]._overviewWidth = overviewImage.width;
				this._images[this.showIndex]._overviewHeight = overviewImage.height;
				overviewImage.style.position = "static";
			}

			if(this._images[this.showIndex]._overviewHeight < 100) {
				overviewImage.style.marginTop = (45 - this._images[this.showIndex]._overviewHeight / 2) + "px";
			}else{
				overviewImage.style.marginTop = "0";
			}

			this._changeOverviewDimensions();

			this._overviewImageCounter.innerHTML = (this.showIndex + 1) + " | " + this._images.length;

			this._overviewArea.getElementsByTagName("b")[0].style.visibility = "visible";
			dojo.style(this._overviewArea.getElementsByTagName("b")[0], "opacity", "1");
			dojo.style(this._overviewArea, "background", "");
			dojo.fadeIn({node: this._overviewArea.getElementsByTagName("span")[0], duration: this.protraction, easing: this._fadeEasing}).play();

		}, 400);
	},

	_destroyOverview: function() {
		if(this._overviewArea) {
			if(dojo.style(this._overviewArea.getElementsByTagName("span")[0], "opacity") > 0.5) {
				dojo.fadeOut({node: this._overviewArea, duration: this.protraction / 2, easing: this._fadeEasing}).play();
				this._delayExec(function() {
					if(this._overviewArea) {
						dojo.body().removeChild(this._overviewArea);
						dojo.body().removeChild(this._overviewBackground);
						delete this._overviewArea;
						delete this._overviewBackground;
						this.showIndex = this._oldShowIndex;
						delete this._oldShowIndex;

						dojo.removeClass(dojo.body(), "OverviewAreaVisible");
					}
				}, this.protraction / 2.2);
			}
		}
	},

	_checkGlobalOverview: function() {
		var isPlainImage = this._images[this.showIndex].image.match("dijit/themes/epages/images/plain.gif") || this._images[this.showIndex].image.match("transparentpixel.gif");
		if(this._globalOverview && !isPlainImage){this._overviewClick()}
	},

	_showImageDetails: function() {

		var info = this._images[this.showIndex];
		this.descriptionNode.innerHTML = info.description;
		this.dateNode.innerHTML = info.date;
		this.countNode.innerHTML = (this.showIndex + 1) + " | " + (this._images.length);

		this.miscNode.style.top = this.miscNodeClipper.offsetHeight + "px";
		this.descriptionNode.style.top = "-" + this.descriptionNode.offsetHeight + "px";

		dojo.style(this.miscNode, "opacity", 0.8);
		dojo.style(this.descriptionNode, "opacity", 0.8);

		if(this.descriptionNode.innerHTML.length > 1) {
			dojo.fx.slideTo({node: this.descriptionNode, duration: 400, easing: this._fadeEasing}).play();
		}
		dojo.fx.slideTo({node: this.miscNode, duration: 400, easing: this._fadeEasing}).play();

	},

	_hideImageDetails: function() {
		dojo.fadeOut({node: this.miscNode, duration: 200, easing: this._fadeEasing}).play();
		dojo.fadeOut({node: this.descriptionNode, duration: 200, easing: this._fadeEasing}).play();
	},

	_setNextImage: function() {
		if(this._overviewArea) {
			if(dojo.style(this._overviewArea.getElementsByTagName("span")[0], "opacity") >= 0.5) {
				this._setImage(this.showIndex > this._images.length - 2 ? 0 : this.showIndex + 1);
			}
		}else{
			this._setImage(this.showIndex > this._images.length - 2 ? 0 : this.showIndex + 1);
		}
	},

	_setPrevImage: function() {
		if(this._overviewArea) {
			if(dojo.style(this._overviewArea.getElementsByTagName("span")[0], "opacity") >= 0.5) {
				this._setImage(this.showIndex == 0 ? this._images.length - 1 : this.showIndex - 1);
			}
		}else{
			this._setImage(this.showIndex == 0 ? this._images.length - 1 : this.showIndex - 1);
		}
	},

	_setImage: function(imageIndex) {
		if(this._images[imageIndex]) {
			if(this._overviewArea) {
				if(!this._oldShowIndex) {
					this._oldShowIndex = this.showIndex;
				}

				var overviewImageContainer = this._overviewArea.getElementsByTagName("span")[0];
				var overviewImage = overviewImageContainer.getElementsByTagName("img")[0];

				dojo.style(this._overviewArea, "background", "url(" + this.loaderImagePath + ") center no-repeat");
				dojo.fadeOut({node: overviewImageContainer, duration: this.protraction, easing: this._fadeEasing}).play();

				this._delayExec(function() {
					var info = this._images[this.showIndex];
					dojo.style(overviewImageContainer, "opacity", 0);
					var overviewImageSrc = info.imageLarge || info.image;
					var overviewImage = this._overviewArea.getElementsByTagName("img")[0];
					overviewImage.src = overviewImageSrc;
					overviewImage.style.width = "auto";
					overviewImage.style.height = "auto";
					overviewImage.style.filter = "alpha(opacity=0)";
					overviewImage.parentNode.style.filter = "progid:DXImageTransform.Microsoft.AlphaImageLoader(src='" + overviewImageSrc + "', sizingMethod='crop')";

					if(overviewImage.complete) {this._delayExec(this._drawOverviewImage, 1)};
				}, this.protraction);

			}else {
				var oldContainer = this._images[this.showIndex].imageTable.getElementsByTagName("b")[0];
				var newContainer = this._images[imageIndex].imageTable.getElementsByTagName("b")[0];

				//be sure that the slideshow stops fading when it is not visible (display:none)
				if(oldContainer.offsetWidth == 0) {
					this._pauseClick();
				}else{
					var newImage = newContainer.getElementsByTagName("img")[0];
					if(typeof(newImage) != "undefined") {
						if(this._images[imageIndex].fit == "true" || this._images[imageIndex].fit == true) {
							if(newImage.offsetHeight > newImage.offsetWidth) {
								newImage.style.height = (this.mainNode.offsetHeight) + "px";
								if(newImage.offsetWidth >= this.mainNode.offsetWidth) {
									newImage.style.height = "auto";
									newImage.style.width = (this.mainNode.offsetWidth) + "px";
								}
							}else{
								newImage.style.width = (this.mainNode.offsetWidth) + "px";
								if(newImage.offsetHeight >= this.mainNode.offsetHeight) {
									newImage.style.width = "auto";
									newImage.style.height = (this.mainNode.offsetHeight) + "px";
								}
							}
						}else if(this._images[imageIndex].fit == null) {
							this._delayExec(function() { 
								if(newImage.offsetHeight > newImage.offsetWidth) {
									if(newImage.offsetHeight > this.mainNode.offsetHeight) {
										newImage.style.height = (this.mainNode.offsetHeight) + "px";
										newImage.style.width = "auto";
									}
								}else{
									if(newImage.offsetWidth > this.mainNode.offsetWidth) {
										newImage.style.width = (this.mainNode.offsetWidth) + "px";
										newImage.style.height = "auto";
									}
								}
							},1);
						}
					}

					if(this.showIndex != imageIndex) {
						dojo.fadeOut({node: oldContainer, duration: this.protraction, easing: this._fadeEasing}).play();
						this._delayExec(function(){this._fadeOutHide(oldContainer)}, this.protraction);
					}

					dojo.style(newContainer, "visibility", "visible");
					dojo.fadeIn({node: newContainer, duration: this.protraction, easing: this._fadeEasing}).play();

					this._hideImageDetails();
					this._delayExec(this._showImageDetails, this.protraction / 2);
					this._images[this.showIndex].imageTable.style.zIndex = 0;
					this._images[imageIndex].imageTable.style.zIndex = 1;
				}
			}
		}
		this.showIndex = imageIndex;
	},

	_fadeOutHide: function(domNode) {
		if(dojo.style(domNode, "opacity") == 0) {
			dojo.style(domNode, "visibility", "hidden");
		}
	},

	_loadImagesSoft: function() {
		dojo.disconnect(this._images[this._loadedIndex].onloadHandle);

		if(this._images[this._loadedIndex].imageToStage) {
			this._setImage(this._images[this._loadedIndex].imageToStage);
		}else if (this._loadedIndex == this.showIndex && this._overviewArea == null) {
			this._setImage(this.showIndex);
		}

		if(this._loadedImages < this._images.length) {
			this._delayExec(this._loadImages, 1000);
		}else{
			this.mainNode.style.background = "none";
		}
	},

	_loadImages: function() {
		this._loadedImages++;

		this._loadedIndex++;
		if(this._loadedIndex >= this._images.length){this._loadedIndex = 0}

		var info = this._images[this._loadedIndex];
		var imageTable = info.imageTable;

		this.mainNode.style.background = "transparent url(" + this.loaderImagePath + ") center no-repeat";

		imageTable.style.zIndex = 0;

		var imageHolder = imageTable.getElementsByTagName("b")[0];
		imageHolder.style.zoom = "1";
		dojo.style(imageHolder, "opacity", 0);
		dojo.style(imageHolder, "visibility", "hidden");
		imageHolder.appendChild(this._getAlphaImage(info.image, true));

		var image = imageHolder.getElementsByTagName("img")[0];
		dojo.connect(image, "onerror", this, "_drawErrorImage");
		//if(image.complete) {
		//	this._delayExec(this._loadImagesSoft, 1);
		//}else{
		
			// timeoutfix HACK 
			
			var test = new Image();
				test.onload = dojo.hitch(this,'_loadImagesSoft');
				test.src = image.src;
			//info.onloadHandle = dojo.connect(image, "onload", this, "_loadImagesSoft");
		//}
	},

	_createImageTable: function() {
		var imageTable = this.imageTable.cloneNode(true);
		imageTable.className = "ImageTable";
		this.imageTable.parentNode.appendChild(imageTable);
		return imageTable;
	},

	_handleKeyEvent: function(evt) {
		evt = evt ? evt: window.event;
		switch (evt.which || evt.keyCode) {
			case 27:
				this._destroyOverview();
				break;
			case 37:
				this._setPrevImage();
				break;
			case 39:
				this._setNextImage();
				break;
		}
	},

	openOverview: function(image) {
		this._overviewClick(image);
	},

	setImage: function(imageIndex) {
		this._pauseClick();
		this._setImage(imageIndex);
	},

	setNextImage: function() {
		this._setNextImage();
	},

	setPrevImage: function() {
		this._setPrevImage();
	},

	addImage: function(insObject, setImage) {
		if(insObject.image) {
			var alreadyAddedIn;
			for(var i = 0;i < this._images.length; i++) {
				if(this._images[i].image == insObject.image) {
					alreadyAddedIn = i;
				}
			}
			if(!alreadyAddedIn) {
				this._images.push(insObject);
				this._images[this._images.length - 1].imageTable = this._createImageTable();
				this._images[this._images.length - 1].imageToStage = this._images.length - 1;
				this._loadImages();
			}else {
				this._setImage(alreadyAddedIn);
			}
		}
	},

	postCreate: function() {
		epages.widget.Slides.superclass.postCreate.apply(this, arguments);
		this._images = [];

		this.domNode.style.width = this.imageWidth;
		this.domNode.style.height = this.imageHeight;
		this.mainNode.parentNode.style.width =  this.imageWidth;

		if(!this.showDescription) {this.descriptionNode.style.display = "none"}
		if(!this.showMisc) {this.miscNode.style.display = "none"}

		var controlButtons = this._drawControlBar();

		for(var i = 0; i < controlButtons.length; i++) {
			if(controlButtons[i]) {
				dojo.connect(controlButtons[i], "onmouseover", this, "_controlBarButtonOver");
				dojo.connect(controlButtons[i], "onmouseout", this, "_controlBarButtonOut");
				dojo.connect(controlButtons[i], "onmousedown", this, "_controlBarButtonDown");
				dojo.connect(controlButtons[i], "onmouseup", this, "_controlBarButtonUp");
			}
		}

		if(this.srcNodeRef) {
			if(this.srcNodeRef.className) {
				this.mainNode.parentNode.parentNode.className += " " + this.srcNodeRef.className;
			}
			// build images from source node
			var sourceDom = this.srcNodeRef.getElementsByTagName("ins");
			for(var i = 0; i < sourceDom.length; i++) {
				var currentImageNode = {};
				currentImageNode.image = sourceDom[i].getAttribute("image") || epages.themeUrl('images') + "/plain.gif";
				currentImageNode.imageLarge = sourceDom[i].getAttribute("imageLarge");
				currentImageNode.name = sourceDom[i].getAttribute("name");
				currentImageNode.description = sourceDom[i].innerHTML;
				currentImageNode.date = sourceDom[i].getAttribute("date");
				currentImageNode.time = sourceDom[i].getAttribute("time");
				currentImageNode.fit = sourceDom[i].getAttribute("fit");

				currentImageNode.imageTable = this._createImageTable();

				this._images.push(currentImageNode);
			}
		}

		this.showIndex = new Number(this.showIndex);
		// start one before, because loadimages increment it first
		this._loadedIndex = this.showIndex - 1;
		if (this._loadedIndex == -1) this._loadedIndex = this._images.length;

		dojo.connect(this.mainNode, "onmouseover", this, "_showControlBar");
		dojo.connect(this.mainNode, "onmouseout", this, "_hideControlBar");

		if(this.imageClickOverview && this.overview) {
			dojo.connect(this.miscNode.offsetHeight ? this.miscNode : this.mainNode, "onclick", this, "_checkGlobalOverview");
			dojo.connect(this.controlNode, "onmouseover", dojo.hitch(this, function(){this._globalOverview = false}));
			dojo.connect(this.controlNode, "onmouseout", dojo.hitch(this, function(){this._globalOverview = true}));
		}else{
			this.mainNode.style.cursor = "default";
		}

		dojo.connect(window, "onscroll", this, "_reAlignOverview");
		dojo.connect(window, "onresize", this, "_changeOverviewDimensions");
		dojo.connect(document, "onkeyup", this, "_handleKeyEvent");

		if(this._images.length > 0) {
			this._delayExec(this._loadImages, 1);
			if(!this.stopped && this._images.length > 1){
				this._delayExec(this._playClickStartTimer, 1000);
			}
		}

	}
});
