/*
	Copyright (c) 2006-2007, epages Software GmbH
	All Rights Reserved.

	epages.cartridges.de_epages.externalcontent.googlemap $Revision: 1.25.2.2 $

*/
dojo.provide('epages.cartridges.de_epages.externalcontent.googlemap');
dojo.require('epages.cartridges.de_epages.design.mcecontent');

epages.cartridges.de_epages.externalcontent.GoogleMap = epages.Class();

dojo.extend(epages.cartridges.de_epages.externalcontent.GoogleMap,
	epages.cartridges.de_epages.design.MceContent.prototype, {
	geocoder        	: undefined, //neuer Geocoder um Adressen umzuwandeln
	// overviewControl 	: undefined,
	controls        	: true, // remember that map has controls
	markers				: {},
	preset				: undefined,
	centerLat       	: 51.165,
	centerLng       	: 10.455278,
	zoom           		: 5,
	markersPlain 		: [],
	type            	: 'roadmap',
	map             	: undefined,
	domObject       	: undefined,
	title           	: 'googlemaps',
	preview         	: epages.vars.StoreRoot + '/BO/icons/navelement_googlemap.png',
	doInsertDom     	: false,
	navId           	: null,
	isPreview			: false,
	additionalCallback 	: undefined,

	changeControls: function(checkbox){
		this.controls= !this.controls;
		this.setControls(this.controls);
	},
	setControls:function(enabled){
		this.map.setOptions({
			navigationControl:enabled, 
			mapTypeControl:enabled,
			draggable:enabled,
			disableDoubleClickZoom:!enabled,
			keyboardShortcuts:enabled,
			scrollwheel:enabled
		});
	},
	resize: function (width, height) {
		if(this.domObject){
			this.domObject.style.width = width;
			this.domObject.style.height = height;
		}
	},

	changeSize: function(domObject){
		if (domObject) {
			dojo.addClass(domObject.focusNode,"DialogError");
			if(!domObject.value.match(/^(\s*[0-9]+)\s*x(\s*[0-9]+\s*)$/)){
				return;
			}
			var size = domObject.value.split("x");
			this.resize(dojo.trim(size[0])+"px", dojo.trim(size[1])+"px");
			dojo.removeClass(domObject.focusNode,"DialogError");
		}
	},

	addEvents: function(markerObj){
		google.maps.event.addListener(markerObj.marker, "click", dojo.hitch(this,function(){
			markerObj.info.open(this.map,markerObj.marker);
			markerObj.infoIsOpen = true;
		}));
		google.maps.event.addListener(markerObj.info, "closeclick",dojo.hitch(this,function(){
			markerObj.infoIsOpen = false;
		}));
	},

	addMarker: function (/*String*/id, /*LatLng*/point, /*String*/titleText, /*String*/markerText, /*Boolean*/draggable,/*Boolean*/infoIsOpen) {
		if (point === undefined)
			point = this.map.getCenter();
		if(infoIsOpen === undefined){infoIsOpen = false;}
		var marker = new google.maps.Marker({
			map:this.map,
			title: titleText,
			draggable: draggable,
			position:point
		});
		if(this.markers[id]){
			this.removeMarker(id);
		}
		this.markers[id] = {
			"searchString" : titleText,
			"marker" : marker,
			"infoIsOpen" : infoIsOpen,
			"info"   : new google.maps.InfoWindow({ 
				content: markerText,
				size: new google.maps.Size(50,50)
			})
		};
		this.map.setCenter(point);
		this.addEvents(this.markers[id]);
		if(infoIsOpen){
			this.markers[id].info.open(this.map,this.markers[id].marker);
		}
	},
	removeMarker : function(/*String*/id){
		if (this.markers[id]) {
			this.markers[id].info.close();
			this.markers[id].marker.setMap(null); // remove marker from map
			delete this.markers[id];
		}
	},
	findAll : function(domObject){
		if(domObject===undefined){
			return;
		}
		var tableNodes = dojo.query("[name=googleMapsMarkerTable]",domObject);
		for(var i=0;i<tableNodes.length;i++){
			var addressInput = dojo.query("input",tableNodes[i]);
			var textInput = dojo.query("textarea",tableNodes[i]);
			if(!addressInput.length || !textInput.length){continue;}
			this.findAddress(addressInput[0], textInput[0]);
		}
	},
	findAddress: function (domObject, infoText,draggable) {
		if(domObject === undefined) return;
		if (draggable ===undefined) draggable = true;
		if(!this.geocoder){this.geocoder = new google.maps.Geocoder();}
		if(this.markers[domObject.name] && this.markers[domObject.name].searchString == domObject.value){
			//searchstring didn't change, so we don't need to search again
			return;
		} 
		this.geocoder.geocode({
			"address": domObject.value
		}, dojo.hitch(this,function(results,status){
			 if (status == google.maps.GeocoderStatus.OK) {
				this.addMarker(domObject.name,results[0].geometry.location, domObject.value, domObject.value, draggable);
				if(infoText){
					this.changeMarkerText(domObject.name, infoText);
				}
				dojo.removeClass(domObject,"DialogError");
	        } else {
				dojo.addClass(domObject,"DialogError");
				//"description":"Address not found '"+ address +"'."
	        }
		}));
	},

	setInfoText: function (id,infoText) {
		if (this.markers[id]) {
			this.markers[id].info.setContent(infoText);
		}
	},

	changeMarkerText: function (id,domObject) {
		if (domObject) {
			var tooltiptext = domObject.value;
			tooltiptext = tooltiptext.replace(/\n/g,'<br/>');
			this.setInfoText(id, tooltiptext);
		}
	},
	encodeInfo: function(info) {
		return this.base64.encode(dojo.toJson(info));
	},
	validateNew : function(info) {
		if(!info.markersPlain) return;
		for(var i=0;i<info.markersPlain.length;i++){
			this.addMarker("googlemaps:address:"+i,new google.maps.LatLng(info.markersPlain[i].lat, info.markersPlain[i].lng),
				info.markersPlain[i].title,info.markersPlain[i].infoText,this.isPreview,info.markersPlain[i].infoIsOpen);
		}
	},
	validateOld : function(info) {
		// some sort of compatibility feature
	},
	loadMap: function( domObject, info) {
		if(domObject === undefined){
			console.warn("Google Map couldn't be loaded. Dom object missing");
			return;
		} 
		this.domObject = domObject;
		parent.epages.cartridges.de_epages.externalcontent.googlemaps = this.markers;
		if (info === undefined) {
			var latlng = new google.maps.LatLng(this.centerLat, this.centerLng);
			this.map = new google.maps.Map(domObject, {
				zoom : this.zoom,
				center: latlng,
				mapTypeId: google.maps.MapTypeId.ROADMAP,
				navigationControl:false
			});
		} else {
			info = this.decodeInfo(info);
			var latlng = new google.maps.LatLng(info.centerLat, info.centerLng);
			info.zoom = parseInt(info.zoom, 10);
			this.controls = info.controls;
			this.resize(info.width, info.height);
			
			var mapOptions = {
				zoom : info.zoom,
				center: latlng,
				mapTypeId: info.type,
				navigationControl: info.controls
			}
			this.map = new google.maps.Map(domObject, mapOptions);
			this.setControls(this.controls);
			
			this.markers = {};
			if(info.markersPlain){
				this.validateNew(info);
			} else {
				this.validateOld(info);
			}
			this.markersPlain = [];
			this.map.setCenter(latlng);
		}
	},
	createMarkersInfo : function(){
		var result = [];
		if(this.markersPlain.length){
			return this.markersPlain;
		}
		for(var i in this.markers){
			result.push({
				infoIsOpen : this.markers[i].infoIsOpen,
				lng: this.markers[i].marker.getPosition().lng(),
				lat: this.markers[i].marker.getPosition().lat(),
				title:this.markers[i].marker.getTitle(),
				infoText: this.markers[i].info.content
			});
		}
		return result;
	},
	/*
	info: function () {
		// Old info function, keep in mind for compatibility when editing an already insered old goolemaps gadget
		// save map detail at object
		var map = this.map;
		if (map !== undefined) {
			dojo.mixin(this, {
				centerLat : map.getCenter().lat(),
				centerLng : map.getCenter().lng(),
				type    : map.getCurrentMapType().getUrlArg(),
				width   : map.getSize().width,
				height  : map.getSize().height,
				zoom    : map.getZoom()
			});
		}
		// get info details from object
		var info = epages.cartridges.de_epages.design.MceContent.prototype.info.apply(this);
		dojo.mixin(info, {
			centerLat : this.centerLat,
			centerLng : this.centerLng,
			type    : this.type,
			zoom    : this.zoom,
			markerLng    : this.markerLng,
			markerLat    : this.markerLat,
			markerText   : this.markerText,
			markerIsOpen : this.markerIsOpen,
			controls: this.controls,
			key: this.key
		});
		return info;
	},*/
	info: function () {
		// save map detail at object
		var map = this.map;
		if (map !== undefined) {
			dojo.mixin(this, {
				centerLat : map.getCenter().lat(),
				centerLng : map.getCenter().lng(),
				type 	: map.getMapTypeId(),
				width   : map.getDiv().style.width,
				height  : map.getDiv().style.height,
				zoom    : map.getZoom()
			});
		}
		// get info details from object
		var info = epages.cartridges.de_epages.design.MceContent.prototype.info.apply(this);
		dojo.mixin(info, {
			centerLat 	: this.centerLat,
			centerLng 	: this.centerLng,
			type    	: this.type,
			zoom    	: this.zoom,
			markersPlain: this.createMarkersInfo(),
			controls	: this.controls
		});
		return info;
	},

	LOAD: function (domObject, info) {
		var googleMap = new epages.cartridges.de_epages.externalcontent.GoogleMap();
		googleMap.loadMap(domObject, info);
	},

	contentString: function (info) {
		var timestamp = (new Date()).getTime();
		var aStrings = [
			'<script type="text/javascript">',
			'dojo["require"]("epages.cartridges.de_epages.externalcontent.googlemap");',
			'window.googleMapsCallback = function(){ ',
			'epages.cartridges.de_epages.externalcontent.GoogleMap.prototype.LOAD($(\''+ timestamp +'\'), \'' + this.encodeInfo(info) + '\');',
			'};',
			'</script>',
			'<script src="http://maps.google.com/maps/api/js?v=3&sensor=false&callback=googleMapsCallback" type="text/javascript"></script>',
			'<div id="'+ timestamp +'" style="width: '+ info.width +'; height: '+info.height+'">\n</div>'
		];
		return aStrings.join("\n");
	},

	loadPreview: function (domObject){
		if (this.map === undefined) {
			this.isPreview = true;
			window["epages_vars_externalcontent_" + this.navId + "_callback"] = dojo.hitch(this, function(){
				this.loadMap($('map'), this.encodeInfo(this.info()));
				if (this.additionalCallback) {
					this.additionalCallback();
				}
			});
			
			var headNode = document.getElementsByTagName("head")[0];
			var script = document.createElement("script");
			script.setAttribute("src", "http://maps.google.com/maps/api/js?v=3&sensor=false&callback=epages_vars_externalcontent_" + this.navId + "_callback");
			script.setAttribute("type", "text/javascript");
			headNode.appendChild(script);
			
			var outerDiv = document.createElement('div');
			outerDiv.innerHTML = '<div id="map" style="width: 300px; height: 300px; max-height: 430px; max-width: 400px;">Loading Map...</div>';
			outerDiv.id = "PluginPreviewFrame";
			outerDiv.style.overflow = "auto";
			
			domObject.parentNode.replaceChild(outerDiv, domObject);
		}
	}
});

