/*
	Copyright (c) 2006-2007, epages Software GmbH
	All Rights Reserved.

	epages.cartridges.de_epages.externalcontent.wikipediade $Revision: 1.20.2.3 $

*/
dojo.provide('epages.cartridges.de_epages.externalcontent.wikipediade');
dojo.require('epages.cartridges.de_epages.design.mcecontent');

epages.cartridges.de_epages.externalcontent.WikipediaDe = epages.Class();

dojo.extend(epages.cartridges.de_epages.externalcontent.WikipediaDe,
	epages.cartridges.de_epages.design.MceContent.prototype, {
	contentClass         : 'epages.cartridges.de_epages.externalcontent.WikipediaDe',
	title                : 'Wikipedia',
	width                : 110,
	height               : 110,
	preview              : epages.vars.StoreRoot + '/BO/icons/navelement_wikipediade.png',

	wikibutton           : epages.vars.StoreRoot + '/SF/Gadgets/wikipedia_button.png',
	wikibanner           : epages.vars.StoreRoot + '/SF/Gadgets/wikipedia_banner.png',

	selectionType        : 'Button',
	fixedLocaleCode			 : 'user',

	info: function () {
		//summary: mixin the needed values for the gadget in SF
		var info = epages.cartridges.de_epages.design.MceContent.prototype.info.apply(this);
		dojo.mixin(info, {
			fixedLocaleCode: this.fixedLocaleCode,
			selectionType: this.selectionType
		});
		return info;
	},

	getLocalisation: function(local){
		//summary: calculate the ISO language symbol for the selected setting
		switch(local){
			case 'current': return "\""+window.epages.vars.Locale.language+"\"";
			case 'english': return "\"en\"";
			//calculate iso symbol for the language when displayed in SF
			default:  			return "window.epages.vars.Locale.language";
		}
	},
	retrieveFormData: function(){
		//summary: exclude the settings of the gadget from the MBO form
		for(var i = 0 ;i < this.formData.get('WikipediaSelection').length;i++){
			if(this.formData.get('WikipediaSelection')[i] != ''){
				this.selectionType = this.formData.get('WikipediaSelection')[i];
			}
		}
		for(i = 0 ;i < this.formData.get('WikipediaLocalisation').length;i++){
			if(this.formData.get('WikipediaLocalisation')[i] != ''){
				this.fixedLocaleCode = this.formData.get('WikipediaLocalisation')[i];
			}
		}
	},
	loadPreview: function (domObject) {
		// summary: load the gadget in the preview iframe in MBO
		top.console.debug('load Preview');
		var timestamp = (new Date()).getTime();
		var iframe = this.getIframeDocument(domObject);
		var head = iframe.getElementsByTagName('head').item(0);
		var oldscript = iframe.getElementById("previewJavascript");

		var newscript = iframe.createElement('script');
		newscript.id = "previewJavascript";
		newscript.type ="text/javascript";

		this.retrieveFormData();
		newscript.text = this.getScript(timestamp,true);

		this.getIframeDocument(domObject).getElementsByTagName('body')[0].innerHTML = this.contentString(this.info(), true, timestamp);

		if(oldscript){
			//replace the old script if the gadget was added before, needed to avoid nameconflicts
			head.replaceChild(newscript,oldscript);
		}else{
			head.appendChild(newscript);
		}
	},

	getScript: function(timestamp,isPreview){
		// summary: create the functions as a string needed to include the gadget in SF
		var localCodeString = this.getLocalisation(this.fixedLocaleCode);
		var translationLocale = "auto";
		if(isPreview){
			translationLocale = epages.vars.Locale.language;
		}
		var finalstring = [
				'dojo.require("epages.io.translation");',
				'var translation = new epages.io.Translation(dojo.moduleUrl("epages.cartridges.de_epages.externalcontent", "templates/wikipediade"), "'+translationLocale+'");',
				'console.debug("translation",translation,epages.vars.Locale.language);',
				'var sourceNode = document.getElementById("div'+timestamp+'");',
				'sourceNode.innerHTML = translation.replaceLanguageTags(sourceNode.innerHTML);'
				].join("\n");
		if(!(this.selectionType) || this.selectionType == "Button"){
			finalstring += [
				'function searchwikipediaSelection(){',
				'var selected;',
				'var localeCode = '+localCodeString+';',
				'if (window.getSelection){selected = window.getSelection();}',
				'else if (document.getSelection){selected = document.getSelection();}',
				'else if (document.selection){selected = document.selection.createRange().text;}',
				'var link = "http://"+localeCode+".wikipedia.org/w/index.php?go=Go&search="+selected;',
				'var newwindow = window.open(link,"Wikipedia");',
				'newwindow.focus();',
				'}'
				].join("\n");
		}else if(this.selectionType == "Searchmask"){
			finalstring += [
				'function searchwikipediaInput(){',
				'var localeCode = '+localCodeString+';',
				'var searchString = document.getElementById("'+timestamp+'").value;',
				'var link = "http://"+localeCode+".wikipedia.org/w/index.php?go=Go&search="+searchString;',
				'var newwindow = window.open(link,"Wikipedia");',
				'newwindow.focus();',
				'}'
				].join("\n");
		}
		return finalstring;
	},

	contentString: function (info, ispreview,timestamp) {
		// summary: create gadget
		top.console.debug('content String');
		var finalstring;
		var backgroundColorString = "";
		if(timestamp === undefined){
			timestamp = (new Date()).getTime();
		}

		if(info.selectionType){
			backgroundColorString="background-color:"+info.formData.get('wikipediaDeBgColor')[0];
		}
		if(!(info.selectionType) || info.selectionType == "Button"){
			finalstring = [
			'<div id="div'+timestamp+'" style="width:'+this.width+'px; margin: 10px; padding: 5px; '+backgroundColorString+'">',
			'<br/>{WikiDescr}',
			'<img src="'+this.wikibutton+'" alt="{wikibuttonalt}" onmousedown="searchwikipediaSelection();" style="cursor:pointer;"></div>',
			].join("\n");
		}
		else if(info.selectionType == "Searchmask"){
			this.width = 137;
			finalstring = [
			'<div id="div'+timestamp+'" style="width:'+this.width+'px; margin: 10px; padding: 5px; background-color:'+info.formData.get('wikipediaDeBgColor')[0]+';">',
			'<a href="http://wikipedia.org" target="_blank">\n',
			'<img src="'+this.wikibanner+'" border="0" alt="{wikibanneralt}" /></a><br/>\n',
			'<form name="search" onsubmit="searchwikipediaInput();return false;">\n',
			'{Search}<br/><input type="text" name="search" id="'+timestamp+'" size="15" /><br />\n',
			'<input type="submit" name="go" value="{gobutton}" /><input type="submit" name="fulltext" value="{fulltextbutton}" />\n',
			'</form>\n',
			'</div>\n'
			].join("\n");
		}
		//add the needed javascript functions
		if(!ispreview){
			finalstring += '<script type="text/javascript" language="Javascript">'+this.getScript(timestamp)+'<'+'/script>';
		}
		return finalstring;
	}
});