use strict;
use Test::More tests => 42;

##no critic qw(ProhibitCaptureWithoutTest)
sub _testClipFishRegEx {
    my $clipfishregex = 'videoid=(.+?)([\"&\']|$)';
    my $regex_url = '\/video\/([\w-]+)';

    my $embed = '?as=0&videoid=NDEyOTB8Mjc5NDQzOQ==&r=1&c=FF0000';
    $embed =~ ($clipfishregex);
    cmp_ok($1, 'eq', "NDEyOTB8Mjc5NDQzOQ==", "ClipFish pattern 1 match (url)");

    $embed = '?as=0&videoid=NDEyOTB8Mjc5NDQzOQ%3D%3D&r=1';
    $embed =~ ($clipfishregex);
    cmp_ok($1, 'eq', "NDEyOTB8Mjc5NDQzOQ%3D%3D", "ClipFish pattern 2 match (url)");

    $embed = "<object classid=\'clsid:d27cdb6e-ae6d-11cf-96b8-444553540000\' codebase=\'http://fpdownload.macromedia.com/pub/shockwave/cabs/flash/swflash.cab#version=8,0,0,0\' width=\'464\' height=\'380\' id=\'player\' align=\'middle\'><param name=\'allowScriptAccess\' value=\'always\' /><param name=\'movie\' value=\'http://www.clipfish.de/videoplayer.swf?as=0&vid=263762&r=1\' /><param name=\'quality\' value=\'high\' /><param name=\'bgcolor\' value=\'#FF0000\' /><param name=\'allowFullScreen\' value=\'true\' /><embed src=\'http://www.clipfish.de/videoplayer.swf?as=0&videoid=MTAwMjIyfDI%3D&r=1\' quality=\'high\' bgcolor=\'#FF0000\' width=\'464\' height=\'380\' name=\'player\' align=\'middle\' allowFullScreen=\'true\' allowScriptAccess=\'always\' type=\'application/x-shockwave-flash\' pluginspage=\'http://www.macromedia.com/go/getflashplayer\'></embed></object><a href=\'http://www.clipfish.de/special/dsds/home/\' title=\'DSDS Videos: alle Castings, Recalls und Mottoshows bei Clipfish.de\' style=\'display:block;margin-left:45px;margin-top:3px;color:#0099CC;cursor:pointer;font-family:verdana,geneva,arial,helvetica,sans-serif;font-size:10px;font-size-adjust:none;font-stretch:normal;font-style:normal;font-variant:normal;font-weight:bold;line-height:normal;text-decoration:underline;\'>DSDS Videos: alle Castings, Recalls und Mottoshows bei Clipfish.de</a>";
    $embed =~ ($clipfishregex);
    cmp_ok($1, 'eq', "MTAwMjIyfDI%3D", "ClipFish pattern 3 match (object)");

    $embed = "videoid=MTAwMjIyfDI%3D";
    $embed =~ ($clipfishregex);
    cmp_ok($1, 'eq', "MTAwMjIyfDI%3D", "ClipFish pattern 4 match (attribute)");

    $embed = '<object width="390" height="260" id="player" align="middle"><param name="movie" value="http://www.clipfish.de/videoplayer.swf?r=1&as=0&vid=263762" /><param name="quality" value="high" /><param name="allowFullScreen" value="true" /><embed src="http://www.clipfish.de/videoplayer.swf?r=1&as=0&videoid=MTAwMjIyfDI%3D" quality="high" width="390" height="260" name="player" align="middle" allowFullScreen="true"></embed></object>';
    $embed =~ ($clipfishregex);
    cmp_ok($1, 'eq', "MTAwMjIyfDI%3D", "ClipFish pattern 5 match (object)");

    $embed = "vid=263762&videoid=MTAwMjIyfDI%3D";
    $embed =~ ($clipfishregex);
    cmp_ok($1, 'eq', "MTAwMjIyfDI%3D", "ClipFish pattern 6 match (attribute in query string)");

    my $permalink = 'http://www.clipfish.de/video/28572/maenner-sehn-alles-/';
    $permalink =~ ($regex_url);
    cmp_ok($1, 'eq', "28572", "Clipfish url pattern match (url)");

    $permalink = 'http://www.clipfish.de/video/28572';
    $permalink =~ ($regex_url);
    cmp_ok($1, 'eq', "28572", "Clipfish url pattern match (permalink)");
}

sub _testMsnVideoRegEx {
    my $msnfishregex = 'v=([\w-]+)';

    my $embed = '<embed src="http://images.video.msn.com/flash/soapbox1_1.swf" width="432" height="364" id="iqor6t1u" type="application/x-shockwave-flash" allowFullScreen="true" allowScriptAccess="always" pluginspage="http://macromedia.com/go/getflashplayer" flashvars="c=v&v=e08b9f5c-c5c1-4a2f-8cbc-91d16593213b&ifs=true&fr=msnvideo&mkt=de-DE"></embed><noembed><br/><a href="http://video.msn.com/video.aspx?vid=e08b9f5c-c5c1-4a2f-8cbc-91d16593213b" target="_new" title="&quot;supervise this&quot; - Vancouver Cancucks going for the 2008-09 stanley cup">Video: &quot;supervise this&quot; - Vancouver Cancucks going for the 2008-09 stanley cup</a></noembed>';
    $embed =~ ($msnfishregex);
    cmp_ok($1, 'eq', "e08b9f5c-c5c1-4a2f-8cbc-91d16593213b", "MsnVideo pattern 1 match (embed)");

    $msnfishregex = 'vid=([\w-]+)';
    $embed =~ ($msnfishregex);
    cmp_ok($1, 'eq', "e08b9f5c-c5c1-4a2f-8cbc-91d16593213b", "MsnVideo pattern 2 match (attribute)");
}

sub _testCultClipRegEx {
    my $cultclipregex_url = 'video\/([\w-]+)';
    my $cultclipregex_embed = 'mediaid=([\w-]+)';

    my $embed = '<object classid="clsid:d27cdb6e-ae6d-11cf-96b8-444553540000" codebase="http://fpdownload.macromedia.com/pub/shockwave/cabs/flash/swflash.cab#version=8,0,0,0" width="480" height="360" id="player" align="middle"><param name="allowScriptAccess" value="always" /><param name="movie" value="http://www.cultclip.tv/ostubeplayer.swf?mediaid=119101&hosturl=http://www.cultclip.tv/&themecolor=0x000000&symbolcolor=0xffffff&backgroundcolor=0x000000&autostart=false&loop=false&overlay=http://www.cultclip.tv//media/custom/player_emb.png&&" />  <param name="quality" value="high" /><param name="bgcolor" value="#000000" /><param name="width" value="480" /><param name="height" value="360" /><param name="scale" value="noscale" /><param name="allowFullScreen" value="true" /><embed src="http://www.cultclip.tv/ostubeplayer.swf?mediaid=119101&hosturl=http://www.cultclip.tv/&themecolor=0x000000&symbolcolor=0xffffff&backgroundcolor=0x000000&autostart=false&loop=false&overlay=http://www.cultclip.tv//media/custom/player_emb.png&&" quality="high" bgcolor="#000000" height="360" width="480" name="player" align="middle" allowFullScreen="true" allowScriptAccess="always" scale="noscale" type="application/x-shockwave-flash" pluginspage="http://www.macromedia.com/go/getflashplayer"></embed></object>';
    $embed =~ ($cultclipregex_embed);
    cmp_ok($1, 'eq', "119101", "Cultclip pattern 1 match (embed)");

    my $permalink = 'http://www.cultclip.tv/video/119101/swisstech-microplus-8in1-multitool---bei-obramo-security.de';
    $permalink =~ ($cultclipregex_url);
    cmp_ok($1, 'eq', "119101", "Cultclip pattern 2 match (permalink)");

    my $browserurl = 'http://www.cultclip.tv/video/119101/swisstech-microplus-8in1-multitool---bei-obramo-security.de';
    $browserurl =~ ($cultclipregex_url);
    cmp_ok($1, 'eq', "119101", "Cultclip pattern 3 match (browser url)");
}

sub _testDailyMotionRegEx {
    my $dailymotionregex_url = 'video\/([\w-]+)';
    my $dailymotionregex_embed = 'swf\/([\w-]+)';
    my $dailymotionregex_input = 'v=([\w-]+)';

    my $embed = '<div><object width="420" height="339"><param name="movie" value="http://www.dailymotion.com/swf/x7c7m9" /><param name="allowFullScreen" value="true" /><param name="allowScriptAccess" value="always" /><embed src="http://www.dailymotion.com/swf/x7c7m9" type="application/x-shockwave-flash" width="420" height="339" allowFullScreen="true" allowScriptAccess="always"></embed></object><br /><b><a href="http://www.dailymotion.com/swf/x7c7m9">BMW HP2 Dig the Dirt</a></b><br /><i>by <a href="http://www.dailymotion.com/Filmschmied">Filmschmied</a></i></div>';
    $embed =~ ($dailymotionregex_embed);
    cmp_ok($1, 'eq', "x7c7m9", "DailyMotion pattern 1 match (embed)");

    my $browserurl = 'http://www.dailymotion.com/video/x7c7m9_bmw-hp2-dig-the-dirt_auto';
    $browserurl =~ ($dailymotionregex_url);
    cmp_ok($1, 'eq', "x7c7m9_bmw-hp2-dig-the-dirt_auto", "DailyMotion pattern 2 match (link)");

    my $input = 'v=x7c7m9';
    $input =~ ($dailymotionregex_input);
    cmp_ok($1, 'eq', "x7c7m9", "DailyMotion pattern 3 match (input)");

    $input = 'v=x7c7m9_bmw-hp2-dig-the-dirt_auto';
    $input =~ ($dailymotionregex_input);
    cmp_ok($1, 'eq', "x7c7m9_bmw-hp2-dig-the-dirt_auto", "DailyMotion pattern 4 match (input)");
}

sub _testMyVideoRegEx {
    my $regex_embed = 'movie\/([\w-]+)';
    my $regex_url = 'watch\/([\w-]+)';
    my $regex_input = 'movie=([\w-]+)';

    my $embed = '<object style="width:470px;height:406px;" width="470" height="406" type="application/x-shockwave-flash" data="http://www.myvideo.de/movie/3783872"><param name="movie" value="http://www.myvideo.de/movie/3783872"></param><param name="AllowFullscreen" value="true"></param><param name="AllowScriptAccess" value="always"></param><embed src="http://www.myvideo.de/movie/3783872" width="470" height="406"></embed></object><br/><a href="http://www.myvideo.de/watch/3783872/Filmfehler" title="Filmfehler - MyVideo">Filmfehler - MyVideo</a>';
    $embed =~ ($regex_embed);
    cmp_ok($1, 'eq', "3783872", "MyVideo pattern 1 match (embed)");

    my $permalink = 'http://www.myvideo.de/watch/3783872/Filmfehler';
    $permalink =~ ($regex_url);
    cmp_ok($1, 'eq', "3783872", "MyVideo pattern 2 match (permalink)");

    my $input = 'movie=3783872';
    $input =~ ($regex_input);
    cmp_ok($1, 'eq', "3783872", "MyVideo pattern 3 match (input)");
}

sub _testSevenLoadRegEx {
    my $regex_embed = 'pl\/([\w-]+)';
    my $regex_url = 'v=([\w-]+)';

    my $embed = '<script type="text/javascript" src="http://de.sevenload.com/pl/BqV1zFl/500x408/0"></script><p>Link: <a href="http://de.sevenload.com/videos/BqV1zFl-Ein-Kaefer-mischt-auf-Minuscule"><img src="http://static.sevenload.com/img/sevenload.png" width="66" height="10" alt="Ein Kfer mischt auf! - Minuscule" /></a></p>';
    $embed =~ ($regex_embed);
    cmp_ok($1, 'eq', "BqV1zFl", "Sevenload pattern 1 match (embed)");

    my $permalink = 'http://de.sevenload.com/videos/BqV1zFl-Ein-Kaefer-mischt-auf-Minuscule';
    $permalink =~ ($regex_url);
    cmp_ok($1, 'eq', "BqV1zFl", "Sevenload pattern 2 match (url)");

    my $input = 'v=BqV1zFl';
    $input =~ ($regex_url);
    cmp_ok($1, 'eq', "BqV1zFl", "Sevenload pattern 3 match (input)");
}

sub _testVeohRegEx {
    my $regex_embed = 'permalinkId=([\w-]+)';
    my $regex_url = 'watch\/([\w-]+)';
    my $regex_input = 'v=([\w-]+)';

    my $embed = '<object width="410" height="341" id="veohFlashPlayer" name="veohFlashPlayer"><param name="movie" value="http://www.veoh.com/static/swf/webplayer/WebPlayer.swf?version=AFrontend.5.4.2.14.1006&permalinkId=v18443537qE3rfxCb&player=videodetailsembedded&videoAutoPlay=0&id=anonymous"></param><param name="allowFullScreen" value="true"></param><param name="allowscriptaccess" value="always"></param><embed src="http://www.veoh.com/static/swf/webplayer/WebPlayer.swf?version=AFrontend.5.4.2.14.1006&permalinkId=v18443537qE3rfxCb&player=videodetailsembedded&videoAutoPlay=0&id=anonymous" type="application/x-shockwave-flash" allowscriptaccess="always" allowfullscreen="true" width="410" height="341" id="veohFlashPlayerEmbed" name="veohFlashPlayerEmbed"></embed></object><br /><font size="1">Watch <a href="http://www.veoh.com/browse/videos/category/animation/watch/v18443537qE3rfxCb">Audi Art Night 2009</a> in <a href="http://www.veoh.com/browse/videos/category/animation">Zeichentrick</a>&nbsp;&nbsp;|&nbsp;&nbsp;View More <a href="http://www.veoh.com">Free Videos Online at Veoh.com</a></font>';
    $embed =~ ($regex_embed);
    cmp_ok($1, 'eq', "v18443537qE3rfxCb", "Veoh pattern 1 match (embed)");

    my $permalink = 'http://www.veoh.com/browse/videos/category/animation/watch/v18443537qE3rfxCb';
    $permalink =~ ($regex_url);
    cmp_ok($1, 'eq', "v18443537qE3rfxCb", "Veoh pattern 2 match (link)");

    my $input = 'v=v18443537qE3rfxCb';
    $input =~ ($regex_url);
    cmp_ok($1, 'eq', "v18443537qE3rfxCb", "Veoh pattern 3 match (input)");
}

sub _testVimeoRegEx {
    my $regex_embed = 'clip_id=([\w-]+)';
    my $regex_url = 'vimeo.com\/([\w-]+)';
    my $regex_input = 'v=([\w-]+)';

    my $embed = '<object width="400" height="230"><param name="allowfullscreen" value="true" /><param name="allowscriptaccess" value="always" /><param name="movie" value="http://vimeo.com/moogaloop.swf?clip_id=4470589&amp;server=vimeo.com&amp;show_title=1&amp;show_byline=1&amp;show_portrait=0&amp;color=&amp;fullscreen=1" /><embed src="http://vimeo.com/moogaloop.swf?clip_id=4470589&amp;server=vimeo.com&amp;show_title=1&amp;show_byline=1&amp;show_portrait=0&amp;color=&amp;fullscreen=1" type="application/x-shockwave-flash" allowfullscreen="true" allowscriptaccess="always" width="400" height="230"></embed></object><p><a href="http://vimeo.com/4470589">Oceansize</a> from <a href="http://vimeo.com/user1694667">Oceansize Team</a> on <a href="http://vimeo.com">Vimeo</a>.</p>';
    $embed =~ ($regex_embed);
    cmp_ok($1, 'eq', "4470589", "Vimeo pattern 1 match (embed)");

    my $permalink = 'http://vimeo.com/4470589';
    $permalink =~ ($regex_url);
    cmp_ok($1, 'eq', "4470589", "Vimeo pattern 2 match (link)");

    my $input = 'v=4961238';
    $input =~ ($regex_url);
    cmp_ok($1, 'eq', "4470589", "Vimeo pattern 3 match (input)");
}

sub _testWideoRegEx {
    my $regex_embed = 'video\/([\w-]+)';
    my $regex_url = 'id=\"([\w-]+)';
    my $regex_input = 'v=([\w-]+)';

    my $embed = '<object name="iLyROoafM49c" id="iLyROoafM49c" type="application/x-shockwave-flash" data="http://sa.kewego.com/swf/p3/epix.swf" width="400" height="300">  <param name="flashVars" value="language_code=fr&playerKey=0df9b773a15b&skinKey=7109c4112f57&sig=iLyROoafM49c&autostart=false&advertise=1" />  <param name="movie" value="http://sa.kewego.com/swf/p3/epix.swf" />  <param name="allowFullScreen" value="true" />  <param name="allowscriptaccess" value="always" /></object><div style="width: 400px;"><a href="http://www.wideo.fr/video/iLyROoafM49c.html">les incongrus dechaines(suite) - wideo</a> <br />ecoledurireorange.fr<div>Mots-cls : <a href="http://www.wideo.fr/search/?q=theatre">theatre</a> <a href="http://www.wideo.fr/search/?q=humour">humour</a> <a href="http://www.wideo.fr/search/?q=rire">rire</a> </div> <div style="text-align: right;"> <a href="http://www.wideo.fr/video/iLyROoafM49c.html">Video</a> de <a href="http://www.wideo.fr/search/?q=user:lecoledurire">lecoledurire</a> </div></div>';
    $embed =~ ($regex_embed);
    cmp_ok($1, 'eq', "iLyROoafM49c", "wideo pattern 1 match (embed)");

    my $permalink = 'http://www.wideo.fr/video/iLyROoafM49c.html';
    $permalink =~ ($regex_url);
    cmp_ok($1, 'eq', "iLyROoafM49c", "wideo pattern 2 match (browser url)");

    $permalink = '<a href="http://www.wideo.fr/video/iLyROoafM49c.html" title="les incongrus dechaines(suite) - wideo"><img src="http://ta.kewego.com/t/4/0720/154x114_iLyROoafM49c_2.jpg" alt="les incongrus dechaines(suite) - wideo"/></a><br /><a href="http://www.wideo.fr/video/iLyROoafM49c.html">les incongrus dechaines(suite) - wideo</a>';
    $permalink =~ ($regex_url);
    cmp_ok($1, 'eq', "iLyROoafM49c", "wideo pattern 3 match (html code)");

    $permalink = '[url=http://www.wideo.fr/video/iLyROoafM49c.html][img]http://ta.kewego.com/t/4/0720/154x114_iLyROoafM49c_2.jpg[/img][/url][url=http://www.wideo.fr/video/iLyROoafM49c.html]les incongrus dechaines(suite) - wideo[/url]';
    $permalink =~ ($regex_url);
    cmp_ok($1, 'eq', "iLyROoafM49c", "wideo pattern 4 match (BBCode)");

    $permalink = '[((http://ta.kewego.com/t/4/0720/154x114_iLyROoafM49c_2.jpg|les incongrus dechaines(suite) - wideo))|http://www.wideo.fr/video/iLyROoafM49c.html][les incongrus dechaines(suite) - wideo|http://www.wideo.fr/video/iLyROoafM49c.html]';
    $permalink =~ ($regex_url);
    cmp_ok($1, 'eq', "iLyROoafM49c", "wideo pattern 5 match (wiki Code)");

    $permalink = '[les incongrus dechaines(suite) - wideo|http://www.wideo.fr/video/iLyROoafM49c.html]';
    $permalink =~ ($regex_url);
    cmp_ok($1, 'eq', "iLyROoafM49c", "wideo pattern 6 match (Instant messenger html code)");

    $permalink = '[url=http://www.wideo.fr/video/iLyROoafM49c.html]les incongrus dechaines(suite) - wideo[/url]';
    $permalink =~ ($regex_url);
    cmp_ok($1, 'eq', "iLyROoafM49c", "wideo pattern 7 match (Instant messenger BBCode)");

    $permalink = '[les incongrus dechaines(suite) - wideo|http://www.wideo.fr/video/iLyROoafM49c.html]';
    $permalink =~ ($regex_url);
    cmp_ok($1, 'eq', "iLyROoafM49c", "wideo pattern 8 match (Instant messenger wiki code)");

    my $input = 'v=iLyROoafM49c';
    $input =~ ($regex_input);
    cmp_ok($1, 'eq', "iLyROoafM49c", "wideo pattern 9 match (input)");
}

sub _testYoutubeRegEx {
    my $regex_embed = 'v\/([\w-]+)';
    my $regex_url = 'v=([\w-]+)';

    my $embed = '<object width="425" height="344"><param name="movie" value="http://www.youtube.com/v/-NHpyPY3Q8M&hl=de&fs=1&"></param><param name="allowFullScreen" value="true"></param><param name="allowscriptaccess" value="always"></param><embed src="http://www.youtube.com/v/-NHpyPY3Q8M&hl=de&fs=1&" type="application/x-shockwave-flash" allowscriptaccess="always" allowfullscreen="true" width="425" height="344"></embed></object>';
    $embed =~ ($regex_embed);
    cmp_ok($1, 'eq', "-NHpyPY3Q8M", "Youtube pattern 1 match (embed)");

    my $permalink = 'http://www.youtube.com/watch?v=-NHpyPY3Q8M&feature=related';
    $permalink =~ ($regex_url);
    cmp_ok($1, 'eq', "-NHpyPY3Q8M", "Youtube pattern 2 match (link)");

    $permalink = 'http://www.youtube.com/watch?v=-NHpyPY3Q8M';
    $permalink =~ ($regex_url);
    cmp_ok($1, 'eq', "-NHpyPY3Q8M", "Youtube pattern 3 match (link)");

    my $input = 'v=-NHpyPY3Q8M';
    $input =~ ($regex_url);
    cmp_ok($1, 'eq', "-NHpyPY3Q8M", "Youtube pattern 4 match (input)");
}

_testClipFishRegEx();
_testMsnVideoRegEx();
_testCultClipRegEx();
_testDailyMotionRegEx();
_testMyVideoRegEx();
_testSevenLoadRegEx();
_testVeohRegEx();
_testVimeoRegEx();
_testWideoRegEx();
_testYoutubeRegEx();
