use strict;
use Test::Simple tests => 86;

use Getopt::Long;
use DE_EPAGES::Installer::API::XMLFilePatcher;
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Object::API::XMLCompare qw ( CompareXML );
use DE_EPAGES::Core::API::File qw ( DeleteFile ExistsFile );

sub Main {

    local $| = 1;

    my $DeleteFiles = 0;
    my $Help;

    GetOptions(
        'help'          => \$Help,
        'deletefiles'  => \$DeleteFiles,
    );

    usage() if $Help;

    my $time1 = time;

    my $hOptions = {'checkdelete' => '1', 'checkreference' => '1', 'printmessage' => '1'};

    #test 01 - Permissions
    my $OldFile = 'Permissions_old.xml';
    my $NewFile = 'Permissions_new.xml';
    my $ImportPatchFile = 'Permissions_patch_import.xml';
    my $DeletePatchFile = 'Permissions_patch_delete.xml';

    my $FilePatcher=DE_EPAGES::Installer::API::XMLFilePatcher->new();
    $FilePatcher->patchXMLFiles($OldFile, $NewFile, $DeletePatchFile, 1);
    ok (CompareXML($DeletePatchFile, 'Permissions_del.xml', $hOptions), 'permission delete 1st');
    ok (CompareXML('Permissions_del.xml', $DeletePatchFile, $hOptions), 'permission delete 2nd');
    DeleteFile($DeletePatchFile) if ($DeleteFiles);

    $FilePatcher->patchXMLFiles($OldFile, $NewFile, $ImportPatchFile, 0);
    ok (CompareXML($ImportPatchFile, 'Permissions_imp.xml',  $hOptions), 'permission import 1st');
    ok (CompareXML('Permissions_imp.xml', $ImportPatchFile,  $hOptions), 'permission import 2nd');
    DeleteFile($ImportPatchFile) if ($DeleteFiles);

    #test 02 - Attributes
    $OldFile = 'AttributesShop_old.xml';
    $NewFile = 'AttributesShop_new.xml';
    $ImportPatchFile = 'AttributesShop_patch_import.xml';

    $FilePatcher->patchXMLFiles($OldFile, $NewFile, $ImportPatchFile, 0);
    ok (CompareXML($ImportPatchFile, 'AttributesShop_imp.xml', $hOptions), 'path 1st');
    ok (CompareXML('AttributesShop_imp.xml', $ImportPatchFile, $hOptions), 'path 2nd');
    DeleteFile($ImportPatchFile) if ($DeleteFiles);

    #test 02 - Attributes
    $OldFile = 'Attributes_old.xml';
    $NewFile = 'Attributes_new.xml';
    $ImportPatchFile = 'Attributes_patch_import.xml';
    $DeletePatchFile = 'Attributes_patch_delete.xml';

    $FilePatcher->patchXMLFiles($OldFile, $NewFile, $DeletePatchFile, 1);
    ok (CompareXML($DeletePatchFile, 'Attributes_del.xml', $hOptions), 'attributes delete 1st');
    ok (CompareXML('Attributes_del.xml', $DeletePatchFile, $hOptions), 'attributes delete 2nd');
    DeleteFile($DeletePatchFile) if ($DeleteFiles);

    $FilePatcher->patchXMLFiles($OldFile, $NewFile, $ImportPatchFile, 0);
    ok (CompareXML($ImportPatchFile, 'Attributes_imp.xml', $hOptions), 'attributes import 1st');
    ok (CompareXML('Attributes_imp.xml', $ImportPatchFile, $hOptions), 'attributes import 2nd');
    DeleteFile($ImportPatchFile) if ($DeleteFiles);

    #test 03 - Actions
    $OldFile = 'Actions_old.xml';
    $NewFile = 'Actions_new.xml';
    $ImportPatchFile = 'Actions_patch_import.xml';
    $DeletePatchFile = 'Actions_patch_delete.xml';

    $FilePatcher->patchXMLFiles($OldFile, $NewFile, $DeletePatchFile, 1);
    ok (CompareXML($DeletePatchFile, 'Actions_del.xml', $hOptions), 'actions delete 1st');
    ok (CompareXML('Actions_del.xml', $DeletePatchFile, $hOptions), 'actions delete 2nd');
    DeleteFile($DeletePatchFile) if ($DeleteFiles);

    $FilePatcher->patchXMLFiles($OldFile, $NewFile, $ImportPatchFile, 0);
    ok (CompareXML($ImportPatchFile, 'Actions_imp.xml', $hOptions), 'actions import 1st');
    ok (CompareXML('Actions_imp.xml', $ImportPatchFile, $hOptions), 'actions import 2nd');
    DeleteFile($ImportPatchFile) if ($DeleteFiles);

    #test 04 - Hooks
    $OldFile = 'Hooks_old.xml';
    $NewFile = 'Hooks_new.xml';
    $ImportPatchFile = 'Hooks_patch_import.xml';
    $DeletePatchFile = 'Hooks_patch_delete.xml';

    $FilePatcher->patchXMLFiles($OldFile, $NewFile, $DeletePatchFile, 1);
    ok (CompareXML($DeletePatchFile, 'Hooks_del.xml', $hOptions), 'hooks delete 1st');
    ok (CompareXML('Hooks_del.xml', $DeletePatchFile, $hOptions), 'hooks delete 2nd');
    DeleteFile($DeletePatchFile) if ($DeleteFiles);

    $FilePatcher->patchXMLFiles($OldFile, $NewFile, $ImportPatchFile, 0);
    ok (CompareXML($ImportPatchFile, 'Hooks_imp.xml', $hOptions), 'hooks import 1st');
    ok (CompareXML('Hooks_imp.xml', $ImportPatchFile, $hOptions), 'hooks import 2nd');
    DeleteFile($ImportPatchFile) if ($DeleteFiles);

    #test 05 - Translation
    $OldFile = 'Translation.Test_old_de.xml';
    $NewFile = 'Translation.Test_new_de.xml';
    $ImportPatchFile = 'Translation.Test_patch_import_de.xml';
    $DeletePatchFile = 'Translation.Test_patch_delete_de.xml';

    $FilePatcher->patchXMLFiles($OldFile, $NewFile, $DeletePatchFile, 1);
    ok (CompareXML($DeletePatchFile, 'Translation.Test_del_de.xml', $hOptions), 'localized delete 1st');
    ok (CompareXML('Translation.Test_del_de.xml', $DeletePatchFile, $hOptions), 'localized delete 2nd');
    DeleteFile($DeletePatchFile) if (ExistsFile($DeletePatchFile) and $DeleteFiles);

    $FilePatcher->patchXMLFiles($OldFile, $NewFile, $ImportPatchFile, 0);
    ok (CompareXML('Translation.Test_imp_de.xml', $ImportPatchFile, $hOptions), 'localized attributes import 2nd');
    ok (CompareXML('Translation.Test_imp_de.xml', $ImportPatchFile, {'checkdelete' => '1', 'checkreference' => '0', 'printmessage' => '1'}), 'localized attributes import 2nd');
    DeleteFile($ImportPatchFile) if ($DeleteFiles);

    #test 06 - TemplateTypes
    $OldFile = 'TemplateTypesSystem_old.xml';
    $NewFile = 'TemplateTypesSystem_new.xml';
    $ImportPatchFile = 'TemplateTypesSystem_patch_import.xml';
    $DeletePatchFile = 'TemplateTypesSystem_patch_delete.xml';

    $FilePatcher->patchXMLFiles($OldFile, $NewFile, $DeletePatchFile, 1);
    ok (CompareXML($DeletePatchFile, 'TemplateTypesSystem_del.xml', $hOptions), 'templates delete 1st');
    ok (CompareXML('TemplateTypesSystem_del.xml', $DeletePatchFile, $hOptions), 'templates delete 2nd');
    DeleteFile($DeletePatchFile) if ($DeleteFiles);

    $FilePatcher->patchXMLFiles($OldFile, $NewFile, $ImportPatchFile, 0);
    ok (CompareXML($ImportPatchFile, 'TemplateTypesSystem_imp.xml', $hOptions), 'templates import 1st');
    ok (CompareXML('TemplateTypesSystem_imp.xml', $ImportPatchFile, $hOptions), 'templates import 2nd');
    DeleteFile($ImportPatchFile) if ($DeleteFiles);

    #test 07 - Countries
    $OldFile = 'Countries_3166_old.xml';
    $NewFile = 'Countries_3166_new.xml';
    $ImportPatchFile = 'Countries_3166_patch_import.xml';
    $DeletePatchFile = 'Countries_3166_patch_delete.xml';

    $FilePatcher->patchXMLFiles($OldFile, $NewFile, $DeletePatchFile, 1);
    ok (CompareXML($DeletePatchFile, 'Countries_3166_del.xml', $hOptions), 'countries delete 1st');
    ok (CompareXML('Countries_3166_del.xml', $DeletePatchFile, $hOptions), 'countries delete 2nd');
    DeleteFile($DeletePatchFile) if ($DeleteFiles);

    $FilePatcher->patchXMLFiles($OldFile, $NewFile, $ImportPatchFile, 0);
    ok (CompareXML($ImportPatchFile, 'Countries_3166_imp.xml', $hOptions), 'countries import 1st');
    ok (CompareXML('Countries_3166_imp.xml', $ImportPatchFile, $hOptions), 'countries import 2nd');
    DeleteFile($ImportPatchFile) if ($DeleteFiles);

    #doesnt work at the moment because of new  line "&#10;" in xml file
    #test 08 - Countries
    #my $OldFile = 'Locales_old.xml';
    #my $NewFile = 'Locales_new.xml';
    #my $ImportPatchFile = 'Locales_patch_import.xml';
    #my $DeletePatchFile = 'Locales_patch_delete.xml';
    #ok (CompareXML('Locales_del.xml', $DeletePatchFile, $hOptions), 'Locales delete');
    #DeleteFile($DeletePatchFile);
    #ok (CompareXML('Locales_imp.xml', $ImportPatchFile, $hOptions), 'Locales import');
    #DeleteFile($ImportPatchFile);

    #test 09 - Currencies
    $OldFile = 'Currencies_4217_old.xml';
    $NewFile = 'Currencies_4217_new.xml';
    $ImportPatchFile = 'Currencies_4217_patch_import.xml';
    $DeletePatchFile = 'Currencies_4217_patch_delete.xml';

    $FilePatcher->patchXMLFiles($OldFile, $NewFile, $DeletePatchFile, 1);
    ok (CompareXML($DeletePatchFile, 'Currencies_4217_del.xml', $hOptions), 'currenies delete 1st');
    ok (CompareXML('Currencies_4217_del.xml', $DeletePatchFile, $hOptions), 'currenies delete 2nd');
    DeleteFile($DeletePatchFile) if ($DeleteFiles);

    $FilePatcher->patchXMLFiles($OldFile, $NewFile, $ImportPatchFile, 0);
    ok (CompareXML($ImportPatchFile, 'Currencies_4217_imp.xml', $hOptions), 'currencies import 1st');
    ok (CompareXML('Currencies_4217_imp.xml', $ImportPatchFile, $hOptions), 'currencies import 2nd');
    DeleteFile($ImportPatchFile) if ($DeleteFiles);

    #test 10 - Search
    $OldFile = 'SearchProducts_old.xml';
    $NewFile = 'SearchProducts_new.xml';
    $ImportPatchFile = 'SearchProducts_patch_import.xml';
    $DeletePatchFile = 'SearchProducts_patch_delete.xml';

    $FilePatcher->patchXMLFiles($OldFile, $NewFile, $DeletePatchFile, 1);
    ok (CompareXML($DeletePatchFile, 'SearchProducts_del.xml', $hOptions), 'search definitions delete 1st');
    ok (CompareXML('SearchProducts_del.xml', $DeletePatchFile, $hOptions), 'search definitions delete 2nd');
    DeleteFile($DeletePatchFile) if ($DeleteFiles);

    $FilePatcher->patchXMLFiles($OldFile, $NewFile, $ImportPatchFile, 0);
    ok (CompareXML($ImportPatchFile, 'SearchProducts_imp.xml', $hOptions), 'search definitions import 1st');
    ok (CompareXML('SearchProducts_imp.xml', $ImportPatchFile, $hOptions), 'search definitions import 2nd');
    DeleteFile($ImportPatchFile) if ($DeleteFiles);

    #test 11 - PageTypes
    $OldFile = 'PageTypesMBO_old.xml';
    $NewFile = 'PageTypesMBO_new.xml';
    $ImportPatchFile = 'PageTypesMBO_patch_import.xml';
    $DeletePatchFile = 'PageTypesMBO_patch_delete.xml';

    $FilePatcher->patchXMLFiles($OldFile, $NewFile, $DeletePatchFile, 1);
    ok (CompareXML($DeletePatchFile, 'PageTypesMBO_del.xml', $hOptions), 'pagetypes delete 1st');
    ok (CompareXML('PageTypesMBO_del.xml', $DeletePatchFile, $hOptions), 'pagetypes delete 2nd');
    DeleteFile($DeletePatchFile) if ($DeleteFiles);

    $FilePatcher->patchXMLFiles($OldFile, $NewFile, $ImportPatchFile, 0);
    ok (CompareXML($ImportPatchFile, 'PageTypesMBO_imp.xml', $hOptions), 'pagetypes import 1st');
    ok (CompareXML('PageTypesMBO_imp.xml', $ImportPatchFile, $hOptions), 'pagetypes import 2nd');
    DeleteFile($ImportPatchFile) if ($DeleteFiles);

    #test 11 - PageTypes
    $OldFile = 'PageTypesMBO2_old.xml';
    $NewFile = 'PageTypesMBO2_new.xml';
    $ImportPatchFile = 'PageTypesMBO2_patch_import.xml';
    $DeletePatchFile = 'PageTypesMBO2_patch_delete.xml';

    $FilePatcher->patchXMLFiles($OldFile, $NewFile, $ImportPatchFile, 0);
    ok (CompareXML($ImportPatchFile, 'PageTypesMBO2_imp.xml', $hOptions), 'pagetypes import 1st');
    ok (CompareXML('PageTypesMBO2_imp.xml', $ImportPatchFile, $hOptions), 'pagetypes import 2nd');
    DeleteFile($ImportPatchFile) if ($DeleteFiles);

    #test 12 - ShopChildDependencies
    $OldFile = 'ShopChildDependencies_old.xml';
    $NewFile = 'ShopChildDependencies_new.xml';
    $ImportPatchFile = 'ShopChildDependencies_patch_import.xml';
    $DeletePatchFile = 'ShopChildDependencies_patch_delete.xml';

    $FilePatcher->patchXMLFiles($OldFile, $NewFile, $DeletePatchFile, 1);
    ok (CompareXML($DeletePatchFile, 'ShopChildDependencies_del.xml', $hOptions), 'shopchild dependencies delete 1st');
    ok (CompareXML('ShopChildDependencies_del.xml', $DeletePatchFile, $hOptions), 'shopchild dependencies delete 2nd');
    DeleteFile($DeletePatchFile) if ($DeleteFiles);

    $FilePatcher->patchXMLFiles($OldFile, $NewFile, $ImportPatchFile, 0);
    ok (CompareXML($ImportPatchFile, 'ShopChildDependencies_imp.xml', $hOptions), 'shopchild dependencies import 1st');
    ok (CompareXML('ShopChildDependencies_imp.xml', $ImportPatchFile, $hOptions), 'shopchild dependencies import 2nd');
    DeleteFile($ImportPatchFile) if ($DeleteFiles);

    #test 12 - Multi Cartridge Tags
    $OldFile = 'PageTypesMBO_MultiCartridge_old.xml';
    $NewFile = 'PageTypesMBO_MultiCartridge_new.xml';
    $ImportPatchFile = 'PageTypesMBO_MultiCartridge_patch_import.xml';

    $FilePatcher->patchXMLFiles($OldFile, $NewFile, $ImportPatchFile, 0);
    ok (CompareXML($ImportPatchFile, 'PageTypesMBO_MultiCartridge_imp.xml', $hOptions), 'templates with multi cartridge tags import 1st');
    ok (CompareXML('PageTypesMBO_MultiCartridge_imp.xml', $ImportPatchFile, $hOptions), 'templates with multi cartridge tags import 2nd');
    DeleteFile($ImportPatchFile) if ($DeleteFiles);

    #test 12 - Test Defaults
    $OldFile = 'AttributesDefaultTest_old.xml';
    $NewFile = 'AttributesDefaultTest_new.xml';
    $ImportPatchFile = 'AttributesDefaultTest_patch_import.xml';

    $FilePatcher->patchXMLFiles($OldFile, $NewFile, $ImportPatchFile, 0);
    ok (CompareXML($ImportPatchFile, 'AttributesDefaultTest_imp.xml', $hOptions), 'import defaults 1st');
    ok (CompareXML('AttributesDefaultTest_imp.xml', $ImportPatchFile, $hOptions), 'import defaults 2nd');
    DeleteFile($ImportPatchFile) if ($DeleteFiles);

    #test 12 - Features, Use of Path/Alias
    $OldFile = 'Features_old.xml';
    $NewFile = 'Features_new.xml';
    $ImportPatchFile = 'Features_patch_import.xml';

    $FilePatcher->patchXMLFiles($OldFile, $NewFile, $ImportPatchFile, 0);
    ok (CompareXML($ImportPatchFile, 'Features_imp.xml', $hOptions), 'features import 1st');
    ok (CompareXML('Features_imp.xml', $ImportPatchFile, $hOptions), 'features import 2nd');
    DeleteFile($ImportPatchFile) if ($DeleteFiles);

    #test 13 - Ignore tags without expression
    $OldFile = 'Permissions2_old.xml';
    $NewFile = 'Permissions2_new.xml';
    $ImportPatchFile = 'Permissions2_patch_import.xml';
    $DeletePatchFile = 'Permissions2_patch_delete.xml';

    $FilePatcher->patchXMLFiles($OldFile, $NewFile, $DeletePatchFile, 1);
    ok (! ExistsFile($DeletePatchFile), 'ignore tags without expression 1st');
    DeleteFile($DeletePatchFile) if ($DeleteFiles);

    $FilePatcher->patchXMLFiles($OldFile, $NewFile, $ImportPatchFile, 0);
    ok (! ExistsFile($ImportPatchFile), 'ignore tags without expression 2nd');
    DeleteFile($ImportPatchFile) if ($DeleteFiles);

    #test 15 - Search4B
    $OldFile = 'Search4BTransactions_old.xml';
    $NewFile = 'Search4BTransactions_new.xml';
    $ImportPatchFile = 'Search4BTransactions_patch_import.xml';
    $DeletePatchFile = 'Search4BTransactions_patch_delete.xml';

    $FilePatcher->patchXMLFiles($OldFile, $NewFile, $DeletePatchFile, 1);
    ok (CompareXML($DeletePatchFile, 'Search4BTransactions_del.xml', $hOptions), '4B transactions delete 1st');
    ok (CompareXML('Search4BTransactions_del.xml', $DeletePatchFile, $hOptions), '4B transactions delete 2nd');
    DeleteFile($DeletePatchFile) if ($DeleteFiles);

    $FilePatcher->patchXMLFiles($OldFile, $NewFile, $ImportPatchFile, 0);
    ok (CompareXML($ImportPatchFile, 'Search4BTransactions_imp.xml', $hOptions), '4B transactions import 1st');
    ok (CompareXML('Search4BTransactions_imp.xml', $ImportPatchFile, $hOptions), '4B transactions import 2nd');
    DeleteFile($ImportPatchFile) if ($DeleteFiles);

    #test 16 - <notrans> tag
    $OldFile = 'Translation.Actions_old_de.xml';
    $NewFile = 'Translation.Actions_new_de.xml';
    $ImportPatchFile = 'Translation.Actions_patch_import_de.xml';
    $DeletePatchFile = 'Translation.Actions_patch_delete_de.xml';

    $FilePatcher->patchXMLFiles($OldFile, $NewFile, $ImportPatchFile, 0);
    ok (CompareXML($ImportPatchFile, 'Translation.Actions_imp_de.xml', $hOptions), 'notrans tag check 1st');
    ok (CompareXML('Translation.Actions_imp_de.xml', $ImportPatchFile, $hOptions), 'notrans tag check 2nd');
    DeleteFile($ImportPatchFile) if ($DeleteFiles);

    #test 17 - <notrans> tag
    $OldFile = 'ShopChildDependencies2_old.xml';
    $NewFile = 'ShopChildDependencies2_new.xml';
    $ImportPatchFile = 'ShopChildDependencies2_patch_import.xml';
    $DeletePatchFile = 'ShopChildDependencies2_patch_delete.xml';

    $FilePatcher->patchXMLFiles($OldFile, $NewFile, $ImportPatchFile, 0);
    ok (CompareXML($ImportPatchFile, 'ShopChildDependencies2_imp.xml', $hOptions), 'shopchild check 2 1st');
    ok (CompareXML('ShopChildDependencies2_imp.xml', $ImportPatchFile, $hOptions), 'shopchild check 2 2nd');
    DeleteFile($ImportPatchFile) if ($DeleteFiles);

    #test 18 - Translation Attributes
    $OldFile = 'Translation.ActionsTest_old_de.xml';
    $NewFile = 'Translation.ActionsTest_new_de.xml';
    $ImportPatchFile = 'Translation.ActionsTest_patch_import_de.xml';
    $DeletePatchFile = 'Translation.ActionsTest_patch_delete_de.xml';

    $FilePatcher->patchXMLFiles($OldFile, $NewFile, $DeletePatchFile, 1);
    ok (CompareXML($DeletePatchFile, 'Translation.ActionsTest_del_de.xml', $hOptions), 'localized delete 1st');
    ok (CompareXML('Translation.ActionsTest_del_de.xml', $DeletePatchFile, $hOptions), 'localized delete 2nd');
    DeleteFile($DeletePatchFile) if (ExistsFile($DeletePatchFile) and $DeleteFiles);

    $FilePatcher->patchXMLFiles($OldFile, $NewFile, $ImportPatchFile, 0);
    ok (CompareXML($ImportPatchFile, 'Translation.ActionsTest_imp_de.xml', {'checkdelete' => '1', 'checkreference' => '0', 'printmessage' => '1'}), 'check translation 1st');
    ok (CompareXML('Translation.ActionsTest_imp_de.xml', $ImportPatchFile, {'checkdelete' => '1', 'checkreference' => '0', 'printmessage' => '1'}), 'check translation 2nd');
    DeleteFile($ImportPatchFile) if (ExistsFile($ImportPatchFile) and $DeleteFiles);

    #test 19 - Translation Attributes
    $OldFile = 'HooksFeatures_old.xml';
    $NewFile = 'HooksFeatures_new.xml';
    $ImportPatchFile = 'HooksFeatures_patch_import.xml';
    $DeletePatchFile = 'HooksFeatures_patch_delete.xml';

    $FilePatcher->patchXMLFiles($OldFile, $NewFile, $ImportPatchFile, 0);
    ok (CompareXML($ImportPatchFile, 'HooksFeatures_imp.xml', $hOptions), 'check translation 1st');
    ok (CompareXML('HooksFeatures_imp.xml', $ImportPatchFile, $hOptions), 'check translation 2nd');
    DeleteFile($ImportPatchFile) if (ExistsFile($ImportPatchFile) and $DeleteFiles);

    #test 20 - NavElements
    $OldFile = 'NavElements_old.xml';
    $NewFile = 'NavElements_new.xml';
    $DeletePatchFile = 'NavElements_patch_delete.xml';

    $FilePatcher->patchXMLFiles($OldFile, $NewFile, $DeletePatchFile, 1);
    ok (CompareXML($DeletePatchFile, 'NavElements_del.xml', $hOptions), 'navelements delete 1st');
    ok (CompareXML('NavElements_del.xml', $DeletePatchFile, $hOptions), 'navelements delete 2nd');
    DeleteFile($DeletePatchFile) if ($DeleteFiles);

    #test 14 - Forms
    $OldFile = 'Forms_old.xml';
    $NewFile = 'Forms_new.xml';
    $ImportPatchFile = 'Forms_patch_import.xml';
    $DeletePatchFile = 'Forms_patch_delete.xml';

    $FilePatcher->patchXMLFiles($OldFile, $NewFile, $DeletePatchFile, 1);
    ok (CompareXML($DeletePatchFile, 'Forms_del.xml', $hOptions), 'forms delete 1st');
    ok (CompareXML('Forms_del.xml', $DeletePatchFile, $hOptions), 'forms delete 2nd');
    DeleteFile($DeletePatchFile) if ($DeleteFiles);

    $FilePatcher->patchXMLFiles($OldFile, $NewFile, $ImportPatchFile, 0);
    ok (CompareXML($ImportPatchFile, 'Forms_imp.xml', $hOptions), 'forms import 1st');
    ok (CompareXML('Forms_imp.xml', $ImportPatchFile, $hOptions), 'forms import 2nd');
    DeleteFile($ImportPatchFile) if ($DeleteFiles);

    #test 17 - PortalSitesCiao
    $OldFile = 'PortalSitesCiao_old.xml';
    $NewFile = 'PortalSitesCiao_new.xml';
    $ImportPatchFile = 'PortalSitesCiao_patch_import.xml';

    $FilePatcher->patchXMLFiles($OldFile, $NewFile, $ImportPatchFile, 0);
    ok (CompareXML($ImportPatchFile, 'PortalSitesCiao_imp.xml', $hOptions), 'PortalSitesCiao import 1st');
    ok (CompareXML('PortalSitesCiao_imp.xml', $ImportPatchFile, $hOptions), 'PortalSitesCiao import 2nd');
    DeleteFile($ImportPatchFile) if ($DeleteFiles);

    #test 18 - Distributors
    #$OldFile = 'Distributor_old.xml';
    #$NewFile = 'Distributor_new.xml';
    #$ImportPatchFile = 'Distributor_patch_import.xml';

    #$FilePatcher->patchXMLFiles($OldFile, $NewFile, $ImportPatchFile, 0);
    #ok (CompareXML($ImportPatchFile, 'Distributor_imp.xml', $hOptions), 'pathes in attribute value tags 1st');
    #ok (CompareXML('Distributor_imp.xml', $ImportPatchFile, $hOptions), 'pathes in attribute value tags 2nd');
    #DeleteFile($ImportPatchFile) if ($DeleteFiles);

    #test 19 - Permissions
    $OldFile = 'System_old.xml';
    $NewFile = 'System_new.xml';
    $ImportPatchFile = 'System_patch_import.xml';

    $FilePatcher->patchXMLFiles($OldFile, $NewFile, $ImportPatchFile, 0);
    ok (CompareXML($ImportPatchFile, 'System_imp.xml',  $hOptions), 'System test 1st');
    ok (CompareXML('System_imp.xml', $ImportPatchFile,  $hOptions), 'System test 2nd');
    DeleteFile($ImportPatchFile) if ($DeleteFiles);

    #test 20 - PaymentLogos
    $OldFile = 'PaymentLogos_old.xml';
    $NewFile = 'PaymentLogos_new.xml';
    $ImportPatchFile = 'PaymentLogos_patch_import.xml';

    $FilePatcher->patchXMLFiles($OldFile, $NewFile, $ImportPatchFile, 0);
    ok (CompareXML($ImportPatchFile, 'PaymentLogos_imp.xml',  $hOptions), 'System test 3rd');
    ok (CompareXML('PaymentLogos_imp.xml', $ImportPatchFile,  $hOptions), 'System test 4th');
    DeleteFile($ImportPatchFile) if ($DeleteFiles);

    #test 21 - translation attributes under System tag
    $OldFile = 'Translation.Notrans.System_old_en.xml';
    $NewFile = 'Translation.Notrans.System_new_en.xml';
    $ImportPatchFile = 'Translation.Notrans.System_patch_import_en.xml';

    $FilePatcher->patchXMLFiles($OldFile, $NewFile, $ImportPatchFile, 0);
    ok (CompareXML($ImportPatchFile, 'Translation.Notrans.System_new_en.xml', $hOptions), 'translation attributes under system object 1st');
    ok (CompareXML('Translation.Notrans.System_new_en.xml', $ImportPatchFile, $hOptions), 'translation attributes under system object 2nd');
    DeleteFile($ImportPatchFile) if ($DeleteFiles);

    #test 22 - StyleGroups
    $OldFile = 'StyleGroups_old.xml';
    $NewFile = 'StyleGroups_new.xml';
    $ImportPatchFile = 'StyleGroups_patch_import.xml';
    $DeletePatchFile = 'StyleGroups_patch_delete.xml';

    $FilePatcher->patchXMLFiles($OldFile, $NewFile, $ImportPatchFile, 0);
    ok (CompareXML($ImportPatchFile, 'StyleGroups_imp.xml', $hOptions), 'StyleGroups import 1st');
    ok (CompareXML('StyleGroups_imp.xml', $ImportPatchFile, $hOptions), 'StyleGroups import 2nd');
    DeleteFile($ImportPatchFile) if ($DeleteFiles);

    my $time2 = time;
    printf "need %.3f seconds\n", $time2-$time1;

}

sub usage {
    print <<END_USAGE;
Usage:
    perl $0 [options] [flags] datafiles

flags:
    -help          show the command line options
    -deletefiles   delete generated patch files

Example:
    perl $0 -deletefiles
END_USAGE
    exit 2;
}

RunScript( Message => 'long', Sub => \&Main);

1;
