#-*- Makefile -*-

require DE_EPAGES::Core::API::Makefile;
__DATA__
#
#   makefile
#
#   Project : epages 6
#   Copyright   : epages Software GmbH 2004
#   Version : Thu Nov 25 17:17:43 2004
#

!OS_SPECIFIC_DEFINES
DE_EPAGES   = .
COMPANY     = DE_EPAGES
STORE       = YourStore
SOURCEDOC   = $(MAKEDIR)\..\SourceDoc
SOURCEDOCDATE= Fri June 4 12:00:00 2010
EPAGES_VERSION= ePages 6.10
EPAGES_TAG  = release_6_10_0
SITE_NAME   = Site
STORE_TYPE  = Store
SITE_TYPE   = Site
BACKUP_SITE_NAME = Backup
PROVIDER_PATH = /Providers/Distributor
PROVIDER_USER = admin
PROVIDER_PASSWD = admin
DEMOSHOP_ALIAS = DemoShop

!ifndef WINDOWS
EP_WEB_GROUP  = $(shell echo $${EPAGES_WEBGROUP:-ep_web} | sed 's/:.*//')
!endif
CPAN_OSDIR    = $(EPAGES_CARTRIDGES)/CPAN/EPAGES/PerlLib/$(OSNAME)

!ifndef PATCH_DIR
PATCH_DIR   = $(EPAGES_SHARED)\Patches\$(EPAGES_TAG)
!endif
DATA_DIR    = $(EPAGES_SHARED)\var-$(EPAGES_TAG)
BACKUP_DIR  = $(DATA_DIR)\backup\$(HOSTNAME)
LOG_DIR     = $(DATA_DIR)\logs\$(HOSTNAME)
TGT         = $(DATA_DIR)\tgt\$(HOSTNAME)

IMPORT      = $(PERL) $(DE_EPAGES)/Object/Scripts/import.pl -storename=$(STORE)
DELETE      = $(PERL) $(DE_EPAGES)/Object/Scripts/delete.pl -storename=$(STORE)
DBDUMPS     = $(PERL) $(DE_EPAGES)/Database/Scripts/DatabaseDumps.pl
PATCHFILES  = $(PERL) $(DE_EPAGES)/Core/Scripts/PatchFiles.pl
PATCHDB     = $(PERL) $(DE_EPAGES)/Installer/Scripts/patchDatabase.pl
CREATEDB    = $(PERL) $(DE_EPAGES)/Database/Scripts/createDB.pl
DROPDB      = $(PERL) $(DE_EPAGES)/Database/Scripts/dropDB.pl
INSTALLDB   = $(PERL) $(DE_EPAGES)/ShopConfiguration/Scripts/installDatabase.pl
REGISTER    = $(PERL) $(DE_EPAGES)/ShopConfiguration/Scripts/registercartridges.pl
SYNC        = $(PERL) $(DE_EPAGES)/ShopConfiguration/Scripts/synccartridges.pl
DROPTABLES  = $(PERL) $(DE_EPAGES)/Database/Scripts/dropAllTables.pl
STOREREDIRECT= $(PERL) $(DE_EPAGES)/Database/Scripts/redirectStoreConnections.pl
DEL_INI_SECTION= $(PERL) $(DE_EPAGES)/Core/Scripts/DelIniSection.pl
INSTALL_CPAN= $(PERL) $(DE_EPAGES)/Installer/Scripts/installCPAN.pl
START_EPAGES= $(PERL) $(DE_EPAGES)/Installer/Scripts/startService.pl epages
STOP_EPAGES = $(PERL) $(DE_EPAGES)/Installer/Scripts/stopService.pl epages
START_CRON  = $(PERL) $(DE_EPAGES)/Installer/Scripts/startService.pl cron
STOP_CRON   = $(PERL) $(DE_EPAGES)/Installer/Scripts/stopService.pl cron

# device size in MByte
DATSIZE = 100
LOGSIZE = 50

makefile: Makefile.PL
	$(PERL) Makefile.PL
	@$(ECHONL) "Please restart '$(MAKE)' again, makefile was updated." && $(PERL) -e "exit 1;"

test_store: makefile
	$(PERL) -e "exit ('$(STORE)' eq 'YourStore' ? 1 : 0);"

stop: makefile
	$(STOP_EPAGES)

start: makefile
!ifdef WINDOWS
	$(START_EPAGES)
!else
	IGNORE_INI=1 $(START_EPAGES)
!endif

stop_cron: makefile
	$(STOP_CRON)

start_cron: makefile
	$(START_CRON)

!ifdef WINDOWS
########################## WINDOWS ###################################
doc: makefile
	$(PERL) $(DE_EPAGES)/SourceDoc/Scripts/DocDir.pl -source $(EPAGES_CARTRIDGES) -target $(SOURCEDOC) -title "$(EPAGES_VERSION)" -ext pm -date "$(SOURCEDOCDATE)" -chm $(EPAGES_CARTRIDGES)\DE_EPAGES
	-$(HHC) "$(SOURCEDOC)\Project.hhp"
	$(MOVE) "$(SOURCEDOC)\$(EPAGES_VERSION).chm" "$(SOURCEDOC)\epages6.chm"
	$(DEL)  $(SOURCEDOC)\Index.hhk
	$(DEL)  $(SOURCEDOC)\Project.hhp
	$(DEL)  $(SOURCEDOC)\TableOfContents.hhc
!endif
########################## END WINDOWS ###################################

restart: makefile stop start

purge_configuration:
	$(DEL) $(EPAGES_CONFIG)\*.ept
	$(DEL) $(EPAGES_CONFIG)\*\*.ept
	$(DEL) $(EPAGES_CONFIG)\*.initial
	$(DEL) $(EPAGES_CONFIG)\*\*.initial
	$(DEL) $(EPAGES_CONFIG)\*~
	$(DEL) $(EPAGES_CONFIG)\*\*~

clean: makefile
!ifdef WINDOWS
	del /Q /S .*#* *.pdb
	dir /S /B /A:D | grep "Generated$$" | xargs "$(RMDIR)"
	dir /S /B /A:-D | grep "makefile$$" | xargs "$(DEL)"
!endif

test: makefile test_store
	$(PERL) $(DE_EPAGES)/Installer/Scripts/runTests.pl -storename $(STORE)

delete_tables: test_store
	$(DROPTABLES) -storename $(STORE)

cleanfiles_global: makefile
	$(EMPTY_DIR) $(EPAGES_WEBROOT)\_MONITOR_
	$(EMPTY_DIR) $(EPAGES_STORES)\_MONITOR_
	$(EMPTY_DIR) $(EPAGES_SHARED)\Static
	$(DEL) $(EPAGES_LOG)\*

cleanfilesdb: makefile test_store
	$(RMDIR) $(EPAGES_WEBROOT)\$(STORE)
	$(RMDIR) $(EPAGES_STORES)\$(STORE)
	$(DEL) $(EPAGES_CONFIG)\Cartridges.d\$(STORE).conf

cleandb: makefile test_store
	$(PERL) $(DE_EPAGES)/WebInterfacePoolDB/Scripts/deleteStore.pl -storename $(STORE)
	$(MAKE) delete_tables STORE=$(STORE)

import_store: test_store
	@$(PERL) $(DE_EPAGES)/ShopConfiguration/Scripts/store.pl -sitename $(SITE_NAME) -action insert -storename $(STORE)

import_distributor:
	$(IMPORT) $(DE_EPAGES)/ShopConfiguration/Database/XML/Distributor.xml

import_shoptypes: makefile test_store
	$(IMPORT) -path $(PROVIDER_PATH) $(DE_EPAGES)/ShopConfiguration/Database/XML/DistributorShopTypes.xml

install_demoshop: makefile
	$(PERL) $(DE_EPAGES)/ShopConfiguration/Scripts/createShop.pl \
		-wsuser $(PROVIDER_PATH)/Users/$(PROVIDER_USER) -passwd $(PROVIDER_PASSWD) \
		-shoptype Demo -storename $(STORE) \
		-alias $(DEMOSHOP_ALIAS) -scriptname $(DEMOSHOP_ALIAS)
	$(PERL) $(DE_EPAGES)/ShopWizard/Scripts/runWizard.pl -storename $(STORE) -shop $(DEMOSHOP_ALIAS) -config $(EPAGES_CONFIG)/ShopWizard.conf -nooutput
	$(MAKE) permissions PERM_ARGS=-u SECTION=webroot

uninstall_demoshop: makefile
	$(PERL) $(DE_EPAGES)/ShopConfiguration/Scripts/deleteShop.pl -wsuser $(PROVIDER_PATH)/Users/$(PROVIDER_USER) -passwd $(PROVIDER_PASSWD) -alias $(DEMOSHOP_ALIAS)

install_diagnostics: makefile test_store
	cd $(DE_EPAGES)\Diagnostics && $(PERL) Makefile.PL
	cd $(DE_EPAGES)\Diagnostics && $(MAKE) install STORE=$(STORE)

install_store: makefile test_store
	$(INSTALLDB) -storename $(STORE) -sitename $(SITE_NAME)
	$(MAKE) synchronize STORE=$(STORE)
	$(MAKE) build_dojo STORE=$(STORE)
	$(MAKE) permissions PERM_ARGS=-u SECTION=webroot
	$(MAKE) permissions PERM_ARGS=-u SECTION=stores

synchronize: makefile test_store
!ifndef EPAGES_IGNORE_SITE
	$(SYNC) -storename $(STORE) -sitename $(SITE_NAME)
!endif

create_store_conf:
	$(PERL) $(DE_EPAGES)/ShopConfiguration/Scripts/createStoreConf.pl -storename $(STORE) -storetype $(STORE_TYPE) -backupname $(BACKUP_NAME)

new_store:
	$(MAKE) create_store_conf create_db import_store install_store STORE=$(STORE) STORE_TYPE=$(STORE_TYPE) BACKUP_NAME=$(BACKUP_NAME)

install_pooldb: makefile stop stop_cron
	$(DEL) $(EPAGES_CONFIG)\ASPool.db
	$(DEL) $(EPAGES_CONFIG)\Cartridges.d\ASPoolDB.conf
	$(INSTALLDB) -storename ASPoolDB
	$(MAKE) permissions start PERM_ARGS=-u

install_cpan: makefile
	$(TEST) $(CPAN_OSDIR) && $(MAKE) install_cpan_epages || $(ECHONL)

install_cpan_epages:
	$(INSTALL_CPAN) CPAN::EPAGES > $(EPAGES_LOG)\install_cpan.log
	$(MAKE) permissions PERM_ARGS=-u

install: makefile
!ifdef WINDOWS
	$(RMDIR) $(DE_EPAGES)\HSBC\bin\MSWin32\CVS
	$(RMDIR) $(DE_EPAGES)\Presentation\bin\MSWin32\CVS
!endif
	$(TEST) $(CPAN_OSDIR) && $(MAKE) stop install_cpan start || $(ECHONL)
!ifdef EPAGES_IGNORE_SITE
	$(MAKE) create_store_conf create_db install_store STORE=Store
!else
	$(MAKE) create_store_conf create_db install_store STORE=Site STORE_TYPE=Site DATSIZE=50 LOGSIZE=20
	$(MAKE) create_store_conf create_db import_store install_store STORE=Store
	$(MAKE) import_distributor STORE=Site
	$(MAKE) import_shoptypes STORE=Site
!endif
!ifndef EPAGES_IGNORE_DEMOSHOP
	$(MAKE) install_demoshop STORE=Store
!endif
	$(MAKE) start_cron

reinstall: makefile uninstall
	$(MAKE) install
!ifndef EPAGES_IGNORE_SITE
	$(MAKE) install_diagnostics STORE=Site
!endif
	$(MAKE) install_diagnostics STORE=Store

uninstall: makefile cleanfiles_global
	$(MAKE) install_pooldb
	$(DEL) $(EPAGES_CONFIG)\EbayCategories.db
!ifndef EPAGES_IGNORE_SITE
	$(MAKE) drop_db cleanfilesdb STORE=Site
!endif
	$(MAKE) drop_db cleanfilesdb STORE=Store

build_dojo:
!ifndef DOJOBUILD
	$(PERL) $(EPAGES_CARTRIDGES)/DE_EPAGES/Dojo/Scripts/build.pl -storename $(STORE)
!endif

reinstall_store: makefile test_store cleandb cleanfilesdb install_store

# create databases

create_db: makefile
	$(CREATEDB) -storename $(STORE) -datsize $(DATSIZE) -logsize $(LOGSIZE)

drop_db: makefile
	$(DROPDB) -storename $(STORE)

# patch

close_store:
	$(STOREREDIRECT) -storename $(STORE) -redirect StoreClosed.html

open_store:
	$(STOREREDIRECT) -storename $(STORE) -remove

patch_store: makefile test_store
	$(PATCHDB)   -nosync -storename $(STORE)
	$(INSTALLDB) -nosync -storename $(STORE)
	$(MAKE) build_dojo STORE=$(STORE)

update_latest: makefile
	cvs -q update -A -d -P
	$(MAKE) permissions SECTION=cartridges
	$(MAKE) restart

latest: makefile update_latest reinstall
renew: latest

update_stable: makefile
	cvs -q update -d -P -r stable
	$(MAKE) permissions SECTION=cartridges
	$(MAKE) restart

stable: makefile update_stable reinstall

# files (obsolete)

backup_local_files: etc\backup_dir Apache\backup_dir Sybase\backup_dir bin\backup_dir Config\backup_dir Monitor\backup_dir
copy_local_files:   etc\copy_dir Apache\copy_dir Sybase\copy_dir bin\copy_dir Config\copy_dir Monitor\copy_dir Patches\copy_dir
patch_local_files:  backup_local_files copy_local_files
restore_local_files: etc\restore_dir Apache\restore_dir Sybase\restore_dir bin\restore_dir Config\restore_dir Monitor\restore_dir

backup_dir:
!ifdef FORGETFULLY
	$(PATCHFILES) -differential -dir "$(DIR)" -bak "$(BACKUP_DIR)/$(NAME)" -new "$(PATCH_DIR)/$(NAME)"
!else
	$(PATCHFILES) -action $(TGT)\$(NAME)\backup -differential -dir "$(DIR)" -bak "$(BACKUP_DIR)/$(NAME)" -new "$(PATCH_DIR)/$(NAME)"
!endif

copy_dir:
!ifdef FORGETFULLY
	$(PATCHFILES) -dir "$(DIR)" -new "$(PATCH_DIR)/$(NAME)"
!else
	$(PATCHFILES) -action $(TGT)\$(NAME)\copy -dir "$(DIR)" -new "$(PATCH_DIR)/$(NAME)"
!endif

restore_dir:
	$(PATCHFILES) -action $(TGT)\$(NAME)\restore -dir "$(DIR)" -bak "$(BACKUP_DIR)/$(NAME)"

etc\backup_dir etc\copy_dir etc\restore_dir:
	$(MAKE) $(@F) DIR=/etc NAME="$(@D)"
Apache\backup_dir Apache\copy_dir Apache\restore_dir:
	$(MAKE) $(@F) DIR="$(HTTPD_ROOT)" NAME="$(@D)"
Sybase\backup_dir Sybase\copy_dir Sybase\restore_dir:
	$(MAKE) $(@F) DIR="$(SYBASE)" NAME="$(@D)"
bin\backup_dir bin\copy_dir bin\restore_dir:
	$(MAKE) $(@F) DIR="$(EPAGES)/bin" NAME="$(@D)"
Config\backup_dir Config\copy_dir Config\restore_dir:
	$(MAKE) $(@F) DIR="$(EPAGES_CONFIG)" NAME="Shared/$(@D)"
Monitor\backup_dir Monitor\copy_dir Monitor\restore_dir:
	$(MAKE) $(@F) DIR="$(EPAGES_SHARED)/Monitor" NAME="Shared/$(@D)"
Patches\copy_dir:
	$(MAKE) $(@F) DIR="$(EPAGES_SHARED)/Patches" NAME="Shared/$(@D)"

permissions:
!ifndef WINDOWS
	$(ETC_INITD) perm admin $(PERM_ARGS)
!ifdef SECTION
	$(ETC_INITD) perm $(SECTION) $(PERM_ARGS)
!endif
!endif
#
# END
#
