use strict;
use Test::Simple tests => 20;
use Getopt::Long;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::PerlTools qw( IsA );
use DE_EPAGES::Database::API::Connection qw (ConnectTestSite DisconnectSite);
use DE_EPAGES::Object::API::Factory qw (LoadRootObject LoadClassByAlias ExistsClassByAlias);
use DE_EPAGES::Core::API::PerlTools qw (GetCurrentDateTime GetLocalTimeZone);

my $ATTRIBUTE_PACKAGE = 'DE_EPAGES::Object::API::Attributes::IniConfigAttribute';
RunScript(Sub=>\&Main, Debug=>1);

$ATTRIBUTE_PACKAGE = 'DE_EPAGES::Object::API::Attributes::DefaultAttribute';
RunScript(Sub=>\&Main, Debug=>1);

sub Main {
    ConnectTestSite();
    # create test classes
    ok( Init(), "initialization " . $ATTRIBUTE_PACKAGE );
    #'FileName' =>'IniConfigAttribute.conf',
    my $Object = LoadRootObject->insertChild( 'C1', { Alias => 'C1O1' } );

    my $Date = GetCurrentDateTime();
    ok($Date->time_zone()->name(), "tz ".$Date->time_zone()->name() );

    $Object->set({'TestDate' => $Date->clone});

    my $CompareDate = $Object->get('TestDate');
    ok( IsA( $CompareDate, "DateTime" ), "attribute value is DateTime" );
    $CompareDate->set_time_zone( GetLocalTimeZone() ) if $CompareDate->time_zone()->is_floating;

    ok($CompareDate eq $Date, "date ". "${Date} - $CompareDate");
    ok($CompareDate->time_zone()->name() eq $Date->time_zone()->name(), "tz ".$Date->time_zone()->name().':'.$CompareDate->time_zone->name() );

    $Date->set_time_zone( GetLocalTimeZone() ) if $Date->time_zone()->is_floating;
    $Date->set_time_zone( 'America/Chicago' );

    $Object->set({'TestDate' => $Date->clone});
    $CompareDate = $Object->get('TestDate');

    $CompareDate->set_time_zone( GetLocalTimeZone() ) if $CompareDate->time_zone()->is_floating;
    $CompareDate->set_time_zone( 'America/Chicago' );

    ok($CompareDate eq $Date, "date ". "${Date} - $CompareDate");
    ok($CompareDate->time_zone()->name() eq $Date->time_zone()->name(), "tz ".$Date->time_zone()->name().':'.$CompareDate->time_zone->name() );

    $Object->set({
        'TestDate' => undef,
        'TestString' => undef,
    });
    ok( !defined $Object->get('TestDate'), "set DateTime attribute to undef" );
    ok( !defined $Object->get('TestString'), "set String attribute to undef" );

    ok( Cleanup(), 'Cleanup' );
    DisconnectSite();
}

sub Cleanup {
    foreach my $ClassName ( qw( C1 ) ) {
        next unless ExistsClassByAlias( $ClassName );
        LoadClassByAlias( $ClassName )->delete;
    }
    return 1;
}

sub Init {
    Cleanup();
    my $ClassObject = LoadClassByAlias( 'Object' );
    my $Class1 = $ClassObject->insertSubClass({ Alias => 'C1' });
    $Class1->insertAttribute({ Alias => 'TestDate', Type => 'DateTime', Package => $ATTRIBUTE_PACKAGE });
    $Class1->insertAttribute({ Alias => 'TestString', Type => 'String', Package => $ATTRIBUTE_PACKAGE });

    return 1;
}
