#!perl -w
use strict;
use Test::Simple tests => 126;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Object::API::SQLStructureImportHandler;
use DE_EPAGES::Core::API::Error qw ( GetError Error );

##no critic qw(ProhibitExcessComplexity)
sub Main {
    RunScript(Sub => sub {
        my $ImportHandler = DE_EPAGES::Object::API::SQLStructureImportHandler->new('Sybase');
        my $SQLStruct = $ImportHandler->getSQLStructure('addresscompany.sql');

        my $Error;

        #test table names
        ok($SQLStruct->existsTable('addresscompany'), "existsTable 1st");
        ok((not $SQLStruct->existsTable('testanystring')), "existsTable 2st");

        #test columns
        eval{$SQLStruct->existsTableColumn('anystring', 'addressid')};
        $Error = GetError();
        ok(($Error->code() eq 'TableNotExists'), "existsTableColumn 1st");
        ok($SQLStruct->existsTableColumn('addresscompany', 'addressid'), "existsTableColumn 2nd");
        ok($SQLStruct->existsTableColumn('addresscompany', 'vatid'), "existsTableColumn 3rd");
        ok($SQLStruct->existsTableColumn('addresscompany', 'bankcode'), "existsTableColumn 4th");
        ok($SQLStruct->existsTableColumn('addresscompany', 'bankname'), "existsTableColumn 5th");
        ok($SQLStruct->existsTableColumn('addresscompany', 'bankaccountno'), "existsTableColumn 6th");
        ok($SQLStruct->existsTableColumn('addresscompany', 'bankaccountholder'), "existsTableColumn 7th");
        ok($SQLStruct->existsTableColumn('addresscompany', 'url'), "existsTableColumn 8th");
        ok((not $SQLStruct->existsTableColumn('addresscompany', 'anystring')), "existsTableColumn 9th");

        eval{$SQLStruct->getTableColumnPosition('anystring', 'addressid')};
        $Error = GetError();
        ok((($Error->code() eq 'TableNotExists')), "getTableColumnPosition 1st");
        ok(($SQLStruct->getTableColumnPosition('addresscompany', 'addressid') eq '0'), "getTableColumnPosition 2nd");
        ok(($SQLStruct->getTableColumnPosition('addresscompany', 'vatid') eq '1'), "getTableColumnPosition 3rd");
        ok(($SQLStruct->getTableColumnPosition('addresscompany', 'bankcode') eq '2'), "getTableColumnPosition 4th");
        ok(($SQLStruct->getTableColumnPosition('addresscompany', 'bankname') eq '3'), "getTableColumnPosition 5th");
        ok(($SQLStruct->getTableColumnPosition('addresscompany', 'bankaccountno') eq '4'), "getTableColumnPosition 6th");
        ok(($SQLStruct->getTableColumnPosition('addresscompany', 'bankaccountholder') eq '5'), "getTableColumnPosition 7th");
        ok(($SQLStruct->getTableColumnPosition('addresscompany', 'url') eq '6'), "getTableColumnPosition 8th");
        eval{$SQLStruct->getTableColumnPosition('addresscompany', 'anystring')};
        $Error = GetError();
        ok(($Error->code() eq 'ColumnNotExists'), "getTableColumnPosition 9th");

        eval{$SQLStruct->getTableColumnDataType('anystring', 'addressid')};
        $Error = GetError();
        ok((($Error->code() eq 'TableNotExists')), "getTableColumnDataType 1st");
        ok(($SQLStruct->getTableColumnDataType('addresscompany', 'addressid') eq 'int'), "getTableColumnDataType 2nd");
        ok(($SQLStruct->getTableColumnDataType('addresscompany', 'vatid') eq 'varchar(20)'), "getTableColumnDataType 3rd");
        ok(($SQLStruct->getTableColumnDataType('addresscompany', 'bankcode') eq 'varchar(20)'), "getTableColumnDataType 4th");
        ok(($SQLStruct->getTableColumnDataType('addresscompany', 'bankname') eq 'nvarchar(255)'), "getTableColumnDataType 5th");
        ok(($SQLStruct->getTableColumnDataType('addresscompany', 'bankaccountno') eq 'varchar(20)'), "getTableColumnDataType 6th");
        ok(($SQLStruct->getTableColumnDataType('addresscompany', 'bankaccountholder') eq 'nvarchar(255)'), "getTableColumnDataType 7th");
        ok(($SQLStruct->getTableColumnDataType('addresscompany', 'url') eq 'varchar(255)'), "getTableColumnDataType 8th");
        eval{$SQLStruct->getTableColumnDataType('addresscompany', 'anystring')};
        $Error = GetError();
        ok(($Error->code() eq 'ColumnNotExists'), "getTableColumnDataType 9th");

        eval{$SQLStruct->getTableColumnNotNull('anystring', 'addressid')};
        $Error = GetError();
        ok((($Error->code() eq 'TableNotExists')), "getTableColumnNotNull 1st");
        ok(($SQLStruct->getTableColumnNotNull('addresscompany', 'addressid') eq '1'), "getTableColumnNotNull 2nd");
        ok(($SQLStruct->getTableColumnNotNull('addresscompany', 'vatid') eq '0'), "getTableColumnNotNull 3rd");
        ok(($SQLStruct->getTableColumnNotNull('addresscompany', 'bankcode') eq '0'), "getTableColumnNotNull 4th");
        ok(($SQLStruct->getTableColumnNotNull('addresscompany', 'bankname') eq '0'), "getTableColumnNotNull 5th");
        ok(($SQLStruct->getTableColumnNotNull('addresscompany', 'bankaccountno') eq '0'), "getTableColumnNotNull 6th");
        ok(($SQLStruct->getTableColumnNotNull('addresscompany', 'bankaccountholder') eq '0'), "getTableColumnNotNull 7th");
        ok(($SQLStruct->getTableColumnNotNull('addresscompany', 'url') eq '1'), "getTableColumnNotNull 8th");
        eval{$SQLStruct->getTableColumnNotNull('addresscompany', 'anystring')};
        $Error = GetError();
        ok(($Error->code() eq 'ColumnNotExists'), "getTableColumnNotNull 9th");

        eval{$SQLStruct->getTableColumnDefault('anystring', 'addressid')};
        $Error = GetError();
        ok((($Error->code() eq 'TableNotExists')), "getTableColumnDefault 1st");
        ok(($SQLStruct->getTableColumnDefault('addresscompany', 'addressid') eq '_SQLStruct-Default'), "getTableColumnDefault 2nd");
        ok(($SQLStruct->getTableColumnDefault('addresscompany', 'vatid') eq '_SQLStruct-Default'), "getTableColumnDefault 3rd");
        ok(($SQLStruct->getTableColumnDefault('addresscompany', 'bankcode') eq '_SQLStruct-Default'), "getTableColumnDefault 4th");
        ok(($SQLStruct->getTableColumnDefault('addresscompany', 'bankname') eq '_SQLStruct-Default'), "getTableColumnDefault 5th");
        ok(($SQLStruct->getTableColumnDefault('addresscompany', 'bankaccountno') eq '_SQLStruct-Default'), "getTableColumnDefault 6th");
        ok(($SQLStruct->getTableColumnDefault('addresscompany', 'bankaccountholder') eq '_SQLStruct-Default'), "getTableColumnDefault 7th");
        ok(($SQLStruct->getTableColumnDefault('addresscompany', 'url') eq 'www.test.de/test01'), "getTableColumnDefault 8th");
        eval{$SQLStruct->getTableColumnDefault('addresscompany', 'anystring')};
        $Error = GetError();
        ok(($Error->code() eq 'ColumnNotExists'), "getTableColumnDefault 9th");

        my $hInfo = {};
        eval{$SQLStruct->getTableColumnInfo('anystring', 'addressid')};
        $Error = GetError();
        ok(($Error->code() eq 'TableNotExists'), "getTableColumnInfo 1st");

        $hInfo = $SQLStruct->getTableColumnInfo('addresscompany', 'addressid');
        ok( (
                exists $hInfo->{'Position'} and
                exists $hInfo->{'Default'} and
                exists $hInfo->{'DataType'} and
                exists $hInfo->{'NotNull'} and
                $hInfo->{'Position'} eq '0' and
                $hInfo->{'Default'} eq '_SQLStruct-Default' and
                $hInfo->{'NotNull'} eq '1' and
                $hInfo->{'DataType'} eq 'int' and
                scalar keys %$hInfo eq '4'
            ),
            "getTableColumnInfo 2nd"
        );
        $hInfo = $SQLStruct->getTableColumnInfo('addresscompany', 'vatid');
        ok( (
                exists $hInfo->{'Position'} and
                exists $hInfo->{'Default'} and
                exists $hInfo->{'DataType'} and
                exists $hInfo->{'NotNull'} and
                $hInfo->{'Position'} eq '1' and
                $hInfo->{'Default'} eq '_SQLStruct-Default' and
                $hInfo->{'NotNull'} eq '0' and
                $hInfo->{'DataType'} eq 'varchar(20)' and
                scalar keys %$hInfo eq '4'
            ),
            "getTableColumnInfo 3rd"
        );
        $hInfo = $SQLStruct->getTableColumnInfo('addresscompany', 'bankcode');
        ok( (
                exists $hInfo->{'Position'} and
                exists $hInfo->{'Default'} and
                exists $hInfo->{'DataType'} and
                exists $hInfo->{'NotNull'} and
                $hInfo->{'Position'} eq '2' and
                $hInfo->{'Default'} eq '_SQLStruct-Default' and
                $hInfo->{'NotNull'} eq '0' and
                $hInfo->{'DataType'} eq 'varchar(20)' and
                scalar keys %$hInfo eq '4'
            ),
            "getTableColumnInfo 4th"
        );
        $hInfo = $SQLStruct->getTableColumnInfo('addresscompany', 'bankname');
        ok( (
                exists $hInfo->{'Position'} and
                exists $hInfo->{'Default'} and
                exists $hInfo->{'DataType'} and
                exists $hInfo->{'NotNull'} and
                $hInfo->{'Position'} eq '3' and
                $hInfo->{'Default'} eq '_SQLStruct-Default' and
                $hInfo->{'NotNull'} eq '0' and
                $hInfo->{'DataType'} eq 'nvarchar(255)' and
                scalar keys %$hInfo eq '4'
            ),
            "getTableColumnInfo 5th"
        );
        $hInfo = $SQLStruct->getTableColumnInfo('addresscompany', 'bankaccountno');
        ok( (
                exists $hInfo->{'Position'} and
                exists $hInfo->{'Default'} and
                exists $hInfo->{'DataType'} and
                exists $hInfo->{'NotNull'} and
                $hInfo->{'Position'} eq '4' and
                $hInfo->{'Default'} eq '_SQLStruct-Default' and
                $hInfo->{'NotNull'} eq '0' and
                $hInfo->{'DataType'} eq 'varchar(20)' and
                scalar keys %$hInfo eq '4'
            ),
            "getTableColumnInfo 6th"
        );
        $hInfo = $SQLStruct->getTableColumnInfo('addresscompany', 'bankaccountholder');
        ok( (
                exists $hInfo->{'Position'} and
                exists $hInfo->{'Default'} and
                exists $hInfo->{'DataType'} and
                exists $hInfo->{'NotNull'} and
                $hInfo->{'Position'} eq '5' and
                $hInfo->{'Default'} eq '_SQLStruct-Default' and
                $hInfo->{'NotNull'} eq '0' and
                $hInfo->{'DataType'} eq 'nvarchar(255)' and
                scalar keys %$hInfo eq '4'
            ),
            "getTableColumnInfo 7th"
        );
        $hInfo = $SQLStruct->getTableColumnInfo('addresscompany', 'url');
        ok( (
                exists $hInfo->{'Position'} and
                exists $hInfo->{'Default'} and
                exists $hInfo->{'DataType'} and
                exists $hInfo->{'NotNull'} and
                $hInfo->{'Position'} eq '6' and
                $hInfo->{'Default'} eq 'www.test.de/test01' and
                $hInfo->{'NotNull'} eq '1' and
                $hInfo->{'DataType'} eq 'varchar(255)' and
                scalar keys %$hInfo eq '4'
            ),
            "getTableColumnInfo 8th"
        );
        eval{$SQLStruct->getTableColumnInfo('addresscompany', 'anystring')};
        $Error = GetError();
        ok(($Error->code() eq 'ColumnNotExists'), "getTableColumnInfo 9th");

        #test primary key
        ok($SQLStruct->existsPrimaryKey('addresscompany', 'pk_address_company'), "existsPrimaryKey 1st");
        ok(($SQLStruct->getPrimaryKeyName('addresscompany') eq 'pk_address_company'), "getPrimaryKeyName 1st");

        my @Columns = @{$SQLStruct->getPrimaryKeyColumns('addresscompany')};
        my $hControl = {};
        foreach my $Column (@Columns){
            $hControl->{$Column} = '1';
        }
        ok(($hControl->{'addressid'} eq '1'), "getPrimaryKeyColumns 1st");
        ok((not $hControl->{'anystring'}), "getPrimaryKeyColumns 2nd");

        my $hData = $SQLStruct->getPrimaryKeyInfo('addresscompany');

        ok(($hData->{'SortOrder'} eq '_SQLStruct-Default' ),"getPrimaryKeyData 1st");
        ok(($hData->{'Clustered'} eq '_SQLStruct-Default' ),"getPrimaryKeyData 2nd");
        ok(($hData->{'Name'} eq 'pk_address_company' ),"getPrimaryKeyData 3rd");

        #test foreign key
        eval{$SQLStruct->existsTableForeignKey('anystring','fk_addresscompany_address')};
        $Error = GetError();
        ok(($Error->code() eq 'TableNotExists'), "existsTableForeignKey 1st");
        ok((not $SQLStruct->existsTableForeignKey('addresscompany','anystring')), "existsTableForeignKey 2nd");
        ok($SQLStruct->existsTableForeignKey('addresscompany','fk_addresscompany_address'), "existsTableForeignKey 3rd");

        ok(($SQLStruct->getTableForeignKeyRefTable('addresscompany','fk_addresscompany_address') eq 'address'), "getTableForeignKeyRefTable 1st");
        eval{$SQLStruct->getTableForeignKeyRefTable('anystring','fk_addresscompany_address')};
        $Error = GetError();
        ok(($Error->code() eq 'TableNotExists'), "getTableForeignKeyRefTable 2nd");
        eval{$SQLStruct->getTableForeignKeyRefTable('addresscompany','anystring')};
        $Error = GetError();
        ok(($Error->code() eq 'ForeignKeyNotExists'), "getTableForeignKeyRefTable 3rd");

        my $hColumnPairs = $SQLStruct->getTableForeignKeyColumnPairs('addresscompany','fk_addresscompany_address');

        ok((exists $hColumnPairs->{'addressid'} and $hColumnPairs->{'addressid'} eq 'addressid' and (scalar keys %$hColumnPairs eq '1')), "getTableForeignKeyColumnPairs 1st");
        eval{$SQLStruct->getTableForeignKeyColumnPairs('anystring','fk_addresscompany_address')};
        $Error = GetError();
        ok(($Error->code() eq 'TableNotExists'), "getTableForeignKeyColumnPairs 2nd");
        eval{$SQLStruct->getTableForeignKeyColumnPairs('addresscompany','anystring')};
        $Error = GetError();
        ok(($Error->code() eq 'ForeignKeyNotExists'), "getTableForeignKeyColumnPairs 3rd");

        my $SQLStruct2 = $ImportHandler->getSQLStructure('amazonoffer.sql');

        #test foreign keys
        my $aFKeys = $SQLStruct2->getTableForeignKeys('amazonoffer');

        ok(
            (
                $aFKeys->[0] eq 'fk_amazonoffer_amazoncatalog' and
                $aFKeys->[1] eq 'fk_amazonoffer_amazonoffer' and
                $aFKeys->[2] eq 'fk_amazonoffer_amazonproductty' and
                $aFKeys->[3] eq 'fk_amazonoffer_currency' and
                $aFKeys->[4] eq 'fk_amazonoffer_object' and
                $aFKeys->[5] eq 'fk_amazonoffer_portalconfig' and
                $aFKeys->[6] eq 'fk_amazonoffer_product' and
                scalar @$aFKeys eq '7'
            ),
            "getTableForeignKeys 1st"
        );
        eval{$SQLStruct2->getTableForeignKeys('anystring')};
        $Error = GetError();
        ok(($Error->code() eq 'TableNotExists'), "getTableColumns 2nd");

        #test index
        eval{$SQLStruct2->existsTableIndex('anystring','u_portalconfigid_productid')};
        $Error = GetError();
        ok(($Error->code() eq 'TableNotExists'), "existsTableIndex 1st");
        ok((not $SQLStruct2->existsTableIndex('amazonoffer','anystring')), "existsTableIndex 2nd");
        ok((not $SQLStruct->existsTableIndex('addresscompany','u_portalconfigid_productid')), "existsTableIndex 3rd");
        ok($SQLStruct2->existsTableIndex('amazonoffer','u_portalconfigid_productid'), "existsTableIndex 4th");
        ok($SQLStruct2->existsTableIndex('amazonoffer','i_productid'), "existsTableIndex 5th");
        ok($SQLStruct2->existsTableIndex('amazonoffer','i_portalconfigid_status'), "existsTableIndex 6th");
        ok($SQLStruct2->existsTableIndex('amazonoffer','i_parentofferid'), "existsTableIndex 7th");

        my @TableIndexColumns = @{$SQLStruct2->getTableIndexColumns('amazonoffer','u_portalconfigid_productid')};
        $hControl = {};
        foreach my $Column (@TableIndexColumns){
            $hControl->{$Column} = '1';
        }
        ok(
            (
                $hControl->{'portalconfigid'} eq '1' and
                $hControl->{'productid'} and
                scalar keys %$hControl eq '2'
            ),
            "getTableIndexRefColumns 1st"
        );

        @TableIndexColumns = @{$SQLStruct2->getTableIndexColumns('amazonoffer','i_productid')};
        $hControl = {};
        foreach my $Column (@TableIndexColumns){
            $hControl->{$Column} = '1';
        }
        ok(
            (
                $hControl->{'productid'} and
                scalar keys %$hControl eq '1'
            ),
            "getTableIndexRefColumns 2nd"
        );

        my $SQLStruct3 = $ImportHandler->getSQLStructure('couponcondition.sql');

        @TableIndexColumns = @{$SQLStruct3->getTableIndexColumns('couponcondition','i_coupcond_campaign')};
        $hControl = {};
        foreach my $Column (@TableIndexColumns){
            $hControl->{$Column} = '1';
        }
        ok(
            (
                $hControl->{'couponcampaignid'} and
                scalar keys %$hControl eq '1'
            ),
            "getTableIndexRefColumns 3rd"
        );

        ok(($SQLStruct2->existsTableIndexColumn('amazonoffer','u_portalconfigid_productid', 'portalconfigid')), "existsTableIndexColumn 1st");
        ok(($SQLStruct2->existsTableIndexColumn('amazonoffer','u_portalconfigid_productid', 'productid')), "existsTableIndexColumn 2nd");
        ok(($SQLStruct2->existsTableIndexColumn('amazonoffer','i_productid', 'productid')), "existsTableIndexColumn 3rd");
        ok(($SQLStruct2->existsTableIndexColumn('amazonoffer','i_portalconfigid_status', 'portalconfigid')), "existsTableIndexColumn 4th");
        ok(($SQLStruct2->existsTableIndexColumn('amazonoffer','i_portalconfigid_status', 'onlinestatus')), "existsTableIndexColumn 5th");
        ok(($SQLStruct2->existsTableIndexColumn('amazonoffer','i_parentofferid', 'parentofferid')), "existsTableIndexColumn 6th");
        ok(($SQLStruct3->existsTableIndexColumn('couponcondition','i_coupcond_campaign', 'couponcampaignid')), "existsTableIndexColumn 7th");
        ok(($SQLStruct3->existsTableIndexColumn('couponcondition','i_coupcond_object', 'conditionobjectid')), "existsTableIndexColumn 8th");
        eval{$SQLStruct3->existsTableIndexColumn('anystring','i_coupcond_object', 'conditionobjectid')};
        $Error = GetError();
        ok(($Error->code() eq 'TableNotExists'), "existsTableIndexColumn 9th");
        eval{$SQLStruct3->existsTableIndexColumn('couponcondition','anystring', 'conditionobjectid')};
        $Error = GetError();
        ok(($Error->code() eq 'IndexNotExists'), "existsTableIndexColumn 10th");
        ok((not $SQLStruct3->existsTableIndexColumn('couponcondition','i_coupcond_object', 'anystring')), "existsTableIndexColumn 11th");

        ok(($SQLStruct3->getTableIndexColumnSortOrder('couponcondition', 'i_coupcond_campaign','couponcampaignid') eq 'DESC'), "getTableIndexColumnSortOrder 1st");
        ok(($SQLStruct3->getTableIndexColumnSortOrder('couponcondition', 'i_coupcond_object','conditionobjectid') eq 'ASC'), "getTableIndexColumnSortOrder 2nd");
        ok(($SQLStruct2->getTableIndexColumnSortOrder('amazonoffer', 'u_portalconfigid_productid','portalconfigid') eq '_SQLStruct-Default'), "getTableIndexColumnSortOrder 3rd");
        eval{$SQLStruct3->getTableIndexColumnSortOrder('anystring', 'i_coupcond_object','conditionobjectid')};
        $Error = GetError();
        ok(($Error->code() eq 'TableNotExists'), "getTableIndexColumnSortOrder 4th");
        eval{$SQLStruct3->getTableIndexColumnSortOrder('couponcondition', 'anystring','conditionobjectid')};
        $Error = GetError();
        ok(($Error->code() eq 'IndexNotExists'), "getTableIndexColumnSortOrder 5th");
        eval{$SQLStruct3->getTableIndexColumnSortOrder('couponcondition', 'i_coupcond_object','anystring')};
        $Error = GetError();
        ok(($Error->code() eq 'IndexColumnNotExists'), "getTableIndexColumnSortOrder 6th");

        my $hColumnSortOrderPairs = $SQLStruct3->getTableIndexColumnSortOrderPairs('couponcondition','i_coupcond_campaign');
        ok(
            (
                exists $hColumnSortOrderPairs->{'couponcampaignid'} and
                $hColumnSortOrderPairs->{'couponcampaignid'} eq 'DESC' and
                scalar keys %$hColumnSortOrderPairs eq '1'
            ),
            "getTableIndexColumnSortOrderPairs 1st"
        );

        $hColumnSortOrderPairs = $SQLStruct3->getTableIndexColumnSortOrderPairs('couponcondition','i_coupcond_object');
        ok(
            (
                exists $hColumnSortOrderPairs->{'conditionobjectid'} and
                $hColumnSortOrderPairs->{'conditionobjectid'} eq 'ASC' and
                scalar keys %$hColumnSortOrderPairs eq '1'
            ),
            "getTableIndexColumnSortOrderPairs 2nd"
        );

        $hColumnSortOrderPairs = $SQLStruct2->getTableIndexColumnSortOrderPairs('amazonoffer','u_portalconfigid_productid');
        ok(
            (
                exists $hColumnSortOrderPairs->{'portalconfigid'} and
                exists $hColumnSortOrderPairs->{'productid'} and
                $hColumnSortOrderPairs->{'portalconfigid'} eq '_SQLStruct-Default' and
                $hColumnSortOrderPairs->{'productid'} eq 'ASC' and
                scalar keys %$hColumnSortOrderPairs eq '2'
            ),
            "getTableIndexColumnSortOrderPairs 3rd"
        );
        eval{$SQLStruct3->getTableIndexColumnSortOrderPairs('anystring','i_coupcond_object')};
        $Error = GetError();
        ok(($Error->code() eq 'TableNotExists'), "getTableIndexColumnSortOrderPairs 4th");
        eval{$SQLStruct3->getTableIndexColumnSortOrderPairs('couponcondition','anystring')};
        $Error = GetError();
        ok(($Error->code() eq 'IndexNotExists'), "getTableIndexColumnSortOrderPairs 5th");

        ok((not $SQLStruct3->getTableIndexUnique('couponcondition', 'i_coupcond_campaign')), "getTableIndexUnique 1st");
        ok(($SQLStruct2->getTableIndexUnique('amazonoffer', 'u_portalconfigid_productid')), "getTableIndexUnique 2nd");
        eval{$SQLStruct2->getTableIndexUnique('anystring', 'u_portalconfigid_productid')};
        $Error = GetError();
        ok(($Error->code() eq 'TableNotExists'), "getTableIndexUnique 3rd");
        eval{$SQLStruct2->getTableIndexUnique('amazonoffer', 'anystring')};
        $Error = GetError();
        ok(($Error->code() eq 'IndexNotExists'), "getTableIndexUnique 4th");

        ok(($SQLStruct3->getTableIndexClustered('couponcondition', 'i_coupcond_campaign') eq 'CLUSTERED'), "getTableIndexClustered 1st");
        ok(($SQLStruct3->getTableIndexClustered('couponcondition', 'i_coupcond_object') eq 'NONCLUSTERED'), "getTableIndexClustered 2nd");
        ok(($SQLStruct2->getTableIndexClustered('amazonoffer', 'u_portalconfigid_productid') eq '_SQLStruct-Default'), "getTableIndexClustered 3rd");
        eval{$SQLStruct2->getTableIndexClustered('anystring', 'u_portalconfigid_productid')};
        $Error = GetError();
        ok(($Error->code() eq 'TableNotExists'), "getTableIndexClustered 4th");
        eval{$SQLStruct2->getTableIndexClustered('amazonoffer', 'anystring')};
        $Error = GetError();
        ok(($Error->code() eq 'IndexNotExists'), "getTableIndexClustered 5th");

        $hInfo = $SQLStruct3->getTableIndexInfo('couponcondition', 'i_coupcond_campaign');
        ok(
            (
                exists $hInfo->{'Clustered'} and
                exists $hInfo->{'Unique'} and
                exists $hInfo->{'ColumnSortOrderPairs'} and
                exists $hInfo->{'ColumnSortOrderPairs'}->{'couponcampaignid'} and
                $hInfo->{'Clustered'} eq 'CLUSTERED' and
                $hInfo->{'Unique'} eq '0' and
                $hInfo->{'ColumnSortOrderPairs'}->{'couponcampaignid'} eq 'DESC' and
                scalar keys %$hInfo eq '3' and
                scalar keys %{$hInfo->{'ColumnSortOrderPairs'}} eq '1'
            ),
            "getTableIndexInfo 1st"
        );

        $hInfo = $SQLStruct3->getTableIndexInfo('couponcondition', 'i_coupcond_object');
        ok(
            (
                exists $hInfo->{'Clustered'} and
                exists $hInfo->{'Unique'} and
                exists $hInfo->{'ColumnSortOrderPairs'} and
                exists $hInfo->{'ColumnSortOrderPairs'}->{'conditionobjectid'} and
                $hInfo->{'Clustered'} eq 'NONCLUSTERED' and
                $hInfo->{'Unique'} eq '0' and
                $hInfo->{'ColumnSortOrderPairs'}->{'conditionobjectid'} eq 'ASC' and
                scalar keys %$hInfo eq '3' and
                scalar keys %{$hInfo->{'ColumnSortOrderPairs'}} eq '1'
            ),
            "getTableIndexInfo 2nd"
        );

        $hInfo = $SQLStruct2->getTableIndexInfo('amazonoffer', 'u_portalconfigid_productid');
        ok(
            (
                exists $hInfo->{'Clustered'} and
                exists $hInfo->{'Unique'} and
                exists $hInfo->{'ColumnSortOrderPairs'} and
                exists $hInfo->{'ColumnSortOrderPairs'}->{'portalconfigid'} and
                exists $hInfo->{'ColumnSortOrderPairs'}->{'productid'} and
                $hInfo->{'Clustered'} eq '_SQLStruct-Default' and
                $hInfo->{'Unique'} eq '1' and
                $hInfo->{'ColumnSortOrderPairs'}->{'portalconfigid'} eq '_SQLStruct-Default' and
                $hInfo->{'ColumnSortOrderPairs'}->{'productid'} eq 'ASC' and
                scalar keys %$hInfo eq '3' and
                scalar keys %{$hInfo->{'ColumnSortOrderPairs'}} eq '2'
            ),
            "getTableIndexInfo 3rd"
        );
        eval{$SQLStruct2->getTableIndexInfo('anystring', 'u_portalconfigid_productid')};
        $Error = GetError();
        ok(($Error->code() eq 'TableNotExists'), "getTableIndexInfo 4th");
        eval{$SQLStruct2->getTableIndexInfo('amazonoffer', 'anystring')};
        $Error = GetError();
        ok(($Error->code() eq 'IndexNotExists'), "getTableIndexInfo 5th");

        my $SQLStruct4 = $ImportHandler->getSQLStructure('test.sql');
        my @Tables = @{$SQLStruct4->getTables()};
        $hControl = {};
        foreach my $Table (@Tables){
            $hControl->{$Table} = '1';
        }
        ok(
            (
                exists $hControl->{'test01'} and
                exists $hControl->{'test02'} and
                exists $hControl->{'test03'} and
                scalar @Tables eq '3'
            ),
            "getTables 1st"
        );

        my $aColumns = $SQLStruct3->getTableColumns('couponcondition');

        ok(
            (
                $aColumns->[0] eq 'couponconditionid' and
                $aColumns->[1] eq 'couponcampaignid' and
                $aColumns->[2] eq 'conditionobjectid' and
                $aColumns->[3] eq 'type' and
                scalar @$aColumns eq '4'
            ),
            "getTableColumns 1st"
        );

        $aColumns = $SQLStruct2->getTableColumns('amazonoffer');

        ok(
            (
                $aColumns->[0] eq 'amazonofferid' and
                $aColumns->[1] eq 'productid' and
                $aColumns->[2] eq 'currencyid' and
                $aColumns->[3] eq 'amazoncatalogid' and
                $aColumns->[4] eq 'amazonproducttypeid' and
                $aColumns->[5] eq 'portalconfigid' and
                $aColumns->[6] eq 'amazonprice' and
                $aColumns->[7] eq 'laststocklevel' and
                $aColumns->[8] eq 'laststockupdate' and
                $aColumns->[9] eq 'onlinestatus' and
                $aColumns->[10] eq 'lastbestpriceupdate' and
                $aColumns->[11] eq 'bestprice' and
                $aColumns->[12] eq 'ispricechanged' and
                $aColumns->[13] eq 'parentofferid' and
                $aColumns->[14] eq 'asin' and
                scalar @$aColumns eq '15'
            ),
            "getTableColumns 2nd"
        );

        eval{$SQLStruct2->getTableColumns('anystring')};
        $Error = GetError();
        ok(($Error->code() eq 'TableNotExists'), "getTableColumns 3rd");

        #test indices
        my $aIndices = $SQLStruct2->getTableIndices('amazonoffer');
        $hControl = {};
        foreach my $Index (@$aIndices){
            $hControl->{$Index} = '1';
        }

        ok(
            (
                exists $hControl->{'u_portalconfigid_productid'} and
                exists $hControl->{'i_productid'} and
                exists $hControl->{'i_portalconfigid_status'} and
                exists $hControl->{'i_parentofferid'} and
                scalar @$aIndices eq '4'
            ),
            "getTableIndices 1st"
        );
        eval{$SQLStruct2->getTableColumns('anystring')};
        $Error = GetError();
        ok(($Error->code() eq 'TableNotExists'), "getTableIndices 2nd");


    }, Debug=>1);
}

Main();

1;