/*
	Copyright (c) 2006-2007, epages Software GmbH
	All Rights Reserved.

	epages.cartridges.de_epages.presentation.widget.searchbar $Revision: 1.10.4.4.2.1 $

*/

dojo.provide("epages.cartridges.de_epages.presentation.SearchBar");
dojo.require("epages.storage");
dojo.require("epages.event");
dojo.require("dojo.fx");
dojo.require("dojox.fx");
dojo.require("dojo.fx.easing");

epages.cartridges.de_epages.presentation.SearchBar = {

	_ELEMENT_DOJOTYPE: " dojoType='epages.widget.FormElement' ",				// FormElement widget dojotype attribute
	_SEARCH_INPUT_PREFIX: "searchInput_",																// prefix on which all inputs ids are extended on
	_SEARCHBAR_ANI_DURATION: 350,																				// default animation duration / delay

	_STORAGE_ID: "Presentation::SearchBar.Status",	// namespace of this module inside of epages.Storage
	_STORAGE_EXTENDED_KEYWORD: "max",															// value for the extended SearchBar stored in epages.Storage
	_STORAGE_MINIMIZED_KEYWORD: "min",														// value for the minimized SearchBar stored in epages.Storage

	searchBarContainer: null,																						// parent node of the SearchBar
	searchBarHTMLArray: [],																							// collection of html snipplets which is parsed and merged on init

	searchBarId: null,																									// id of the SearchBar

	searchBarNodes: {																										// collection of important node references for easy access
		searchBarPlaceHolder: null,
		searchBar:null,

		wildSearch:null,
		wildSearchInput:null,
		wildSearchButton:null,
		extender:null,

		detailedSearch:null,
		detailedSearchPrimary:null,
		detailedSearchSecondary:null,

		searchBarButton:null,
		minimizer:null,

		widthContainer:null
	},

	_searchInputCounter: 0,																							// length of all inputs inside of a SearchBar
	_searchBarFormAction: "",																						// merged action of the SearchBar <form>

	_forcedDetailedSearch: false,																				// is there a wild search on this page?
	_isExtended: false,																									// is the detailed search form about to be expanded?
	_isExtendedToEnd: false,																						// is the detailed serach form expanded?
	_allowHardRecalc: true,																							// is it allowed to update all dimension properties of the SearchBar?

	_detailedSearchBarHeight: 0,																				// height of the detailed search form in px

	_mainDetailInputNode: null,																					// main input of the detail search

	_secondaryFieldsHighlightBeacon: null,															// timer reference for the opacity animation delay of the secondary search fields

	_getStoredSearchBarStatus: function() {
		var hash = epages.localStorage.getItem(this._STORAGE_ID) || {};
		return hash[this.searchBarId] || null;
	},
	_setStoredSearchBarStatus: function(keyword /* String */) {
		var hash = epages.localStorage.getItem(this._STORAGE_ID) || {};
			hash[this.searchBarId] = keyword;
			
		epages.localStorage.setItem(this._STORAGE_ID, hash);
	},

	createSearchBar: function(searchBarId /* String */, extendAtStartup /* Boolean */, action /* Object */, generalTranslations /* Object */, fields /* Object */) {
		// summary:
		//		Creates a new SearchBar instance
		// description:
		//		Calling this method will cause an injection of a new SearchBar node in the page at the point where the script is declared.
		// searchBarId:
		//		A unique ID for the SearchBar.
		// extendAtStartup:
		//		States whether the detailed search or the wild search form is visible on startup.
		// action:
		//		ViewAction and ObjectID for the SearchBar form.
		// generalTranslations:
		//		Collection of page specific translation string for the SearchBar.
		// fields:
		//		Collection of input fields.
		// example:
		//	// Have a look at "MBO*.Toolbar.html"

		// no need to proceed if there is no id given
		if(!searchBarId) {
			return;
		}

		this.searchBarId = searchBarId;

		if(!generalTranslations.wildSearchLabel) {
			// no label > no wild search
			this._forcedDetailedSearch = extendAtStartup = this._isExtendedToEnd = true;
		}else if(!fields.primary.length) {
			// no primary fields > no detailed search
			this._forcedDetailedSearch = extendAtStartup = this._isExtendedToEnd = false;
		}

		this._isExtended = extendAtStartup;

		// merge the form action
		this._searchBarFormAction = "?ViewAction=" + action.viewAction + "&ObjectID=" + action.objectId;

		// inject the container nodes
		document.write("<div id='SearchBar' class='SearchBar HideElementSoft'></div><div id='SearchBarPlaceHolder' class='SearchBarPlaceHolder'></div>");
		this.searchBarContainer = $("SearchBar");

		// convert the object to a html collection
		var formString =			"<form action='" + this._searchBarFormAction + "' method='post' id='WildSearch' class='WildSearch'><div>" +
														"<div id='Extender' class='" + (fields.primary.length ? "" : "HideElementSoft ") + "Extender EnableHover' title='" + generalTranslations.extendButtonTitle + "'></div>" +
														"<label unselectable='unselectable' for='WildSearchInput'>" + generalTranslations.wildSearchLabel+ "</label>";
		if(generalTranslations.wildSearchCustomInput) {
			var inp = generalTranslations.wildSearchCustomInput;
			formString +=						"<input value='" + (generalTranslations.wildSearchString || "") + "' type='" + inp.type + "' name='WildSearchInput_JSONTagEditor' inputName='"+ inp.inputName +"' id='WildSearchInput' class='WildSearchInput' dojoType='"+inp.dojotype+"' inputStyle='"+inp.inputStyle+"' />";
		} else {
			formString += 					"<input value='" + (generalTranslations.wildSearchString || "") + "' " + this._ELEMENT_DOJOTYPE + " type='text' name='WildSearchString' id='WildSearchInput' class='WildSearchInput'/>";
		}
		formString += 					"<i id='WildSearchButton' class='SearchBarMiniButton Sprite dialog_ico_s_loupe CursorPointer' title='" + generalTranslations.searchButtonTitle + "'></i>" +
														"<input type='submit' class='HideElementSoft'/>";
		this.searchBarHTMLArray.push(formString);
		for( var i=0,iLength=fields.hidden.length ; i<iLength ; i++ ) {
			this.searchBarHTMLArray.push("<input type='hidden' name='" + fields.hidden[i].name + "' value='" + fields.hidden[i].value + "'/>");
		}

		this.searchBarHTMLArray.push("</div></form>");

		this.searchBarHTMLArray.push("<form action='" + this._searchBarFormAction + "' method='post' id='DetailedSearch' class='DetailedSearch" + (extendAtStartup ? "" : " HideElementSoft") + "'><div>");

		for( var i=0,iLength=fields.hidden.length ; i<iLength ; i++ ) {
			this.searchBarHTMLArray.push("<input type='hidden' name='" + fields.hidden[i].name + "' value='" + fields.hidden[i].value + "'/>");
		}

		if(epages.Browser.engine != "MSIE") {
			this.searchBarHTMLArray.push("<div id='DetailedSearchWidthContainer' class='WidthContainer'>");

			var fieldsPrimary = this._getFieldCount(fields.primary);
			var fieldsSecondary = this._getFieldCount(fields.secondary);

			var widthLimitingFields = (fieldsPrimary.length >= fieldsSecondary.length) ? fieldsPrimary : fieldsSecondary;
				for( var i=0,iLength=widthLimitingFields.length ; i<iLength ; i++ ) {
					if(widthLimitingFields[i].type == "startgroup" || widthLimitingFields[i].type == "multioptions") {
						this.searchBarHTMLArray.push("<div class='PlaceHolderGroup'><div class='PlaceHolder'></div><div class='PlaceHolder'></div></div>");
					}else {
						this.searchBarHTMLArray.push("<div class='PlaceHolder'></div>");
					}
				}
			this.searchBarHTMLArray.push("</div>");
		}

		this.searchBarHTMLArray.push("<div id='DetailedSearchPrimary' class='Primary'>");
		this._buildFormElements(fields.primary);

		this.searchBarHTMLArray.push("</div><div id='DetailedSearchSecondary' class='Secondary'>");

		if(fields.secondary.length) {
			this._buildFormElements(fields.secondary);
		}

		this.searchBarHTMLArray.push("</div><div id='Minimizer' class='Minimizer EnableHover " + (this._forcedDetailedSearch ? "HideElement" : "") + "' title='" + generalTranslations.minimizeButtonTitle + "'></div>");

		this.searchBarHTMLArray.push("<div class='SearchBarButtonContainer'><div><input dojoType='epages.widget.Button' type='submit' id='SearchBarButton' class='SearchBarButton Button' value='" + generalTranslations.searchButtonTitle + "' iconSrc='dialog_ico_s_loupe'/></div><div class='RoundCornerPatch'></div></div></form>");

		this._initSearchBar();
	},

	_getFieldCount: function(fieldArray) {
		var fieldArrayFiltered = [];
		var isGroupOpened = false;
		for( var i=0,iLength=fieldArray.length ; i<iLength ; i++ ) {
			if(isGroupOpened) {
				if(fieldArray[i].type == "endgroup") {
					isGroupOpened = false;
				}
			}else {
				if(fieldArray[i].type == "startgroup") {
					isGroupOpened = true;
				}
				fieldArrayFiltered.push(fieldArray[i]);
				fieldArrayFiltered.push(fieldArray[i]);
			}
		}
		return fieldArrayFiltered;
	},

	_buildFormElements: function(elementArray) {
		// traverse the element array and build up the specific html code for every entry
		for( var i=0,iLength=elementArray.length ; i<iLength ; i++ ) {
			switch(elementArray[i].type) {
				case "select": this._formElements.getSelect(elementArray[i], this._getInputId(), this);break;
				case "checkbox": this._formElements.getCheckbox(elementArray[i], this._getInputId(), this);break;
				case "date": this._formElements.getDateInput(elementArray[i], this._getInputId(), this);break;
				case "datetime": this._formElements.getDateTimeInput(elementArray[i], this._getInputId(), this);break;
				case "multioptions": this._formElements.getMultiOptionsInput(elementArray[i], this._getInputId(), this);break;
				case "tag": this._formElements.getTagInput(elementArray[i], this._getInputId(), this);break;
				case "startgroup": this._formElements.getGroupStart(elementArray[i], this);break;
				case "endgroup": this._formElements.getGroupEnd(this);break;
				case "custom": this._formElements.getCustom(elementArray[i],this);break;
				default: this._formElements.getInput(elementArray[i], this._getInputId(), this);break;
			}
		}
	},

	_formElements: {
		// collection of all available input types inside of a SearchBar (called via "_buildFormElements")
		_getLabel: function(label, id, adjacentText) {
			return "<label unselectable='unselectable' for='" + id + "'>" + label + this._getAdjacentText(adjacentText) + "</label>";
		},
		_getAdjacentText: function(text) {
			return (text ? "<br/><span>" + text + "</span>" : "");
		},
		getSelect: function(arrayEntry, id, scope) {
			scope.searchBarHTMLArray.push("<span class='EnableHover SearchGroup SelectGroup " + (arrayEntry.cssClass || "") + "'>" + this._getLabel(arrayEntry.label, id, arrayEntry.adjacentText) +
																		"<select " + scope._ELEMENT_DOJOTYPE + " id='" + id + "' name='" + arrayEntry.name + "'>");
			for( var i=0,iLength=arrayEntry.options.length ; i<iLength ; i++ ) {
				scope.searchBarHTMLArray.push("<option value='" + arrayEntry.options[i].value + "' " + (arrayEntry.options[i].selected ? "selected='selected'" : "") + ">" + arrayEntry.options[i].label + "</option>");
			}
			scope.searchBarHTMLArray.push("</select></span>");
		},
		getCheckbox: function(arrayEntry, id, scope) {
			scope.searchBarHTMLArray.push("<span class='EnableHover SearchGroup CheckboxGroup " + (arrayEntry.cssClass || "") + "'>" + this._getLabel(arrayEntry.label, id, arrayEntry.adjacentText) +
																		"<input value='" + arrayEntry.defaultValue + "' id='" + id + "' " + scope._ELEMENT_DOJOTYPE + " type='text' class='Checkbox' name='" + arrayEntry.name + "' " + (arrayEntry.checked ? "checked='checked'" : "") + "/>" +
																		"</span>");
		},
		getDateInput: function(arrayEntry, id, scope) {
			scope.searchBarHTMLArray.push("<span class='EnableHover SearchGroup DateGroup " + (arrayEntry.cssClass || "") + "'>" + this._getLabel(arrayEntry.label, id, arrayEntry.adjacentText) +
																		"<input onchange='epages.cartridges.de_epages.presentation.SearchBar._checkErrorState()' dojoType='epages.widget.Datetimeinput' empty='true' selector='date' class='SearchDate' id='" + id + "' type='text' name='" + arrayEntry.name + "' value='" + (arrayEntry.defaultValue || "") + "'/></span>");
		},
		getDateTimeInput: function(arrayEntry, id, scope) {
			scope.searchBarHTMLArray.push("<span class='EnableHover SearchGroup DateTimeGroup " + (arrayEntry.cssClass || "") + "'>" + this._getLabel(arrayEntry.label, id, arrayEntry.adjacentText) +
																		"<input onchange='epages.cartridges.de_epages.presentation.SearchBar._checkErrorState()' dojoType='epages.widget.Datetimeinput' selector='datetime' class='SearchDate' id='" + id + "' type='text' empty='true' name='" + arrayEntry.name + "' value='" + (arrayEntry.defaultValue || "") + "'/></span>");
		},
		getMultiOptionsInput: function(arrayEntry, id, scope) {
			scope.searchBarHTMLArray.push("<span class='EnableHover SearchGroup SelectGroup MultiOptionsGroup " + (arrayEntry.cssClass || "") + "'>" + this._getLabel(arrayEntry.label, id, arrayEntry.adjacentText));

			var optionCollection = [];
			var preSelectedValueCollection = [];
			for( var i=0,iLength=arrayEntry.options.length ; i<iLength ; i++ ) {
				optionCollection.push("<option ep_name='" + arrayEntry.options[i].name + "'>" + arrayEntry.options[i].label + "</option>");
				if(arrayEntry.options[i].defaultValue) {
					preSelectedValueCollection.push(scope._getMultipleOptionsPreSelected(arrayEntry.options[i].name, arrayEntry.options[i].label, arrayEntry.translations[3], arrayEntry.translations[0], arrayEntry.translations[1], (arrayEntry.options[i].defaultValue == 2 ? true : false)));
				}
			}

			scope.searchBarHTMLArray.push("<span class='MultiOptionsGroupSelectContainer'><span class='SelectedEntries'>" + preSelectedValueCollection.join("") + "</span>" +
																		"<select onchange='epages.cartridges.de_epages.presentation.SearchBar._addMultipleOptionsOption(this)' " + scope._ELEMENT_DOJOTYPE + " id='" + id + "' ep_nottranslation='" + arrayEntry.translations[3] + "' ep_removetranslation='" + arrayEntry.translations[4] + "'>");

			scope.searchBarHTMLArray = scope.searchBarHTMLArray.concat(optionCollection);

			scope.searchBarHTMLArray.push("</select><span class='DetailSelector LeftPadding'>" +
																		"<input id='radio_" + id + "_true' name='radio_" + id + "_true' type='text' disabled='disabled' class='Disabled Checked Radio' checked='checked'/><label class='RightPadding' for='radio_" + id + "_true'>" + arrayEntry.translations[0] + "</label>" +
																		"<input id='radio_" + id + "_false' name='radio_" + id + "_false' type='text' disabled='disabled' class='Disabled Radio' /><label for='radio_" + id + "_false'>" + arrayEntry.translations[1] + "</label>" +
																		"</span></span></span>");
		},
		getTagInput: function(arrayEntry, id, scope) {
			scope.searchBarHTMLArray.push("<span class='EnableHover SearchGroup TagGroup " + (arrayEntry.cssClass || "") + "'>" + this._getLabel(arrayEntry.label, id, arrayEntry.adjacentText) +
																		"<input dojoType='epages.cartridges.de_epages.content.widget.TagEditor' class='SearchDate' id='" + id + "' type='text' empty='true' name='" + arrayEntry.name + "' value='" + (arrayEntry.defaultValue || "") +
																		"' availableTags='" + (arrayEntry.availableTags || "") + "'/></span>");
		},
		getInput: function(arrayEntry, id, scope) {
			scope.searchBarHTMLArray.push("<span class='EnableHover SearchGroup InputGroup " + (arrayEntry.cssClass || "") + "'>" + this._getLabel(arrayEntry.label, id, arrayEntry.adjacentText) +
																		"<input " + scope._ELEMENT_DOJOTYPE + " id='" + id + "' type='text' name='" + arrayEntry.name + "' value='" + (arrayEntry.defaultValue || "") + "'/></span>");
		},
		getGroupStart: function(arrayEntry, scope) {
			scope.searchBarHTMLArray.push("<span class='EnableHover Group " + (arrayEntry.cssClass || "") + "'>");
		},
		getGroupEnd: function(scope) {
			scope.searchBarHTMLArray.push("</span>");
		},
		getCustom: function(arrayEntry, scope) {
			scope.searchBarHTMLArray.push(arrayEntry.html);
		}
	},

	_getMultipleOptionsPreSelected: function(inputName, selectValue, removeTranslation, yesTranslation, noTranslation, selectReverse) {
		if($(inputName + "_true")) {
			return "";
		}
		return "<span class='SelectedEntry'>" +
						"<span class='SelectedEntryLabel'>" + selectValue + "</span>" +
						"<input id='" + inputName + "_true' value='1' name='" + inputName + "' type='text' class='Radio' dojoType='epages.widget.FormElement' " + (selectReverse ? "" : "checked='checked'") + "/><label class='RightPadding' for='" + inputName + "_true'>" + yesTranslation + "</label>" +
						"<input id='" + inputName + "_false' value='2' name='" + inputName + "' type='text' class='Radio' dojoType='epages.widget.FormElement' " + (selectReverse ? "checked='checked'" : "") + "/><label for='" + inputName + "_false'>" + noTranslation + "</label>" +
						"<i title='" + removeTranslation + "' class='RemoveButton Sprite ico_s_remove' onclick='epages.cartridges.de_epages.presentation.SearchBar._removeMultipleOptionsSelector(this)'></i>" +
						"</span>";
	},

	_addMultipleOptionsOption: function(selectNode) {

		if(selectNode.selectedIndex === 0) {
			return;
		}

		var selectContainer = selectNode.parentNode.parentNode;
		selectContainer = epages.Browser.engine == "MSIE" ? selectContainer.parentNode : selectContainer;

		var entriesContainer = dojo.query(".SelectedEntries", selectContainer)[0];
		var selectLabels = dojo.query("label", selectContainer);
		var inputName = selectNode.options[selectNode.selectedIndex].getAttribute("ep_name");

		var removeTranslation = selectNode.getAttribute("ep_removetranslation");
		var selectValue = selectNode.value || selectNode.options[selectNode.selectedIndex].innerHTML;

		var newSelectedEntryHTML = this._getMultipleOptionsPreSelected(inputName, selectValue, removeTranslation, selectLabels[1].innerHTML, selectLabels[2].innerHTML, false);
		var tempContainer = document.createElement("div");

		if(newSelectedEntryHTML.length > 2) {
			tempContainer.innerHTML = newSelectedEntryHTML;
			entriesContainer.appendChild(tempContainer.firstChild);
			dojo.parser.parse(entriesContainer.lastChild);
		}

		selectNode.selectedIndex = 0;
		this._hardRecalcSearchBarHeight();
	},

	_removeMultipleOptionsSelector: function(iconNode) {
		var entry = iconNode.parentNode;
		var entriesContainer = entry.parentNode;

		dojo.query(".Radio", entry).forEach(function(radioWidget) {
			$$(radioWidget.id).destroy();
		});

		entriesContainer.removeChild(entry);

		this._hardRecalcSearchBarHeight();
	},

	_getInputId: function() {
		// returns the ID for a new SearchBar element
		this._searchInputCounter++;
		return this._SEARCH_INPUT_PREFIX + this._searchInputCounter;
	},

	_injectSearchBarCode: function() {
		// merge the html snipplets
		//this.searchBarContainer.innerHTML = this.searchBarHTMLArray.join("");
		
		// parse the merged html
		//dojo.parser.parse(this.searchBarContainer);
		
		// workaround parseOnLoad <> dojo.parser conflict
		var temp = document.createElement('div');
			temp.innerHTML = this.searchBarHTMLArray.join("");
			dojo.parser.parse(temp);
			dojo.query('*[dojoType]',temp).forEach(function( node, i ) {
				dojo.removeAttr( node, 'dojoType' );
			})
		this.searchBarContainer.appendChild( temp );
		

		// link the node references
		with(this.searchBarNodes) {
			searchBarPlaceHolder = $("SearchBarPlaceHolder");
			searchBar = $("SearchBar");

			wildSearch = $("WildSearch");
			wildSearchInput = $("WildSearchInput");
			wildSearchButton = $("WildSearchButton");
			extender = $("Extender");

			detailedSearch = $("DetailedSearch");
			detailedSearchPrimary = $("DetailedSearchPrimary");
			detailedSearchSecondary = $("DetailedSearchSecondary");

			searchBarButton = $("SearchBarButton");
			minimizer = $("Minimizer");

			widthContainer = $("DetailedSearchWidthContainer") || document.createElement("div");
		}

		// is there an equivalent wild search input element inside of the detail search mask?
		this._mainDetailInputNode = dojo.query(".MainDetailInputNode input", this.searchBarNodes.detailedSearchPrimary)[0];
	},

	_appendSearchBarEvents: function() {
		dojo.connect(this.searchBarNodes.wildSearchButton, "click", this, "_searchBarMiniButtonClick");
		dojo.connect(this.searchBarNodes.extender, "click", this, "_extenderClick");

		dojo.connect(this.searchBarNodes.searchBarButton, "click", this, "_searchBarButtonClick");
		dojo.connect(this.searchBarNodes.searchBar, "keyup", this, "_checkErrorState");
		dojo.connect(this.searchBarNodes.minimizer, "click", this, "_minimizerClick");
	},

	_showSearchBarFinally: function() {
		if(this._mainDetailInputNode &&  this.searchBarNodes.wildSearchInput.value) {
			this._mainDetailInputNode.value = this.searchBarNodes.wildSearchInput.value;
		}
		this.searchBarNodes.searchBarPlaceHolder.style.height = this._detailedSearchBarHeight + "px";
		this.searchBarNodes.detailedSearch.style.right = "0";
		this.searchBarNodes.detailedSearch.style.left = "auto";
		dojo.removeClass(this.searchBarNodes.detailedSearch, "HideElementSoft");
		this.searchBarNodes.wildSearch.style.top = "-9999px";
		setTimeout(dojo.hitch(this, function() {
			this._allowHardRecalc = true;
			this._isExtendedToEnd = true;
			if(this._mainDetailInputNode) {
				setTimeout(dojo.hitch(this, function() {
					epages.event.fire(this._mainDetailInputNode, "mouseup");
					try {
						this._mainDetailInputNode.focus();
						this._mainDetailInputNode.select();
					}catch(e) {}
				}), 200);
			}
		}), 10);
	},

	_hideSearchBarFinally: function() {
		if(this._mainDetailInputNode &&  this._mainDetailInputNode.value) {
			this.searchBarNodes.wildSearchInput.value = this._mainDetailInputNode.value;
		}
		this.searchBarNodes.searchBarPlaceHolder.style.height = "0";
		dojo.addClass(this.searchBarNodes.detailedSearch, "HideElementSoft");
		this.searchBarNodes.wildSearch.style.top = "0";
		setTimeout(dojo.hitch(this, function() {
			this._allowHardRecalc = true;
			this._isExtendedToEnd = false;
			setTimeout(dojo.hitch(this, function() {
				epages.event.fire(this.searchBarNodes.wildSearchInput, "mouseup");
				try {
					this.searchBarNodes.wildSearchInput.focus();
					this.searchBarNodes.wildSearchInput.select();
				}catch(e) {}
			}), 200);
		}), 10);
	},

	showSearchBar: function(soft /* Boolean */) {
		// summary:
		//		Hides the wild search and reveales the detailed search.
		// soft:
		//		true - Animated process (default animation time)
		//		false- Static process
		this._allowHardRecalc = false;
		if(soft) {
			dojox.fx.wipeTo({node: this.searchBarNodes.searchBarPlaceHolder,
				height: this._detailedSearchBarHeight,
				duration: this._SEARCHBAR_ANI_DURATION,
				easing: dojo.fx.easing.quadIn
			}).play();

			var innerBarWidth = this.searchBarNodes.detailedSearchSecondary.parentNode.offsetWidth + 10;
			this.searchBarNodes.detailedSearch.style.top = (-this.searchBarNodes.detailedSearch.offsetHeight + this.searchBarNodes.wildSearch.offsetHeight) + "px";
			this.searchBarNodes.detailedSearch.style.left = (this.searchBarNodes.searchBar.offsetWidth - 10 - this.searchBarNodes.wildSearch.offsetWidth) + "px";
			dojo.removeClass(this.searchBarNodes.detailedSearch, "HideElementSoft");
			dojo.fx.slideTo({node: this.searchBarNodes.detailedSearch,
				top: 0,
				left: this.searchBarNodes.searchBar.offsetWidth - innerBarWidth,
				duration: this._SEARCHBAR_ANI_DURATION,
				easing: dojo.fx.easing.quadOut,
				onEnd: dojo.hitch(this, "_showSearchBarFinally")
			}).play();
		}else {
			this._showSearchBarFinally();
		}
		this._setStoredSearchBarStatus(this._STORAGE_EXTENDED_KEYWORD);
		this._isExtended = true;
	},

	hideSearchBar: function(/* Boolean */soft) {
		// summary:
		//		Hides the detailed search and reveales the wild search.
		// soft:
		//		true - Animated process (default animation time)
		//		false- Static process
		this._allowHardRecalc = false;
		if(soft) {
			dojox.fx.wipeTo({node: this.searchBarNodes.searchBarPlaceHolder,
				height: 0,
				duration: this._SEARCHBAR_ANI_DURATION,
				easing: dojo.fx.easing.quadOut
			}).play();

			this.searchBarNodes.detailedSearch.style.top = "0px";
			this.searchBarNodes.detailedSearch.style.left = (this.searchBarNodes.searchBar.offsetWidth - this.searchBarNodes.detailedSearch.offsetWidth) + "px";
			this.searchBarNodes.detailedSearch.style.right = "auto";
			dojo.fx.slideTo({node: this.searchBarNodes.detailedSearch,
				top: (-this.searchBarNodes.detailedSearch.offsetHeight + this.searchBarNodes.wildSearch.offsetHeight),
				left: (this.searchBarNodes.searchBar.offsetWidth - 10 - this.searchBarNodes.wildSearch.offsetWidth),
				duration: this._SEARCHBAR_ANI_DURATION,
				easing: dojo.fx.easing.quadIn,
				onEnd: dojo.hitch(this, "_hideSearchBarFinally")
			}).play();
		}else {
			this._hideSearchBarFinally();
		}
		this._setStoredSearchBarStatus(this._STORAGE_MINIMIZED_KEYWORD);
		this._isExtended = false;
	},

	_minimizerClick: function(evt) {
		this.hideSearchBar(true);
	},

	_extenderClick: function(evt) {
		this.showSearchBar(true);
	},

	_searchBarButtonClick: function(evt) {
		this.submitForm(this.searchBarNodes.detailedSearch);
	},

	_checkErrorState: function() {
		var searchBarButtonWidget = $$(this.searchBarNodes.searchBarButton.id);
		if(dojo.query("input.DialogError", this.searchBarNodes.detailedSearch).length) {
			searchBarButtonWidget.disable();
		}else {
			searchBarButtonWidget.enable();
			dojo.removeClass(this.searchBarNodes.searchBarButton.parentNode, "Active");
		}
	},

	_searchBarMiniButtonClick: function(evt) {
		this.submitForm(this.searchBarNodes.wildSearch);
	},

	submitForm: function(formNode) {
		if(!dojo.query("input.DialogError", formNode).length) {
			formNode.submit();
		}
	},

	_hardRecalcSearchBarHeight: function() {
		this._detailedSearchBarHeight = this.searchBarNodes.detailedSearch.offsetHeight - 10;
		setTimeout(dojo.hitch(this, function() {
			this._detailedSearchBarHeight = this.searchBarNodes.detailedSearch.offsetHeight - 10;
			if(this._isExtended) {
				with(this.searchBarNodes.searchBarPlaceHolder.style) {
					height = "auto";
					height = this._detailedSearchBarHeight + "px";
				}
			}
		}), 1);
	},

	_hardRecalcDimensions: function() {
		if(!this._allowHardRecalc) {
			return;
		}
		var limitNode = this.searchBarNodes.detailedSearchSecondary.parentNode;
		this.searchBarNodes.searchBar.style.visibility = "hidden";
		limitNode.style.width = "auto";
		limitNode.style.filter = "alpha()";
		this.searchBarNodes.widthContainer.style.display = "inline";
		setTimeout(dojo.hitch(this, function() {
			var hiddenClassSet = dojo.hasClass(this.searchBarNodes.detailedSearch, "HideElementSoft");
			dojo.removeClass(this.searchBarNodes.detailedSearch, "HideElementSoft");
			limitNode.style.width = (epages.Browser.engine == "MSIE" ? this.searchBarNodes.detailedSearchSecondary : this.searchBarNodes.widthContainer).offsetWidth + "px";
			this.searchBarNodes.widthContainer.style.display = "block";
			if(hiddenClassSet) {
				dojo.addClass(this.searchBarNodes.detailedSearch, "HideElementSoft");
			}
			limitNode.style.filter = "none";
			this.searchBarNodes.searchBar.style.visibility = "visible";
		}), 1);
		this._hardRecalcSearchBarHeight();
	},

	_initSearchBar: function() {
		this._injectSearchBarCode();
		this._appendSearchBarEvents();

		if(!this._forcedDetailedSearch) {
			var searchBarStatus = this._getStoredSearchBarStatus();
			if(searchBarStatus == this._STORAGE_EXTENDED_KEYWORD) {
				this.showSearchBar(false);
			}else {
				this.hideSearchBar(false);
			}
		}

		this.searchBarNodes.searchBar.style.visibility = "hidden";
		dojo.removeClass(this.searchBarContainer, "HideElementSoft");

		this._hardRecalcDimensions();

		setTimeout(dojo.hitch(this, function() {

			this._hardRecalcDimensions();
			if(!this._isExtended) {
				this._allowHardRecalc = false;
			}

			dojo.connect(window, "resize", this, "_hardRecalcDimensions");

			// only IE fires the resize event when the sidebar status changed
			if(epages.Browser.engine != "MSIE") {
				var sideBarController = $("Sidebarcontroller");
				if(sideBarController) {
					dojo.subscribe(dojo.query("[id*=Sidebar]", sideBarController)[0].id + "/afterShowSidebar", dojo.hitch(this, function() {
						setTimeout(dojo.hitch(this, function() {
							this._hardRecalcDimensions();
						}), 1);
					}));
				}
			}

			// sometimes it is necessary to update the dimension styles after all surrounding elements are rendered
			setTimeout(dojo.hitch(this, function() {
				var temp_allowHardRecalc = this._allowHardRecalc;
				this._allowHardRecalc = true;
				this._hardRecalcDimensions();
				this._allowHardRecalc = temp_allowHardRecalc;

				this._checkErrorState();
				dojo.publish("SearchBar/ready", [this.searchBarNodes.searchBar]);
			}), 1);

		}), 10);
	}

};
