dojo.provide("epages.cartridges.de_epages.presentation.richtext");
dojo.require("epages.html.element");
dojo.require("epages.lang.uri");

epages.cartridges.de_epages.presentation.richtext = {

	// reference to opened editor 
	_editor: null,
	_translation: new epages.io.Translation(dojo.moduleUrl('epages.cartridges.de_epages.presentation', "templates/translation"), "auto"),

	_HEIGHT : 480,
	_WIDTH : 900,

	// stub for overloading
	getValue: function(value) {
		return value;
	},

	setValue: function(value){
		return value;
	},

	cleanUp: function(value, insertCompleteUrl) {
		if (insertCompleteUrl) {
			value = value.replace(/((?:src|href)\s*=\s*[\'\"])(.*?)([\'\"])/ig, function(match, beforeUrl, url, afterUrl) {
				if(url.match(/^mailto\:/i)){
					//ignore mailto:
					return beforeUrl + url + afterUrl;
				}
				// ignore attachments
				if (!url.match(/^cid\:/i)) {
					url = epages.lang.uri.parts(url);
					if (!url.host && !url.path && !url.query && url.anchor) return match;
					var baseUrl = epages.lang.uri.parts(epages.vars.StoreFrontUrl);
					if (url.host == "") {
						if (url.path.match(/^\//)) {
							return beforeUrl + baseUrl.protocol + '://' + baseUrl.host + url.href + afterUrl
						} else {
							return beforeUrl + baseUrl.protocol + '://' + baseUrl.host + url.href + afterUrl
						}
					}
					return beforeUrl + url.href + afterUrl;
				}
				return beforeUrl + url + afterUrl;
			});
		}
		return value;
	},

	openDialog: function(options){
		epages.require('ep_shop_tree','dijit.Tree');
		epages.require('ep_shop_preview','epages.cartridges.de_epages.presentation.objectupdater');
		if (options.value == null) options.value = $(options.name).value;
		// value is a callback
		if (typeof value == 'function') options.value = options.value($(options.name).value);
		if (options.siteId == null) options.siteId = epages.vars.SiteID;
		if (options.title == null) options.title = $E(options.name).getLabel();
		if (options.title == undefined) options.title = '';
		options.title = this._translation.get('EditText')+': '+options.title;
		if (options.getValueCallback == null) options.getValueCallback = dojo.hitch(epages.cartridges.de_epages.presentation.richtext,"getValue");
		if (options.setValueCallback == null) options.setValueCallback = dojo.hitch(epages.cartridges.de_epages.presentation.richtext,"setValue");
		var modalDialog = $$('epages.cartridges.de_epages.presentation.richtext.modalDialog');
		var buttons = [{
			label: '{Apply}',
			cssClass: 'Active',
			onclick: function() {
				var rt = epages.cartridges.de_epages.presentation.richtext;
				var input = $(options.name);
				input.value = rt.cleanUp(options.getValueCallback(rt._editor.getContent()), options.insertCompleteUrl)
				if(options.savePage) {
					if (input.form.onsubmit) input.form.onsubmit();
					input.form.submit();
				} else {
					epages.event.fire(input, "change");
				}
				this._modalDialogWidget.hide();
			}
		},{
			label: '{Cancel}',
			onclick: function() { this._modalDialogWidget.hide(); }
		}];
		if (modalDialog == null) {
			var div = document.createElement('div');
			document.getElementsByTagName('body')[0].appendChild(div);
			dojo.require("epages.io.json");
			var json = new epages.io.Json();
			var openerId = epages.vars.currentSelectedObjectID ? epages.vars.currentSelectedObjectID : new Date().getTime();
			try {
				var snippet = json.loadSync('?', {
					'ObjectID':           parent.epages.vars.ObjectID,
					'TemplateAction':     parent.epages.vars.ViewAction,
					'TemplateLanguageID': parent.epages.vars.Locale.languageId,
					'TemplateName':       'INC-RichTextEditor',
					'ViewAction':         'JSONSnippet',
					'UseAbsolutePath':		options.useAbsolutePath ? '1' : '0',
					'OpenerID' : openerId
				}).data.snippet;
			} catch (e) {
				throw new Error('Could not load richtext snippet. Reason: ' + e.message);
			}
			dojo.require("epages.widget.Modaldialog");
			modalDialog = new epages.widget.Modaldialog({
				id:             'epages.cartridges.de_epages.presentation.richtext.modalDialog',
				options: {
					content:        snippet,
					width:          this._WIDTH,
					height:         this._HEIGHT,
					containerClass: 'NoPadding',
					title:					options.title
				},
				buttons:          buttons
			}, div);
			if(epages.TinyMCELicenceText !== undefined) {
				var copyRightDiv = document.createElement("div");
				copyRightDiv.className="TinyMCELicenceText";
				copyRightDiv.innerHTML = epages.TinyMCELicenceText;
				var innerButtonBarNode = modalDialog.innerButtonBarNode;
				innerButtonBarNode.insertBefore(copyRightDiv, innerButtonBarNode.firstChild);
			}
			var tinyToolBar = $('TinyMCETools'+openerId);
			if(tinyToolBar !== undefined) {
				dojo.removeClass(tinyToolBar, 'NavBarInvisible');
			}

			var rte = $('RichTextEditor');
			rte.value = options.setValueCallback(options.value);
			var settings = dojo.mixin(tinyMCE.settings, {
				width: this._WIDTH,
				height: (this._HEIGHT - epages.vars.Layout && epages.vars.Layout.toolBarHeight ? epages.vars.Layout.toolBarHeight : 0),
				body_class: 'ShowVisualAid mceShowContent'
			});

			var editor = new tinymce.Editor('RichTextEditor', settings);
			editor.render();
			epages.cartridges.de_epages.presentation.richtext._editor = editor;
		} else {
			epages.cartridges.de_epages.presentation.richtext._editor.setContent(options.setValueCallback(options.value));
			modalDialog.setupButtons(buttons);
			modalDialog.setTitle(options.title);
		}

		if(options.initCallback) {
			options.initCallback(modalDialog);
		}

		modalDialog.show();

	}

};