#!perl -w
use strict;
use DE_EPAGES::Core::API::PerlTools qw( IsA );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw( LoadRootObject );
use DE_EPAGES::Presentation::API::Form;
use HTTP::Request;
use Test::More;

RunScript( Sub => sub {
    # Run on test store, needed for locale/object
    RunOnStore( Store => GetTestSite(), Sub => sub {
        plan tests => 7;

        my $Request = HTTP::Request->new('GET');
        my $Form = DE_EPAGES::Presentation::API::Form->new($Request);
        $Form->{'Object'} = LoadRootObject();

        # test: float numbers in German using ","
        $Form->setlocale('de_DE');
        my $Value = $Form->formField( '10.000,00', { Name => 'Age', Type => 'reg_integer' });
        cmp_ok( $Value, 'eq', '10000', "integer de_DE" );

        # test: DateTime parser
        $Value = $Form->formField( '01.02.2009', { Name => 'CreationDate', Type => 'reg_date' });
        ok( IsA($Value, 'DateTime'), "date de_DE" );
        cmp_ok( $Value->iso8601, 'eq', '2009-02-01T00:00:00', "date de_DE" );

        # 00:00 datetime
        $Value = $Form->formField( '00:00', { Name => 'CloseAfternoon', Type => 'reg_time' });
        ok( IsA($Value, 'DateTime'), "time midnight" );
        cmp_ok( $Value->iso8601, 'eq', '2000-01-01T00:00:00', "time midnight" );

        # 24:00 datetime
        $Value = $Form->formField( '24:00', { Name => 'CloseAfternoon', Type => 'reg_time' });
        ok( IsA($Value, 'DateTime'), "time midnight 24:00" );
        cmp_ok( $Value->iso8601, 'eq', '2000-01-01T00:00:00', "time midnight 24:00" );

    });
});

