/*
	Copyright (c) 2006-2010, epages Software GmbH
	All Rights Reserved.

*/
dojo.provide("epages.cartridges.de_epages.product.widget.ProductPicker");//an dieser Stelle wird das neue Widget registriert
dojo.require("epages.widget.LocalizedWidget");
dojo.require("epages.cartridges.de_epages.product.widget.ProductPickerList");
dojo.require("epages.cartridges.de_epages.product.widget.ProductPickerTree");

dojo.declare("epages.cartridges.de_epages.product.widget.ProductPicker", [epages.widget.Modaldialog], {
		innerTemplatePath : dojo.moduleUrl('epages.cartridges.de_epages.product.widget', "templates/ProductPicker.html"),
		translationName   : dojo.moduleUrl('epages.cartridges.de_epages.product.widget', "templates/translation"),

		/*
		 * public properties
		*/

		searchTableHead          : undefined,  // String - column headlines
		onSubmitHandler          : undefined,  // function - custom handler which will be called when the modaldialog is going to get submitted
		requestObjectID          : undefined,  // String - #Shop.ProductFolder.ID
		options:{
				title                : '',
				width                : 450,
				height               : 250,
				content              : undefined,
				destroyWidgetOnClose : true
		}, // object - modaldialog settings

		/*
		 * private properties
		*/
		buttons                  : [], // [] - modaldialog buttons
		widgetsInTemplate        : true,

		// widget references
		_productList             : undefined,
		_productTree             : undefined,

		_viewIsTree              : true,
		_modalDialogSubmitButton : undefined, // DomNode - dom node of add button

		postMixInProperties: function() {
			// summary: set up modaldialog buttons
			this.buttons = [{
				label: '{Apply}',
				cssClass: 'Active',
				buttonId : 'modalDialogAddProductButton',
				onclick: dojo.hitch(this,this.onSubmitClick)
			 },{
				label: '{Cancel}',
				onclick: dojo.hitch(this,this.onCancelClick)
			}];
			if(this.searchTableHead === undefined) {
			this.searchTableHead = [this.translate('Name')];
			}
			dijit.Dialog.prototype.postMixInProperties.apply(this, arguments);
			this.inherited("postMixInProperties", arguments);
		},

		postCreate  : function() {
			// summary: Connect to events and create widgets
			dijit.Dialog.prototype.postCreate.apply(this, arguments);
			this.inherited("postCreate", arguments);

			dojo.connect(this.chooseTypeProduct.domNode,"onclick",this,"selectType");
			dojo.connect(this.chooseTypeAppointment.domNode,"onclick",this,"selectType");
			dojo.subscribe(this.id+"/tabChange",this,"onTabChange");

			// create search widget
			this._productList = new epages.cartridges.de_epages.product.widget.ProductPickerList({
				tableHead       : this.searchTableHead,
				requestObjectID : this.requestObjectID
			});
			// create tree widget
			this._productTree = new epages.cartridges.de_epages.product.widget.ProductPickerTree();
			// listen on select to disable submit button of modaldialog if selected item is not valid
			dojo.subscribe(this._productTree.id+'/select',this,"toggleAddButton");
			// add widgets to dom
			this.listNode.appendChild(this._productList.domNode);
			this.treeNode.appendChild(this._productTree.domNode);
		},
		show : function(){
			// summary: clean up all checkboxes
			this._productList.uncheckAll();
			this.inherited("show", arguments);
		},
		startSearch : function(evt){
			// summary: start search of productList widget
			dojo.stopEvent(evt);
			evt.returnValue = false;
			var isAppointment = this.chooseTypeAppointment.isChecked()?'1':'0';
			this._productList.startSearch(this.searchString.value,isAppointment);
		},
		selectedItems : function(){
			// summary: retrieve selected items on the current visible tab
			if(this._viewIsTree){
				var selectedItem = this._productTree.selectedItem();
				if(selectedItem){
					// convert result to look like the _productList.selectedItems() result
					var obj = {};
					obj[selectedItem.objectId] = {
						"Alias":selectedItem.alias,
						"ObjectID": selectedItem.objectId,
						"NameOrAlias":selectedItem.nameOrAlias
					};
					return obj; //Object
				} else {
					return {}; //Object
				}
			} else {
				return this._productList.selectedItems(); //Object
			}
		},
		toggleAddButton : function(/*Object*/ currentSelectedItem){
			// summary: disable submit button if selected item is not valid
			if(!this.buttons || !this.buttons[0] || !this.buttons[0].buttonId){
				return;
			}
			if(currentSelectedItem === undefined){
				this.disableButton(this.buttons[0].buttonId);
			} else  {
				this.enableButton(this.buttons[0].buttonId);
			}
		},
		onTabChange : function(/*DOMNode*/ targetTab){
			// summary: keep in mind which tab is selected to submit the correct selected items
			this._viewIsTree = targetTab==this.treeTab;
		},
		onSubmitClick : function(/*Object*/ evt){
			// summary: call custom handler and hide dialog
			if(!evt.currentTarget.className.match("Disabled")) {
				// if a custom handler is specified call it
				if(this.onSubmitHandler){
					this.onSubmitHandler(evt,this.selectedItems());
				}
				this.hide();
			}
		},
		onCancelClick : function(/*Object*/ evt){
			// summary: hide modaldialog
			this.hide();
		},
		selectType : function(/*Object*/ evt){
			// summary: change displayed text of search label
			this.searchLabel.innerHTML = evt.target.value;
		}
	}
);