use strict;
use Test::More;
use DE_EPAGES::Core::API::PerlTools qw( UsePackage );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( GetTestSite RunOnStore GetCurrentDBHandle);

sub GetDirtyReferences() {

    my $DBIPackage = GetCurrentDBHandle()->driverName =~ /mysql/i
        ? 'DE_EPAGES::ShopTransfer::API::DBImySQL'
        : 'DE_EPAGES::ShopTransfer::API::DBISybase';
    UsePackage($DBIPackage);
    my $ShopTransferDBI = $DBIPackage->new;

    my %Tables = ();
    my $Count = 0;
    foreach my $Table (@{$ShopTransferDBI->getTables}) {
        my @References = $ShopTransferDBI->getReferenceColumns($Table);
        $Tables{$Table} = \@References;
        $Count+= scalar @References;
    }

    plan tests => $Count;

    my $dbi = GetCurrentDBHandle();

    foreach my $Table (sort keys %Tables) {
        foreach my $Column (@{$Tables{$Table}}) {
            my $time1 = time;
            my @Master = $ShopTransferDBI->getMasterOfReference($Table, $Column);
            my $sql = sprintf 'SELECT t.%s FROM %s t LEFT JOIN %s m ON (m.%s = t.%s) WHERE m.%s IS NULL AND t.%s IS NOT NULL',
                $Column, $Table, $Master[0], $Master[1], $Column, $Master[1], $Column;
            my $aaResult = $dbi->execute($sql);
            ok((not scalar @$aaResult), $Table.'.'.$Column.'=>'.$Master[0].'.'.$Master[1].(scalar @$aaResult ? ' sql:'.$sql : ''));
        }
    }
    return;
}

sub Main {
    RunOnStore( Store => GetTestSite(), Sub => \&GetDirtyReferences );
}

RunScript(Sub=>\&Main, Debug=>1);

1;
