use Test;
use strict;
use DE_EPAGES::Core::API::Error qw ( GetError ExistsError Error);
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::File  qw ( GetFileContent WriteFile DeleteFile );
use DE_EPAGES::TLE::API::TLE    qw ( ReplaceTLEVars GetByteCodeByText );
use DE_EPAGES::TLE::t::tlevars;

our $TEST_ONE;

sub Main {
    Initialize();
    if ( defined $TEST_ONE ) {
        plan tests => 1;
        ok ( Test_Package($TEST_ONE ) );
    }
    else {
        my @Files = glob '*_t.txt';
        plan tests => scalar @Files;

        foreach my $File ( @Files ) {
            if($File =~ /^(.+)_t\.txt$/) {
                my $TestCase = $1;
                ok ( Test_Package($TestCase ) );
            }
            else {
                die "invalid file name $File";
            }
        }
    }
}

local $| = 1;

RunScript(
    'Sub' => \&Main
);

sub ErrorDefinition { return {
    NO_EQUAL_LINE => "Output is different at line #AtLine"
}; }

sub Test_Package {
    my ($i) = @_;
    my $isContent;
    eval {
        no strict 'refs'; ##no critic qw( ProhibitNoStrict )
        my $Loops =  DE_EPAGES::TLE::t::tlevars::GetLoops($i);
        my $Vars  =  DE_EPAGES::TLE::t::tlevars::GetVars($i);

        local $DE_EPAGES::TLE::API::Execute::TEMPLATE_SOURCE = $i.'_t.txt';
        $isContent = ProcessFileTemplate($i.'_t.txt', $Vars , $Loops);
        my $rExContent = GetFileContent($i.'_e.txt');
        CompareContent($rExContent, \$isContent);
    };
    if (ExistsError()) {
        my $Error = GetError();
        $Error->addVars( { 'Test' => $i } );
        print $Error->longMessage();
        WriteFile($i.'_i.txt', \$isContent) if defined $isContent;
        return 0;
    } else {
        DeleteFile($i.'_i.txt');
    }
    return 1;
}

sub ProcessFileTemplate {
    my ( $FileName, $hVars, $hLoops ) = @_;

    my $rTemplate = GetFileContent( $FileName );

    my $byteCode = GetByteCodeByText( $rTemplate );
    my %TempVars = (%$hVars, %$hLoops);
    return ReplaceTLEVars($byteCode, \%TempVars );
}

sub CompareContent {
    my ($rExpected, $rIs) = @_;

    my @aExpected = split ("\n", $$rExpected);
    my @aIs = split ("\n", $$rIs);
    if (scalar @aIs != scalar @aExpected) {
        Error('NO_EQUAL_LINES', {'CountLinesIs' => scalar @aIs, 'CountLinesExpected' => scalar @aExpected } );
    }
    for (my $i=0;$i < scalar @aExpected; $i++) {
        if ($aExpected[$i] ne $aIs[$i]) {
            Error('NO_EQUAL_LINE', {'AtLine' => $i + 1, 'Exp'=>$aExpected[$i], 'Ist'=> $aIs[$i] } );
        }
    }
    return 1;
}

sub Initialize {
    ##no critic qw(ProhibitPackageVars)
    $bin::ISSRV::AppServerIP = 12.12.12.12 if not defined $bin::ISSRV::AppServerIP;
    $bin::ISSRV::AppServerPort = 1234 if not defined $bin::ISSRV::AppServerPort;
    local $ENV{SCRIPT_NAME} = 'TLE.t';

}

sub Cleanup {
}

1;

