use Test;
use strict;
use DE_EPAGES::Core::API::Error qw ( GetError ExistsError Error);
use DE_EPAGES::Core::API::File  qw ( GetFileContent WriteFile DeleteFile );
use DE_EPAGES::Core::API::PerlTools qw(UsePackage);
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::TLE::API::Template qw ( ProcessFileTemplate );

our $TEST_ONE;

sub Main {
    Initialize();
    if ( defined $TEST_ONE ) {
	    plan tests => 1;
        ok ( Test_Package($TEST_ONE ) );
    }
    else {
	    my @Files = glob '*_t.tmpl';
	    plan tests => (defined $TEST_ONE ? 1 : scalar @Files);

        foreach my $File ( @Files ) {
            if ($File =~ /^(.+)_t\.tmpl$/) {
                my $TestCase = $1;
                ok ( Test_Package($TestCase ) );
            }
            else {
                die "invalid file name $File";
            }
        }
    }
}

local $| = 1;
RunScript( Sub => sub {
    Main(@ARGV);
});

if (ExistsError()) {
    print GetError()->shortMessage(),"\n";
    exit 1;
} else {
    print ".\n";
}

sub ErrorDefinition { return {
    NO_EQUAL_LINE => "Output is different at line #AtLine"
}; }

sub Test_Package {
    my ($i) = @_;
    my $Package = 'DE_EPAGES::TLE::t::tlevars';
    UsePackage( $Package );
    my $isContent;
    eval {
        no strict 'refs';  ##no critic qw( ProhibitNoStrict )
        my $Loops =  &{"$Package\:\:GetLoops"}($i);
        my $Vars  =  &{"$Package\:\:GetVars"}($i);

        $isContent = ProcessFileTemplate($i.'_t.tmpl', { %$Vars , %$Loops});
        my $rExContent = GetFileContent($i.'_e.tmpl');
        CompareContent($rExContent, \$isContent);
    };
    if (ExistsError()) {
        my $Error = GetError();
        print $Error->shortMessage();
        $Error->{'Vars'}->{'Test'} = $i;
        WriteFile($i.'_i.tmpl', \$isContent) if defined $isContent;
        return 0;
    } else {
        DeleteFile($i.'_i.tmpl');
    }
    return 1;
}

sub CompareContent {
    my ($rExpected, $rIs) = @_;

    my @aExpected = split ("\n", $$rExpected);
    my @aIs = split ("\n", $$rIs);
    if (scalar @aIs != scalar @aExpected) {
        Error('NO_EQUAL_LINES', {'CountLinesIs' => scalar @aIs, 'CountLinesExpected' => scalar @aExpected } );
    }
    for (my $i=0;$i < scalar @aExpected; $i++) {
        if ($aExpected[$i] ne $aIs[$i]) {
            Error('NO_EQUAL_LINE', {'AtLine' => $i + 1, 'Exp'=>$aExpected[$i], 'Ist'=> $aIs[$i] } );
        }
    }
    return 1;
}

sub Initialize {
    ##no critic qw(ProhibitPackageVars)
    $bin::ISSRV::AppServerIP = 12.12.12.12 if not defined $bin::ISSRV::AppServerIP;
    $bin::ISSRV::AppServerPort = 1234 if not defined $bin::ISSRV::AppServerPort;
    local $ENV{SCRIPT_NAME} = 'TLE.t';

}

sub Cleanup {
}

1;

