#!/bin/env perl.exe
#========================================================================================
# package      monitor.pl
# state        public
#----------------------------------------------------------------------------------------
# description  Use this Perl script if you have trouble displaying the hourglass pages.
#               This may be necessary if the $EPAGES_WEBROOT directory is shared using NFS.
#               Typically this file is installed into the directory $EPAGES_SHARED/Monitor.
#
#               Installation on Linux:
#                 1. Make sure the file monitor.pl is executable for the user "ep_web".
#                 2. Configure the Apache web server:
#                      uncomment:
#                        LoadModule actions_module modules/mod_actions.so
#                      add (where "/opt/epages/eproot" is your path $EPAGES):
#                        ScriptAlias /monitor.pl "/opt/epages/eproot/WebAdapter/monitor.pl"
#                        <Location /WebRoot/_MONITOR_/>
#                           AddHandler monitor .html
#                           Action monitor /monitor.pl
#                        </Location>
#
#               Installation on Windows with IIS:
#                 1. New virtual directory /WebRoot/_MONITOR_
#                      add script mapping:
#                        .html -> C:\epages\Perl\bin\perl.exe C:\epages\WebAdapter\monitor.pl %s
#                        [x] script engine
#                        [x] check that file exists
#========================================================================================
use strict;
# print "Content-Type: text/html\n\nHallo $ENV{PATH_INFO}<br>\n";
eval {
    $ENV{PATH_INFO} =~ /^\/WebRoot(\/_MONITOR_\/[0-9A-F\-]+\.html)$/i or die "Invalid Path $ENV{PATH_INFO}";
    my $FilePath = "$ENV{EPAGES_WEBROOT}$1";
    my $FileOpen = 0;
    my $MaxTries = 5;
    my $FILE;
    while( !$FileOpen && $MaxTries-- ) {
        eval { open $FILE, "<", $FilePath or die "Can't read $ENV{PATH_INFO}: $!" };
        if( $@ ) {
            sleep 1;
        }
        else {
            $FileOpen = 1;
        }
    }

    die $@ unless $FileOpen;

    print "Content-Type: text/html\n"; # charset=utf-8\n"; - specified inside HTML
    print "Cache-Control: private\n";
    print "Cache-Control: no-cache\n";
    print "\n";

#    flock FILE, 1 or die "Can't lock $FilePath: $!"; # LOCK_SH
    print while <$FILE>;
#    flock FILE, 8 or die "Can't unlock $FilePath: $!"; # LOCK_UN
    close $FILE or die "Can't close $FilePath: $!";
};
if( $@ ) {
    $@ =~ s/\s+at \S+ line \d+.*//; # remove perl file and line number
#    print "Status: 404 Not Found\n";
    print "Content-Type: text/html\n";
    print "\n";
    print "Error: $@\n";
};
