use strict ;
use Benchmark;
use DBI;

{
	my $FileName = 'ASPool.db';
	print "connecting database $FileName\n";
	my $dbh = DBI->connect( 'dbi:SQLite:dbname='.$FileName);

#	print "connecting database aspooldb\n";
#	my $dbh = DBI->connect( 'dbi:mysql:database=aspooldb', 'root', 'epages');
	$dbh->{RaiseError} = 1;

	my $TestCase = DBIBenchmark->new( $dbh );

	print "Benchmark started\n";
	timethis( 1, sub { $TestCase->DeleteAll }, 'DeleteAll' );
	timethis( 1, sub { $TestCase->Insert }, 'Insert' );
	timethis( 10, sub { $TestCase->CountAll }, 'CountAll' );
	timethis( 10, sub { $TestCase->Select }, 'Select' );
	timethis( 10, sub { $TestCase->SelectAll }, 'SelectAll' );
	# timethis( 1, sub { $TestCase->DeleteAll }, 'DeleteAll' );

	$dbh->disconnect;
}

package DBIBenchmark;
use strict;
use DE_EPAGES::Core::API::PerlTools qw( GenerateGUID );
use DE_EPAGES::Core::API::Log qw (GetLog);

sub new {
    my $class = shift;
    my $dbh = shift or die "Missing databaase handle";

    return bless { dbh => $dbh }, $class;
}

sub dbh {
    return shift->{dbh};
}

sub Insert {
    my $self = shift;

	my $dbh = $self->dbh;
    local $dbh->{AutoCommit} = 0;

    print "filling initial data...\n";
    $dbh->do('INSERT INTO pool (poolid, name) VALUES (1,\'DefaultPool\')');
    $dbh->do('INSERT INTO store (storeid, poolid, name) VALUES (1, 1,\'Store\')');
    $dbh->do('INSERT INTO store (storeid, poolid, name) VALUES (2, 1,\'Site\')');
    $dbh->do("INSERT INTO site (siteid, storeid, guid, objectpath) VALUES (1, 1, 'Site', '/')");

    my $MAX_SITE = 100000;
    my $MAX_HOST = 10000;
    my $MAX_COMMIT = 1000;

    my $StoreID = 1;

    print "filling values...\n";
    my $Counter = 0;

    # insert hosts
    my $sth_host = $dbh->prepare("INSERT INTO host (name, siteid) VALUES (?, ?)");
        $sth_host->execute('www.myshop.de', 1);
        my $HostID = $dbh->last_insert_id(undef, undef, undef, undef);
    foreach my $HostNo (1..$MAX_HOST) {
        my $HttpHost = 'www.myshop' . $HostNo . '.de';
        $sth_host->execute($HttpHost, 1);
    }

    my $sth_site = $dbh->prepare("INSERT INTO site (storeid, guid, objectpath) VALUES (?, ?, ?)");
    my $sth_request = $dbh->prepare("INSERT INTO request (siteid, hostid, uri) VALUES (?, ?, ?)");

    foreach my $SiteNo (1..$MAX_SITE) {
        my $SiteAlias = 'Site' . $SiteNo;
        my $SiteGUID = GenerateGUID();
        my $ObjectPath = '/Shops/'. $SiteNo;
        $sth_site->execute($StoreID, $SiteGUID, $ObjectPath);

        my $SiteID = $dbh->last_insert_id(undef, undef, undef, undef);

        my $URI = '/epages/'. $SiteNo . '.';
        $sth_request->execute($SiteID, $HostID, $URI);
        if( 0 == $SiteNo % $MAX_COMMIT ) {
                $dbh->commit;
                print "commit $SiteNo\n";
        }
    }
    $dbh->commit;
    $sth_site->finish;
    $sth_request->finish;
    return;
}

sub __Select {
    my $self = shift;

    print "reading 10000 values by key...\n";
    for(50000..60000) {
        my $HttpHost = 'www.myshop1.de';
        my $URI = '/epages/' . $_ . '.';

#           my $sth = $self->dbh->prepare("SELECT * FROM request WHERE httphost = ? AND uri = ?");
            my $sth = $self->dbh->prepare("SELECT storename, poolname FROM v_lookup WHERE httphost = ? AND uri = ?");
        $sth->execute($HttpHost, $URI);
        my ($x) = $sth->fetchrow_array or die "Not found ($HttpHost, $URI)";

        do {
            while( $sth->fetchrow_arrayref ) { ; }
        } while( $sth->{syb_more_results} && $sth->{syb_result_type} != 4043 );
        if ($sth->{syb_result_type} == 4043) {
            my ($status) = $sth->fetchrow_array;
            GetLog->warn("return status = $status") unless $status;
        }

            $sth->finish;

    #   print "$x\n";
    }
    return;
}

sub Select {
    my $self = shift;

    print "reading 10000 values by key...\n";
    my $sth = $self->dbh->prepare("SELECT storename, poolname FROM v_lookup WHERE httphost = ? AND uri = ?");
    for(50000..60000) {
        my $HttpHost = 'www.myshop.de';
        my $URI = '/epages/' . $_ . '.';
        $sth->execute($HttpHost, $URI);
        my ($x) = $sth->fetchrow_array or die "Not found ($HttpHost, $URI)";

        do {
            while( $sth->fetchrow_arrayref ) { ; }
        } while( $sth->{syb_more_results} && $sth->{syb_result_type} != 4043 );
        if ($sth->{syb_result_type} == 4043) {
            my ($status) = $sth->fetchrow_array;
            GetLog->warn("return status = $status") unless $status;
        }

    #   print "$x\n";
    }
    $sth->finish;
    return;
}

sub CountAll {
    my $self = shift;

    print "CountAll\n";
    my $aaRows = $self->dbh->selectall_arrayref("SELECT COUNT(*) FROM v_lookup");
    print "Total: $aaRows->[0]->[0] rows\n";
    return;
}

sub SelectAll {
    my $self = shift;

    print "reading all values sequencially\n";
    my $sth = $self->dbh->prepare("SELECT storename, poolname, httphost, uri, storename FROM v_lookup");
    $sth->execute;
    while( my($key, $value) = $sth->fetchrow_array ) {
        my $x = "$key, $value\n";
    }

    $sth->finish;
    return;
}

sub DeleteAll {
    my $self = shift;

    print "deleting all values\n";
    $self->dbh->do("DELETE FROM request");
    $self->dbh->do("DELETE FROM host");
    $self->dbh->do("DELETE FROM site");
    $self->dbh->do("DELETE FROM store");
    $self->dbh->do("DELETE FROM pool");
#    $self->dbh->do("VACUUM");
	return;
}
