#========================================================================================
# package      main
# state        private
# description  Creates an operating system specific Makefile
#----------------------------------------------------------------------------------------
# example      # Makefile.PL
#               our $MakeMake = "MakeMake" # used by Makefile.pm
#               require "$MakeMake/Makefile.pm";
#               __DATA__
#               # Makefile content
#========================================================================================
use strict;
use Config;
use feature qw( switch );
use vars qw($MakeMake *DATA); ##no critic qw(ProhibitPackageVars)

my ($mday, $mon, $year) = (localtime)[3,4,5];
my $IsDate = sprintf "%02d/%02d/%02d", $mon+1, $mday, $year % 100;

my $preprocessor = $Config{osname} eq 'MSWin32' ? '!' : '';

open my $COM, "<", "$MakeMake/common.mk" or die "Can't read $MakeMake/common.mk";
my $COMMON = do { local $/ = undef; <$COM> };
$COMMON =~ s/\n\#(include|ifn?def|ifeq|else|endif)/\n$preprocessor$1/gx;
$COMMON =~ s/\n(!if)eq([^,]*),/\n$1$2==/g if $Config{osname} eq 'MSWin32';;
$COMMON =~ s,\\(.),/$1,g unless $Config{osname} eq 'MSWin32';
close $COM;

$MakeMake =~ s,\\(.),/$1,g unless $Config{osname} eq 'MSWin32';

my $ARCH = _GetArchitecture();

open  my $MAKEFILE, ">", "makefile" or die "Can't write makefile\n";
print $MAKEFILE "WINDOWS  = 1\n" if $Config{osname} eq 'MSWin32';
print $MAKEFILE <<HERE
IS_DATE = $IsDate
ARCH    = $ARCH
OSNAME  = $Config{osname}
OSVERS  = $Config{osvers}
PERL    = $Config{perlpath}
MAKEMAKE= $MakeMake
HERE
    ;

while (<DATA>) {
    s/^\#include\ .*common.mk\n/$COMMON/x;
    s/^\#(include|ifn?def|ifeq|else|endif)/$preprocessor$1/x;
    s/^(!if)eq([^,]*),/\n$1$2==/g if $Config{osname} eq 'MSWin32';
    s,\\(.),/$1,g unless $Config{osname} eq 'MSWin32';
    print $MAKEFILE $_;
}
close $MAKEFILE or die $!;

sub _GetArchitecture {
    given( $Config{osname} ) {
        when( 'linux' ) { return 'i686-linux' }
        when( 'solaris' ) {
            my $Architecture = $Config{myarchname};
            if( $Config{osvers} =~ /\.(.*)/ ) {
                $Architecture .= $1 > 9 ? "-2.10" : "-2.9";
            }
            return $Architecture;
        }
        when( 'MSWin32' ) { return 'MSWin32' }
        default { return 'unknown' }
    }
}

1;
