﻿use Test::More;
use strict;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::WebInterface::API::FakeRequest;
use DE_EPAGES::Core::API::File qw (GetFileMD5Digest);
use DE_EPAGES::Core::API::File qw (GetFileContent);
use DE_EPAGES::WebInterface::API::Form;
use Digest::MD5 qw(md5 md5_hex);
use HTTP::Cookies;
use HTTP::Request;

plan tests => 10;

sub Main {
    # Test with default input name (don't supply one)
    UploadImage('test.jpg');

    # Test with specific input name
    UploadImage('test.jpg', 'TestImage');

    return;
}

sub UploadImage {
    my ($TestImageFilename, $TestImageInputname) = @_;
    my $ServerName = 'localhost';
    # build request
    my $FakeRequest = DE_EPAGES::WebInterface::API::FakeRequest->new;
    my $URL = 'http://'.$ServerName.'/epages/TestRequest.sf';
    my $URI = URI->new($URL);
    my $Request = HTTP::Request->new( 'POST', $URI->path_query );
    $Request->header( 'Host' => $FakeRequest->_host($URI) );
    $Request->header( 'handleashtml', 1);
    my $Disp = 'attachement; filename="'.$TestImageFilename.'";';
    $Disp .= ' name="'.$TestImageInputname.'"' if $TestImageInputname;
    $Request->header('content-disposition', $Disp);

    $Request->content_type( "application/octet-stream" );
    my $TestFileContent = GetFileContent($TestImageFilename, { 'binary' => 1 });
    $Request->content(\$TestFileContent);

    # send
    my $Response = $FakeRequest->request( $Request );
    cmp_ok( $Response->code, '==', 200, '200 OK' );

    # default input name for reading input values of none is supplied
    $TestImageInputname = 'NewFile' unless $TestImageInputname;

    # check form data
    my $Form = DE_EPAGES::WebInterface::API::Form->new( $Request );
    ok((defined $Form and defined $Form->{'Files'} and $Form->{'Files'} != {}), 'form defined');
    ok(defined $Form->{'Files'}->{$TestImageInputname}, 'form has one file');

    cmp_ok(@{$Form->{'Values'}->{$TestImageInputname}}[0], 'eq', $TestImageFilename, 'form knows file name');

    my $FileContentInForm = ${${@{$Form->{'Files'}->{$TestImageInputname}}[0]}};
    my $MD5InForm = md5_hex($FileContentInForm);
    my $MD5TestFile = GetFileMD5Digest($TestImageFilename);
    cmp_ok($MD5InForm, 'eq', $MD5TestFile, 'md5 hash matches');

    return;
}

RunScript(Sub=>\&Main, Debug=>1);
