#-*- Makefile -*-

require DE_EPAGES::Core::API::Makefile;
__DATA__

!include host-$(HOSTNAME).mk

PATCH_TAG	= patch_6_10_2
PATCH_VERSION	= 6.10.2
PATCH_LEVEL	= 2
DE_EPAGES_DIR	= $(EPAGES_CARTRIDGES)\DE_EPAGES
DATA_DIR	= $(EPAGES_SHARED)\var-$(PATCH_TAG)
LOG_DIR		= $(DATA_DIR)\logs\$(HOSTNAME)
TGT		= $(DATA_DIR)\tgt\$(HOSTNAME)
BACKUP_DIR	= $(DATA_DIR)\backup\$(HOSTNAME)
BACKUP_DB	= 1

!OS_SPECIFIC_DEFINES
PRL		= $(PERL) -IInclude
!ifdef WINDOWS
RMDIR		= rmdir /S /Q
DIFF		= gdiff
CHANGE_ETC	= 0
!else
RMDIR		= rm -rf
DIFF		= diff
EPAGES_PROJECT	?= epages6
CHANGE_ETC	= 1
EPAGES_ETC	= $(shell dirname "$(EPAGES)")\etc
!endif

SYBASE_DUMPS 	= $(PRL) Cartridges/DE_EPAGES/Database/Scripts/DatabaseDumps.pl
MYSQL_DUMPS 	= $(PRL) Cartridges/DE_EPAGES/Database/Scripts/MySQLDumps.pl
START_SERVICE   = $(PRL) $(DE_EPAGES_DIR)/Installer/Scripts/startService.pl -ifexists
STOP_SERVICE    = $(PRL) $(DE_EPAGES_DIR)/Installer/Scripts/stopService.pl -ifexists
CREATE_CTMPLS   = $(PRL) $(DE_EPAGES_DIR)/Presentation/Scripts/createCtmpls.pl

!ifeq ("$(PATCH_LEVEL)", "0")
SINGLE_HOST_DUMP =
!else
SINGLE_HOST_DUMP = -all
!endif

!ifdef INITIAL_VERSION_LT_6010001000
DE_PATCH_DIR	= $(PATCH_DIR)\Cartridges\DE_EPAGES
!else
DE_PATCH_DIR	= $(DE_EPAGES_DIR)
!endif

STORE		= dummy

.force:

# we always need following target for patch.pl:
get_var: .force
	@echo LOG_DIR=$(LOG_DIR)
	@echo BACKUP_DIR=$(BACKUP_DIR)
	@echo TGT_DIR=$(DATA_DIR)\tgt
	@echo TGT=$(TGT)

# how to install the epages patch (see install_en.txt)
# ****************************************************
#-------------------------------------------------------------------------------------------
#    | execute                             | on  UNIX                  | on Windows        |
#-------------------------------------------------------------------------------------------
# 1. | set environment                     | . /etc/default/epages6    |                   |
# 2. | create makefile (needs epages perl) | perl Makefile.PL          | perl Makefile.PL  |
# 3. | pre-installation tasks              | make preinstall           | nmake preinstall  |
# 4. | set environment                     | . /etc/default/epages6    |                   |
# 5. | installation tasks* (see NOTE)      | make install              | nmake install     |
#-------------------------------------------------------------------------------------------
# *) if epages runs on more than 1 computer, execute this step only on your main application server

preinstall: $(TGT)\preinstall
$(TGT)\preinstall:
	$(MAKE) acl stop_epages limit_httpd stop_cron initial_backup_db
	$(DEL) $(DATA_DIR)\tgt\posthosts.txt
	$(AWK) "/^$(HOSTNAME)$$/{i=1}END{exit i!=1}" $(DATA_DIR)\tgt\prehosts.txt 2>$(NUL) || echo $(HOSTNAME)>> $(DATA_DIR)\tgt\prehosts.txt
	$(MAKE) touch DIR=$(@D) ADDF=$(@F)

install: $(TGT)\preinstall patch_files restart_epages patch update_ctmpls
	echo $(HOSTNAME)> $(DATA_DIR)\tgt\alphahost.txt
!ifdef WINDOWS
	if exist $(TGT)\restore_style $(RMDIR) $(TGT)\restore_style
!else
	[ ! -d $(TGT)\restore_style ] || $(RMDIR) $(TGT)\restore_style
!endif
	$(MAKE) touch DIR=$(DATA_DIR)\tgt ADDF=install DELF=restore
	$(DIFF) -q $(DATA_DIR)\tgt\alphahost.txt $(DATA_DIR)\tgt\prehosts.txt >$(NUL) 2>&1 &&\
  $(MAKE) postinstall IS_ALPHA=1 POSTINSTALL_COMPLETE=1 || $(ECHONL)
	$(MAKE) perm

postinstall:
!ifdef IS_ALPHA
!ifneq ("$(POSTINSTALL_COMPLETE)", "1")
	@echo "Postinstall on alpha server failed: First postinstall all other servers."
	@$(PERL) -e "exit 1;"
!endif
!else
	$(MAKE) patch_files
!endif
	$(MAKE) $(DATA_DIR)\tgt\install final_backup_db start_cron new/unlimit_httpd start_epages
!ifdef IS_ALPHA
	$(MAKE) open_stores
	$(MAKE) sync_stores
!endif
	echo $(HOSTNAME)>> $(DATA_DIR)\tgt\posthosts.txt

# how to restore the prevoius epages version (see install_en.txt)
# ***************************************************************
#-------------------------------------------------------------------------------------------
#    | execute                             | on  UNIX                  | on Windows        |
#-------------------------------------------------------------------------------------------
# 1. | set environment                     | . /etc/default/epages6    |                   |
# 2. | create makefile (needs epages perl) | perl Makefile.PL          | perl Makefile.PL  |
# 3. | pre-restore tasks                   | make prerestore           | nmake prerestore  |
# 4. | restore tasks* (see NOTE)           | make restore              | nmake restore     |
# 5. | set environment                     | . /etc/default/epages6    |                   |
#-------------------------------------------------------------------------------------------
# *) if epages runs on more than 1 computer, execute this step only on your main application server

prerestore: acl stop_epages new/limit_httpd restore_files stop_cron restart_epages restore_db

restore: prerestore
!ifdef WINDOWS
	if exist $(TGT)\patch_store $(RMDIR) $(TGT)\patch_store
	if exist $(TGT)\sync_store $(RMDIR) $(TGT)\sync_store
	if exist $(TGT)\test_style $(RMDIR) $(TGT)\test_style
	if exist $(TGT)\create_ctmpl $(RMDIR) $(TGT)\create_ctmpl
	if exist $(TGT)\update_ctmpl $(RMDIR) $(TGT)\update_ctmpl
!else
	[ ! -d $(TGT)\patch_store ] || $(RMDIR) $(TGT)\patch_store
	[ ! -d $(TGT)\sync_store ] || $(RMDIR) $(TGT)\sync_store
	[ ! -d $(TGT)\test_style ] || $(RMDIR) $(TGT)\test_style
	[ ! -d $(TGT)\create_ctmpl ] || $(RMDIR) $(TGT)\create_ctmpl
	[ ! -d $(TGT)\update_ctmpl ] || $(RMDIR) $(TGT)\update_ctmpl
!endif
	$(MAKE) update_ctmpls DEL_SAVED_INCLUDES=1
!ifdef WINDOWS
	if exist $(TGT)\update_ctmpl $(RMDIR) $(TGT)\update_ctmpl
!ifdef INITIAL_VERSION_LT_6010002000
	if exist "$(EPAGES_WEBROOT)\StoreTypes" $(RMDIR) "$(EPAGES_WEBROOT)\StoreTypes"
!endif
!else
	[ ! -d $(TGT)\update_ctmpl ] || $(RMDIR) $(TGT)\update_ctmpl
!ifdef INITIAL_VERSION_LT_6010002000
	[ ! -d "$(EPAGES_WEBROOT)\StoreTypes" ] || $(RMDIR) "$(EPAGES_WEBROOT)\StoreTypes"
!endif
!endif
	$(MAKE) start_epages
	$(MAKE) restore_styles
	$(MAKE) start_cron
	$(MAKE) touch DIR=$(DATA_DIR)\tgt ADDF=restore DELF=install
	$(MAKE) postrestore perm IS_ALPHA=1

postrestore: $(DATA_DIR)\tgt\restore
	$(DEL) $(TGT)\preinstall
	$(DEL) $(DATA_DIR)\tgt\prehosts.txt
	$(DEL) $(DATA_DIR)\tgt\alphahost.txt
!ifdef IS_ALPHA
	$(MAKE) open_stores
!else
	$(MAKE) start_cron start_epages
!endif
	$(MAKE) unlimit_httpd RESTART_APACHE=1
!ifdef WINDOWS
	if exist $(TGT)\sync_store $(RMDIR) $(TGT)\sync_store
!else
	[ ! -d $(TGT)\sync_store ] || $(RMDIR) $(TGT)\sync_store
!endif

# main targets (patch files)
# ==========================

patch_files:
!ifeq ("$(CHANGE_CONFIG)", "1")
	$(MAKE) Config\patch
!endif
!ifeq ("$(CHANGE_CARTRIDGES)", "1")
	$(MAKE) Cartridges\patch
!endif
!ifeq ("$(CHANGE_APACHE)", "1")
	$(MAKE) patch_apache
!endif
!ifeq ("$(PATCH_LEVEL)", "0")
!ifeq ("$(CHANGE_PERL)", "1")
	$(MAKE) patch_cpan
!endif
!ifeq ("$(CHANGE_SYBASE)", "1")
	$(MAKE) Sybase\patch
!endif
!ifeq ("$(CHANGE_REQROUTER)", "1")
	$(MAKE) RequestRouter\patch
!endif
!endif
!ifeq ("$(CHANGE_WEBADAPTER)", "1")
	$(MAKE) WebAdapter\patch
!endif
!ifeq ("$(CHANGE_ETC)", "1")
	$(MAKE) etc\patch
!endif
!ifeq ("$(CHANGE_BIN)", "1")
	$(MAKE) bin\patch
!endif
!ifeq ("$(CHANGE_WEBROOT)", "1")
	$(MAKE) WebRoot\patch
!endif
!ifeq ("$(CHANGE_STORES)", "1")
	$(PRL) Scripts/patch_shared.pl -cp Stores -backup "$(BACKUP_DIR)" -version $(PATCH_VERSION)
	$(DEL) "$(EPAGES_STORES)"\.$(PATCH_VERSION)-restore
	$(MAKE) touch DIR="$(EPAGES_STORES)" ADDF=.$(PATCH_VERSION)-patch
!ifndef WINDOWS
	chmod 640 "$(EPAGES_STORES)"/.$(PATCH_VERSION)-patch
	chown $${EPAGES_APPUSER:-eprunapp}:epages "$(EPAGES_STORES)"/.$(PATCH_VERSION)-patch
!endif
!endif

restore_files:
!ifeq ("$(PATCH_LEVEL)", "0")
!ifeq ("$(CHANGE_SYBASE)", "1")
	$(MAKE) Sybase\restore
!endif
!ifeq ("$(CHANGE_REQROUTER)", "1")
	$(MAKE) RequestRouter\restore
!endif
!endif
!ifeq ("$(CHANGE_WEBADAPTER)", "1")
	$(MAKE) WebAdapter\restore
!endif
!ifeq ("$(CHANGE_ETC)", "1")
	$(MAKE) etc\restore
!endif
!ifeq ("$(CHANGE_BIN)", "1")
	$(MAKE) bin\restore
!endif
!ifeq ("$(CHANGE_WEBROOT)", "1")
	$(MAKE) WebRoot\restore
!endif
!ifeq ("$(CHANGE_STORES)", "1")
	$(MAKE) Stores\restore
!endif
!ifeq ("$(CHANGE_CONFIG)", "1")
	$(MAKE) Config\restore
!endif
!ifeq ("$(CHANGE_CARTRIDGES)", "1")
	$(MAKE) Cartridges\restore
!endif

# main targets (patch database)
# =============================

# backup databases
initial_backup_db: $(TGT)\initial_backup_db
final_backup_db: $(TGT)\final_backup_db
$(TGT)\initial_backup_db $(TGT)\final_backup_db full_backup_db:
!ifdef BACKUP_DB
!ifeq ("$(MYSQL_DATASOURCE)", "1")
	$(MYSQL_DUMPS) $(SINGLE_HOST_DUMP) -target $@
!endif
!ifeq ("$(SYBASE_DATASOURCE)", "1")
	$(SYBASE_DUMPS) $(SINGLE_HOST_DUMP) -target $@
!endif
!endif

# install the patch
patch: patch_site patch_stores patch_pooldb patch_version

# restore databases
restore_db: $(TGT)\restore_db
$(TGT)\restore_db:
!ifdef BACKUP_DB
!ifeq ("$(SYBASE_RUNS)", "1")
!ifndef WINDOWS
!ifneq ("$(EXECUTOR)", "root")
	-$(ETC_INITD) perm sybase -u
!endif
!endif
	$(SYBASE_DUMPS) -target $@
!endif
!ifeq ("$(MYSQL_DATASOURCE)", "1")
	$(MYSQL_DUMPS) -own -target $@
!else
!ifneq ("$(SYBASE_RUNS)", "1")
	$(MAKE) touch DIR=$(@D) ADDF=$(@F) DELF=initial_backup_db
	$(DEL) $(@D)\final_backup_db
!endif
!endif
!endif

# targets (patch files)
# *********************

# patch: etc

etc\patch: $(EPAGES_ETC)\.$(PATCH_VERSION)-patch
$(EPAGES_ETC)\.$(PATCH_VERSION)-patch:
	$(PRL) Scripts/patch_files.pl -patch etc -backup "$(BACKUP_DIR)" -version $(PATCH_VERSION)
	./Scripts/patch_etc.sh -P "$(EPAGES_PROJECT)" -i "$(INITIAL_INTERNAL_VERSION)" -p $(PATCH_TAG) -b "$(BACKUP_DIR)"

etc\restore: $(EPAGES_ETC)\.$(PATCH_VERSION)-restore
$(EPAGES_ETC)\.$(PATCH_VERSION)-restore:
	$(PRL) Scripts/patch_files.pl -restore etc -backup "$(BACKUP_DIR)" -version $(PATCH_VERSION)
	./Scripts/patch_etc.sh -R -P "$(EPAGES_PROJECT)" -i "$(INITIAL_INTERNAL_VERSION)" -p $(PATCH_TAG) -b "$(BACKUP_DIR)"

# patch: bin

bin\patch: $(EPAGES)\bin\.$(PATCH_VERSION)-patch
$(EPAGES)\bin\.$(PATCH_VERSION)-patch:
!ifndef WINDOWS
	$(PRL) Scripts/patch_files.pl -patch bin -backup "$(BACKUP_DIR)" -version $(PATCH_VERSION)
!endif

bin\restore: $(EPAGES)\bin\.$(PATCH_VERSION)-restore
$(EPAGES)\bin\.$(PATCH_VERSION)-restore:
!ifndef WINDOWS
	$(PRL) Scripts/patch_files.pl -restore bin -backup "$(BACKUP_DIR)" -version $(PATCH_VERSION)
!endif

# patch: WebAdapter

WebAdapter\patch: $(EPAGES)\WebAdapter\.$(PATCH_VERSION)-patch
$(EPAGES)\WebAdapter\.$(PATCH_VERSION)-patch:
	$(PRL) Scripts/patch_files.pl -patch WebAdapter -backup "$(BACKUP_DIR)" -version $(PATCH_VERSION)

WebAdapter\restore: $(EPAGES)\WebAdapter\.$(PATCH_VERSION)-restore
$(EPAGES)\WebAdapter\.$(PATCH_VERSION)-restore:
	$(PRL) Scripts/patch_files.pl -restore WebAdapter -backup "$(BACKUP_DIR)" -version $(PATCH_VERSION)

# patch: RequestRouter

RequestRouter\patch: $(EPAGES)\RequestRouter\.$(PATCH_VERSION)-patch
$(EPAGES)\RequestRouter\.$(PATCH_VERSION)-patch:
	$(PRL) Scripts/patch_files.pl -patch RequestRouter -backup "$(BACKUP_DIR)" -version $(PATCH_VERSION)

RequestRouter\restore: $(EPAGES)\RequestRouter\.$(PATCH_VERSION)-restore
$(EPAGES)\RequestRouter\.$(PATCH_VERSION)-restore:
	$(PRL) Scripts/patch_files.pl -restore RequestRouter -backup "$(BACKUP_DIR)" -version $(PATCH_VERSION)

# patch: Cartridges

Cartridges\patch: $(EPAGES_CARTRIDGES)\.$(PATCH_VERSION)-patch
$(EPAGES_CARTRIDGES)\.$(PATCH_VERSION)-patch:
	$(MAKE) close_stores
!ifeq ("$(PATCH_LEVEL)", "0")
	$(PRL) Scripts/patch_shared.pl -cp Cartridges -backup "$(BACKUP_DIR)" -version $(PATCH_VERSION)
	$(PRL) Scripts/patch_files.pl -patch Cartridges -backup "$(BACKUP_DIR)" -version $(PATCH_VERSION) -ignore CPAN
!else
	$(PRL) Scripts/patch_files.pl -patch Cartridges -backup "$(BACKUP_DIR)" -version $(PATCH_VERSION)
!endif
!ifdef BACKUP_DIR
	$(MKDIR) "$(BACKUP_DIR)\Cartridges"
!ifdef WINDOWS
	$(CP) "$(EPAGES_CARTRIDGES)"\Cartridges.xml "$(BACKUP_DIR)\Cartridges"\Cartridges.xml
!else
	cp -p "$(EPAGES_CARTRIDGES)"\Cartridges.xml "$(BACKUP_DIR)\Cartridges"\Cartridges.xml
!endif
!endif
	$(MAKE) make_makefile
	$(PRL) Scripts/patchCartrigesXML.pl
	$(PRL) $(DE_EPAGES_DIR)/Installer/Scripts/patchAllCartridgeDirectories.pl ALL

Cartridges\restore: $(EPAGES_CARTRIDGES)\.$(PATCH_VERSION)-restore
$(EPAGES_CARTRIDGES)\.$(PATCH_VERSION)-restore:
	$(MAKE) close_stores
	$(PRL) Scripts/patch_shared.pl -rm Cartridges -version $(PATCH_VERSION)
	$(PRL) Scripts/patch_files.pl -restore Cartridges -backup "$(BACKUP_DIR)" -version $(PATCH_VERSION)

# patch: CPAN

patch_cpan:
	$(PRL) $(DE_EPAGES_DIR)/Installer/Scripts/installCPAN.pl CPAN::EPAGES

# patch: Apache

patch_apache:
!ifndef WINDOWS
	if cd $(EPAGES)/Apache/bin ; then [ ! -f httpd ] || cp -p httpd httpd.sh ; fi
	cat $(OSNAME)/Apache/bin/httpd.sh > $(EPAGES)/Apache/bin/httpd.sh
!endif

# patch: Sybase

Sybase\patch: $(EPAGES)\Sybase\.$(PATCH_VERSION)-patch
$(EPAGES)\Sybase\.$(PATCH_VERSION)-patch:
	$(PRL) Scripts/patch_files.pl -patch Sybase\bin -backup "$(BACKUP_DIR)" -version $(PATCH_VERSION)

Sybase\restore: $(EPAGES)\Sybase\.$(PATCH_VERSION)-restore
$(EPAGES)\Sybase\.$(PATCH_VERSION)-restore:
	$(PRL) Scripts/patch_files.pl -restore Sybase\bin -backup "$(BACKUP_DIR)" -version $(PATCH_VERSION)

# patch: WebRoot

WebRoot\patch: $(EPAGES_WEBROOT)\.$(PATCH_VERSION)-patch
$(EPAGES_WEBROOT)\.$(PATCH_VERSION)-patch:
	$(PRL) Scripts/patch_shared.pl -cp WebRoot -backup "$(BACKUP_DIR)" -version $(PATCH_VERSION)
	$(PRL) Scripts/patch_files.pl -patch WebRoot -backup "$(BACKUP_DIR)" -version $(PATCH_VERSION) -ignore Doc/API/DE_EPAGES
!ifeq ("$(PATCH_LEVEL)", "0")
	$(PRL) Scripts/patch_files.pl -patch Monitor -backup "$(BACKUP_DIR)" -version $(PATCH_VERSION)
!endif

WebRoot\restore: $(EPAGES_WEBROOT)\.$(PATCH_VERSION)-restore
$(EPAGES_WEBROOT)\.$(PATCH_VERSION)-restore:
	$(PRL) Scripts/patch_shared.pl -rm WebRoot -version $(PATCH_VERSION)
	$(PRL) Scripts/patch_files.pl -restore WebRoot -backup "$(BACKUP_DIR)" -version $(PATCH_VERSION) -alpha "$(IS_ALPHA)"
!ifeq ("$(PATCH_LEVEL)", "0")
	$(PRL) Scripts/patch_files.pl -restore Monitor -backup "$(BACKUP_DIR)" -version $(PATCH_VERSION)
!endif

# patch: Stores

Stores\restore: $(EPAGES_STORES)\.$(PATCH_VERSION)-restore
$(EPAGES_STORES)\.$(PATCH_VERSION)-restore:
	$(PRL) Scripts/patch_shared.pl -rm Stores -version $(PATCH_VERSION)
	$(PRL) Scripts/patch_files.pl -restore Stores -backup "$(BACKUP_DIR)" -version $(PATCH_VERSION) -alpha "$(IS_ALPHA)"

# patch: Config

Config\patch: $(EPAGES_CONFIG)\.$(PATCH_VERSION)-patch
$(EPAGES_CONFIG)\.$(PATCH_VERSION)-patch:
	$(PRL) Scripts/patch_files.pl -patch Config -backupall "$(BACKUP_DIR)" -version $(PATCH_VERSION)
!ifdef INITIAL_VERSION_LT_6010000000
	$(PRL) Scripts/patchConfigFiles.pl
	$(PRL) Scripts/EditDatabaseConf.pl -ini database.ini
!endif
!ifdef INITIAL_VERSION_LT_6010002000
	$(PRL) Scripts/EditDatabaseD.pl -version $(PATCH_VERSION)
!endif

Config\restore: $(EPAGES_CONFIG)\.$(PATCH_VERSION)-restore
$(EPAGES_CONFIG)\.$(PATCH_VERSION)-restore:
	$(PRL) Scripts/patch_files.pl -restore Config -backup "$(BACKUP_DIR)" -version $(PATCH_VERSION) -alpha "$(IS_ALPHA)"
!ifdef INITIAL_VERSION_LT_6010000000
!ifdef WINDOWS
	if exist $(EPAGES_CONFIG)\Database.d $(RMDIR) $(EPAGES_CONFIG)\Database.d
!else
	[ ! -d $(EPAGES_CONFIG)\Database.d ] || $(RMDIR) $(EPAGES_CONFIG)\Database.d
!endif
!endif
!ifdef INITIAL_VERSION_LT_6010002000
!ifndef INITIAL_VERSION_LT_6010000000
!ifdef WINDOWS
	if exist $(EPAGES_CONFIG)\Cartridges.d $(RMDIR) $(EPAGES_CONFIG)\Cartridges.d
!else
	[ ! -d $(EPAGES_CONFIG)\Cartridges.d ] || $(RMDIR) $(EPAGES_CONFIG)\Cartridges.d
!endif
!endif
!endif

# targets (patch database)
# ************************

close_stores: loop_stores.mak
	$(MAKE) make_makefile DE_EPAGES_DIR=$(DE_PATCH_DIR)
!ifndef EPAGES_IGNORE_SITE
	$(CD) $(DE_PATCH_DIR) && $(MAKE) close_store STORE=Site STORE_TYPE=Site
!endif
	$(MAKE) -f loop_stores.mak close_stores TGT="$(TGT)"
close_store:
	$(CD) $(DE_PATCH_DIR) && $(MAKE) close_store STORE=$(STORE) STORE_TYPE=$(STORE_TYPE)
	$(DEL) $(TGT)\open_store\$(STORE)

open_stores: loop_stores.mak
	$(MAKE) make_makefile DE_EPAGES_DIR=$(DE_PATCH_DIR)
!ifndef EPAGES_IGNORE_SITE
	$(CD) $(DE_PATCH_DIR) && $(MAKE) open_store STORE=Site STORE_TYPE=Site
!endif
	$(MAKE) -f loop_stores.mak open_stores TGT="$(TGT)"
open_store:
	$(CD) $(DE_PATCH_DIR) && $(MAKE) open_store STORE=$(STORE) STORE_TYPE=$(STORE_TYPE)
	$(DEL) $(TGT)\close_store\$(STORE)

patch_site:
!ifndef EPAGES_IGNORE_SITE
	$(CD) $(DE_EPAGES_DIR) && $(MAKE) patch_store STORE=Site STORE_TYPE=Site
!endif

patch_stores: loop_stores.mak
	$(MAKE) -f loop_stores.mak patch_stores TGT="$(TGT)"
patch_store:
	$(CD) $(DE_EPAGES_DIR) && $(MAKE) patch_store STORE=$(STORE) STORE_TYPE=$(STORE_TYPE)

patch_pooldb:
	$(CD) $(DE_EPAGES_DIR) && $(MAKE) patch_store STORE=ASPoolDB STORE_TYPE=ASPoolDB DOJOBUILD=0

sync_stores: loop_stores.mak
!ifndef EPAGES_IGNORE_SITE
	$(MAKE) sync_store STORE=Site
!endif
	$(MAKE) -f loop_stores.mak sync_stores TGT="$(TGT)"
sync_store:
	$(CD) $(DE_EPAGES_DIR) && $(MAKE) synchronize STORE=$(STORE)

restore_styles: loop_stores.mak
	$(MAKE) -f loop_stores.mak restore_styles TGT="$(TGT)"
restore_style:
	$(PRL) $(DE_EPAGES_DIR)/Design/Scripts/invalidateAllStyles.pl -storename $(STORE)

create_ctmpls: loop_stores.mak
!ifndef EPAGES_IGNORE_SITE
	$(CREATE_CTMPLS) -storename Site -nooutput
!endif
	$(MAKE) -f loop_stores.mak create_ctmpls TGT="$(TGT)"
create_ctmpl:
	$(CREATE_CTMPLS) -storename $(STORE) -nooutput

update_ctmpls: loop_stores.mak
!ifndef EPAGES_IGNORE_SITE
!ifdef CTMPL_TXT
	$(CREATE_CTMPLS) -storename Site -listfile ctmpl-Site.txt -nooutput
!else
	$(CREATE_CTMPLS) -storename Site -delete -nooutput
!endif
!ifdef DEL_SAVED_INCLUDES
!ifdef WINDOWS
	if exist $(EPAGES_STATIC)\Site\SavedIncludes $(RMDIR) $(EPAGES_STATIC)\Site\SavedIncludes
!else
	[ ! -d $(EPAGES_STATIC)\Site\SavedIncludes ] || $(RMDIR) $(EPAGES_STATIC)\Site\SavedIncludes
!endif
!endif
!endif
	$(MAKE) -f loop_stores.mak update_ctmpls TGT="$(TGT)"
update_ctmpl:
!ifdef CTMPL_TXT
	$(CREATE_CTMPLS) -storename $(STORE) -listfile ctmpl-$(STORE).txt -nooutput
!else
	$(CREATE_CTMPLS) -storename $(STORE) -delete -nooutput
!endif
!ifdef DEL_SAVED_INCLUDES
!ifdef WINDOWS
	if exist $(EPAGES_STATIC)\$(STORE)\SavedIncludes $(RMDIR) $(EPAGES_STATIC)\$(STORE)\SavedIncludes
!else
	[ ! -d $(EPAGES_STATIC)\$(STORE)\SavedIncludes ] || $(RMDIR) $(EPAGES_STATIC)\$(STORE)\SavedIncludes
!endif
!endif

test_styles: loop_stores.mak
	$(MAKE) -f loop_stores.mak test_styles TGT="$(TGT)"
test_style:
	$(PRL) $(DE_EPAGES_DIR)/Installer/Scripts/compareCartridgeData.pl -storename $(STORE) -type $(STORE_TYPE)

make_makefile:
	$(CD) $(DE_EPAGES_DIR) && $(PRL) Makefile.PL

loop_stores.mak:
	$(PRL) Scripts/LoopStores.pl -mak makefile> $@.out
	$(MOVE) $@.out $@

patch_version:
	echo $(PATCH_TAG)>> $(EPAGES_CONFIG)\version.txt

# auxiliary targets
# *****************

stop_cron:
!ifneq ("$(PATCH_LEVEL)", "0")
!ifdef WINDOWS
	if not exist "$(DATA_DIR)"\tgt\prehosts.txt $(PRL) Scripts/EditSchedulerConf.pl -stop $(CRON_FILES)
!else
	[ -f "$(DATA_DIR)"\tgt\prehosts.txt ] || $(PRL) Scripts/EditSchedulerConf.pl -stop $(CRON_FILES)
!endif
!endif
	$(MAKE) stop_sic

start_cron:
!ifneq ("$(PATCH_LEVEL)", "0")
!ifdef WINDOWS
	if not exist "$(DATA_DIR)"\tgt\posthosts.txt $(PRL) Scripts/EditSchedulerConf.pl -start $(CRON_FILES)
!else
	[ -f "$(DATA_DIR)"\tgt\posthosts.txt ] || $(PRL) Scripts/EditSchedulerConf.pl -start $(CRON_FILES)
!endif
!endif
	$(MAKE) start_sic

start_sic: $(TGT)\start_sic
$(TGT)\start_sic:
!ifeq ("$(CHANGE_SIC)", "1")
	$(START_SIC)
	$(MAKE) touch DIR=$(@D) ADDF=$(@F)
!endif

stop_sic:
!ifeq ("$(CHANGE_SIC)", "1")
	$(STOP_SIC)
	$(DEL) $(TGT)\start_sic
!endif

stop_web:
!ifeq ("$(PATCH_LEVEL)", "0")
!ifdef APACHE_LOCAL
	$(APACHE_STOP)
!else
	$(STOP_SERVICE) webserver
!endif
!ifndef WINDOWS
!BEGIN_VERBATIM
	-kill $$(ps -u$(EPAGES_WEBUSER) -uep_web -opid= -oppid= -ocomm= 2>/dev/null | while read pid ppid comm ;\
 do [ "$${comm#httpd}" != "$$comm" ] && echo -e "$$pid\n$$ppid" ; done | sort | uniq | tr '\n' ' ') 2>/dev/null
!END_VERBATIM
!endif
!endif

start_web:
!ifeq ("$(PATCH_LEVEL)", "0")
!ifdef APACHE_LOCAL
	$(APACHE_START)
!else
	$(START_SERVICE) webserver
!endif
!endif

stop_epages: .force stop_web
!ifeq ("$(PATCH_LEVEL)", "0")
	$(STOP_SERVICE) cron
	$(STOP_SERVICE) epages
!ifndef WINDOWS
	if pgrep RequestRouter ; then $(ETC_INITD) stop_rr || pkill -9 RequestRouter ; fi
!endif
!endif

start_epages: .force start_web
!ifeq ("$(PATCH_LEVEL)", "0")
	$(START_SERVICE) epages
	$(START_SERVICE) cron
!endif

restart_epages:
!ifneq ("$(PATCH_LEVEL)", "0")
!ifdef IS_ALPHA
	$(PRL) $(DE_EPAGES_DIR)/WebInterface/Scripts/shutdownAppSrv.pl -sleep 0
!endif
!endif

limit_httpd unlimit_httpd new/limit_httpd new/unlimit_httpd:
!ifeq ("$(PATCH_LEVEL)", "0")
!ifdef APACHE_CONF
	./Apache/$(@F).sh -v "$(@D)" -c host-$(HOSTNAME).mk -i "$(INITIAL_INTERNAL_VERSION)"
	[ "$(@F)" != limit_httpd -a -z "$(RESTART_APACHE)" ] || $(APACHE_START)
!endif
!endif

acl:
!ifdef EPAGES_ACL
!ifneq ("$(EXECUTOR)", "root")
	-$(ETC_INITD) +acl all
!endif
!endif

perm:
!ifndef WINDOWS
!ifneq ("$(EXECUTOR)", "root")
	-$(ETC_INITD) perm all -u
!endif
	-$(ETC_INITD) -acl all
	-$(ETC_INITD) perm admin
!endif

touch:
	$(MKDIR) $(DIR)
	$(DEL) $(DIR)\$(DELF)
	$(ECHONL) > $(DIR)\$(ADDF)

del_backup_files:
!ifdef BACKUP_DIR
!ifdef WINDOWS
	if exist "$(BACKUP_DIR)" $(RMDIR) "$(BACKUP_DIR)"
!else
	[ ! -d "$(BACKUP_DIR)" ] || $(RMDIR) "$(BACKUP_DIR)"
!endif
!endif
	$(DEL) $(EPAGES_CARTRIDGES)/.6.*
	$(DEL) $(EPAGES_SHARED)/Monitor/.6.*
	$(DEL) $(EPAGES_STORES)/.6.*
	$(DEL) $(EPAGES_WEBROOT)/.6.*
	$(DEL) $(EPAGES_CONFIG)/.6.*
	$(DEL) $(EPAGES)/Service/.6.*
	$(DEL) $(EPAGES)/RequestRouter/.6.*
	$(DEL) $(EPAGES)/WebAdapter/.6.*
	$(DEL) $(HTTPD_ROOT)/.6.*
	$(DEL) $(EPAGES_PERL)/.6.*
	$(DEL) $(SYBASE)/.6.*
	$(DEL) $(SYBASE)/bin/.6.*
	$(DEL) $(EPAGES)/bin/.6.*
!ifndef WINDOWS
	$(DEL) `dirname $(EPAGES)`/etc/.6.*
!endif

#-*- Makefile -*-
