#!/bin/env bash
#-----BEGIN PGP SIGNATURE-----
#Version: GnuPG v1.2.6 (GNU/Linux)
#Comment: 2008-11-24 ePages Software GmbH (release key) <rnd@epages.de>
#
#iD8DBQBMpK1ll5jMabHSgyARAsIVAJ9H84lR1ecI9WWIh7YsiFQ+fmxUAgCfZXsa
#ViuRV0Vxcy+2o7Fs3WLP7wc=
#=gPyi
#-----END PGP SIGNATURE-----
#line 2

BN=`basename $0`

cn=`hostname --fqdn 2>/dev/null` ||  cn=`hostname`
bits=2048
days=1
httpdconf=/tmp/cert.$$

usage () {
    echo "Usage:"
    echo "  $BN [-h HOST] [-b BITS] [-d DAYS] [-c CONF_DIR]"
    echo
    echo "Options:"
    echo " -h host: certificate host name (default: $cn)"
    echo " -d days: certificate validation period in days (default: $days)"
    echo " -b bits: key size in bits (default: $bits)"
    echo " -c dir:  store certificates in apache conf directory (default: $httpdconf)"
    echo "Example:"
    echo "  $BN -d 365 -b 4092 -c /etc/httpd"
    exit 1
}

while getopts h:b:d:c: c ; do
  case $c in
    h) cn=$OPTARG ;;
    b) bits=$OPTARG ;;
    d) days=$OPTARG ;;
    c) httpdconf=$OPTARG ;;
    *) usage ;;
  esac
done

openssl version || exit 1

host=( $(tr '.' '\n' <<< $cn | tac) )
host[1]=${host[1]:-MyCompany}
host[2]=${host[2]:-MyDepartment}

algo=RSA

crtdir="$httpdconf/conf/ssl.crt"
csrdir="$httpdconf/conf/ssl.csr"
keydir="$httpdconf/conf/ssl.key"

mkdir -m 755 -p $crtdir $csrdir $keydir || exit 1

# find some random files

unset randfiles
for file in /var/log/messages /etc/hosts /etc/group /etc/resolv.conf /bin/ls; do
    [ -r $file ] && randfiles="$randfiles:$file"
done

cfgfile=$httpdconf/.mkcert.cfg
randfiles="${randfiles#:}"
export RANDFILE=$httpdconf/.mkcert.rnd

# create_req_cfg

create_req_cfg() {
cat > $cfgfile <<!
[ req ]
default_bits            = $bits
distinguished_name      = req_DN
prompt                  = no
[ req_DN ]
countryName             = ${host[0]}
stateOrProvinceName     = ${host[1]}
localityName            = ${host[2]}
organizationName        = ${host[1]}
organizationalUnitName  = $1
commonName              = $cn
emailAddress            = $1@${host[1]}.${host[0]}
!
}

# create_x509_cfg

create_x509_cfg() {
            cat > $cfgfile <<!
extensions = x509v3
[ x509v3 ]
subjectAltName   = email:copy
nsComment        = "mod_ssl generated test server certificate"
nsCertType       = $1
!
[ $1 = sslCA ] && cat >> $cfgfile <<!
basicConstraints = CA:true,pathlen:0
!
}

# create CA certificate

openssl genrsa -rand $randfiles -out $keydir/dummy-ca.key $bits || exit 1

create_req_cfg CA

openssl req -config $cfgfile -new -key $keydir/dummy-ca.key -out $csrdir/dummy-ca.csr || exit 1

create_x509_cfg sslCA

openssl x509 -extfile $cfgfile -days $days -signkey $keydir/dummy-ca.key -in $csrdir/dummy-ca.csr -req -out $crtdir/dummy-ca.crt || exit 1

# create server certificate

openssl genrsa -rand $randfiles -out $keydir/dummy-server.key $bits || exit 1

create_req_cfg Server

openssl req -config $cfgfile -new -key $keydir/dummy-server.key -out $csrdir/dummy-server.csr || exit 1

create_x509_cfg server

openssl x509 -extfile $cfgfile -days $days -set_serial $(date +%s) -CA $crtdir/dummy-ca.crt -CAkey $keydir/dummy-ca.key -in $csrdir/dummy-server.csr -req -out $crtdir/dummy-server.crt || exit 1

chmod 600 $httpdconf/conf/ssl.*/dummy-*
chmod 644 $httpdconf/conf/ssl.crt/dummy-*

echo

find $httpdconf/conf/ssl.* -type f -exec ls \-l {} \;
