#!/bin/bash
#-----BEGIN PGP SIGNATURE-----
#Version: GnuPG v1.2.6 (GNU/Linux)
#Comment: 2008-11-24 ePages Software GmbH (release key) <rnd@epages.de>
#
#iD8DBQBMpN7Bl5jMabHSgyARAqYwAJsGmPDlBL702cmZxfv/WQIEUifmYACg2KgN
#pmPdrxNYfPcN1djk1oyUmVA=
#=+IXD
#-----END PGP SIGNATURE-----
#line 2

DN=`dirname "$0"`
cd "$DN"

if [ ${0##*/} = uninstall-epages.sh ] ; then
  unset ALLMATCHES
  case "$@" in
    *-\?*|*--help*)  echo "Usage: $0 [rpm erase options]" ; exit 1 ;;
    *--allmatches*) ALLMATCHES=1 ;;
  esac

  EPAGES_INSTALL_PREFIX=`cd ../.. ; pwd`
  RPM_LIST=$(for i in "" -setup -sybase -docs -appsrv -reqsrv -mysqld -mysql -sybsrv -sybcli -sybcmn -websrv -webconf -perl -patch -common -requires -release ; do
    PKG=$(rpm -q --queryformat '%{NAME}-%{VERSION}-%{RELEASE} %{INSTALLPREFIX}\n' epages$i)
    [ $? -eq 0 ] && echo "$PKG" | while read PKG INSTALLPREFIX ; do
      [ -n "$ALLMATCHES" -o "$EPAGES_INSTALL_PREFIX" = "$INSTALLPREFIX" -o "x$i" = "x-release" ] && echo -e "$PKG "
     done
  done)

  if [ -z "$RPM_LIST" ] ; then
    echo "${0##*/}: No matching ePages RPMs found."
  else
    echo $RPM_LIST | tr ' ' '\n'
    echo -n "Do you really want to uninstall those ePages RPMs? (y/N): "
    read ans
    if [ "$ans" = y -o "$ans" = Y ] ; then
      for i in $RPM_LIST ; do
        echo rpm -e "$@" $i
        rpm -e --allmatches "$@" $i
      done
    fi
  fi
  exit 0
fi

PATCH_PKGS="`find * -name 'epages-patch_*' -printf '%f ' | sed 's/epages-//g;s/-[0-9][^ ]*//g'`"
DEFAULT_PKGS="release docs requires common $PATCH_PKGS patch perl"
if ls */epages-sybsrv-*.rpm >/dev/null 2>&1 ; then
  SYBASE_ARGS=" [appsrv-syb] [sybsrv] [websrv]"
  DEFAULT_PKGS="$DEFAULT_PKGS sybcmn sybcli appsrv sybsrv reqsrv websrv setup"
else
  unset SYBASE_ARGS
  DEFAULT_PKGS="$DEFAULT_PKGS mysql appsrv mysqld reqsrv webconf 6"
fi

usage() {
  echo "Usage: $0 [appsrv] [reqsrv] [mysqld] [webconf]$SYBASE_ARGS [rpm install options]"
  exit 1
}

unset REPLACE
case "$@" in
  *-\?*|*--help*) usage ;;
  *--replacepkgs*|*--replacefiles*) REPLACE=1 ;;
esac

unset PKG_LIST REQ
while [ $# -gt 0 ] ; do
  case "$1" in
    appsrv|mysqld) REQ="release docs requires common $PATCH_PKGS patch perl mysql" ;;
    reqsrv|webconf) REQ="release docs requires common $PATCH_PKGS patch perl" ;;
    appsrv-syb) REQ="release docs requires common $PATCH_PKGS patch perl sybcmn sybcli" ;;
    sybsrv) [ -z "$SYBASE_ARGS" ] && usage || REQ="release docs requires common $PATCH_PKGS patch perl sybcmn sybcli" ;;
    websrv) [ -z "$SYBASE_ARGS" ] && usage || REQ="release docs requires common $PATCH_PKGS patch perl" ;;
    -*) break ;;
    *) usage ;;
  esac
  for j in $REQ $1 ; do
    [[ "$PKG_LIST " == *" "$j" "* ]] || PKG_LIST="$PKG_LIST $j"
  done
  shift
done

[ ! -f /etc/sysconfig/epages6 ] || . /etc/sysconfig/epages6
EPAGES_INSTALL_PREFIX=${EPAGES_INSTALL_PREFIX:-/srv/epages}

case `uname -r` in
  2.6.16[.-]*) KERNEL=2.6.16s ;;
  2.6.18[.-]*) KERNEL=2.6.18r ;;
  2.6.27[.-]*) KERNEL=2.6.27s ;;
  2.6.32-*)    KERNEL=2.6.32r ;;
  2.6.32.*)    KERNEL=2.6.32s ;;
esac

unset REQ ALL RELEASE
[[ $(uname -m) = x86_64 ]] && ARCH=x86_64 || ARCH=i386

for i in ${PKG_LIST:-$DEFAULT_PKGS} ; do
  PKG=`ls */epages-$i[.-]*.$ARCH.rpm 2>/dev/null`
  [ -z "$PKG" ] && PKG=`ls */epages-$i[.-]*.rpm kernel/$KERNEL/epages-$i[.-]*.rpm 2>/dev/null`
  [ -z "$PKG" ] && { ls */epages-$i[.-]*.$ARCH.rpm */epages-$i[.-]*.rpm kernel/$KERNEL/epages-$i[.-]*.rpm; exit 1; }
  ALL="$ALL $PKG"
  [ $i = release ] && QF=$EPAGES_INSTALL_PREFIX || QF='%{INSTALLPREFIX}'
  INSTALLPREFIX=$(rpm -q `basename $PKG .rpm` --queryformat "$QF") \
  && [ -z "$REPLACE" -a "$INSTALLPREFIX" = "$EPAGES_INSTALL_PREFIX" ] && continue
  [ $i = release ] && RELEASE=$PKG || REQ="$REQ $PKG"
done

if [ \( "$REQ" != "${ALL//sybsrv/}" -a -z "${EPAGES_INSTALL_DB_HOST%:ip6}" \) -o \
     \( "$REQ" != "${REQ//appsrv/}" -a -z "${EPAGES_INSTALL_AS_HOST%:ip6}" \) -o \
     \( "$REQ" != "${REQ//reqsrv/}" -a -z "${EPAGES_INSTALL_RR_HOST%:ip6}" \) ] ; then
  host `hostname` >/dev/null 2>&1 || {
    echo "Error: host \`hostname\`: $(host `hostname`)"
    exit 1
  }
fi

if [ -z "$REQ" ] ; then
  # just show that everything is already installed
  eval rpm -hiv --test --replacefiles $ALL
else
  # install epages packages
  rpm -qa | grep -qi gpg-pubkey-b1d28320- || rpm --import pubring.txt
  export LANG=C
  [ -n "$RELEASE" ] && eval rpm -hiv "$@" $RELEASE 2>&1 | tee /tmp/$$.pid
  eval rpm -hiv --prefix $EPAGES_INSTALL_PREFIX "$@" $REQ 2>&1 | tee -a /tmp/$$.pid
fi

unset REQ

# some capabilities needed for epages packages are missing
for cap in $(awk '/ is needed /{print $1}' /tmp/$$.pid 2>/dev/null) ; do

  case $cap in
    apache2|apache2-worker|audit-libs|bind-utils|coreutils) REQ="$REQ $cap" ;;
    gawk|httpd|libstdc++|make|mod_ssl|net-tools|pam) REQ="$REQ $cap" ;;
    procps|python-urlgrabber|rpm-python|sed) REQ="$REQ $cap" ;;
    jre-openjdk) REQ="$REQ java-1.6.0-openjdk" ;;
    jre-sun) REQ="$REQ java-1_4_2-sun" ;;
    jre-ibm) REQ="$REQ java-1_6_0-ibm" ;;
  esac

  K="$KERNEL-$ARCH"
  case $K in
    *r-*)
    case $cap in
      libc.so.6) REQ="$REQ glibc" ;;
      libaio.so.1) REQ="$REQ libaio" ;;
      libasound.so.2) REQ="$REQ alsa-lib" ;;
      libXi.so.6) REQ="$REQ libXi" ;;
      libXtst.so.6) REQ="$REQ libXtst" ;;
      libICE.so.6) REQ="$REQ libICE" ;;
      libSM.so.6) REQ="$REQ libSM" ;;
      libX11.so.6) REQ="$REQ libX11" ;;
      libXext.so.6) REQ="$REQ libXext" ;;
      libXp.so.6) REQ="$REQ libXp" ;;
      libXt.so.6) REQ="$REQ libXt" ;;
      /lib*/libgcc_s.so.1) REQ="$REQ libgcc" ;;
      /lib*/libpam.so.0) REQ="$REQ pam" ;;
      /lib*/libaudit.so.0) REQ="$REQ audit-libs" ;;
      /usr/lib*/libstdc++.so.5) REQ="$REQ  compat-libstdc++-33" ;;
      /usr/lib*/libstdc++.so.6) REQ="$REQ libstdc++" ;;
      /sbin/chkconfig) REQ="$REQ chkconfig" ;;
      /usr/bin/crontab) [ $K = ${K#2.6.18} ] && REQ="$REQ cronie" || REQ="$REQ vixie-cron" ;;
      /usr/bin/getconf) REQ="$REQ glibc-common" ;;
      /usr/bin/mysql) REQ="$REQ mysql" ;;
      /usr/bin/mysqld_safe) REQ="$REQ mysql-server" ;;
      /usr/lib/libjpeg.so.62) REQ="$REQ libjpeg" ;;
      /usr/lib/libpng12.so.0) REQ="$REQ libpng" ;;
      /usr/sbin/sendmail) REQ="$REQ sendmail" ;;
      libgssapi_krb5.so.2) REQ="$REQ krb5-libs" ;;
      */libz.so.0) REQ="$REQ zlib" ;;
      /lib/libexpat.so.*) REQ="$REQ expat" ;;
      /usr/lib/libexpat.so.0) REQ="$REQ compat-expat1" ;;
      /usr/lib/libgdbm.so.2) REQ="$REQ gdbm" ;;
      gnupg) REQ="$REQ gnupg2" ;;
      libssl.so.6) REQ="$REQ openssl" ;;
      /usr/lib/libstdc++-libc6.2-2.so.3) REQ="$REQ compat-libstdc++-296" ;;
    esac ;;
    *s-*)
    case $cap in
      libc.so.6) REQ="$REQ glibc" ;;
      libaio.so.1) REQ="$REQ libaio" ;;
      libasound.so.2) [ $K != ${K#2.6.16} ] && REQ="$REQ alsa" || {
        [ $K != ${K%x86_64} ] && REQ="$REQ libasound2 libasound2-32bit" || REQ="$REQ libasound2"; } ;;
      /lib/libgcc_s.so.1) [ $K != ${K#2.6.16} ] && REQ="$REQ libgcc" || {
        [ $K != ${K%x86_64} ] && REQ="$REQ libgcc43-32bit" || REQ="$REQ libgcc43"; } ;;
      /lib64/libgcc_s.so.1) [ $K != ${K#2.6.16} ] && REQ="$REQ libgcc" || REQ="$REQ libgcc43" ;;
      /lib/libpam.so.0) [ $K != ${K%x86_64} ] && REQ="$REQ pam-32bit" || REQ="$REQ pam" ;;
      /lib64/libpam.so.0) REQ="$REQ pam" ;;
      /lib/libaudit.so.0) [ $K != ${K%x86_64} ] && REQ="$REQ audit-libs-32bit" || REQ="$REQ audit-libs" ;;
      /lib64/libaudit.so.0) REQ="$REQ audit-libs" ;;
      /usr/lib/libstdc++.so.5) [ $K != ${K#2.6.16} ] && REQ="$REQ compat-libstdc++" || {
        [ $K != ${K%x86_64} ] && REQ="$REQ libstdc++33-32bit" || REQ="$REQ libstdc++33"; } ;;
      /usr/lib64/libstdc++.so.5) [ $K != ${K#2.6.16} ] && REQ="$REQ compat-libstdc++" || REQ="$REQ libstdc++33" ;;
      /usr/lib/libstdc++.so.6) [ $K != ${K#2.6.16} ] && REQ="$REQ libstdc++" || {
        [ $K != ${K%x86_64} ] && REQ="$REQ libstdc++43-32bit" || REQ="$REQ libstdc++43"; } ;;
      /usr/lib64/libstdc++.so.6) [ $K != ${K#2.6.16} ] && REQ="$REQ libstdc++" || REQ="$REQ libstdc++43" ;;
      /sbin/chkconfig) REQ="$REQ aaa_base" ;;
      /usr/bin/crontab) REQ="$REQ cron" ;;
      /usr/bin/getconf) REQ="$REQ glibc" ;;
      /usr/bin/mysql) REQ="$REQ mysql-client" ;;
      /usr/bin/mysqld_safe) REQ="$REQ mysql" ;;
      /usr/lib/libjpeg.so.62) [ $K != ${K%x86_64} ] && REQ="$REQ libjpeg-32bit" || REQ="$REQ libjpeg" ;;
      /usr/lib/libpng12.so.0) [ $K != ${K%x86_64} ] && REQ="$REQ libpng-32bit" || REQ="$REQ libpng" ;;
      /usr/sbin/sendmail) REQ="$REQ postfix" ;;
      libgssapi_krb5.so.2) [ $K != ${K%x86_64} ] && REQ="$REQ krb5-32bit" || REQ="$REQ krb5" ;;
      */libz.so.0) [ $K != ${K%x86_64} ] && REQ="$REQ zlib-32bit"  || REQ="$REQ zlib" ;;
      /lib/libexpat.so.1) [ $K != ${K%x86_64} ] && REQ="$REQ libexpat1-32bit"  || REQ="$REQ libexpat1" ;;
      /usr/lib/libexpat.so.1) [ $K != ${K%x86_64} ] && REQ="$REQ expat-32bit"  || REQ="$REQ expat" ;;
      gnupg) [ $K = ${K#2.6.16} ] && REQ="$REQ gpg2" || REQ="$REQ gpg" ;;
      /usr/lib/libssl.so.0.9.8) [ $K != ${K#2.6.16} ] && {
        [ $K != ${K%x86_64} ] && REQ="$REQ openssl-32bit" || REQ="$REQ openssl"; } || {
        [ $K != ${K%x86_64} ] && REQ="$REQ libopenssl0_9_8-32bit" || REQ="$REQ libopenssl0_9_8"; } ;;
      /usr/lib/libstdc++-libc6.2-2.so.3) [ $K != ${K%x86_64} ] && REQ="$REQ compat-32bit"  || REQ="$REQ compat" ;;
    esac

    if  [[ $K != ${K#2.6.16} && -n $(expr $cap : "lib.*\.so\.6$") ]] ; then
      REQ="$REQ xorg-x11-libs" ; [ $K != ${K%x86_64}  ] && REQ="$REQ xorg-x11-libs-32bit"
    else
      case $cap in
        libXi.so.6|libXtst.so.6) i=xorg-x11-libs ; REQ="$REQ $i";  [ $K = ${K%x86_64} ] || REQ="$REQ $i-32bit" ;;
        lib*.so.6) i=xorg-x11-${cap%.so.6} ; REQ="$REQ $i";  [ $K = ${K%x86_64} ] || REQ="$REQ $i-32bit";;
      esac
    fi ;;
  esac

done

if [ -n "$REQ" ] ; then
  REQ=$(echo $REQ | tr ' ' '\n' | sort | uniq | tr '\n' ' ')
  rpm -e epages-release 2>/dev/null
  echo "First install missing RPM packages by executing:"
  [ $KERNEL = ${KERNEL%s} ] && INSTALLER=yum || INSTALLER=zypper
  echo "$INSTALLER install $REQ"
  RET=1
else
  RET=0
fi

rm -f /tmp/$$.pid
exit $RET
