#!/bin/bash
#-----BEGIN PGP SIGNATURE-----
#Version: GnuPG v1.2.6 (GNU/Linux)
#Comment: 2008-11-24 ePages Software GmbH (release key) <rnd@epages.de>
#
#iD8DBQBMjfNjl5jMabHSgyARAhFtAKCxKQYc+8pN+AIVWtAcIGHWUUFboACgufE4
#gucTERR+02HjB5FxqhrOVCE=
#=EFHM
#-----END PGP SIGNATURE-----
#line 2

BN=`basename $0`
DN=`cd $(dirname $0) && pwd`

[[ "x$1" = "x-pre" ]] && PREREQUISITES=1 || unset PREREQUISITES

umask 022

if [[ ! -w /etc/passwd ]] ; then
  echo "Error: Only 'root' can execute this program!"
  exit 1
fi

[[ ! -f /etc/sysconfig/epages6 ]] || . /etc/sysconfig/epages6
EPAGES_PROJECT=${EPAGES_PROJECT:-$EPAGES_INSTALL_RUN_SCRIPT}
EPAGES_PROJECT=${EPAGES_PROJECT:-epages6}

. /etc/default/$EPAGES_PROJECT

[[ -z $PREREQUISITES ]] && EPAGES_IGNORE_CRON=1 /etc/init.d/$EPAGES_PROJECT start

# -----------------------------------------------------------------------------
# find current version
# -----------------------------------------------------------------------------

if grep -q '^\[Cartridges\]' $EPAGES_CONFIG/Database.d/*.conf >/dev/null 2>&1 ; then
  CONF=$EPAGES_CONFIG/Database.d/\*.conf
  CONFFILE=$EPAGES_CONFIG/Database.d/*.conf
elif ls $EPAGES_CONFIG/Cartridges.d/*.conf >/dev/null 2>&1 ; then
  CONF=$EPAGES_CONFIG/Cartridges.d/\*.conf
  CONFFILE=$EPAGES_CONFIG/Cartridges.d/*.conf
elif [[ -f $EPAGES_CONFIG/Cartridges.conf ]] ; then
  CONF=$EPAGES_CONFIG/Cartridges.conf
  CONFFILE=$EPAGES_CONFIG/Cartridges.conf
else
  echo "Install failed, at least one of following files must exist:"
  echo "  \$EPAGES_CONFIG/Database.d/*.conf"
  echo "  \$EPAGES_CONFIG/Cartridges.d/*.conf"
  echo "  \$EPAGES_CONFIG/Cartridges.conf"
  exit 1
fi

CURRENT_VERSION=$(awk -F= '/^DE_EPAGES::Core=/{print $2;exit 0}' $CONFFILE)
unset CURRENT_REL
while IFS=. read -a i ; do CURRENT_REL=(${i[@]}) ; done <<<$CURRENT_VERSION

if [[ -z ${CURRENT_REL[2]} ]] ; then
  echo "Install failed: $CONF: DE_EPAGES::Core=$CURRENT_VERSION: format must be X.Y.Z"
  exit 1
fi

if [[ ${CURRENT_REL[0]} -ne 6 ]] ; then
  echo "Install failed: $CONF: DE_EPAGES::Core=$CURRENT_VERSION: must be 6.X.Y"
  exit 1
fi

if [[ ${CURRENT_REL[1]} -lt 9 ]] ; then
  echo "Install failed: $CONF: DE_EPAGES::Core=$CURRENT_VERSION: must be at least 6.9.X"
  exit 1
fi

# -----------------------------------------------------------------------------
# disable epages-setup.repo
# -----------------------------------------------------------------------------

if [ -f /etc/yum.repos.d/epages-setup.repo ] ; then
  $EPAGES/bin/ini-config.sh -f /etc/yum.repos.d/epages-setup.repo -s epages-setup -k enabled -v 0 -i
fi

# -----------------------------------------------------------------------------
# find latest version
# -----------------------------------------------------------------------------

cd $EPAGES/Shared/Patches >/dev/null 2>&1 || exit 0
ls -d patch_* >/dev/null 2>&1 || exit 0

LATEST_REL=(0 0 0)
for PATCH_TAG in patch_* ; do
  PATCH_TAG=${PATCH_TAG#patch_} 
  [[ -n ${PATCH_TAG//[0-9_]/} ]] && continue

  while IFS=_ read -a i ; do j=(${i[@]}) ; done <<<${PATCH_TAG#patch_}
  [[ -z ${j[2]} ]] && continue

  if [[ ( ${j[0]} -gt ${LATEST_REL[0]} ) || ( ${j[0]} -eq ${LATEST_REL[0]} && ${j[1]} -gt ${LATEST_REL[1]} ) || \
        ( ${j[0]} -eq ${LATEST_REL[0]} && ${j[1]} -eq ${LATEST_REL[1]} && ${j[2]} -gt ${LATEST_REL[2]} ) ]] ; then
    LATEST_REL=( ${j[@]} )
  fi
done

rpm -q epages-appsrv >/dev/null 2>&1 && INSTALL=install || INSTALL=preinstall

# -----------------------------------------------------------------------------
# find language packs
# -----------------------------------------------------------------------------

i=${CURRENT_REL[1]}
j=${CURRENT_REL[2]}

if [[ -n $PREREQUISITES ]] ; then
  [[ $INSTALL = install ]] || exit 0

  LANGUAGES=$($PERL -e '
use DE_EPAGES::Database::API::Connection qw( GetCurrentDBHandle RunOnStore );
my %Languages;
eval {
    foreach my $StoreName (@ARGV) {
        eval { RunOnStore( "Store"=> $StoreName, "Sub" => sub {
            $Languages{$_} = 1 foreach map { $_->[0] }
            @{GetCurrentDBHandle()->execute("SELECT code2 FROM language WHERE isused = 1")};
                           });
        };
    };
};
delete $Languages{de};
delete $Languages{en};
print join(" ", sort keys %Languages);
' `ls $CONFFILE | xargs -i{} basename {} .conf` 2>/dev/null)

  [[ -z $LANGUAGES ]] && exit 0

  while [[ $i -lt ${LATEST_REL[1]} ]] ; do
    j=0
    i=$(($i + 1))
    for k in $LANGUAGES ; do
      LANG_PACK=langpack-$k-epages${LATEST_REL[0]}${i}p$j
      ls $EPAGES/Shared/Patches/$LANG_PACK.tar.gz >/dev/null || exit 1
    done
  done

  while [[ $j -lt ${LATEST_REL[2]} ]] ; do
    j=$(($j + 1))
    for k in $LANGUAGES ; do
      LANG_PACK=langpack-$k-epages${LATEST_REL[0]}${LATEST_REL[1]}p$j
      ls $EPAGES/Shared/Patches/$LANG_PACK.tar.gz >/dev/null || exit 1
    done
  done
  exit 0
fi

# -----------------------------------------------------------------------------
# install all patches
# -----------------------------------------------------------------------------

i=${CURRENT_REL[1]}
j=${CURRENT_REL[2]}

# install minor releases

while [[ $i -lt ${LATEST_REL[1]} ]] ; do
  j=0
  i=$(($i + 1))
  PATCH_TAG=patch_${LATEST_REL[0]}_${i}_$j
  ls $EPAGES/Shared/Patches/$PATCH_TAG >/dev/null || continue
  [ -f $EPAGES/Shared/Patches/$PATCH_TAG/patch.sh ] && PATCHIT=./patch.sh || PATCHIT='$PERL patch.pl'
  if [[ -n $EPAGES_DONT_PATCH ]] ; then
    echo "Now run: cd \$EPAGES/Shared/Patches/$PATCH_TAG && $PATCHIT $INSTALL"
  elif [[ "$PATCHIT" = ./patch.sh ]] ; then
    cd $EPAGES/Shared/Patches/$PATCH_TAG && { ./patch.sh $INSTALL || exit 1; }
  else
    cd $EPAGES/Shared/Patches/$PATCH_TAG && { $PERL patch.pl $INSTALL || exit 1; }
  fi
done

# install patch releases

while [[ $j -lt ${LATEST_REL[2]} ]] ; do
  j=$(($j + 1))
  PATCH_TAG=patch_${LATEST_REL[0]}_${LATEST_REL[1]}_$j
  ls $EPAGES/Shared/Patches/$PATCH_TAG >/dev/null || continue
  [ -f $EPAGES/Shared/Patches/$PATCH_TAG/patch.sh ] && PATCHIT=./patch.sh || PATCHIT='$PERL patch.pl'
  if [[ -n $EPAGES_DONT_PATCH ]] ; then
    echo "Now run: cd \$EPAGES/Shared/Patches/$PATCH_TAG && $PATCHIT $INSTALL"
  elif [[ "$PATCHIT" = ./patch.sh ]] ; then
    cd $EPAGES/Shared/Patches/$PATCH_TAG && { ./patch.sh $INSTALL || exit 1; }
  else
    cd $EPAGES/Shared/Patches/$PATCH_TAG && { $PERL patch.pl $INSTALL || exit 1; }
  fi
done

exit 0
