#!/bin/env bash
#-----BEGIN PGP SIGNATURE-----
#Version: GnuPG v1.2.6 (GNU/Linux)
#Comment: 2008-11-24 ePages Software GmbH (release key) <rnd@epages.de>
#
#iD8DBQBMpXrml5jMabHSgyARAmykAJ9fbqfOeaGbFwl4OEsBLHrBzFRwHgCfSKL1
#aqXl4ouZWcR0j8oEM/Biacc=
#=6CFS
#-----END PGP SIGNATURE-----
#line 2

BN=${0%%*/}
EPOCH=`date "+%s"`

if [[ ! -w /etc/passwd ]] ; then
  echo "$BN: Only 'root' can execute this script."
  exit 1
fi

rpm --quiet -q httpd && HTTPD_RPM=1 || unset HTTPD_RPM
rpm --quiet -q apache2 && APACHE2_RPM=1 || unset APACHE2_RPM

if [[ ( -d /etc/httpd && -n "$HTTPD_RPM" ) || ( ! -d /etc/apache2 && -z "$APACHE2_RPM" ) ]] ; then
  WEB_SERVER=httpd
elif [[ -d /etc/apache2 && ! -d /etc/httpd ]] ; then
  WEB_SERVER=apache2
else
  WEB_SERVER=httpd
fi

usage() {
  echo "Usage:"
  echo "  $BN [-c CONF_DIR] [-g WEB_GROUP] [-u] [-r]"
  echo "Options:"
  echo "  -c: web server configuration directory (default: /etc/$WEB_SERVER)"
  echo "  -g: web server group"
  echo "  -u: undo changes on local web server"
  echo "  -r: rewrite configuration in web server configuration directory"
  exit 1
}

unset CONF_DIR OWN_CONF_DIR WEB_GROUP UNDO REWRITE
while getopts c:g:ur c ; do
  case $c in
    c) CONF_DIR=$OPTARG ; OWN_CONF_DIR=1 ;;
    g) WEB_GROUP=$OPTARG ;;
    u) UNDO=1 ;;
    r) REWRITE=1 ;;
    *) usage ;;
  esac
done

EPAGES_PROJECT=${EPAGES_PROJECT:-epages6}

FILE=/etc/default/"$EPAGES_PROJECT"

ls "$FILE" >/dev/null || exit 1
. "$FILE"
. $EPAGES/bin/configure-epages.sh || exit 1

CONF_DIR=${CONF_DIR:-/etc/$WEB_SERVER}
CONF_PARENT=$(dirname "$CONF_DIR")

[[ -d "$CONF_DIR"/vhosts.d ]] && CONF_D="$CONF_DIR"/vhosts.d || {
  [[ -d "$CONF_DIR"/conf.d ]] && CONF_D="$CONF_DIR"/conf.d || CONF_D="$CONF_DIR"/extra
}

if [[ -z "$OWN_CONF_DIR" ]] ; then
  RUN_SCRIPT=/etc/init.d/$WEB_SERVER
  [[ $WEB_SERVER = httpd ]] && DEFAULT_GROUP=apache || DEFAULT_GROUP=www
  WEB_GROUP=${WEB_GROUP:-$DEFAULT_GROUP}
else
  RUN_SCRIPT="$CONF_PARENT"/bin/apachectl
  if [ -z "$WEB_GROUP" ] ; then
    WEB_GROUP=`ls  "$CONF_DIR"/httpd.conf "$CONF_DIR"/*.conf "$CONF_D"/*.conf 2>/dev/null \
               | xargs cat | grep '^[[:space:]]*Group[[:space:]]' | tail -1 | awk '{print $2}'`
  fi
fi

if [[ -n $OWN_CONF_DIR && -f $CONF_PARENT/bin/httpd2 ]] ; then
    HTTPD_EXE=$CONF_PARENT/bin/httpd2
    WEB_SERVER=httpd
elif [[ -n $OWN_CONF_DIR && -f $CONF_PARENT/bin/httpd ]] ; then
    HTTPD_EXE=$CONF_PARENT/bin/httpd
    WEB_SERVER=httpd
elif [[ -f /usr/sbin/httpd2 ]] ; then
  HTTPD_EXE=/usr/sbin/httpd2
elif [[ -f /usr/sbin/httpd ]] ; then
  HTTPD_EXE=/usr/sbin/httpd
else
  HTTPD_EXE=`ls /usr/sbin/httpd2* 2>/dev/null | head -1`
  if [[ -n "$HTTPD_EXE" ]] ; then
    ln -s $HTTPD_EXE /usr/sbin/httpd2
  else
    HTTPD_EXE=`ls /usr/sbin/httpd* | head -1`
  fi
fi
[[ -z "$HTTPD_EXE" ]] && HTTPD_EXE=`which httpd`
[[ -z "$HTTPD_EXE" ]] && HTTPD_EXE=`which apache2`

export EPAGES_REWRITE_CONF="$CONF_D"/zzz-epages-rewrite.conf

#-----------------------------------------------------------------------
# reset configuration
#-----------------------------------------------------------------------

if [[ -n "$UNDO" ]] ; then

  if [[ -n "$CONF_DIR" ]] ; then
    # reset $CONF_D/*epages-*.conf
    echo "Undo: $CONF_DIR/*epages-*.conf"
    find "$CONF_DIR" -follow -name "*epages-*.conf" -exec rm \-f {} \; 2>/dev/null

    # reset $CONF_DIR/httpd.conf
    for i in `find "$CONF_DIR" -name httpd.conf ! -regex '.*/original/.*' 2>/dev/null` ; do
      if grep -q "#epages#" $i 2>/dev/null ; then
        ORIG=`ls $i.~*~ 2>/dev/null | head -1`
        echo "Undo: $i"
        [[ -n "$ORIG" ]] && cat "$ORIG" > $i
        rm -f $i.~*~
        cp -p $i $i.tmp
        sed 's/#epages#//g' $i.tmp > $i
      fi
      rm -f $i.tmp $i.~*~
    done
  fi

  # reset $EPAGES/Apache
  if [[ -d "$EPAGES"/Apache.unused ]] ; then
    rm -rf "$EPAGES"/Apache.new
    [[ -d "$EPAGES"/Apache ]] && mv "$EPAGES"/Apache "$EPAGES"/Apache.new
    mv "$EPAGES"/Apache.unused "$EPAGES"/Apache
    echo "Undo: $EPAGES/Apache"

    NEW_WEB_GROUP=`find "$EPAGES"/Apache -maxdepth 0 -follow -printf "%g\n"`
    if [[ "$NEW_WEB_GROUP" != "${EPAGES_WEBGROUP%%:*}" ]] ; then
      # change web group
      chgrp -L root / >/dev/null 2>&1 && CHGRP_L=-L || unset CHGRP_L
      chgrp -fR $CHGRP_L "${EPAGES_WEBGROUP%%:*}" "$EPAGES"/Apache/
    fi
  fi

  [[ -n "$OWN_CONF_DIR" ]] && {
    echo "Undo: /etc/init.d/httpd-epages"
    rm -f /etc/init.d/httpd-epages
  }

  if [[ -n "$RUN_SCRIPT" ]] ; then
    # reset $RUN_SCRIPT
    if grep -q "/etc/default/.\?$EPAGES_PROJECT.\?$" $RUN_SCRIPT 2>/dev/null ; then
      echo "Undo: $RUN_SCRIPT"
      ORIG=`ls $RUN_SCRIPT.~*~ 2>/dev/null | head -1`
      [[ -n "$ORIG" ]] && cat "$ORIG" > $RUN_SCRIPT
      cp -p $RUN_SCRIPT $RUN_SCRIPT.tmp
      egrep -v "(/etc/default/.\?$EPAGES_PROJECT.\?$|update-rewrite-conf\.sh|^export (LD_LIBRARY_)\?PATH=|setsebool)" \
        $RUN_SCRIPT.tmp > $RUN_SCRIPT
    fi
    rm -f $RUN_SCRIPT.tmp $RUN_SCRIPT.~*~
    $RUN_SCRIPT stop >/dev/null 2>&1
    echo Starting web server ...
    sleep 2
    $RUN_SCRIPT start
    sleep 2
    ps -f -C ${HTTPD_EXE##*/}
  fi

  exit 0
fi

#-----------------------------------------------------------------------
# define environment
#-----------------------------------------------------------------------

ls "$CONF_D" >/dev/null || usage
ls "$RUN_SCRIPT" >/dev/null || exit 1
[[ -z "$HTTPD_EXE" ]] && { echo "Error: No httpd binary found.";  exit 1; }
ls "$EPAGES"/WebAdapter/zzz-epages-{ssl,httpd}.conf.ept >/dev/null || exit 1

WEB_GROUP_ID=$(getent group "$WEB_GROUP" | awk -F: '{print $3}')
[[ -z "$WEB_GROUP_ID" ]] && { echo "$BN: group not found: $WEB_GROUP"; exit 1; }

if [[ -z "$IGNORE_SELINUX" && "`PATH=$PATH:/usr/sbin getenforce 2>/dev/null | tr A-Z a-z`" = enforcing ]] ; then
  echo "$BN: Can't start $HTTPD_EXE: SELinux security policy is enforced."
  echo "Disable SELinux by executing following command:"
  echo '  $EPAGES/bin/ini-config.sh -f /etc/selinux/config -k SELINUX -v permissive -i'
  echo '  PATH=$PATH:/usr/sbin setenforce Permissive'
  echo "Or configure SELinux so that $HTTPD_EXE can start, then run:"
  echo "  IGNORE_SELINUX=1 $0"
  exit 1
fi

pathmunge () {
  unset OUT
  OIFS=$IFS
  IFS=:
  for i in $1 ; do
    [ -z "$i" ] && continue
    [ -d "$i" ] || continue
    [ "${OUT%:$i:*}" = "${OUT#*:$i:}" ] || continue
    OUT="$OUT:$i"
  done
  IFS=$OIFS
  echo $OUT
}

case `uname -r` in
  2.6.16[.-]*) KERNEL=2.6.16s ;;
  2.6.18[.-]*) KERNEL=2.6.18r ;;
  2.6.27[.-]*) KERNEL=2.6.27s ;;
  2.6.32-*)    KERNEL=2.6.32r ;;
  2.6.32.*)    KERNEL=2.6.32s ;;
esac

LDLP=`pathmunge "$HTTPD_ROOT/lib:/lib:/usr/lib:/usr/X11R6/lib:$LD_LIBRARY_PATH"`
LDLP=`pathmunge "$EPAGES_PERL/lib32/$KERNEL:$EPAGES_PERL/lib32/ssl:$EPAGES_PERL/bin:$LDLP"`
LD_LIBRARY_PATH=`echo $LDLP | tr ':' '\n' | grep -v $EPAGES/Apache | tr '\n' ':'`

$HTTPD_EXE -v 2>/dev/null | grep -q 'Apache/2\.2[^0-9]' || {
  echo "$BN: requires Apache 2.2"
  $HTTPD_EXE -v
  exit 1
}

file -L $HTTPD_EXE | grep -q 'ELF 64-bit' && WA_LIB=lib64/ || WA_LIB=

if [[ -n $OWN_CONF_DIR ]] ; then
  export SR=$(find "$CONF_DIR" -name "*.conf" ! -regex '.*/original/.*' -exec awk '/^[[:space:]]*ServerRoot[[:space:]]+/{print $2}' {} \; | sort | uniq)
  MODULES_PATH=$(find "$CONF_DIR" -name "*.conf" ! -regex '.*/original/.*' -exec awk '/^[[:space:]]*LoadModule[[:space:]]+/{sub(/^[^\/]*$/,"",$3);
                      sub(/\/[^\/]*$/,"",$3); if ($3 !~ /^\//) printf "%s/", ENVIRON["SR"]; print $3}' {} \; | sort | uniq)
elif [[ -f /etc/SuSE-release ]] ; then
  [[ -z $WA_LIB ]] && MODULES_PATH=/usr/lib/apache2* || MODULES_PATH=/usr/lib64/apache2*
else
  [[ -z $WA_LIB ]] && MODULES_PATH=/usr/lib/httpd/modules || MODULES_PATH=/usr/lib64/httpd/modules
fi

#-----------------------------------------------------------------------
# set configuration
#-----------------------------------------------------------------------

unset SERVER_ALIAS
if [[ $EPAGES_INSTALL_WS_ADDR != ${EPAGES_INSTALL_WS_FQDN%%.*} ]] ; then
  SERVER_ALIAS="ServerAlias ${EPAGES_INSTALL_WS_FQDN%%.*}"
fi
if [[ $EPAGES_INSTALL_WS_ADDR != $EPAGES_INSTALL_WS_FQDN && $EPAGES_INSTALL_WS_FQDN != ${EPAGES_INSTALL_WS_FQDN%%.*} ]] ; then
  SERVER_ALIAS="$SERVER_ALIAS\nServerAlias $EPAGES_INSTALL_WS_FQDN"
fi

# create dummy certificate
[[ -s "$CONF_DIR"/conf/ssl.crt/dummy-server.crt ]] && USE_SSL=1 || {
  echo "Run: $EPAGES/bin/dummycert.sh -h $EPAGES_INSTALL_WS_FQDN -c $CONF_DIR"
  "$EPAGES"/bin/dummycert.sh -h $EPAGES_INSTALL_WS_FQDN -c "$CONF_DIR" >/dev/null 2>&1
   [ -s "$CONF_DIR"/conf/ssl.crt/dummy-server.crt ] && USE_SSL=1 || unset USE_SSL
}

# edit $CONF_D/zzz-epages-lb.local
[[ -s "$CONF_D"/zzz-epages-lb.local && -z "$REWRITE" ]] || {
  echo "Edit: $CONF_D/zzz-epages-lb.local"
  sed "
  s,_CONF_DIR_,$CONF_DIR,g;
  s,_SERVERALIAS_,$SERVER_ALIAS,g;
  s,_HOST_,$EPAGES_INSTALL_WS_ADDR,g;
  s,_IP_,$EPAGES_INSTALL_WS_IP,g;
  s,_REWRITE_CONF_,$EPAGES_REWRITE_CONF,g;
  " "$EPAGES"/WebAdapter/zzz-epages-lb.local > "$CONF_D"/zzz-epages-lb.local
}

# edit $CONF_D/zzz-epages-lb.conf.loop.ept
[[ -s "$CONF_D"/zzz-epages-lb.conf.loop && -z "$REWRITE" ]] || {
  echo "Edit: $CONF_D/zzz-epages-lb.conf.loop"
  sed "s,_HOST_,$EPAGES_INSTALL_WS_ADDR,g" \
  "$EPAGES"/WebAdapter/zzz-epages-lb.conf.loop.ept >  "$CONF_D"/zzz-epages-lb.conf.loop
}

# edit $CONF_D/zzz-epages-ssl.conf
[[ -s "$CONF_D"/zzz-epages-ssl.conf && -z "$REWRITE" ]] || {
  echo "Edit: $CONF_D/zzz-epages-ssl.conf"
  sed "
  s,_CONF_DIR_,$CONF_DIR,g;
  s,_EPAGES_,$EPAGES,g;
  s,_SERVERALIAS_,$SERVER_ALIAS,g;
  s,_HOST_,$EPAGES_INSTALL_WS_ADDR,g;
  s,_PORT_,$EPAGES_INSTALL_WS_PORT,g;
  s,_SSLPORT_,$EPAGES_INSTALL_WS_SSL_PORT,g;
  s,_LOCALPORT_,$EPAGES_INSTALL_WS_LOCAL_PORT,g;
  s,_LOCALSSLPORT_,$EPAGES_INSTALL_WS_LOCAL_SSLPORT,g;
  s,_IP_,$EPAGES_INSTALL_WS_IP,g;
  s,_REWRITE_CONF_,$EPAGES_REWRITE_CONF,g;
  " "$EPAGES"/WebAdapter/zzz-epages-ssl.conf.ept > "$CONF_D"/zzz-epages-ssl.conf
}

MODULES="authz_host deflate log_config env expires headers setenvif mime
autoindex cgi vhost_alias negotiation dir actions alias rewrite ssl"

FOUND_MODULES=

# edit $CONF_D/zzz-epages-httpd.conf
[[ -s "$CONF_D"/zzz-epages-httpd.conf ]] || {
  echo "Edit: $CONF_D/zzz-epages-httpd.conf"
  [ "$(/bin/uname -p)" = x86_64 ] && LIB64=lib64/ || unset LIB64

  sed "
  s,_EPAGES_PERL_,$EPAGES_PERL,g;
  s,_EPAGES_,$EPAGES,g;
  s,_HOST_,$EPAGES_INSTALL_WS_ADDR,g;
  s,_PORT_,$EPAGES_INSTALL_WS_PORT,g;
  s,_LOCALPORT_,$EPAGES_INSTALL_WS_LOCAL_PORT,g;
  s,_KERNEL_,$KERNEL,g;
  s,_LIB_,$LIB64,g;
  " "$EPAGES"/WebAdapter/zzz-epages-httpd.conf.ept > "$CONF_D"/zzz-epages-httpd.conf

  for i in $MODULES ; do
    MODULE=$(find $MODULES_PATH -follow -name "mod_$i.so" 2>/dev/null | head -1)
    if [[ -n "$MODULE" ]] ; then
      FOUND_MODULES="$FOUND_MODULES $i"
      sed "s, modules/mod_$i\.so, $MODULE," -i "$CONF_D"/zzz-epages-httpd.conf
    else
      sed "s,\(.* modules/mod_$i\.so\),#\1," -i "$CONF_D"/zzz-epages-httpd.conf
    fi
  done
}

# edit $CONF_D/ssl.conf
if [[ $WEB_SERVER = httpd ]] && grep -q '_default_:443' "$CONF_D"/ssl.conf 2>/dev/null ; then
  echo "Edit: $CONF_D/ssl.conf"
  sed -i 's,_default_:443,127.0.0.1:443,' "$CONF_D"/ssl.conf
fi

# edit /etc/sysconfig/$WEB_SERVER
if [[ -f /etc/sysconfig/$WEB_SERVER ]] ; then
  echo "Edit: /etc/sysconfig/$WEB_SERVER"
  ( . /etc/sysconfig/$WEB_SERVER
    if [[ -n ${!APACHE_MODULES*} ]] ; then
      am=" $APACHE_MODULES "
      for i in $FOUND_MODULES ; do
        [[ "${am/ $i /}" = "$am" ]] && APACHE_MODULES="$APACHE_MODULES $i"
      done
      if [[ $am != " $APACHE_MODULES " ]] ; then
        sed 's,^\([[:space:]]*APACHE_MODULES=\).*,\1"'"$APACHE_MODULES"'",' -i /etc/sysconfig/$WEB_SERVER
      fi
    fi
    if [ -n "$USE_SSL" ] ; then
      if [[ $WEB_SERVER = apache2 && -z ${!APACHE_SERVER_FLAGS*} ]] ; then
        echo 'APACHE_SERVER_FLAGS="SSL"' >> /etc/sysconfig/$WEB_SERVER
      elif [[ $WEB_SERVER = apache2 && "${APACHE_SERVER_FLAGS/SSL/}" = "$APACHE_SERVER_FLAGS" ]] ; then
        sed 's,^\([[:space:]]*APACHE_SERVER_FLAGS=\).*,\1"'"$APACHE_SERVER_FLAGS SSL"'",' -i /etc/sysconfig/$WEB_SERVER
      fi
      if [[ -z ${!OPTIONS*} ]] ; then
        echo OPTIONS=-DSSL >> /etc/sysconfig/$WEB_SERVER
      elif [[ "${OPTIONS/-DSSL/}" = "$OPTIONS" ]] ; then
        sed 's,^\([[:space:]]*OPTIONS=\).*,\1"'"$OPTIONS -DSSL"'",' -i /etc/sysconfig/$WEB_SERVER
      fi
    fi
  )
fi

# edit $CONF_D/zzz-epages-rewrite.conf
echo "Edit: $EPAGES_REWRITE_CONF"
"$EPAGES"/bin/update-rewrite-conf.sh

chmod 644 $CONF_D/zzz-epages*

# edit $CONF_DIR/httpd.conf
for i in `find "$CONF_DIR" -name httpd.conf ! -regex '.*/original/.*'` ; do
  grep -q '#epages#' "$i" && continue
  echo "Edit: $i"
  cp -p "$i" "$i".~$EPOCH~
  ( sed 's,^\([[:space:]]*DocumentRoot[[:space:]]\),#epages#\1,' $i.~$EPOCH~
    echo -e '<IfModule !FCGIAdapter_mod>\nInclude '"$CONF_D"'/zzz-epages-*.conf\n</IfModule>'
  ) > $i
done

# edit DocumentRoot in $CONF_DIR
for i in `find "$CONF_DIR" -name '*.conf' ! -regex '.*/original/.*' ` ; do
  rm -f $i.tmp
  cp -p "$i" "$i".~$EPOCH~
  awk '/^[[:space:]]*DocumentRoot/{print $2}' < $i | while read j ; do
    eval e=$j
    if [[ ! -d "$j" && ! -d "$e" ]] ; then
      echo "Edit: $i"
      cp -p "$i" "$i".tmp
      grep -v "DocumentRoot[[:space:]].*$j" $i.tmp > $i
    fi
  done
  [[ -e "$i".tmp ]] && rm -f "$i".tmp || rm -f "$i".~$EPOCH~
done

# edit /etc/default/$EPAGES_PROJECT
cp -p "$FILE" "$FILE".~$EPOCH~
echo "Edit: $FILE"
set -- `getent group "$WEB_GROUP" | tr : ' '`
$EPAGES/bin/ini-config.sh -f "$FILE" -k EPAGES_WEBGROUP -v "$1:$3" -i
$EPAGES/bin/ini-config.sh -f "$FILE" -k EPAGES_REWRITE_CONF -v "$EPAGES_REWRITE_CONF" -i

# edit /etc/group
for i in ep_appl ${EPAGES_APPUSER:-eprunapp} ; do
  MEMBERS=$(awk -F: '{if ($1 == "'$WEB_GROUP'") {print $4; exit 0}}' /etc/group)
  MEMBERS=${MEMBERS:+$MEMBERS,}
  if [[ ",$MEMBERS" !=  *",$i,"* ]] ; then
    echo "Edit: /etc/group: $i"
    cp -p /etc/group /etc/group.~$EPOCH~
    sed "s/^\($WEB_GROUP:.*:\).*/\1$MEMBERS$i/" /etc/group.~$EPOCH~ > /etc/group
  fi
done

# edit $RUN_SCRIPT
grep -q "bin/update-rewrite-conf\.sh" $RUN_SCRIPT
if [[ $? -ne 0 ]] ; then
  echo "Edit: $RUN_SCRIPT"
  cp -p $RUN_SCRIPT $RUN_SCRIPT.~$EPOCH~
  grep -q '()' $RUN_SCRIPT && WS_PATTERN='()' || WS_PATTERN='^$'
  export WS_PATTERN
  ( sed -n "1,/$WS_PATTERN/p" $RUN_SCRIPT.~$EPOCH~ | grep -v "$WS_PATTERN"
    grep -q "/etc/default/.\?$EPAGES_PROJECT.\?$" $RUN_SCRIPT.~$EPOCH~ || {
      echo ". /etc/default/\"$EPAGES_PROJECT\""
      echo 'export LD_LIBRARY_PATH'
      echo 'export PATH=$PATH:$EPAGES_PERL/bin:/usr/bin:/usr/sbin:/sbin'
      echo 'if type -p setsebool >/dev/null 2>&1 ; then'
      echo '  setsebool httpd_can_network_connect=1 >/dev/null 2>&1'
      echo '  setsebool httpd_can_network_connect_db=1 >/dev/null 2>&1'
      echo '  setsebool httpd_can_network_relay=1 >/dev/null 2>&1'
      echo '  setsebool httpd_disable_trans=1 >/dev/null 2>&1'
      echo 'fi # setsebool'
    }
    echo '[ -x "$EPAGES"/bin/update-rewrite-conf.sh ] && "$EPAGES"/bin/update-rewrite-conf.sh'
    sed -n "/$WS_PATTERN/,"'$p' $RUN_SCRIPT.~$EPOCH~
  ) > $RUN_SCRIPT
fi

# stop epages web server
/etc/init.d/"$EPAGES_PROJECT" stop_httpd

OLD_WEB_GROUP=$EPAGES_WEBGROUP

. "$FILE"

if [[ "$OLD_WEB_GROUP" != "$EPAGES_WEBGROUP" ]] ; then
  # change web group
  chgrp -L root / >/dev/null 2>&1 && CHGRP_L=-L || unset CHGRP_L
  chgrp -fR $CHGRP_L $WEB_GROUP "$EPAGES_LOG/" "$EPAGES_WEBROOT/" "$EPAGES_SHARED"/Monitor/ "$EPAGES"/WebAdapter/
fi

# restart web server
if [[ -d "$EPAGES"/Apache ]] ; then
  rm -rf "$EPAGES"/Apache.unused
  mv "$EPAGES"/Apache "$EPAGES"/Apache.unused
fi

if [[ $RUN_SCRIPT != ${RUN_SCRIPT#/etc/init.d/} ]] ; then
  PERL5LIB= /sbin/chkconfig --del $WEB_SERVER >/dev/null 2>&1
  sed -i 's/\(chkconfig:\) -/\1 35/' $RUN_SCRIPT
  chmod 755 $RUN_SCRIPT
  PERL5LIB= /sbin/chkconfig --add $WEB_SERVER >/dev/null 2>&1
else
  echo "Edit: /etc/init.d/httpd-epages"
  rm -f /etc/init.d/httpd-epages
  ln -s $RUN_SCRIPT /etc/init.d/httpd-epages
fi
$RUN_SCRIPT stop >/dev/null 2>&1
echo Starting web server ...
sleep 2
$RUN_SCRIPT start
sleep 2
ps -f -C ${HTTPD_EXE##*/}
